/*
 * Copyright 2008, 2010
 * Boris Popov <popov.b@gmail.com>
 */

#include "XMLBuildOfTree.hpp"

//-----------------------------------------------
XMLBuildOfTree::XMLBuildOfTree ()
{
	zero_xmldeclar = true;//нет декларации
	root = NULL;//нет root
	last_container = root;
	message = "";
	last_in_declar = false;//последний - декларация
	xmldeclar = NULL;
	return;
}
//-----------------------------------------------

//-----------------------------------------------
void
XMLBuildOfTree::make_open_teg (std::string t)
{
	if ((root != 0) && (container.empty ()))
	{
		//корень закрылся
		message = message + 
			"more than one root element: " +
			t + "\n";
		return;
	}
	last_in_declar = false;
	XMLContain* pC;
	pC = new XMLContain ();
	(pC->Content) = t;
	if (root == 0)
	{
		root = pC;
	}
	last_container = pC;
	if (!container.empty ())
	{
		//тогда сцепить
		XMLContain* pParent;
		pParent = container.top ();
		(*pParent) += (*pC);
	}
	container.push (pC);
	return;
}
//-----------------------------------------------

//-----------------------------------------------
void
XMLBuildOfTree::make_close_teg (std::string t)
{
	if ((root == 0) ||
		(container.empty ()) ||
		(t != ((container.top ())->Content)))
	{
		message = message +
			"close teg in an unknown place: " +
			t + "\n";
		return;
	}
	container.pop ();
	last_container = 0;
	return;
}
//-----------------------------------------------

//-----------------------------------------------
void
XMLBuildOfTree::make_null_teg (std::string t)
{
	if ((root != 0) && (container.empty ()))
	{
		//корень закрылся
		message = message + 
			"more than one root element: " +
			t + "\n";
		return;
	}
	last_in_declar = false;
	XMLContain* pC;
	pC = new XMLContain ();
	(pC->Content) = t;
	if (root == 0)
	{
		root = pC;
	}
	last_container = pC;
	if (!container.empty ())
	{
		//тогда сцепить
		XMLContain* pParent;
		pParent = container.top ();
		(*pParent) += (*pC);
	}
	return;
}
//-----------------------------------------------

//-----------------------------------------------
void
XMLBuildOfTree::make_text (std::string t)
{
	if ((root == 0) || (container.empty ()))
	{
		message = message +
			"text in an unknown place: " +
			t + "\n";

	}
	XMLText* pT;
	pT = new XMLText ();
	pT->Content = t;
	XMLContain* pC;
	pC = container.top ();
	(*pC) += (*pT);
	last_container = 0;
	return;
}
//-----------------------------------------------

//-----------------------------------------------
void
XMLBuildOfTree::put_att (std::string a,
		std::string v)
{
	if ((last_container == 0) &&
			(root == 0) &&
			(container.empty ()))
	{
		//в зоне пролога
		if (last_in_declar)
		{
			if ((xmldeclar->GetAtt (a)) !=
					v)
			{
				xmldeclar->AddAtt (a, v);
			}
		}
	}
	else
	{
		//в зоне боди
		last_container->AddAtt (a, v);

	}
	return;
}
//-----------------------------------------------

//-----------------------------------------------
void
XMLBuildOfTree::make_decl ()
{
	if (root != 0)
	{
		message = message +
			"declaration in an unknown place\n";
		return;
	}
	if (zero_xmldeclar)
	{
		//создать
		xmldeclar = new XMLDeclar;
		zero_xmldeclar = false;
		last_in_declar = true;
	}
	else
	{
		//отказать и уведомит
		message = message + 
			"more than one of declaration\n";
	}
	return;
}
//-----------------------------------------------

//-----------------------------------------------
XMLBuildOfTreeResult
XMLBuildOfTree::get ()
{
	XMLBuildOfTreeResult rez;
	rez.declar = xmldeclar;
	if (root == 0)
	{
		message = message +
			"root element is not found" +
			"\n";
	}
	rez.cont = root;
	rez.messg = &message;
	return rez;
}
//-----------------------------------------------

void XMLBuildOfTree::Make_open_teg (const char* t) {

	make_open_teg
		( clear_from_delim (std::string (t)) );
	return;
}

void XMLBuildOfTree::Make_close_teg (const char* t) {

	make_close_teg
		( clear_from_delim (std::string (t)) );
	return;
}

void XMLBuildOfTree::Make_text (const char* t) {

	std::string text1 (t);
	text1 = clear_from_delim (std::string (text1));
	std::string text2 = m_fMirror.Fetch (text1);
	make_text (text2);
	return;
}

void XMLBuildOfTree::Put_att (AttValue av) {

	std::string at1 (std::string (av.attribute));
	std::string va1 (std::string (av.value));
	at1 = clear_from_delim (at1);
	va1 = clear_from_delim (va1);
	std::string va2 = m_fMirror.Fetch (va1);
	put_att (at1, va2);
	return;
}

void XMLBuildOfTree::Make_decl () {

	make_decl ();
	return;
}

//-----------------------------------------------
bool
XMLBuildOfTree::m_IsItDelimiter (const char ch) {
   bool DELIMITER = true;
   bool NO_DELIMITER = false;
   switch (ch) {
      case ' ':
	 return DELIMITER;
	 break;
      case '\n':
	 return DELIMITER;
	 break;
      case '\t':
	 return DELIMITER;
	 break;
      case '\f':
	 return DELIMITER;
	 break;
      case '\r':
	 return DELIMITER;
	 break;
      default:
	 return NO_DELIMITER;
   }
}
//-----------------------------------------------

//-----------------------------------------------
std::string
XMLBuildOfTree::clear_from_delim (std::string what)
{
	/* удалить разделители
	 * в начале и конце what
	 */

	std::string temp;
	size_t pos = 0;
	size_t start = 0;
	size_t end = what.size () - 1;
	const char* my_string = what.c_str ();
	while (m_IsItDelimiter (my_string [pos]))
	{
		pos++;
	}
	start = pos;//начало без разделителей

	pos = end;
	while (m_IsItDelimiter (my_string [pos]))
	{
		pos--;
	}
	end = pos;//конец без разделителей
	temp.assign (what, start, end - start + 1);
	return temp;
}
//-----------------------------------------------

//-----------------------------------------------
void XMLBuildOfTree::Put_message (const char* t) {

	message = message + std::string (t);
	return;
}
//-----------------------------------------------
