/*
 * Copyright 2008, 2009, 2010 Boris Popov <popov.b@gmail.com>
 */

#define NO_DEBUG_XMLCONTAIN
#ifdef DEBUG_XMLCONTAIN
#include <iostream>
#endif

#include "XMLDocument.hpp"
#include "XMLMirroring.hpp"
#include "XMLLxpe.hpp"

typedef const char* cCString;

//-----------------------------------------------
XMLContain::XMLContain () :
	XMLNode (XMLDocument::Container)
{
	place = in;
	//flag_of_find = false;	
	return;
}
//-----------------------------------------------

//-----------------------------------------------
XMLContain::XMLContain (std::string str) :
	XMLNode (XMLDocument::Container)
{
	place = out;
	//flag_of_find = false;	
	Set (str);
	return;
}
//-----------------------------------------------

//-----------------------------------------------
void
XMLContain::DelAtt (std::string a)
{
	if (ListOfAttr.empty ())
	{
		return;
	}
	std::list <XMLAttr>::iterator p;
	p = ListOfAttr.begin ();
	while (p != ListOfAttr.end ())
	{
		if ((*p).att == a)
		{
			ListOfAttr.erase (p);
			break;
		}
		p++;
	}
	return;
}
//-----------------------------------------------

//-----------------------------------------------
void
XMLContain::AddAtt (std::string a, std::string v)
{
	XMLAttr atemp;
	atemp.att = a;
	atemp.val = v;
	ListOfAttr.push_back (atemp);
	return;
}
//-----------------------------------------------

//-----------------------------------------------
bool
XMLContain::HasItAttr () const
{
	if (ListOfAttr.empty ())
	{
		return false;
	}
	return true;
}
//-----------------------------------------------

//-----------------------------------------------
std::string
XMLContain::GetAtt (std::string a) const
{
	std::string val;
	if (ListOfAttr.empty ())
	{
		return val;
	}
	std::list <XMLAttr>::const_iterator p;
	p = ListOfAttr.begin ();
	while (p != ListOfAttr.end ())
	{
		if ((*p).att == a)
		{
			val = (*p).val;
			break;
		}
		p++;
	}
	return val;
}
//-----------------------------------------------

//-----------------------------------------------

XMLContain&
XMLContain::operator << (XMLContain& e)
{
	return this->operator += (e);
}

//-----------------------------------------------

//-----------------------------------------------

XMLContain&
XMLContain::operator << (XMLContain const &e)
{
	return this->operator += (e);
}

//-----------------------------------------------

//-----------------------------------------------
XMLContain&
XMLContain::operator << (XMLText& t)
{
	return this->operator += (t);
}
//-----------------------------------------------

//-----------------------------------------------
XMLContain&
XMLContain::operator << (XMLText const &t)
{
	return this->operator += (t);
}
//-----------------------------------------------

//-----------------------------------------------
XMLContain::~XMLContain ()
{
	if (!ListOfAttr.empty ())
	{
		ListOfAttr.clear ();
	}
	if (!ListOfChildren.empty ())
	{
		std::list <XMLNode*>::iterator p;
		p = ListOfChildren.begin ();
		while (p != ListOfChildren.end ())
		{
			delete (*p);
			p++;
		}
		ListOfChildren.clear ();
	}
	return;
}
//-----------------------------------------------

//-----------------------------------------------
XMLBodyIter
XMLContain::BeginBodyIter ()
{
	XMLNode* pN;
	if (ListOfChildren.empty ())
	{
		pN = 0;
	}
	else
	{
		pN = this;
	}
	XMLBodyIter XBI (pN);
	if (!ListOfChildren.empty ())
	{
		XBI++;
	}
	return XBI;
}
//-----------------------------------------------

//-----------------------------------------------
XMLBodyIter
XMLContain::EndBodyIter ()
{
	XMLNode* pN = 0;
	XMLBodyIter XBI (pN);
	return XBI;
}
//-----------------------------------------------

//-----------------------------------------------
XMLAttrIter
XMLContain::BeginAttr ()
{
	XMLAttrIter ITA;
	ITA.IPA = ListOfAttr.begin ();
	return ITA;
}
//-----------------------------------------------

//-----------------------------------------------
XMLAttrIter
XMLContain::EndAttr ()
{
	XMLAttrIter ITA;
	ITA.IPA = ListOfAttr.end ();
	return ITA;
}
//-----------------------------------------------

//-----------------------------------------------
XMLContain& 
XMLContain::operator += (XMLContain& e)
{
	//
	//возможно два варианта:
	//e внутри или снаружи.
	//
	if (e.place == in)
	{
		//сцепить
		ListOfChildren.push_back (&e);
		e.pParent = this;
	}
	else
	{
		//справа - контейнер снаружи
		//создать контейнер
		XMLContain* xmlspr = new XMLContain;
		xmlspr->Set (e.Get ());
		(xmlspr->place) = in;
		//перенести атрибуты
		if (e.HasItAttr ())
		{
			std::list <XMLAttr>::iterator p;
			p = (e.ListOfAttr).begin ();
			while (p != (e.ListOfAttr).end ())
			{
				(xmlspr->ListOfAttr).push_back (*p);
				p++;
			}
		}
			//перенос указателей///////////////////////////
			//
			//
			if (!e.ListOfChildren.empty ())
			{
				std::list <XMLNode*>::iterator p;
				p = (e.ListOfChildren).begin ();
				while (p != (e.ListOfChildren).end ())
				{
					(xmlspr->ListOfChildren).push_back (*p);
					(*p)->pParent = xmlspr;
					p++;
				}
				e.ListOfChildren.clear ();
			}
			//
			///////////////////////////////////////////////

		//сцепить
		ListOfChildren.push_back (xmlspr);
		(xmlspr->pParent) = this;
	}
	return *this;
}
//-----------------------------------------------

//-----------------------------------------------
XMLContain& 
XMLContain::operator += (XMLContain const &e)
{
	XMLContain* xmlspr = new XMLContain;
	xmlspr->Set (e.Get ());
	(xmlspr->place) = in;
	
	if (e.HasItAttr ())
	{
	  std::list <XMLAttr>::const_iterator p;
	  p = (e.ListOfAttr).begin ();
	  while (p != (e.ListOfAttr).end ())
	  {
		   (xmlspr->ListOfAttr).push_back (*p);
		   p++;
	  }
	}
	ListOfChildren.push_back (xmlspr);
	(xmlspr->pParent) = this;
	return *this;
}
//-----------------------------------------------

//-----------------------------------------------
XMLContain&
XMLContain::operator += (XMLText& t)
{
	if (t.place == in)
	{
		//сцепить
		ListOfChildren.push_back (&t);
		t.pParent = this;
	}
	else
	{
		XMLText* xmlspr = new XMLText;
		xmlspr->Set (t.Get ());
		(xmlspr->place) = in;
		//сцепить
		ListOfChildren.push_back (xmlspr);
		(xmlspr->pParent) = this;
	}
	return *this;
}
//-----------------------------------------------

//-----------------------------------------------
XMLContain&
XMLContain::operator += (XMLText const &t)
{
	XMLText* xmlspr = new XMLText;
	xmlspr->Set (t.Get ());
	(xmlspr->place) = in;
	//сцепить
	ListOfChildren.push_back (xmlspr);
	(xmlspr->pParent) = this;
	return *this;
}
//-----------------------------------------------

//-----------------------------------------------
XMLContain&
XMLContain::operator = (XMLContain& c)
{
   c.Get ();
   return *this;
}
//-----------------------------------------------

//-----------------------------------------------
std::string
XMLContain::build ()
{
   std::string shift;
   std::string akkum = "";
   level lev;
   lev = GetLevel ();
   for (level i = 0; i < lev; i++)
   {
      shift = shift + "    ";
   }
   // Для каждого ребёнка
   // вызываем этот метод
   if (!ListOfChildren.empty ())
   {
      std::list<XMLNode*>::iterator p;
      p = ListOfChildren.begin ();
      while (p != ListOfChildren.end ())
      {
	 akkum = akkum + ((*p)->build ());
	 p++;
      }
   }
   std::string one_tag;
   std::string two_tag;
   if (ListOfChildren.empty ())
   {
      // нулевой тег
      if (!ListOfAttr.empty ())
      {
	 // есть атрибуты
	 std::string att;
	 att = get_string_attr ();
	 one_tag = shift + "<" + Get () + att + " />\n";
      }
      else
      {
	 one_tag = shift + "<" + Get () + " />\n";
      }
      return one_tag;
   }
   else
   {
      // полный тег
      two_tag = shift + "</" + Get () + ">\n";
      if (!ListOfAttr.empty ())
      {
	 std::string att;
	 att = get_string_attr ();
	 one_tag = shift + "<" + Get () + att + ">\n";
      }
      else
      {
	 one_tag = shift + "<" + Get () + ">\n";
      }
      return one_tag + akkum + two_tag;
   }
}
//-----------------------------------------------

//-----------------------------------------------
std::string
XMLContain::get_string_attr ()
{
   mr::ToMirror tMirror;
   std::string out;
   std::list<XMLAttr>::iterator p;
   p = ListOfAttr.begin ();
   while (p != ListOfAttr.end ())
   {
      std::string attstring = (*p).val;
      out = out + " " + (*p).att + "=\"" +
	 tMirror.Fetch (attstring)
	 + "\"";
      p++;
   }
   return out;
}
//-----------------------------------------------

//-----------------------------------------------
void
XMLContain::Search
(const xpath path, XMLSearchResult& result) {
   
   m_SearchFromContain (path, result);
   return;
}
//-----------------------------------------------
		
//-----------------------------------------------
void
XMLContain::m_SearchFromRoot
(const xpath path, XMLSearchResult& result) {

   result.Clear ();
   //обработка path
   cCString xp = path.c_str ();
   xmlxpathextr::XPathExtractor XPE (xp);

   //получить контейнер
   xmlxpathcnt::Container* Cnt = XPE.Get ();
   if (Cnt == 0) {
      
      return;
   }

   //если нет слэшей в начале
   //то это не от рута
   if (!Cnt->GoesFirstStepFromRoot ()) {

      return;
   }
  
   Cnt->GoStart ();
   
   xmlxpath::Step* step = Cnt->Pop ();

   if (step == 0) {

      return;
   }

   step->GetFirstFromRoot (this, result);

   step = Cnt->Pop ();

   while (step != 0) {
      //обработка каждого шага
      //Get ();
      step->Get (result);
      step = Cnt->Pop ();
   }

   return;
}
//-----------------------------------------------

//-----------------------------------------------
void
XMLContain::m_SearchFromContain
(const xpath path, XMLSearchResult& result) {
   
   result.Clear ();
   //обработка path
   cCString xp = path.c_str ();
   xmlxpathextr::XPathExtractor XPE (xp);

   //получить контейнер
   xmlxpathcnt::Container* Cnt = XPE.Get ();
   if (Cnt == 0) {
      
      #ifdef DEBUG_XMLCONTAIN
      std::cerr << "no container";
      #endif
      return;
   }

   if (Cnt->GoesFirstStepFromRoot ()) {

      return;
   }

   Cnt->GoStart ();

   ////

   xmlxpath::Step* step = Cnt->Pop ();

   if (step == 0) {

      return;
   }

   result.m_container.push_back (this);

   while (step != 0) {
      //обработка каждого шага
      //Get ();
      step->Get (result);
      step = Cnt->Pop ();
   }
   ////

   return;
}
//-----------------------------------------------

//-----------------------------------------------
XMLChildIter
XMLContain::BeginChildIter () {

   XMLChildIter it;
   it.m_IOC = ListOfChildren.begin ();
   return it;
}
//-----------------------------------------------

//-----------------------------------------------
XMLChildIter
XMLContain::EndChildIter () {
   
   XMLChildIter it;
   it.m_IOC = ListOfChildren.end ();
   return it;
}
//-----------------------------------------------
