/*
 * Copyright 2008, 2009, 2010 Boris Popov <popov.b@gmail.com>
 */

#include "XMLDocument.hpp"

//-----------------------------------------------
XMLDocument::XMLDocument () :
Prolog (*(new XMLProlog)),
Body (*(new XMLBody)),
m_InputString (NULL) {

	return;
}
//-----------------------------------------------

//-----------------------------------------------
XMLDocument::XMLDocument (XMLContain& c) :
Prolog (*(new XMLProlog)),
Body (*(new XMLBody (c))),
m_InputString (NULL) {

	return;
}
//-----------------------------------------------

//-----------------------------------------------
XMLDocument::XMLDocument
(const String& str) :
Prolog (*(new XMLProlog)),
Body (*(new XMLBody)),
m_InputString (str.c_str ()) {

   m_ParseString ();
   return;
}
//-----------------------------------------------

//-----------------------------------------------
XMLDocument::XMLDocument
(const char* str) :
Prolog (*(new XMLProlog)),
Body (*(new XMLBody)),
m_InputString (str) {

   m_ParseString ();
   return;
}
//-----------------------------------------------

//-----------------------------------------------
void XMLDocument::m_ParseString () {

   if (m_InputString == NULL) {

      return;
   }

   XMLParser parser (m_InputString);
   XMLParserResult ParsRes;
   ParsRes = parser.Get ();
     
   if (ParsRes.messg == "") {
      
      correct (ParsRes);
   }

   return;
}
//-----------------------------------------------

//-----------------------------------------------
bool XMLDocument::Load (std::string FileName) {
	
	message = "";
	bool flag;
	XMLParser* pParser;
	pParser = new XMLParser ( FileName.c_str () );
	XMLParserResult ParsRes;
	ParsRes = pParser->GetFromFile ();
	if (ParsRes.messg != "")
	{
		flag = false;
		message = ParsRes.messg;
	}
	else
	{
		flag = true;
		//откорректировать
		//структуру документа
		//согласно
		//загруженному.
		correct (ParsRes);
	}
	delete pParser;
	return flag;
}

bool
XMLDocument::Load (XMLI& i)
{
	message = "";
	bool flag;
	XMLParser* pParser;
	pParser = new XMLParser ( i.Get () );
	XMLParserResult ParsRes;
	ParsRes = pParser->Get ();
	if (ParsRes.messg != "")
	{
		flag = false;
		message = ParsRes.messg;
	}
	else
	{
		flag = true;
		//откорректировать
		//структуру документа
		//согласно
		//загруженному.
		correct (ParsRes);
	}
	delete pParser;
	return flag;
}
//-----------------------------------------------

//-----------------------------------------------
XMLDocument::~XMLDocument ()
{
	delete &Prolog;
	delete &Body;
	return;
}
//-----------------------------------------------

//-----------------------------------------------
bool
XMLDocument::Save (XMLO& o)
{
	message = "";
	bool flag;
	XMLBuilder* pBuilder;
	pBuilder = new XMLBuilder (Prolog, Body);
	std::string text = pBuilder->Get ();
	if (text == "")
	{
		flag = false;
		message = "nothing to save";
	}
	else
	{
		flag = true;
		o.Put (text.c_str ());
	}
	delete pBuilder;
	return flag;
}
//-----------------------------------------------

//-----------------------------------------------
std::string
XMLDocument::GetMessage ()
{
	return message;
}
//-----------------------------------------------

//-----------------------------------------------
void
XMLDocument::correct (XMLParserResult pr)
{
	delete Body.Root;
	Body.Root = pr.cont;

	
	////декларация
	if (pr.declar == 0)
	{
		//отключить декларацию
		Prolog.GetDeclar ().SW = XMLDeclar::OFF;
	}
	else
	{
		Prolog.GetDeclar ().SW = XMLDeclar::ON;
		Prolog.GetDeclar ().ListOfAtt = 
			(pr.declar)->ListOfAtt;
		delete pr.declar;
	}
	////
	return;
}
//-----------------------------------------------

//-----------------------------------------------
XMLProlog&
XMLDocument::GetProlog () const
{
	return Prolog;
}
//-----------------------------------------------

//-----------------------------------------------
XMLBody&
XMLDocument::GetBody () const
{
	return Body;
}
//-----------------------------------------------

//-----------------------------------------------
XMLDocument&
XMLDocument::operator = (XMLDocument& d)
{
   d.GetMessage ();
   return *this;
}
//-----------------------------------------------
