/*
 * Copyright 2008, Boris Popov <popov.b@gmail.com>
 */

#define NODEBUG
#include "XMLLexemExtractor.hpp"

// =========================================== //
lf::Delimiter::~Delimiter () {
   return;
}

bool
lf::Compare
(const Delimiter* a, const Delimiter* b) {
   return ( (a->m_length) > (b->m_length) );
}

lf::Length
lf::Delimiter::GetLength () const {
   return m_length;
}

bool
lf::Delimiter::Cmp (cCString string) {

   bool SUCCESS = true;
   bool UNSUCCESS = false;

   const void* my = m_delimiter;
   const void* out = string;

   if (memcmp (my, out, m_length) == 0) {

      #ifdef DEBUG
      std::cerr << "= " << m_delimiter << "\n";
      #endif
      return SUCCESS;
   }
   return UNSUCCESS;
}
      
lf::cCString
lf::Delimiter::GetName () const {

   return m_name;
}
// =========================================== //

// =========================================== //
lf::ContainerDelimiters::ContainerDelimiters
(const Delimiter& delimiter) {

   m_listDelimiters.push_back (&delimiter);
   return;
}

lf::ContainerDelimiters::~ContainerDelimiters
() {

   m_listDelimiters.clear ();
   return;
}

void
lf::ContainerDelimiters::Add
(const Delimiter& delimiter) {

   m_listDelimiters.push_back (&delimiter);
   return;
}

void
lf::ContainerDelimiters::Sort () {
   
   m_listDelimiters.sort (lf::Compare);
   return;
}

lf::IteratorContainerDelimiters
lf::ContainerDelimiters::
Begin () {
   
   return m_listDelimiters.begin ();
}

lf::IteratorContainerDelimiters
lf::ContainerDelimiters::
End () {

   return m_listDelimiters.end ();
}

#ifdef DEBUG
void
lf::f_print
(const Delimiter* delimiter) {

   Delimiter* del = const_cast
      <Delimiter*> (delimiter);
   std::cerr << del->GetName () <<"\n"; 
   return;
}

void
lf::ContainerDelimiters::PRINT () {
   
   IteratorContainerOfDelimiters begin;
   IteratorContainerOfDelimiters end;
   begin = m_listDelimiters.begin ();
   end = m_listDelimiters.end ();
   for_each (begin, end, f_print);

   return;
}
#endif
// =========================================== //

// =========================================== //
lf::Lexem::~Lexem () {

   return;
}

lf::TypeLexem
lf::Lexem::GetTypeLexem () const {

   return m_type;
}

lf::Position
lf::Lexem::GetStart () const {

   return m_start;
}

lf::Position
lf::Lexem::GetStop () const {

   return m_stop;
}
// =========================================== //

// =========================================== //
lf::LexemDelimiter::~LexemDelimiter () {

   return;
}
      
const lf::Delimiter&
lf::LexemDelimiter::Get () const {

   return m_delimiter;
}
// =========================================== //

// =========================================== //
void
lf::f_ClearLexemStar (const Lexem* lexem) {
   delete lexem;
   return;
}

lf::ContainerLexems::
ContainerLexems () {

   m_listLexems.clear ();
   return;
}

lf::ContainerLexems::
~ContainerLexems () {

   IteratorContainerLexems begin;
   begin = m_listLexems.begin ();

   IteratorContainerLexems end;
   end = m_listLexems.end ();
   
   for_each (begin, end, f_ClearLexemStar);

   m_listLexems.clear ();
   return;
}

void
lf::ContainerLexems::Add
(const Lexem* lexem) {

   m_listLexems.push_back (lexem);
   return;
}

bool
lf::ContainerLexems::Empty () {

   return m_listLexems.empty ();
}

lf::IteratorContainerLexems
lf::ContainerLexems::Begin () {

   return m_listLexems.begin ();
}

lf::IteratorContainerLexems
lf::ContainerLexems::End () {
   
   return m_listLexems.end ();
}
      
lf::IteratorContainerLexems
lf::ContainerLexems::Insert
(IteratorContainerLexems it,
 const Lexem* lexem) {

   return m_listLexems.insert (it, lexem);
}
// =========================================== //

// =========================================== //
void
lf::FinderLexems::m_Find () {

   if (m_length <= 0) {

      return;
   }

   const char* currentPtr = m_string;

   while (m_pos < m_length) {
     
      Length delta = 1;
      IteratorContainerDelimiters it;
      it = m_containerDelimiters.Begin ();

      IteratorContainerDelimiters itend;
      itend = m_containerDelimiters.End ();

      while (it != itend) {

	 Delimiter* delimiter =
	    const_cast <Delimiter*> (*it);

	 if (delimiter->Cmp (currentPtr)) {

	    delta = delimiter->GetLength ();
	    const LexemDelimiter* lexem =
	       new LexemDelimiter
	       (DELIMITER_LEXEM,
		m_pos, (m_pos + delta - 1),
		**it);
	    m_containerLexems.Add (lexem);
	    break;
	 }
	 else {

	    delta = 1;
	 }
	 it++;
      }
      currentPtr = currentPtr + delta;
      m_pos = m_pos + delta;
   }
   return;
}

void
lf::FinderLexems::m_AddOrdinaryLexems () {

   if (m_containerLexems.Empty ()) {
      // здесь надо тогда добавить
      // единственную
      // лексему
      //
      const Lexem* lexem =
	 new Lexem (ORDINARY_LEXEM,
	       0,
	       (m_length - 1));
      m_containerLexems.Add (lexem);
      return;
   }

   m_pos = 0;

   IteratorContainerLexems it;
   it = m_containerLexems.Begin ();

   IteratorContainerLexems itend;
   itend = m_containerLexems.End ();

   while (it != itend) {

      Position start = (*it)->GetStart ();
      Position stop = (*it)->GetStop ();
      
      #ifdef DEBUG
	 std::cerr << start << " " <<
	    stop << "\n";
      #endif
      
      if (start > m_pos) {
	 
	 const Lexem* lexem =
	    new Lexem
	    (ORDINARY_LEXEM, m_pos, (start - 1));
	 
	 it =
	 m_containerLexems.Insert (it, lexem);
	 it++;

	 #ifdef DEBUG
	 std::cerr << lexem->GetStart () << " " <<
	    lexem->GetStop () << "\n";
	 #endif

      }

      it++;
      m_pos = stop + 1;
   }
   return;
}

void
lf::FinderLexems::m_AddLastLexem () {
   
   IteratorContainerLexems itend;
   itend = m_containerLexems.End ();
   itend--;
   const Lexem* last = *itend;
   
   if ( (m_length - 1) > (last->GetStop ()) ) {
  
      #ifdef DEBUG
      std::cerr << "*\n";
      #endif
      
      const Lexem* lexem =
	 new Lexem (ORDINARY_LEXEM,
	       (last->GetStop () + 1),
	       (m_length - 1));
      m_containerLexems.Add (lexem);
      
   }
   return;
}
// =========================================== //

// =========================================== //
lf::ExtractorLexems::~ExtractorLexems () {

   return;
}

lf::Position
lf::ExtractorLexems::GetPos () const {

   return m_pos;
}

lf::cCString
lf::ExtractorLexems::GetLexem () const {

   return m_lexem;
}

lf::TypeLexem
lf::ExtractorLexems::GetTypeLexem () const {

   return m_type;
}

lf::cCString
lf::ExtractorLexems::GetNameDelimiter () const {

   return m_nameDelimiter;
}
      
bool
lf::ExtractorLexems::Fetch () {

   bool SUCCESS = true;
   bool UNSUCCESS = false;

   if (m_it != m_itend) {

      m_pos = (*m_it)->GetStart ();
      m_type = (*m_it)->GetTypeLexem ();
      Position stop = (*m_it)->GetStop ();
      m_lexem = m_stringExtractor.GetString
	 (m_inputString, m_pos, stop);
      if (m_type == DELIMITER_LEXEM) {

	 const LexemDelimiter* lexemd =
	 (const LexemDelimiter*) (*m_it);
	 const Delimiter& delimiter = lexemd->Get ();
	 m_nameDelimiter = delimiter.GetName ();
      }
      else {

	 m_nameDelimiter = "";
      }
      m_it++;
      return SUCCESS;
   }
   else {

      return UNSUCCESS;
   }
}
// =========================================== //
