/*
 * Copyright 2008, Boris Popov <popov.b@gmail.com>
 */

#ifndef __lexemextractor2__hpp__
#define __lexemextractor2__hpp__

#include <XMLStringExtractor.hpp>
#include <cstring>
#include <list>
#include <algorithm>

#define NODEBUG
#ifdef DEBUG
#include <iostream>
#endif


namespace lf {

typedef const char* cCString;
typedef char* CString;
typedef size_t Position;
typedef size_t Length;

class Delimiter;
class Lexem;

#ifdef DEBUG
void f_print (const Delimiter* delimiter);
#endif

typedef std::list <const Delimiter*>
   ContainerOfDelimiters;

typedef std::list <const Delimiter*>::iterator
   IteratorContainerDelimiters;

typedef std::list <const Delimiter*>::
   iterator IteratorContainerOfDelimiters;

typedef std::list <const Lexem*> ContainerOfLexems;

typedef std::list <const Lexem*>::
   iterator IteratorContainerLexems;

// =========================================== //
bool Compare
   (const Delimiter* a, const Delimiter* b);
// =========================================== //

// =========================================== //
class Delimiter {

   public:
      Delimiter (cCString name,
	    cCString delimiter) :
	 m_name (name),
	 m_delimiter (delimiter),
	 m_length (strlen (delimiter)) {}
      ~Delimiter ();
      Length GetLength () const;
      
      friend bool Compare (const Delimiter* a,
	    const Delimiter* b);
      
      bool Cmp (cCString string);
      cCString GetName () const;

   private:
      cCString m_name;
      cCString m_delimiter;
      const Length m_length;
};
// =========================================== //

// =========================================== //
class ContainerDelimiters {

   public:
      ContainerDelimiters (const Delimiter&
	    delimiter);
      ~ContainerDelimiters ();
      void Add (const Delimiter& delimiter);
      void Sort ();

      IteratorContainerDelimiters Begin ();
      IteratorContainerDelimiters End ();

      #ifdef DEBUG
      void PRINT ();
      #endif

   private:
      ContainerOfDelimiters m_listDelimiters;
};
// =========================================== //

// =========================================== //
enum TypeLexem {

   DELIMITER_LEXEM,
   ORDINARY_LEXEM
};
// =========================================== //

// =========================================== //
class Lexem {

   public:
      Lexem (const TypeLexem type,
	    const Position start,
	    const Position stop) :
	 m_type (type),
	 m_start (start),
	 m_stop (stop) {}
      ~Lexem ();

      TypeLexem GetTypeLexem () const;
      Position GetStart () const;
      Position GetStop () const;

   protected:
      const TypeLexem m_type;
      const Position m_start;
      const Position m_stop;
};
// =========================================== //

// =========================================== //
class LexemDelimiter : public Lexem {
   
   public:
      LexemDelimiter (const TypeLexem type,
	    const Position start,
	    const Position stop,
	    const Delimiter& delimiter) :

	 Lexem (type, start, stop),
	 m_delimiter (delimiter) {}

      ~LexemDelimiter ();

      const Delimiter& Get () const;

   private:
      const Delimiter& m_delimiter;
};
// =========================================== //

// =========================================== //
class ContainerLexems;

class FinderLexems {

   public:
      FinderLexems (cCString string,
	    ContainerDelimiters& cd,
	    ContainerLexems& cl) :

	 m_string (string),
	 m_containerDelimiters (cd),
	 m_length (strlen (string)),
	 m_containerLexems (cl) {
	    
	    m_pos = 0;
	    m_Find ();
	    m_AddOrdinaryLexems ();
	    m_AddLastLexem ();
	    return;
	 }

      ~FinderLexems () {
	 return;
      }

   private:
      void m_AddLastLexem ();
      Position m_pos;
      void m_Find ();
      void m_AddOrdinaryLexems ();
      cCString m_string;
      ContainerDelimiters& m_containerDelimiters;
      const Length m_length;
      ContainerLexems& m_containerLexems;
};
// =========================================== //

// =========================================== //
void f_ClearLexemStar (const Lexem* lexem);
// =========================================== //

// =========================================== //
class ContainerLexems {

   public:
      ContainerLexems ();
      ~ContainerLexems ();

      void Add (const Lexem* lexem);
      bool Empty ();
      IteratorContainerLexems Begin ();
      IteratorContainerLexems End ();

      IteratorContainerLexems
	 Insert (IteratorContainerLexems it,
	       const Lexem* lexem);

   private:
      ContainerOfLexems m_listLexems;
};
// =========================================== //

// =========================================== //
class ExtractorLexems {

   public:
      ExtractorLexems (cCString string,
	    ContainerLexems& cont) :
	 m_inputString (string),
	 m_containerLexems (cont) {

	    m_it = m_containerLexems.Begin ();
	    m_itend = m_containerLexems.End ();
	 }
      ~ExtractorLexems ();

      bool Fetch ();
      
      Position GetPos () const;
      cCString GetLexem () const;
      TypeLexem GetTypeLexem () const;
      cCString GetNameDelimiter () const;

   private:
      IteratorContainerLexems m_it;
      IteratorContainerLexems m_itend;

      se::StringExtractor m_stringExtractor;
      cCString m_inputString;
      ContainerLexems& m_containerLexems;

      Position m_pos;
      cCString m_lexem;
      TypeLexem m_type;
      cCString m_nameDelimiter;
};
// =========================================== //
}
#endif
