/*
 * Copyright 2008, Boris Popov <popov.b@gmail.com>
 * Copyright 2009, Boris Popov <popov.b@gmail.com>
 */

/*
 * Lexem XPath Extractor
 */

#include "XMLLxpe.hpp"
#define NO_LXPE_DEBUG

// =========================================== //
xmlxpathcnt::Container*
xmlxpathextr::
XPathExtractor::Get () {

   #ifdef LXPE_DEBUG
   std::cerr << std::endl;
   std::cerr << m_string << "\n";
   #endif

   if (m_el->Fetch ()) {
      
      if (strcmp (m_el->GetNameDelimiter (),
	      "SLASH") == 0) {
	 
	 m_container = new
	    xmlxpathcnt::Container
	    (xmlxpathcnt::FirstStep_FROM_ROOT);
	    if (m_el->Fetch ()) {

	       string lexem (m_el->GetLexem ());
	       if (m_IsItAll (lexem)) {
		  // (/*)
		  m_MakeChildAny ();
	       }
	       
	       else {
		  // (/affds)
		  m_MakeChildLexem (lexem);
	       }
	    }

	    else {
	       
	       return m_container;
	    }
      }

      else if (strcmp (m_el->GetNameDelimiter (),
	      "DOUBLESLASH") == 0) {

	 m_container = new
	    xmlxpathcnt::Container
	    (xmlxpathcnt::FirstStep_FROM_ROOT);
	    if (m_el->Fetch ()) {
	       
	       string lexem (m_el->GetLexem ());
	       if (m_IsItAll (lexem)) {
		  // (//*)
		  m_MakeChildAnyAll ();
	       }

	       else {
		  // (//affds)
		  m_MakeChildLexemAll (lexem);
	       }
	    }

	    else {
	       
	       return m_container;
	    }
      }

      else {
	 m_container = new
	    xmlxpathcnt::Container
	    (xmlxpathcnt::FirstStep_FROM_CURRENT);
	    string lexem (m_el->GetLexem ());
	    if (m_IsItAll (lexem)) {
	       // (*)
	       m_MakeChildAny ();
	    }

	    else {
	       // (whatever)
	       m_MakeChildLexem (lexem);
	    }
      }
   }

   else {
      
      return m_container;
   }
   m_Fetch ();
   return m_container;
}
	 
void
xmlxpathextr::
XPathExtractor::m_Fetch () {

   while (m_el->Fetch ()) {

      cCString delimiter = m_el->GetNameDelimiter ();

      #ifdef LXPE_DEBUG
      std::cerr << delimiter;
      #endif	 

      if (strcmp (delimiter, "SLASH")
	    == 0) {
	 // (/)
	 if (m_el->Fetch ()) {

	    string lexem (m_el->GetLexem ());
	    if (m_IsItAll (lexem)) {
	       // (/*)
	       m_MakeChildAny ();
	    }
	    else {
	       // (/ars)
	       m_MakeChildLexem (lexem);
	    }
	 }
      }

      else if (strcmp (delimiter, "DOUBLESLASH")
	    == 0) {
	 // (//)
	 if (m_el->Fetch ()) {

	    string lexem (m_el->GetLexem ());
	    if (m_IsItAll (lexem)) {
	       // (/*)
	       m_MakeChildAnyAll ();
	    }
	    else {
	       // (/ars)
	       m_MakeChildLexemAll (lexem);
	    }
	 }
      }

      else if (strcmp (delimiter, "LEFTBRACKET")
	    == 0) {
	 // ([)
	 if (m_el->Fetch ()) {

	    string lexem (m_el->GetLexem ());
	    m_MakePredicat (lexem);
	 }
      }

      else if (strcmp (delimiter, "RIGHTBRACKET")
	    == 0) {
	 // (])
	 // пропускаем!
      }

      else {
	 
	 #ifdef LXPE_DEBUG
	 std::cerr << std::endl;
	 std::cerr << "error in xpath string";
	 std::cerr << std::endl;
	 #endif
      }
   }
   return;
}

bool
xmlxpathextr::
XPathExtractor::m_IsItAll (string str) {

   bool YES = true;
   bool NO = false;

   if (str == "*") {

      return YES;
   }
   return NO;
}

xmlxpathextr::
XPathExtractor::~XPathExtractor () {
   
   if (m_container != 0) {
      m_container->GoStart ();
      const xmlxpath::Step* step = m_container->Pop ();
      while (step != 0) {

	 const xmlxpath::Predicat* predicat=
	    step->GetPredicat ();
	 if (predicat != 0) {

	    delete predicat;
	 }
	 delete step;
	 step = m_container->Pop ();
      }
      delete m_container;
   }
   delete m_el;
   delete m_fl;
   return;
}


void
xmlxpathextr::
XPathExtractor::m_MakeChildAny () {
   // (*)
   m_last = new xmlxpath::ChildAny;
   m_container->Push (*m_last);
   return;
}

void
xmlxpathextr::
XPathExtractor::m_MakeChildLexem (string str) {
   // (whatever)
   m_last = new xmlxpath::ChildLexem
      (str);
   m_container->Push (*m_last);
   return;
}

void
xmlxpathextr::
XPathExtractor::m_MakeChildAnyAll () {
   // (//*)
  m_last = new xmlxpath::ChildAnyAll;
  m_container->Push (*m_last);
   return;
}

void
xmlxpathextr::
XPathExtractor::m_MakeChildLexemAll (string str) {
   // (//affds)
   m_last = new xmlxpath::ChildLexemAll
      (str);
   m_container->Push (*m_last);
   return;
}
	 
void
xmlxpathextr::
XPathExtractor::m_MakePredicat (string str) {

   #ifdef LXPE_DEBUG
   std::cerr << std::endl;
   std::cerr << "making predicat " << str;
   std::cerr << std::endl;
   #endif
   m_MakePredicatAtt (str);
   return;
}

void
xmlxpathextr::
XPathExtractor::m_MakePredicatAtt (string str) {

   XPathExtractorAtt extractor (str);

   #ifdef LXPE_DEBUG
   std::cerr << std::endl;
   std::cerr << "att " << extractor.GetAtt ();
   std::cerr << std::endl;
   std::cerr << "val " << extractor.GetVal ();
   std::cerr << std::endl;
   std::cerr << "all " << extractor.GetAll ();
   std::cerr << std::endl;
   std::cerr << "func " << extractor.GetFunction ();
   std::cerr << std::endl;
   #endif

   string val = extractor.GetVal ();
   string att = extractor.GetAtt ();
   
   if (extractor.GetAll ()) {
      // @* and not(@*)
      if (extractor.GetFunction () == "not") {

	 const xmlxpath::Predicat_ATTALL* predicat =
	    new xmlxpath::Predicat_ATTALL (
		  xmlxpath::FUNC_NOT);
	 m_last->SetPredicat (predicat);
      }

      else {
	 
	 const xmlxpath::Predicat_ATTALL* predicat =
	    new xmlxpath::Predicat_ATTALL ();
	 m_last->SetPredicat (predicat);
      }
   }

   else if (val == "") {
      // @dasd and not(@dasd)
      if (extractor.GetFunction () == "not") {

	 const xmlxpath::Predicat_ATT* predicat =
	    new xmlxpath::Predicat_ATT (att,
		  xmlxpath::FUNC_NOT);
	 m_last->SetPredicat (predicat);
      }

      else {
	 
	 const xmlxpath::Predicat_ATT* predicat =
	    new xmlxpath::Predicat_ATT (att);
	 m_last->SetPredicat (predicat);
      }
   }

   else {
      // @dasd='s' and not(@dasd='s')
      if (extractor.GetFunction () == "not") {
	 
	 const xmlxpath::Predicat_ATTandVAL* predicat =
	    new xmlxpath::Predicat_ATTandVAL (att, val,
		  xmlxpath::FUNC_NOT);
	 m_last->SetPredicat (predicat);
      }

      else {
	 
	 const xmlxpath::Predicat_ATTandVAL* predicat =
	    new xmlxpath::Predicat_ATTandVAL (att, val);
	 m_last->SetPredicat (predicat);
      }
   }

   return;
}
// =========================================== //

// =========================================== //
xmlxpathextr::
XPathExtractorAtt::
XPathExtractorAtt (string lexem) :
   m_att (""),
   m_val (""),
   m_all (false),
   m_function ("") {

      if (lexem.find ("not(") == 0) {

	 m_function = "not";
      }
      
      if (lexem.find ("*") != 
	    NoPosition) {
	 
	 m_all = true;
	 return;
      }

      if (lexem.find ("=") !=
	    NoPosition) {
	 // att & val
	 m_GetAttVal (lexem);
      }
      else {
	 // only att
	 m_GetAtt (lexem);
      }

   return;
}
	 
void
xmlxpathextr::
XPathExtractorAtt::m_GetAtt (string lexem) {

   #ifdef LXPE_DEBUG
   std::cerr << "*****************";
   #endif

   Position star;
   star = lexem.find ("@");
   if (star == NoPosition) {

      return;
   }
   
   Position end;
   end = lexem.find (")");
   if (end == NoPosition) {

      end = lexem.size ();
   }
   else {

      end = end;
   }

   Position start = star + 1;

   if (end > start) {

      string att;
      att.assign (lexem, start, (end - start));
      m_att = att;
   }
   return;
}
	 
void
xmlxpathextr::
XPathExtractorAtt::m_GetAttVal (string lexem) {
   
   Position star;
   star = lexem.find ("@");
   if (star == NoPosition) {

      return;
   }

   Position eq;
   eq = lexem.find ("=");

   Position end = eq;
   Position start = star + 1;
   if (end > start) {

      string att;
      att.assign (lexem, start, (end - start));
      m_att = att;
   }

   else {

      return;
   }

   start = eq + 2;
   if (m_function != "") {

      end = lexem.size () - 1 - 1;
   }

   else {

      end = lexem.size () - 1;
   }

   if (end > start) {

      string val;
      val.assign (lexem, start, (end - start));
      m_val = val;
   }
   return;
}
// =========================================== //
