/*
 * Copyright 2008, Boris Popov <popov.b@gmail.com>
 * Copyright 2009, Boris Popov <popov.b@gmail.com>
 */

/*
 * Lexem XPath Extractor
 * Из лексем генерируем
 * объекты шагов локализации.
 */

#ifndef __lxpe__hpp__
#define __lxpe__hpp__

#define LXPE_NODEBUG
#ifdef LXPE_DEBUG
#include <iostream>
#endif

#include <XMLLexemExtractor.hpp>
#include <XMLSteps.hpp>
#include <XMLContainerSteps.hpp>
#include <string>
#include <cstring>

namespace xmlxpathextr {

   typedef const char* cCString;
   typedef std::string string;
   typedef std::string::size_type Position;
   
   #define NoPosition (std::string::npos)

// =========================================== //
   class XPathExtractor {

      public:
	 XPathExtractor (cCString string) :
	    m_string (string),
	    m_slash (lf::Delimiter
		  ("SLASH", "/")),
	    m_doubleSlash (lf::Delimiter
		  ("DOUBLESLASH", "//")),
	    m_left (lf::Delimiter
		  ("LEFTBRACKET", "[")),
	    m_right (lf::Delimiter
		  ("RIGHTBRACKET", "]")),
	    m_cd (lf::ContainerDelimiters
		  (m_slash)),
	    m_container (0) {
	       m_cd.Add (m_doubleSlash);
	       m_cd.Add (m_left);
	       m_cd.Add (m_right);
	       m_cd.Sort ();
	       m_fl = new lf::FinderLexems
		  (m_string, m_cd, m_cl);
	       m_el = new lf::ExtractorLexems
		  (m_string, m_cl);
	    }

	 ~XPathExtractor ();
	 xmlxpathcnt::Container* Get ();

      private:
	 cCString m_string;
	 const lf::Delimiter m_slash;
	 const lf::Delimiter m_doubleSlash;
	 const lf::Delimiter m_left;
	 const lf::Delimiter m_right;
	 lf::ContainerDelimiters m_cd;
	 lf::ContainerLexems m_cl;
	 const lf::FinderLexems* m_fl;
	 lf::ExtractorLexems* m_el;
	 xmlxpathcnt::Container* m_container;
	 void m_Fetch ();
	 bool m_IsItAll (string str);
	 xmlxpath::Step* m_last;
	 void m_MakeChildAny ();
	 void m_MakeChildLexem (string str);
	 void m_MakeChildAnyAll ();
	 void m_MakeChildLexemAll (string str);
	 void m_MakePredicat (string str);
	 void m_MakePredicatAtt (string str);
   };
// =========================================== //

// =========================================== //
   class XPathExtractorAtt {

      public:
	 XPathExtractorAtt (string lexem);
	 ~XPathExtractorAtt () { return; }

	 string GetAtt () {
	    return m_att;
	 }

	 string GetVal () {
	    return m_val;
	 }

	 bool GetAll () {
	    return m_all;
	 }

	 string GetFunction () {
	    return m_function;
	 }

      private:
	 string m_att;
	 string m_val;
	 bool m_all;
	 string m_function;
	 void m_GetAtt (string str);
	 void m_GetAttVal (string str);
   };
// =========================================== //

}

#endif
