/*
 * Copyright (c) 2010, Boris Popov <popov.b@gmail.com>
 *
 * Permission to use, copy, modify, and/or distribute
 * this software for any purpose with or without fee
 * is hereby granted, provided that the above copyright
 * notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS
 * ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL,
 * DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH
 * THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "string_common.h"
#include "bool.h"

#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#ifndef __cbuffer__h__
#define __cbuffer__h__

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

typedef size_t Position;
typedef size_t Size;
typedef FILE* FD;

enum _TypeResult {TypeResultEOF, TypeResultNORMAL};
typedef enum _TypeResult TypeResult;

struct _Result {

		TypeResult type;
		char ch;
};
typedef struct _Result Result;

struct _CBufferFromString {

		CString string;
		Size size;
		Position pos;
};
typedef struct _CBufferFromString* CBufferFromString;

struct _CBufferFromFD {

		char* buffer;
		Size size;
		Size current_size;
		Position pos;
		FD fd;
};
typedef struct _CBufferFromFD* CBufferFromFD;

struct _CBufferFromSTDIN {

		char* buffer;
		Size size;
		Size current_size;
		Position pos;
};
typedef struct _CBufferFromSTDIN* CBufferFromSTDIN;

struct _CBufferFrom {

		char* buffer;
		Size size;
		Size current_size;
		Position pos;
};
typedef struct _CBufferFrom* CBufferFrom;

enum _BufferType {BufferTypeString, BufferTypeFD, BufferTypeSTDIN};
typedef enum _BufferType BufferType;

typedef void* Buffer;

struct _CBuffer {

		BufferType btype;
		Buffer buffer;
};


typedef struct _CBuffer* CBuffer;

Result GetChar (CBuffer);
Bool HasItEof (const Result);
char GetCharFromResult (const Result);

/** 
  * @addtogroup Buffer
  * @{
  */
/** Make buffer from file descriptor */
CBuffer InitCBufferFromFD (const FD);

/** Make buffer from C-string (const char*) */
CBuffer InitCBufferFromString (const CString);

/** Make buffer from stdin */
CBuffer InitCBufferFromSTDIN ();

/** Destroying buffer */
void DestroyCBuffer (CBuffer);
/** @} */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __cbuffer__h__ */
