/*
 * Copyright (c) 2010, Boris Popov <popov.b@gmail.com>
 *
 * Permission to use, copy, modify, and/or distribute
 * this software for any purpose with or without fee
 * is hereby granted, provided that the above copyright
 * notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS
 * ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL,
 * DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH
 * THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include "cstringakk.h"

/*---------------------------------------------------*/
StringAkkN** InitStringAkkN (const SizeBuffer sb) {
	
	StringAkkN** akk;
	akk  = (StringAkkN**) malloc (sizeof (void**));
	*akk = (StringAkkN*)  malloc (sizeof (StringAkkN));
	(*akk)->string = (Byte*) malloc (sb);
	(*akk)->size_buffer = sb;
	(*akk)->step_size_buffer = sb;
	(*akk)->current_size_buffer = 1;
	*((*akk)->string) = '\0';
	return akk;
}

void DestoryStringAkkN (StringAkkN** san) {

	if (san == NULL)  return;
	if ((*san) == NULL) {
	
		free (san);
		return;
	}
		
	if ((*san)->string != NULL) {

			free ((*san)->string);
	}

	free (*san);
	free (san);
	return;
}

void AddByteToStringAkkN (StringAkkN** sakk,
		const Byte byte) {

	if (sakk    == NULL)  return;
	if ((*sakk) == NULL)  return;
	
	if ( ((*sakk)->current_size_buffer + 1) <=
			(*sakk)->size_buffer) {

		*(((*sakk)->string) +
				((*sakk)->current_size_buffer) - 1) = byte;
		*(((*sakk)->string) +
				((*sakk)->current_size_buffer)) = '\0';
		(*sakk)->current_size_buffer++;
	
	} else {

		SizeBuffer newsize = (*sakk)->size_buffer +
			(*sakk)->step_size_buffer;
		SizeBuffer newstepsize = (*sakk)->step_size_buffer;

		StringAkkN** saknew = InitStringAkkN (newsize);
		if (saknew            == NULL)  return;
		if ((*saknew)         == NULL)  return;
		if ((*saknew)->string == NULL)  return;

		strcpy (((*saknew)->string), ((*sakk)->string));
		*(((*saknew)->string) +
				((*sakk)->current_size_buffer) - 1) = byte;
		*(((*saknew)->string) +
				((*sakk)->current_size_buffer)) = '\0';
		(*saknew)->current_size_buffer =
			((*sakk)->current_size_buffer) + 1;
		(*saknew)->step_size_buffer = newstepsize;
		free ((*sakk)->string);
		free (*sakk);
		(*sakk) = (*saknew);
		free (saknew);
	}

	return;
}

CString GetStringFromStringAkkN (StringAkkN** sakk) {
	
	if (sakk == NULL)       return "";
	if ( (*sakk) == NULL )  return "";
	return (*sakk)->string;
}
/*---------------------------------------------------*/
