///////////////////////////////////////////////////////////////////////////////
// Библиотека алгоритмов небесной механики
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
// Реализация измерения c учетом времени прохождения сигнала
///////////////////////////////////////////////////////////////////////////////
#include "ADelayMeasurement.h"
#include "GreenwichFrame.h"
#include "Consts.h"
///////////////////////////////////////////////////////////////////////////////
// Конструктор
//-----------------------------------------------------------------------------
ADelayMeasurement::ADelayMeasurement() : AMeasurement(),
                                         // По умолчанию назначем точность определения времени распространения
                                         // сигнала равной одной сотой секунды
                                         TimePrec(0.0001 * 1.e-3)
{
}

//-----------------------------------------------------------------------------
// Метод определяет время t, которое соответствует моменту излучения сигнала с 
// объекта, а также его положение на орбите в этот момент времени
//-----------------------------------------------------------------------------
bool ADelayMeasurement::DetectPosition(ITraject& trj, TimeJD& t,
                                       const IFrame* fr, Vect6* f) const
{
    Vect6 fo;
    Vect3 rc;
    double tau, tau1;
    GreenwichFrame gf;

    if (fr == 0)
        fr = &gf;

    if (f == 0)
        f = &fo;

    tau = 0;
    if (!DetectRecieverPos(t, *fr, rc))
        return false;
    for (;;) {
        t = GetTime();
        t -= tau / 86.4;
        if (!trj.Predict(t, *fr, *f))
            return false;
        tau1 = sqrt((rc.x - f->x) * (rc.x - f->x) +
                (rc.y - f->y) * (rc.y - f->y) +
                (rc.z - f->z) * (rc.z - f->z)) /
            Clight;
        if (fabs(tau - tau1) < TimePrec)
            return true;
        tau = tau1;
    }
    return false;
}

///////////////////////////////////////////////////////////////////////////////
