///////////////////////////////////////////////////////////////////////////////
// Библиотека алгоритмов небесной механики
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
// Реализация измерения c учетом времени прохождения сигнала
///////////////////////////////////////////////////////////////////////////////
#ifndef BNOTOOL_ADELAYMEASUREMENT_H
#define BNOTOOL_ADELAYMEASUREMENT_H
///////////////////////////////////////////////////////////////////////////////
#include "IMeasurement.h"

///////////////////////////////////////////////////////////////////////////////
class ADelayMeasurement : public AMeasurement
{
public:
    // Конструктор по умолчанию
    ADelayMeasurement();
    // Установка точности опреления времени распространения сигнала
    void SetTimePrecision(double dt)
    {
        TimePrec = dt;
    };
    // Метод возвращает точность опреления времени распространения сигнала
    double GetTimePrecision() const
    {
        return TimePrec;
    };

protected:
    // Точность, с которой определяется время распространения сигнала
    double TimePrec;
    // Метод определяет время t, которое соответствует моменту излучения сигнала с объекта,
    // а также его положение на орбите в этот момент времени
    virtual bool DetectPosition(ITraject& trj, TimeJD& t, const IFrame* fr = 0,
                                Vect6* f = 0) const;
    // Метод определяет положение приемника сигнала f в момент времени t
    // в системе отсчета, определяемой параметром fr
    virtual bool DetectRecieverPos(const TimeJD& t, const IFrame& fr, Vect3& f) const = 0;
};

///////////////////////////////////////////////////////////////////////////////
#endif
