///////////////////////////////////////////////////////////////////////////////
// Библиотека алгоритмов небесной механики
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
// Класс обеспечивающий кеширование параметров при вычислении ускорений
// Используется приближенный алгоритм вычисления положения КО в Гринвиче
// (учитывается только поворот из-за вращения Земли)
///////////////////////////////////////////////////////////////////////////////
#ifndef BNOTOOL_ACCADAPTER_H
#define BNOTOOL_ACCADAPTER_H
///////////////////////////////////////////////////////////////////////////////
#include "PhasePoint.h"
#include "MMode.h"

///////////////////////////////////////////////////////////////////////////////
class AccAdapter
{
public:
    // Инициализация 
    virtual ~AccAdapter()
    {
    };
    // Инициализация 
    virtual void Init(const TimeJD& t, const MotionMode& m);
    // Вычисление параметров
    virtual void Set(const TimeJD& t, const Vect6& rv_j2000, bool precItrf);

    // Вычисляемые параметры
    // Матрица перехода от J2000 к Гринвичу в момент инициализации
    TransMtx BJ2000toGFCS;
    // Момент инициализации (в шкале TDB)
    TimeJD T0;
    // Текущее время
    TimeJD T_TDB;
    // Координаты КО в J2000
    Vect3 RJ2000;
    // Скорость КО в J2000
    Vect3 VJ2000;
    // Координаты КО в Гринвиче
    Vect3 RGr;
    // Скорость КО в Гринвиче
    Vect3 VGr;
    // Положение Солнца в J2000
    Vect3 SunJ2000;
    // Положение Солнца в Гринвиче
    Vect3 SunGr;
    // Плотность атмосферы
    double Density;
    // Текущая матрица перехода от J2000 к Гринвичу
    TransMtx CJ2000toGFCS;
    // Текущая матрица перехода от Гринвичa к J2000
    TransMtx CGFCStoJ2000;
    // Описание учитваемых факторов
    const MotionMode* MMode;
};

///////////////////////////////////////////////////////////////////////////////
#endif
