///////////////////////////////////////////////////////////////////////////////
// Библиотека алгоритмов небесной механики
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
// Класс координат дальность, прямое восхождение, склонение и скорости их
// изменения
///////////////////////////////////////////////////////////////////////////////
#include "AlphDelD6Coords.h"
///////////////////////////////////////////////////////////////////////////////
// Конструктор по умолчанию
//-----------------------------------------------------------------------------

AlphDelD6Coords::AlphDelD6Coords() : Coords()
{
    Dv = DAlpha = DDelta = 0.0;
}

//-----------------------------------------------------------------------------
// Установка координат по декартовым
//-----------------------------------------------------------------------------
void AlphDelD6Coords::SetXYZ(const Vect6& src)
{
    Coords.SetXYZ(src);

    double sd = sin(Coords.Delta);
    double cd = cos(Coords.Delta);
    double sa = sin(Coords.Alpha);
    double ca = cos(Coords.Alpha);
    double d = Coords.Dist;

    Dv = src.vx * cd * ca + src.vy * cd * sa + src.vz * sd;
    DDelta = (-src.vx * sd * ca - src.vy * sd * sa + src.vz * cd) / (d * 1E3);
    DAlpha = (-src.vx * sa + src.vy * ca) / (d * 1E3 * cd);
}

void AlphDelD6Coords::GetPrtDrv(const Vect6& crd, const FrameConverter6D& frameConverter, DMatrix& m)
{
    double ra2 = crd.Len2();
    double ras2 = crd.x * crd.x + crd.y * crd.y;
    double ras = sqrt(ras2);
    Vect3 dalfaa, ddeltaa, dalfaf, ddeltaf;
    dalfaa[0] = -crd.y / ras2;
    dalfaa[1] = crd.x / ras2;
    dalfaa[2] = 0.0;
    ddeltaa[0] = -crd.x * crd.z / (ras * ra2);
    ddeltaa[1] = -crd.y * crd.z / (ras * ra2);
    ddeltaa[2] = ras / ra2;
    FrameConverter3D inv;
    frameConverter.Converter3D.Invert(inv);
    dalfaf = dalfaa;
    dalfaf.MulLeft(inv.Rotation);
    ddeltaf = ddeltaa;
    ddeltaf.MulLeft(inv.Rotation);

    m.Init(2, 3);
    m.Init(0.0);
    for (int i = 0; i < 3; i++) {
        m(0, i) = dalfaf[i];
        m(1, i) = ddeltaf[i];
    }
}

//-----------------------------------------------------------------------------
// Получение декартовых координат
//-----------------------------------------------------------------------------
bool AlphDelD6Coords::GetXYZ(Vect6& dst) const
{
    Coords.GetXYZ(dst);

    double sd = sin(Coords.Delta);
    double cd = cos(Coords.Delta);
    double sa = sin(Coords.Alpha);
    double ca = cos(Coords.Alpha);
    double d = Coords.Dist;

    dst.vx = Dv * cd * ca - d * sd * ca * DDelta - d * cd * sa * DAlpha;
    dst.vy = Dv * cd * sa - d * sd * sa * DDelta + d * cd * ca * DAlpha;
    dst.vz = Dv * sd + d * cd * DDelta;

    return true;
}

///////////////////////////////////////////////////////////////////////////////
