///////////////////////////////////////////////////////////////////////////////
// Библиотека алгоритмов небесной механики
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
// Класс координат дальность, прямое восхождение, склонение
///////////////////////////////////////////////////////////////////////////////
#include "AlphDelDCoords.h"
#include "Consts.h"
///////////////////////////////////////////////////////////////////////////////
// Конструктор по умолчанию
//-----------------------------------------------------------------------------
AlphDelDCoords::AlphDelDCoords()
{
    Dist = Alpha = Delta = 0.0;
}

//-----------------------------------------------------------------------------
// Установка координат по декартовым
//-----------------------------------------------------------------------------
void AlphDelDCoords::SetXYZ(const Vect3& src)
{
    Dist = src.Len();

    Delta = Dist > 0 ? asin(src.z / Dist) : 0.0;
    Alpha = (src.x != 0 || src.y != 0) ? atan2(src.y, src.x) : 0.0; // <-- [-PI,+PI]
    if (Alpha < 0)
        Alpha += Pi2;
}

//-----------------------------------------------------------------------------
// Получение декартовых координат
//-----------------------------------------------------------------------------
void AlphDelDCoords::GetXYZ(Vect3& dst) const
{
    dst.z = Dist * sin(Delta);
    dst.x = Dist * cos(Delta) * cos(Alpha);
    dst.y = Dist * cos(Delta) * sin(Alpha);
}

///////////////////////////////////////////////////////////////////////////////
