///////////////////////////////////////////////////////////////////////////////
// Библиотека алгоритмов небесной механики
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
#include "AprioriConst.h"
///////////////////////////////////////////////////////////////////////////////
// Связанный список априорных констант
///////////////////////////////////////////////////////////////////////////////
// Добавление в список
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
void DAprioriConst::GetDelVector(const DVector& u, DVector& del) const
{
    del.Init(1U);

    del(0) = Value - u(Index);
}

//-----------------------------------------------------------------------------
// Получить матрицу частных производных
//-----------------------------------------------------------------------------
void DAprioriConst::GetMatrix(const DVector& u, DMatrix& m) const
{
    m.Init(1, u.Dim());
    m.Init(0.0);
    m(0, Index) = 1.0;
}

//-----------------------------------------------------------------------------
// Получить весовую матрицу
//-----------------------------------------------------------------------------
void DAprioriConst::GetWeightMatrix(DSymmMatrix& m) const
{
    m.Init(1U);

    m(0, 0) = 1.0 / (Mse * Mse);
}

//-----------------------------------------------------------------------------
// Создать наследника
//-----------------------------------------------------------------------------
IAprioriConst* DAprioriConst::Clone() const
{
    DAprioriConst* ret = new DAprioriConst;
    *ret = *this;
    return ret;
}

//-----------------------------------------------------------------------------
// Установка нового значения по вектору
//-----------------------------------------------------------------------------
void DAprioriConst::Set(const DVector& u)
{
    Value = u(Index);
}

//-----------------------------------------------------------------------------
// Установка параметров
//-----------------------------------------------------------------------------
void DAprioriConst::SetParams(unsigned index, double v, double mse)
{
    Index = index;
    Value = v;
    Mse = mse;
}

///////////////////////////////////////////////////////////////////////////////
// Связанный список априорных констант
///////////////////////////////////////////////////////////////////////////////
// Добавление в список
//-----------------------------------------------------------------------------
void IAprioriConstList::Add(const IAprioriConst& a)
{
    DeleteByType(a.GetType());
    IAprioriConstEntry* en = (IAprioriConstEntry *)a.Clone();
    insert_last(en, 0);
}

//-----------------------------------------------------------------------------
// Удалените по типу
//-----------------------------------------------------------------------------
void IAprioriConstList::DeleteByType(AprioriConstType t)
{
    IAprioriConstEntry* en = first();
    IAprioriConstEntry* nx;

    while (en != NULL) {
        if (en->GetType() == t) {
            nx = en->next();
            delete_node(en);
            en = nx;
        }
        else
            en = en->next();
    }
};
///////////////////////////////////////////////////////////////////////////////
// Установка априорных констант по вектору
//-----------------------------------------------------------------------------
void IAprioriConstList::Set(const DVector& u)
{
    IAprioriConstEntry* e;
    for (e = first(); e != NULL; e = e->next())
        e->Set(u);
}

///////////////////////////////////////////////////////////////////////////////
