///////////////////////////////////////////////////////////////////////////////
// Библиотека алгоритмов небесной механики
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
// Классы, описывающие таблицы преобразования шкал времени
///////////////////////////////////////////////////////////////////////////////
#include "Defines.h"
#include "Consts.h"
#include "AstroTimeTables.h"
#include <stdio.h>
///////////////////////////////////////////////////////////////////////////////
#define MAX_LENTGH_LINE_UTC2UT1 256
#define BEG_LENTGH_TABLE_UTC2UT1 128
#define MAX_LENTGH_LINE_TAI2UTC 128
///////////////////////////////////////////////////////////////////////////////
//  Класс таблицы преобразования TAI -> UTC
///////////////////////////////////////////////////////////////////////////////
bool TTableTAI2UTC::Init(const char* fname)
{
    int i, i1;
    char *buf, *s;
    FILE* f;

    tableLen = 0;
    table = NULL;
    f = fopen(fname, "r");
    if (f == NULL)
        return false;

    buf = new char[MAX_LENTGH_LINE_TAI2UTC + 1];

    while (!feof(f)) {
        s = fgets(buf, MAX_LENTGH_LINE_TAI2UTC, f);
        if (!s)
            break;
        if (s[0] == '\n')
            break;
        tableLen += 1;
    };

    fseek(f, SEEK_SET, 0);
    table = new TRecTableTAI2UTC[tableLen];
    TRecTableTAI2UTC* t = table;

    for (i = 0; i < tableLen; i++) {
        s = fgets(buf, MAX_LENTGH_LINE_TAI2UTC, f);
        i1 = sscanf(s, "%*d %*s %*d =JD %le TAI-UTC= %le S + (MJD - %le) X %le",
                    &t->JD, &t->B, &t->MJD, &t->S);
        TAssert(i1 == 4);
        t->JD_TAI = (t->JD - (modJD + t->MJD) * t->S * sec2day
            + t->B * sec2day) / (1 - t->S * sec2day);
        t++;
    };

    delete[] buf;
    fclose(f);
    return true;
}

//-----------------------------------------------------------------------------
void TTableTAI2UTC::Clear()
{
    if (table != NULL)
        delete[] table;
    table = NULL;
    tableLen = 0;
};
///////////////////////////////////////////////////////////////////////////////
//  Класс таблицы преобразования UTC -> UT1
///////////////////////////////////////////////////////////////////////////////
bool TTableUTC2UT1::Init(const char* fname)
{
    int i1;
    char *buf, *s;
    FILE* f;
    double t1, *tableNew;
    int maxTableLen, iLen;

    tableLen = 0;
    table = NULL;
    f = fopen(fname, "r");
    if (f == NULL)
        return false;

    buf = new char[MAX_LENTGH_LINE_UTC2UT1 + 1];
    s = fgets(buf, MAX_LENTGH_LINE_UTC2UT1, f);
    i1 = sscanf(s + 7, "%le ", &t1);
    if (i1 != 1)
        return false;
    jdBeg = t1;
    maxTableLen = BEG_LENTGH_TABLE_UTC2UT1;
    table = new double[maxTableLen];

    while (true) {
        i1 = sscanf(s + 57, "%*c %le ", &t1);
        if (i1 != 1)
            break;
        table[tableLen++] = t1;
        if (tableLen == maxTableLen) {
            iLen = maxTableLen;
            maxTableLen += BEG_LENTGH_TABLE_UTC2UT1;
            tableNew = new double[maxTableLen];
            while (iLen > 0) {
                iLen -= 1;
                tableNew[iLen] = table[iLen];
            };
            delete[] table;
            table = tableNew;
        }
        buf[0] = 0;
        s = fgets(buf, MAX_LENTGH_LINE_UTC2UT1, f);
    };

    delete[] buf;
    fclose(f);

    tableNew = new double[tableLen];
    for (iLen = 0; iLen < tableLen; iLen++) {
        tableNew[iLen] = table[iLen];
    };
    delete[] table;
    table = tableNew;

    return true;
};
//-----------------------------------------------------------------------------
void TTableUTC2UT1::Clear()
{
    if (table != NULL)
        delete[] table;
    table = NULL;
    tableLen = 0;
};
///////////////////////////////////////////////////////////////////////////////
// Класс таблицы движения полюсов
///////////////////////////////////////////////////////////////////////////////
#define BEG_LENTGH_TABLE_UTC2UT1 128
#define MAX_LENTGH_LINE_POLUS 256
#define BEG_LENTGH_TABLE_POLUS 128
//-----------------------------------------------------------------------------
void TTablePolus::addValue(double val)
{
    int iLen;

    if (tableLen == maxTableLen) {
        double* tableNew;
        iLen = maxTableLen;
        maxTableLen += BEG_LENTGH_TABLE_UTC2UT1;
        tableNew = new double[maxTableLen];
        while (iLen > 0) {
            iLen -= 1;
            tableNew[iLen] = table[iLen];
        };
        delete[] table;
        table = tableNew;
    };

    table[tableLen++] = val;
};
//-----------------------------------------------------------------------------
bool TTablePolus::Init(const char* fname)
{
    int i1;
    char *buf, *s;
    FILE* f;
    double x, y, t1;
    int iLen;
    double* tableNew;

    tableLen = 0;
    table = NULL;
    f = fopen(fname, "r");
    if (f == NULL)
        return false;

    buf = new char[MAX_LENTGH_LINE_POLUS + 1];
    s = fgets(buf, MAX_LENTGH_LINE_POLUS, f);
    i1 = sscanf(s + 7, "%le ", &t1);
    if (i1 != 1)
        return false;
    jdBeg = t1;
    maxTableLen = BEG_LENTGH_TABLE_POLUS;
    table = new double[maxTableLen];

    while (true) {
        i1 = sscanf(s + 19, "%le %*e %le", &x, &y);
        if (i1 != 2)
            break;
        addValue(x);
        addValue(y);
        buf[0] = 0;
        s = fgets(buf, MAX_LENTGH_LINE_POLUS, f);
    };
    delete[] buf;
    fclose(f);

    tableNew = new double[tableLen];
    for (iLen = 0; iLen < tableLen; iLen++) {
        tableNew[iLen] = table[iLen];
    };
    delete[] table;
    table = tableNew;
    tableLen /= 2;

    return true;
};
//-----------------------------------------------------------------------------
void TTablePolus::Clear()
{
    if (table != NULL)
        delete[] table;
    table = NULL;
    tableLen = 0;
};
///////////////////////////////////////////////////////////////////////////////
