///////////////////////////////////////////////////////////////////////////////
// Библиотека алгоритмов небесной механики
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
// Классы, описывающие таблицы преобразования шкал времени
///////////////////////////////////////////////////////////////////////////////
#ifndef BNOTOOL_ASTROTIMETABLES_H
#define BNOTOOL_ASTROTIMETABLES_H
///////////////////////////////////////////////////////////////////////////////
#include <stdlib.h>
#include "Defines.h"
#define modJD 2400000.5l

///////////////////////////////////////////////////////////////////////////////
// Класс таблицы преобразования UTC -> UT1
///////////////////////////////////////////////////////////////////////////////
class TTableUTC2UT1
{
public:
    TTableUTC2UT1()
    {
        table = NULL;
        tableLen = 0;
    };

    virtual ~TTableUTC2UT1()
    {
        Clear();
    };
    bool Init(const char* fname);
    void Clear();

    int Len() const
    {
        return tableLen;
    };

    double JDBeg() const
    {
        return jdBeg;
    };

    double dUT1UTC(int i) const
    {
        return table[i];
    };

    bool isInit() const
    {
        return (table != NULL ? true : false);
    };

private:
    int tableLen;
    double jdBeg;
    double* table;
};

///////////////////////////////////////////////////////////////////////////////
// Класс таблицы преобразования TAI -> UTC
///////////////////////////////////////////////////////////////////////////////
// Структура для описания одной записи таблицы
struct TRecTableTAI2UTC
{
    double JD;
    double B;
    double MJD;
    double S;
    double JD_TAI;
};

//-----------------------------------------------------------------------------
class TTableTAI2UTC
{
public:
    TTableTAI2UTC()
    {
        table = NULL;
        tableLen = 0;
    };

    ~TTableTAI2UTC()
    {
        Clear();
    };
    bool Init(const char* fname);
    void Clear();

    int Len() const
    {
        return tableLen;
    };

    const TRecTableTAI2UTC* getBlock(int n) const
    {
        return table + n;
    };

    bool isInit() const
    {
        return (table != NULL ? true : false);
    };

private:
    int tableLen;
    TRecTableTAI2UTC* table;
};

///////////////////////////////////////////////////////////////////////////////
// Класс таблицы движения полюсов
///////////////////////////////////////////////////////////////////////////////
class TTablePolus
{
public:
    TTablePolus()
    {
        table = NULL;
    };

    ~TTablePolus()
    {
        Clear();
    };

    bool Init(const char* fname);
    void Clear();

    int Len() const
    {
        return tableLen;
    };

    double JDBeg() const
    {
        return jdBeg;
    };

    double X(int i) const
    {
        return table[i * 2 + 0];
    };

    double Y(int i) const
    {
        return table[i * 2 + 1];
    };

    bool isInit() const
    {
        return (table != NULL ? true : false);
    };

private:
    void addValue(double val);
    int maxTableLen;

    int tableLen;
    double jdBeg;
    double* table;
};

///////////////////////////////////////////////////////////////////////////////
#endif
