#include "CloseAppr.h"

#include "IAstroBase.h"
///////////////////////////////////////////////////////////////////////////////
// Реализации методов класса
///////////////////////////////////////////////////////////////////////////////
// Методы для быстрого приближенного прогнозирования
// Установка опорного узла
//-----------------------------------------------------------------------------
void CAOrbitNodes::SetNode(const TimeJD& jd)
{
    Node_Time = jd;
    if (!IsInsideInterval(jd) || !GetCoordVelocity(Node_Time, Node_R, Node_V)) {
        NodeFlag = false;
        return;
    }

    double miu_r_3 = Node_R.Len();
    miu_r_3 = -(GetAstroBase()->GetEarthMu() / (miu_r_3 * miu_r_3 * miu_r_3));

    int i;
    for (i = 0; i < 3; i++)
        Node_A[i] = Node_R[i] * miu_r_3;

    NodeFlag = true;
}

//-----------------------------------------------------------------------------
// Быстрый прогноз на заданное время
//-----------------------------------------------------------------------------
void CAOrbitNodes::FastPredict(const TimeJD& jd)
{
    F_time = jd;
    TmDay td = jd - Node_Time;
    double dt = td.GetDouble() * 86.4;

    int i;

    F_r = Node_A;
    for (i = 0; i < 3; i++) {
        F_r[i] = Node_R[i] + Node_V[i] * dt + Node_A[i] * dt * dt * 0.5;
        F_v[i] = Node_V[i] + Node_A[i] * dt;
    }
}

//-----------------------------------------------------------------------------
// Точный прогноз. Возвращает true, если он был успешен
//-----------------------------------------------------------------------------
bool CAOrbitNodes::GetCoordVelocity(const TimeJD& jd, Vect3& coord, Vect3& velocity)
{
    if (!IsInsideInterval(jd))
        return false;

    PhasePoint6D pp;

    if (!Ko->GetPos(jd, pp))
        return false;

    J2000Frame j2000;
    pp.Convert(j2000);

    coord = pp.CoordsVel;
    pp.CoordsVel.GetVelocity(velocity);

    return true;
}

//-----------------------------------------------------------------------------
// Проверка вхождения указанного времени в интервал применимости
// траекторных данных
//-----------------------------------------------------------------------------
bool CAOrbitNodes::IsInsideInterval(const TimeJD& jd) const
{
    TimeJD ts, te;

    Ko->GetTimeRange(ts, te);

    return jd >= ts && jd <= te;
}

///////////////////////////////////////////////////////////////////////////////
