///////////////////////////////////////////////////////////////////////////////
// Библиотека алгоритмов небесной механики
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
#include "CloseAppr.h"
///////////////////////////////////////////////////////////////////////////////
// Список КО, участвующих в расчете опасных сближений
///////////////////////////////////////////////////////////////////////////////
// Вычисление диапазона высот, в котором находятся орбиты объектов
//-----------------------------------------------------------------------------
void CASpaceObjectList::GetHeightRange(double& hp, double& ha) const
{
    if (first() == NULL && last() == NULL)
        return;

    ICASpaceObjectEntry* e = first();
    e->GetHeightRange(hp, ha);
    e = e->next();

    while (e != NULL) {
        double hmin, hmax;
        e->GetHeightRange(hmin, hmax);
        if (hmin < hp)
            hp = hmin;
        if (hmax > ha)
            ha = hmax;
        e = e->next();
    }

    return;
}

//-----------------------------------------------------------------------------
// Фильтрация списка по диапазону высот
// (Из списка удаляются объекты, орбиты которых лежат вне указанного
// диапазона высот)
//-----------------------------------------------------------------------------
void CASpaceObjectList::FilterByHeight(double hp, double ha)
{
    ICASpaceObjectEntry *e = first(), *n;

    while (e != NULL) {
        n = e->next();
        double hmin, hmax;
        e->GetHeightRange(hmin, hmax);
        if (hmax < hp || hmin > ha)
            delete_node(e);
        e = n;
    }
}

//-----------------------------------------------------------------------------
// Фильтрация списка по временному диапазону
// (Из списка удаляются объекты, у которых интервалы применимости 
// лежат вне указанного диапазона)
//-----------------------------------------------------------------------------
void CASpaceObjectList::FilterByTime(const TimeJD& ts, const TimeJD& te)
{
    ICASpaceObjectEntry *e = first(), *n;

    while (e != NULL) {
        n = e->next();
        TimeJD tmin, tmax;
        e->GetTimeRange(tmin, tmax);
        if (tmax < ts || tmin > te)
            delete_node(e);
        e = n;
    }
}

///////////////////////////////////////////////////////////////////////////////
