///////////////////////////////////////////////////////////////////////////////
// Библиотека алгоритмов небесной механики
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
// Функции инициализации данных астробаллистической базы
///////////////////////////////////////////////////////////////////////////////
#include "ComplexInit.h"
///////////////////////////////////////////////////////////////////////////////
static BNOAstroBase ABase;
///////////////////////////////////////////////////////////////////////////////
// Инициализация
bool ComplexInit(const char* fname)
{
    SetAstroBase(ABase);

    if (fname != NULL)
        ABase.SetCfgFile(fname);

    return ABase.Init();
}

//-----------------------------------------------------------------------------
// Инициализация базы с явным указанием времени для эфемерид
bool ComplexInit(const TimeGD& s, const TimeGD& e, const char* fname)
{
    SetAstroBase(ABase);

    if (fname != NULL)
        ABase.SetCfgFile(fname);

    return ABase.Init(s, e);
}

//-----------------------------------------------------------------------------
// Инициализация базы с явным указанием времени для эфемерид
bool ComplexInit(const TimeJD& s, const TimeJD& e, const char* fname)
{
    SetAstroBase(ABase);

    if (fname != NULL)
        ABase.SetCfgFile(fname);

    return ABase.Init(s, e);
}

//-----------------------------------------------------------------------------
// Установка модели атмосферы
void SetAtmosphereModel(TAtmosphereId mi)
{
    ABase.SetAtmosphereModel(mi);
}

//-----------------------------------------------------------------------------
// Установка модели гравитации для Земли
bool SetGravityModel(TGravityModelId mi)
{
    return ABase.SetEarthGravityModel(mi);
}

//-----------------------------------------------------------------------------
// Загрузка данных для эфемерид планеты
bool LoadEphData(TPlanetId pl)
{
    return ABase.LoadEphData(pl);
}

//-----------------------------------------------------------------------------
// Выгрузка данных для эфемерид планеты (освобождение памяти)
void UnLoadEphData(TPlanetId pl)
{
    ABase.UnLoadEphData(pl);
}

//-----------------------------------------------------------------------------
// Получение ссылки на астробаллистическую базу ПК БИКОП
BNOAstroBase* GetBNOAstroBase()
{
    return &ABase;
}

///////////////////////////////////////////////////////////////////////////////
