#include "DMinMnkTrAn.h"
#include <iostream>

///////////////////////////////////////////////////////////////////////////////
// Класс траекторной минимизации с выставлением признаков
// аномальности измерений
///////////////////////////////////////////////////////////////////////////////
// Выполнение минимизации
// u0 - начальное приближение
// f - минимизируемая функция
//-----------------------------------------------------------------------------
void DMinMnkTrAn::Minimize(const DVector& u0, IMinTrFun& f)
{
    if (IanBuffer != NULL) {
        delete[] IanBuffer;
    }

    int maxiter = f.GetPacket()->GetSize(), i;
    unsigned j;

    unsigned n = f.GetPacket()->GetSize();
    unsigned iterapr = 0;

    IanBuffer = new unsigned short[(maxiter + 1) * n];
    KIan = 0;

    SaveIans(*f.GetPacket());

    DVector eoc = u0;
    bool fdm = f.GetDigMtx();
    for (i = 0; i < maxiter; i++) {
        DMinMnk::Minimize(eoc, f);

        /*
        if (Res != SUCCESS_RES && !f.GetDigMtx())
        {
        f.SetDigMtx(true);
        if (RegPrint > NONE)
        printf("+++Принудительный расчет част. производных в конечных разностях\n");
        i--;
        continue;
        }*/

        if (Res != SUCCESS_RES || Check == NULL) {
            break;
        }

        // Проверка функционала на априорной орбите
        if (f.HasApr() && iterapr < 3 && MaxFApr < 1.0 && f.GetFa() > MaxFApr * Fk) {
            f.SetApr(U);
            iterapr++;
            i--;
            if (RegPrint > NONE)
                printf("Изменение априорной орбиты\n");
            continue;
        }

        Check->ChangeIans(f);

        if (!Check->CheckRange(f)) {
            Res = BAD_IANS;
            break;
        }

        SaveIans(*f.GetPacket());

        // сравнение признаков аномальности
        if (HasCurIans(*f.GetPacket())) {
            Res = SUCCESS_RES;
            break;
        }

        eoc = U;
        if (RegPrint > NONE) {
            printf("Изменены признаки аномальности:\n");
            unsigned short* p = IanBuffer + i * n;
            unsigned short* c = IanBuffer + (i + 1) * n;
            f.GetPacket()->SetFirst();
            for (j = 0; j < n; j++) {
                const IMeasurement* meas = f.GetPacket()->GetCurrent();
                f.GetPacket()->GoNext();
                if (p[j] != c[j]) {
                    printf(" %d:", j);
                    unsigned k;
                    for (k = 0; k < meas->GetSize(); k++)
                        putc((p[j] & (0x1 << k)) != 0 ? '1' : '0', stdout);
                    printf("->");
                    for (k = 0; k < meas->GetSize(); k++)
                        putc((c[j] & (0x1 << k)) != 0 ? '1' : '0', stdout);
                    printf("\n");
                }
            }
        }
    }

    f.SetDigMtx(fdm);

    if (Res != SUCCESS_RES) {
        RestoreIan(*f.GetPacket());
    }

    delete[] IanBuffer;
    IanBuffer = NULL;
}

//-------------------------------------------------------------
// Добавление признаков аномальности в буфер
//-------------------------------------------------------------
void DMinMnkTrAn::SaveIans(IMeasurementPacket& m)
{
    unsigned n = m.GetSize(), i, j;
    unsigned short* p = IanBuffer + KIan * n;

    m.SetFirst();
    for (i = 0; i < n; i++) {
        IMeasurement* meas = m.GetCurrent();
        m.GoNext();
        p[i] = 0;
        for (j = 0; j < meas->GetSize(); j++) {
            if (meas->GetIan(j))
                p[i] |= 0x01 << j;
        }
    }

    KIan++;
}

//-------------------------------------------------------------
// Проверка существования актуальных признаков аномальности
// в буфере
//-------------------------------------------------------------
bool DMinMnkTrAn::HasCurIans(IMeasurementPacket& m)
{
    unsigned n = m.GetSize(), i, j;
    unsigned short* p = IanBuffer + (KIan - 1) * n;

    for (i = 0; i < KIan - 1; i++) {
        unsigned short* c = IanBuffer + i * n;
        for (j = 0; j < n; j++) {
            if (p[j] != c[j])
                break;
        }
        if (j == n)
            return true;
    }

    return false;
}

//-------------------------------------------------------------
// Восстановление признаков аномальности по первой записи
//-------------------------------------------------------------
void DMinMnkTrAn::RestoreIan(IMeasurementPacket& m)
{
    unsigned n = m.GetSize(), i, j;

    m.SetFirst();
    for (i = 0; i < n; i++) {
        IMeasurement* meas = m.GetCurrent();
        m.GoNext();
        for (j = 0; j < meas->GetSize(); j++) {
            if (IanBuffer[i] & (0x01 << j))
                meas->SetIan(j, true);
            else
                meas->SetIan(j, false);
        }
    }
}

///////////////////////////////////////////////////////////////////////////////
