///////////////////////////////////////////////////////////////////////////////
// Библиотека алгоритмов небесной механики
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
// Классы для оптимизации траектории c изменением признаков аномальности
///////////////////////////////////////////////////////////////////////////////
#ifndef BNOTOOL_DMINMNKTRAN_H
#define BNOTOOL_DMINMNKTRAN_H
///////////////////////////////////////////////////////////////////////////////
#include "IMinMnkTr.h"

///////////////////////////////////////////////////////////////////////////////
// виртуальный класс принятия решений внутри схемы селекции
// аномальных компонент
///////////////////////////////////////////////////////////////////////////////
class DMinMnkAnTune
{
public:
    virtual ~DMinMnkAnTune()
    {
    };
    // процедура корректирует признаки аномальности после очередного решения
    virtual void ChangeIans(IMinTrFun& tf) = 0;
    // процедура проверяет допустимость набора признаков аномальности после очередной коррекции
    virtual bool CheckRange(IMinTrFun& tf)
    {
        return true;
    };
};

///////////////////////////////////////////////////////////////
// Класс траекторной минимизации с выставлением признаков
// аномальности измерений
///////////////////////////////////////////////////////////////
class DMinMnkTrAn : public DMinMnk
{
public:
    // Конструктор
    DMinMnkTrAn(DMinMnkAnTune* f = NULL) : DMinMnk()
    {
        Check = f;
        IanBuffer = NULL;
        KIan = 0;
        MaxFApr = 1.0;
    };
    // Деструктор
    virtual ~DMinMnkTrAn()
    {
        if (IanBuffer != NULL) delete[] IanBuffer;
    };

    // Инициализация
    void Init(DMinMnkAnTune& f)
    {
        Check = &f;
    };

    // Выполнение минимизации
    // u0 - начальное приближение
    // f - минимизируемая функция
    void Minimize(const DVector& u0, IMinTrFun& f);

    // максимальная доля общего функционала, допустимый
    // для априорной орбиты. Если превышен - будет повтор
    // решения с новым начальным приближением. По умолчанию = 1.0
    double MaxFApr;

protected:
    // Класс выставления признаков аномальности
    DMinMnkAnTune* Check;
    // Память для хранения признаков аномальности
    unsigned short* IanBuffer;
    // Счетчик числа элементов в буфере
    unsigned KIan;
    // Добавление признаков аномальности в буфер
    void SaveIans(IMeasurementPacket& p);
    // Проверка существования актуальных признаков аномальности
    // в буфере
    bool HasCurIans(IMeasurementPacket& m);
    // Восстановление признаков аномальности по первой записи
    void RestoreIan(IMeasurementPacket& m);
};

///////////////////////////////////////////////////////////////////////////////
#endif
