///////////////////////////////////////////////////////////////////////////////
// Библиотека алгоритмов небесной механики
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
// Класс, реализующий квадратную матрицу
///////////////////////////////////////////////////////////////////////////////
#include "Matrix.h"

#include <math.h>
///////////////////////////////////////////////////////////////////////////////
// Транспонирование
//-----------------------------------------------------------------------------
void DSqrMatrix::Transpose()
{
    TAssert(Matrix != NULL);
    TAssert(NRows == NCols);

    unsigned i, j;
    double a;
    double* line;

    for (i = 0; i < NRows; i++) {
        line = Matrix[i];
        for (j = i + 1; j < NRows; j++) {
            a = line[j];
            line[j] = Matrix[j][i];
            Matrix[j][i] = a;
        }
    }

    NRowsMem = NRows;
    NColsMem = NRows;

    return;
}

//-----------------------------------------------------------------------------
// Обращение матрицы
//-----------------------------------------------------------------------------
bool DSqrMatrix::Inverse()
{
    TAssert(Matrix != NULL);
    TAssert(NRows == NCols);

    DSqrMatrix src;
    src.Init(NRows);
    XChng(src);
    if (Matrix == NULL)
        return false;

    unsigned* in = new unsigned[NRows];
    if (in == NULL) {
        delete Matrix;
        Matrix = NULL;
        return false;
    }

    unsigned i, j, k, k1, k2, l = 0;
    double s;
    double* line;

    for (k = 0; k < NRows; k++) {
        line = src.Matrix[k];
        for (s = 0.0, k1 = k; k1 < NRows; k1++) {
            if (fabs(line[k1]) > s) {
                l = k1;
                s = fabs(line[k1]);
            }
        }
        in[k] = l;
        for (k2 = 0; k2 < NRows; k2++) {
            s = src.Matrix[k2][l];
            src.Matrix[k2][l] = src.Matrix[k2][k];
            src.Matrix[k2][k] = s;
        }
        if (fabs(line[k]) < ZERO_ACCURACY) {
            delete in;
            NRows = NCols = 0;
            return false;
        }

        for (i = 0; i < NRows; i++) {
            for (j = 0; j < NRows; j++) {
                if (i == k) {
                    if (j == k)
                        Matrix[k][j] = 1.0 / line[k];
                    else
                        Matrix[k][j] = line[j] / line[k];
                }
                else {
                    if (j == k)
                        Matrix[i][j] = -src.Matrix[i][k] / line[k];
                    else
                        Matrix[i][j] = src.Matrix[i][j] - line[j] * src.Matrix[i][k] / line[k];
                }
            }
        }

        if (k < NRows - 1)
            XChng(src);
    }

    for (k1 = 0; k1 < NRows; k1++) {
        k = NRows - 1 - k1;
        l = in[k];
        for (k2 = 0; k2 < NRows; k2++) {
            s = Matrix[l][k2];
            Matrix[l][k2] = Matrix[k][k2];
            Matrix[k][k2] = s;
        }
    }

    delete in;

    return true;
}

//-----------------------------------------------------------------------------
// Реализация запроса значений элементов в виде массива
//-----------------------------------------------------------------------------
void DSqrMatrix::GetArray(double* array) const
{
    TAssert(Matrix != NULL);
    TAssert(array != NULL);

    unsigned i;

    for (i = 0; i < NRows; i++) {
        memcpy(array, Matrix[i], NCols * sizeof(double));
        array += NCols;
    }
}

//-----------------------------------------------------------------------------
// Реализация запроса значений столбца
//-----------------------------------------------------------------------------
void DSqrMatrix::GetCol(unsigned index, double* col) const
{
    TAssert(Matrix != NULL);
    TAssert(index < NCols);
    TAssert(col != NULL);

    unsigned i;

    for (i = 0; i < NRows; i++)
        col[i] = Matrix[i][index];
}

///////////////////////////////////////////////////////////////////////////////
