///////////////////////////////////////////////////////////////////////////////
// Библиотека алгоритмов небесной механики
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
// Служебный класс хранения треугольной матрицы в памяти
///////////////////////////////////////////////////////////////////////////////
#include "Matrix.h"
///////////////////////////////////////////////////////////////////////////////
// Конструктор по умолчанию
//-----------------------------------------------------------------------------
DTriagMem::DTriagMem()
{
    Buffer = NULL;
    Dimension = 0;
    DimMem = 0;
}

//-----------------------------------------------------------------------------
// Деструктор
//-----------------------------------------------------------------------------
DTriagMem::~DTriagMem()
{
    if (Buffer != NULL)
        delete Buffer;
}

//-----------------------------------------------------------------------------
// Обмен содержимого
//-----------------------------------------------------------------------------
void DTriagMem::XChng(DTriagMem& m)
{
    unsigned b;

    b = Dimension;
    Dimension = m.Dimension;
    m.Dimension = b;

    b = DimMem;
    DimMem = m.DimMem;
    m.DimMem = b;

    b = MemSize;
    MemSize = m.MemSize;
    m.MemSize = b;

    double* ptr;
    ptr = Buffer;
    Buffer = m.Buffer;
    m.Buffer = ptr;
}

//-----------------------------------------------------------------------------
// Инициализация
//-----------------------------------------------------------------------------
void DTriagMem::Init(unsigned dim)
{
    if (ReqMemSize(dim) > MemSize || Buffer == NULL) {
        if (Buffer != NULL)
            delete Buffer;
        MemSize = ReqMemSize(dim);
        Buffer = new double[MemSize];
    }

    DimMem = Dimension = dim;
}

//-----------------------------------------------------------------------------
// Изменение размерности
//-----------------------------------------------------------------------------
void DTriagMem::ResetDimension(unsigned dim)
{
    TAssert(dim <= DimMem);
    Dimension = dim;
}

//-----------------------------------------------------------------------------
// Установка единого значения для всех элементов
//-----------------------------------------------------------------------------
void DTriagMem::Init(double value)
{
    TAssert(Buffer != NULL);

    unsigned i, j;

    for (i = 0; i < Dimension; i++)
        for (j = i; j < Dimension; j++)
            Buffer[Index(i, j)] = value;
}

//-----------------------------------------------------------------------------
// Умножение всех элементов на число
//-----------------------------------------------------------------------------
void DTriagMem::Mult(double mltp)
{
    TAssert(Buffer != NULL);

    unsigned i, j;

    for (i = 0; i < Dimension; i++)
        for (j = i; j < Dimension; j++)
            Buffer[Index(i, j)] *= mltp;
}

///////////////////////////////////////////////////////////////////////////////
