///////////////////////////////////////////////////////////////////////////////
// Библиотека алгоритмов небесной механики
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
// Определение 3- и 6-мерных декартовых координат и матриц преобразования
////////////////////////////////////////////////////////////////////////////////
#include "DVectors.h"
////////////////////////////////////////////////////////////////////////////////
// Умножение вектора на матрицу справа
// (т.е. результат - вектор-строка)
//------------------------------------------------------------------------------
Vect3& Vect3::MulRight(const TransMtx& m)
{
    double x_ = x, y_ = y, z_ = z;
    const double* mtx = m.GetConstPtr();
    x = mtx[0] * x_ + mtx[3] * y_ + mtx[6] * z_;
    y = mtx[1] * x_ + mtx[4] * y_ + mtx[7] * z_;
    z = mtx[2] * x_ + mtx[5] * y_ + mtx[8] * z_;
    return *this;
};
//------------------------------------------------------------------------------
// Умножение вектора на матрицу слева
// (т.е. результат - вектор-столбец)
//------------------------------------------------------------------------------
Vect3& Vect3::operator*=(const TransMtx& m)
{
    double x_ = x, y_ = y, z_ = z;
    const double* mtx = m.GetConstPtr();
    x = mtx[0] * x_ + mtx[1] * y_ + mtx[2] * z_;
    y = mtx[3] * x_ + mtx[4] * y_ + mtx[5] * z_;
    z = mtx[6] * x_ + mtx[7] * y_ + mtx[8] * z_;
    return *this;
};
////////////////////////////////////////////////////////////////////////////////
// Вывод на печать 6-мерного вектора
////////////////////////////////////////////////////////////////////////////////
void Vect6::print(const char* name, FILE* out) const
{
    if (name && *name)
        fprintf(out, "%s", name);
    fprintf(out, "coord %23.16e %23.16e %23.16e\n", x, y, z);
    fprintf(out, "veloc %23.16e %23.16e %23.16e\n", vx, vy, vz);
};
////////////////////////////////////////////////////////////////////////////////
// Транспонирование
////////////////////////////////////////////////////////////////////////////////
TransMtx& TransMtx::Transp()
{
    double t1;
    t1 = mtx[1];
    mtx[1] = mtx[3];
    mtx[3] = t1;
    t1 = mtx[2];
    mtx[2] = mtx[6];
    mtx[6] = t1;
    t1 = mtx[5];
    mtx[5] = mtx[7];
    mtx[7] = t1;
    return *this;
};
//------------------------------------------------------------------------------
void TransMtx::Transp(TransMtx& m) const
{
    m.mtx[0] = mtx[0];
    m.mtx[1] = mtx[3];
    m.mtx[2] = mtx[6];
    m.mtx[3] = mtx[1];
    m.mtx[4] = mtx[4];
    m.mtx[5] = mtx[7];
    m.mtx[6] = mtx[2];
    m.mtx[7] = mtx[5];
    m.mtx[8] = mtx[8];
}

//------------------------------------------------------------------------------
// Умножение на матрицу справа: A = A * B
//------------------------------------------------------------------------------
TransMtx& TransMtx::operator*=(double a)
{
    int i;
    for (i = 0; i < 9; i++)
        mtx[i] *= a;

    return *this;
}

//------------------------------------------------------------------------------
// Умножение на матрицу справа: A = A * B
//------------------------------------------------------------------------------
TransMtx& TransMtx::operator*=(const TransMtx& mRight)
{
    double m0 = mtx[0] * mRight.mtx[0] + mtx[1] * mRight.mtx[3] + mtx[2] * mRight.mtx[6];
    double m1 = mtx[0] * mRight.mtx[1] + mtx[1] * mRight.mtx[4] + mtx[2] * mRight.mtx[7];
    double m2 = mtx[0] * mRight.mtx[2] + mtx[1] * mRight.mtx[5] + mtx[2] * mRight.mtx[8];
    double m3 = mtx[3] * mRight.mtx[0] + mtx[4] * mRight.mtx[3] + mtx[5] * mRight.mtx[6];
    double m4 = mtx[3] * mRight.mtx[1] + mtx[4] * mRight.mtx[4] + mtx[5] * mRight.mtx[7];
    double m5 = mtx[3] * mRight.mtx[2] + mtx[4] * mRight.mtx[5] + mtx[5] * mRight.mtx[8];
    double m6 = mtx[6] * mRight.mtx[0] + mtx[7] * mRight.mtx[3] + mtx[8] * mRight.mtx[6];
    double m7 = mtx[6] * mRight.mtx[1] + mtx[7] * mRight.mtx[4] + mtx[8] * mRight.mtx[7];
    double m8 = mtx[6] * mRight.mtx[2] + mtx[7] * mRight.mtx[5] + mtx[8] * mRight.mtx[8];
    mtx[0] = m0;
    mtx[1] = m1;
    mtx[2] = m2;
    mtx[3] = m3;
    mtx[4] = m4;
    mtx[5] = m5;
    mtx[6] = m6;
    mtx[7] = m7;
    mtx[8] = m8;
    return *this;
};
//------------------------------------------------------------------------------
// Умножение на матрицу слева: A = B * A
//------------------------------------------------------------------------------
TransMtx& TransMtx::MulLeft(const TransMtx& mLeft)
{
    double m0 = mLeft.mtx[0] * mtx[0] + mLeft.mtx[1] * mtx[3] + mLeft.mtx[2] * mtx[6];
    double m1 = mLeft.mtx[0] * mtx[1] + mLeft.mtx[1] * mtx[4] + mLeft.mtx[2] * mtx[7];
    double m2 = mLeft.mtx[0] * mtx[2] + mLeft.mtx[1] * mtx[5] + mLeft.mtx[2] * mtx[8];
    double m3 = mLeft.mtx[3] * mtx[0] + mLeft.mtx[4] * mtx[3] + mLeft.mtx[5] * mtx[6];
    double m4 = mLeft.mtx[3] * mtx[1] + mLeft.mtx[4] * mtx[4] + mLeft.mtx[5] * mtx[7];
    double m5 = mLeft.mtx[3] * mtx[2] + mLeft.mtx[4] * mtx[5] + mLeft.mtx[5] * mtx[8];
    double m6 = mLeft.mtx[6] * mtx[0] + mLeft.mtx[7] * mtx[3] + mLeft.mtx[8] * mtx[6];
    double m7 = mLeft.mtx[6] * mtx[1] + mLeft.mtx[7] * mtx[4] + mLeft.mtx[8] * mtx[7];
    double m8 = mLeft.mtx[6] * mtx[2] + mLeft.mtx[7] * mtx[5] + mLeft.mtx[8] * mtx[8];
    mtx[0] = m0;
    mtx[1] = m1;
    mtx[2] = m2;
    mtx[3] = m3;
    mtx[4] = m4;
    mtx[5] = m5;
    mtx[6] = m6;
    mtx[7] = m7;
    mtx[8] = m8;
    return *this;
};
//------------------------------------------------------------------------------
// Умножение матрицы на матрицу m
//------------------------------------------------------------------------------
void TransMtx::mul(const TransMtx& m, TransMtx& res) const
{
    res.mtx[0] = mtx[0] * m.mtx[0] + mtx[1] * m.mtx[3] + mtx[2] * m.mtx[6];
    res.mtx[1] = mtx[0] * m.mtx[1] + mtx[1] * m.mtx[4] + mtx[2] * m.mtx[7];
    res.mtx[2] = mtx[0] * m.mtx[2] + mtx[1] * m.mtx[5] + mtx[2] * m.mtx[8];
    res.mtx[3] = mtx[3] * m.mtx[0] + mtx[4] * m.mtx[3] + mtx[5] * m.mtx[6];
    res.mtx[4] = mtx[3] * m.mtx[1] + mtx[4] * m.mtx[4] + mtx[5] * m.mtx[7];
    res.mtx[5] = mtx[3] * m.mtx[2] + mtx[4] * m.mtx[5] + mtx[5] * m.mtx[8];
    res.mtx[6] = mtx[6] * m.mtx[0] + mtx[7] * m.mtx[3] + mtx[8] * m.mtx[6];
    res.mtx[7] = mtx[6] * m.mtx[1] + mtx[7] * m.mtx[4] + mtx[8] * m.mtx[7];
    res.mtx[8] = mtx[6] * m.mtx[2] + mtx[7] * m.mtx[5] + mtx[8] * m.mtx[8];
}

//------------------------------------------------------------------------------
// Сложение матриц
//------------------------------------------------------------------------------
TransMtx& TransMtx::operator+=(const TransMtx& mtx_)
{
    for (int i = 0; i < 9; i++)
        mtx[i] += mtx_.mtx[i];
    return *this;
};
//------------------------------------------------------------------------------
// Вычитание матриц
//------------------------------------------------------------------------------
TransMtx& TransMtx::operator-=(const TransMtx& mtx_)
{
    for (int i = 0; i < 9; i++)
        mtx[i] -= mtx_.mtx[i];
    return *this;
};
//------------------------------------------------------------------------------
// Присвоение числа всем элементам матрицы
//------------------------------------------------------------------------------
TransMtx& TransMtx::operator=(double val)
{
    for (int i = 0; i < 9; i++)
        mtx[i] = val;
    return *this;
};
//------------------------------------------------------------------------------
// Определитель матрицы
//------------------------------------------------------------------------------
double TransMtx::Det() const
{
    return mtx[0] * mtx[4] * mtx[8] +
        mtx[1] * mtx[5] * mtx[6] +
        mtx[2] * mtx[3] * mtx[7] -
        mtx[0] * mtx[5] * mtx[7] -
        mtx[1] * mtx[3] * mtx[8] -
        mtx[2] * mtx[4] * mtx[6];
}

//------------------------------------------------------------------------------
// Вывод на печать
//------------------------------------------------------------------------------
void TransMtx::print(const char* name, FILE* out) const
{
    if (name && *name)
        fprintf(out, "%s\n", name);
    fprintf(out, "%19.15f %19.15f %19.15f\n", mtx[0], mtx[1], mtx[2]);
    fprintf(out, "%19.15f %19.15f %19.15f\n", mtx[3], mtx[4], mtx[5]);
    fprintf(out, "%19.15f %19.15f %19.15f\n", mtx[6], mtx[7], mtx[8]);
};
//------------------------------------------------------------------------------
// Сравнение на совпадение
//------------------------------------------------------------------------------
bool TransMtx::operator==(const TransMtx& m) const
{
    int i;

    for (i = 0; i < 9; i++) {
        if (mtx[i] != m.mtx[i])
            return false;
    }

    return true;
}

////////////////////////////////////////////////////////////////////////////////
