///////////////////////////////////////////////////////////////////////////////
// Библиотека алгоритмов небесной механики
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
// Определение класса системы отсчета ECI с указанной датой
////////////////////////////////////////////////////////////////////////////////
#include "ECIFrame.h"
#include "IAstroBase.h"
////////////////////////////////////////////////////////////////////////////////
// Установка параметров системы
void ECIFrame::SetParameters(const IFrameParameters& p)
{
    TAssert(p.GetFrameType() == csIdECI);
    const ECIFrameParameters& pm = (const ECIFrameParameters &)p;
    Pars = pm;
}

//------------------------------------------------------------------------------
// Получение объекта, который соответствует переходу от текущей системы отсчета 
// к системе отсчета J2000 на момент времени t0 (для 3-мерных векторов)
void ECIFrame::Get3DJ2000Converter(const TimeJD& t0, FrameConverter3D& result) const
{
    TAssert(Pars.IsInit());

    result.Init();

    GetAstroBase()->MtxPrecession(Pars.GetTime(), TimeJD(0, doubleJ2000),
                                  result.Rotation);
    return;
}

//------------------------------------------------------------------------------
// Получение объекта, который соответствует переходу от текущей системы отсчета 
// к системе отсчета J2000 на момент времени t0 (для 6-мерных векторов)
void ECIFrame::Get6DJ2000Converter(const TimeJD& t0, FrameConverter6D& result) const
{
    TAssert(Pars.IsInit());
    result.Init();
    Get3DJ2000Converter(t0, result.Converter3D);
}

////////////////////////////////////////////////////////////////////////////////
