///////////////////////////////////////////////////////////////////////////////
// Библиотека алгоритмов небесной механики
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
// Утилита создания системы отсчета по ее параметрам
////////////////////////////////////////////////////////////////////////////////
#include "IFrames.h"
#include "J2000Frame.h"
#include "J2000SFrame.h"
#include "B1950Frame.h"
#include "B1900Frame.h"
#include "METEFrame.h"
#include "GreenwichFrame.h"
#include "FGreenwichFrame.h"
#include "ECIFrame.h"
#include "NZEFrame.h"
#include "RNBFrame.h"
#include "LocalJ2000Frame.h"
#include "LocalRLFrame.h"
////////////////////////////////////////////////////////////////////////////////
template <class Frame>
static Frame* FrameCreator(const IFrameParameters& p)
{
    Frame* ret = new Frame;
    ret->SetParameters(p);
    return ret;
}

////////////////////////////////////////////////////////////////////////////////
IFrame* CreateIFrame(const IFrameParameters& p)
{
    TCoordSysId cs = p.GetFrameType();

    switch (cs) {
    case csIdJ2000:
        return FrameCreator<J2000Frame>(p);
    case csIdB1950:
        return FrameCreator<B1950Frame>(p);
    case csIdB1900:
        return FrameCreator<B1950Frame>(p);
    case csIdMETE:
        return FrameCreator<METEFrame>(p);
    case csIdGCS:
        return FrameCreator<GreenwichFrame>(p);
    case csIdGFCS:
        return FrameCreator<FGreenwichFrame>(p);
    case csIdJ2000S:
        return FrameCreator<J2000SFrame>(p);
    case csIdECI:
        return FrameCreator<ECIFrame>(p);
    case csIdNZE:
        return FrameCreator<NZEFrame>(p);
    case csIdRNB:
        return FrameCreator<RNBFrame>(p);
    case csIdLocalJ2000:
        return FrameCreator<LocalJ2000Frame>(p);
    case csIdLocalRL:
        return FrameCreator<LocalRLFrame>(p);
    }

    TAssert(false);
    return NULL;
}
