///////////////////////////////////////////////////////////////////////////////
// Библиотека алгоритмов небесной механики
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
// Класс географических координат
///////////////////////////////////////////////////////////////////////////////
#ifndef BNOTOOL_GEOCOORDS_H
#define BNOTOOL_GEOCOORDS_H
///////////////////////////////////////////////////////////////////////////////
#include "ICoords3D.h"
#include "IModel.h"

///////////////////////////////////////////////////////////////////////////////
class GeoCoords : public ICoords3D
{
public:
    double H; // Высота
    double Phi; // Широта [рад]
    double Lam; // Долгота [рад]

    // Конструктор по умолчанию
    GeoCoords();
    GeoCoords(double Re, double Alpha);
    GeoCoords(const IGravityModel& model);

    // Установка параметров эллипсоида      
    void SetParams(double re, double alpha)
    {
        Re = re;
        Alpha = alpha;
    };
    // Получение параметров эллипсоида
    double GetRe() const
    {
        return Re;
    };

    double GetAlpha() const
    {
        return Alpha;
    };

    // Установка координат по декартовым
    virtual void SetXYZ(const Vect3& src);
    // Получение декартовых координат
    virtual void GetXYZ(Vect3& dst) const;

    // Сравнение на равенство
    bool operator==(const GeoCoords& p) const
    {
        return (H == p.H) && (Phi == p.Phi) && (Lam == p.Lam) && (Re == p.Re) && (Alpha == p.Alpha);
    };

protected:
    double Re; // Экваториальный радиус
    double Alpha; // Коэф. Сжатия

    // Расстояние от центра эллипсоида до его поверхности
    double SDistance() const;
};

///////////////////////////////////////////////////////////////////////////////
class GeoCoordsPrec : public GeoCoords
{
public:
    GeoCoordsPrec() : GeoCoords()
    {
    };

    GeoCoordsPrec(double EarthRadius, double EarthOblatness) : GeoCoords(EarthRadius, EarthOblatness)
    {
    };
    // Установка координат по декартовым
    virtual void SetXYZ(const Vect3& src);
};

///////////////////////////////////////////////////////////////////////////////
#endif
