///////////////////////////////////////////////////////////////////////////////
// Библиотека алгоритмов небесной механики
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
// Таблица индексов солнечной активности и
// геомагнитной возмущенности
///////////////////////////////////////////////////////////////////////////////
#include "GeoSolarFlux.h"
#include <string.h>
///////////////////////////////////////////////////////////////////////////////
// Переменная для возвращаемых индексов
static TGeoSolarFlux RetVal;
///////////////////////////////////////////////////////////////////////////////
// Конструктор по умолчанию
TTableGeoSolarFlux::TTableGeoSolarFlux()
{
    table = NULL;
    constUsing = false;
};
//-----------------------------------------------------------------------------
// Деструктор
TTableGeoSolarFlux::~TTableGeoSolarFlux()
{
    if (table)
        delete[] table;
};
//-----------------------------------------------------------------------------
// Инициализация из файла
bool TTableGeoSolarFlux::Init(const char* fname)
{
    TimeGD grd(1, 6, 1991);
    F107level = grd;
    unsigned int i, kp[8];
    int i1, Year, Month, Day;
    FILE* f;
    TGeoSolarFlux* table1;

    f = fopen(fname, "r");
    if (!f)
        return false;

    // ГГГГММДД  признак f10.7 f81 f90 f135 f81w f135w признак ap(целое) kp
    char buf[256];
    len = 0;
    while (!feof(f)) {
        if (!fgets(buf, 255, f))
            break;
        len++;
    };
    fseek(f, 0, SEEK_SET);

    table = new TGeoSolarFlux[len];
    table1 = table;
    for (i = 0; i < len; i++) {
        if (!fgets(buf, 255, f))
            break;
        i1 = sscanf(buf, "%4d%2d%2d %*d %le %le %le %le %le %le %*d %d %le %d %d %d %d %d %d %d %d", &Year, &Month, &Day
                    , &(table1->f01), &(table1->f81), &(table1->f90), &(table1->f135), &(table1->f81w), &(table1->f135w)
                    , &(table1->ap), &(table1->kp), &(kp[0]), &(kp[1]), &(kp[2]), &(kp[3]), &(kp[4]), &(kp[5]), &(kp[6]), &(kp[7])
        );
        if (i1 != 19) {
            len = i;
            break;
        };
        if (i == 0) {
            grd.Init(Day, Month, Year);
            jdbeg = grd;
            k3first = jdbeg;
            k3first += 1.0 / 16.0;
            Ffirst = jdbeg;
            if (jdbeg >= F107level)
                Ffirst += 20.0 / 24.0;
            else
                Ffirst += 17.0 / 24.0;
            Kfirst = jdbeg;
            Kfirst += 0.5;
        };
        table1->iskp3 = true;
        for (i1 = 0; i1 < 8; i1++) {
            table1->three_hours_kp[i1] = kp[i1] == 99 ? table1->kp : (kp[i1] / 10) + (kp[i1] % 10) * (1 / 3.0);
            if (kp[i1] == 99)
                table1->iskp3 = false;
        };
        table1 += 1;
    };
    fclose(f);

    k3last = jdbeg;
    k3last += len - 1.0 / 16.0;
    Flast = jdbeg;
    Flast += len;
    if (Flast > F107level)
        Flast -= 4.0 / 24.0;
    else
        Flast -= 7.0 / 24.0;
    Klast = jdbeg;
    Klast += len;
    Klast -= 0.5;

    return true;
};
//-----------------------------------------------------------------------------
// Сохранение индексов в файле
void TTableGeoSolarFlux::Save(const TimeJD& ts, const TimeJD& te, std::vector<std::string>& res)
{
    unsigned si, ei, i, j, kp[8];
    double daytime;
    TGeoSolarFlux* table1;

    GetIndex(ts, si, daytime);
    GetIndex(te, ei, daytime);

    if (si > 0)
        si--;
    if (ei < len - 1)
        ei++;

    for (i = si, table1 = table + si; i <= ei; i++, table1++) {
        TimeJD t = jdbeg;
        t += i + 0.5;
        TimeGD g(t);
        for (j = 0; j < 8; j++) {
            if (!table1->iskp3)
                kp[j] = 99;
            else
                kp[j] = (int)(floor(table1->three_hours_kp[j])) * 10 +
                    (int)((table1->three_hours_kp[j] - floor(table1->three_hours_kp[j])) * 3.0 + 0.5);
        }
        const int sn = 1024;
        char f[sn];
        memset(f, 0, sn * sizeof(char));
        sprintf(f, "%4d%02d%02d 1 %f %f %f %f %f %f 8 %03d %f %d %d %d %d %d %d %d %d\n",
                g.Year, g.Month, g.Day, table1->f01, table1->f81, table1->f90, table1->f135, table1->f81w,
                table1->f135w, table1->ap, table1->kp, kp[0], kp[1], kp[2], kp[3], kp[4], kp[5], kp[6], kp[7]);

        res.push_back(f);
    }
}

//-----------------------------------------------------------------------------
// Получение индексов на указанную дату
const TGeoSolarFlux* TTableGeoSolarFlux::Get_F107(const TimeJD& tm) const
{
    if (constUsing)
        return &consts;

    TAssert((table != NULL) && (len > 0));

    TimeJD tu = tm;

    if (tu < Ffirst && tu < Kfirst)
        return table;

    if (tu > Flast && tu > Klast)
        return table + len - 1;

    unsigned index;
    double daytime;
    GetIndex(tu, index, daytime);

    if (tu < Ffirst) {
        RetVal.f135 = table[0].f135;
        RetVal.f90 = table[0].f90;
        RetVal.f81 = table[0].f81;
        RetVal.f135w = table[0].f135w;
        RetVal.f81w = table[0].f81w;
        RetVal.f01 = table[0].f01;
    }
    else if (tu > Flast) {
        RetVal.f135 = table[len - 1].f135;
        RetVal.f90 = table[len - 1].f90;
        RetVal.f81 = table[len - 1].f81;
        RetVal.f135w = table[len - 1].f135w;
        RetVal.f81w = table[len - 1].f81w;
        RetVal.f01 = table[len - 1].f01;
    }
    else {
        TimeJD t = jdbeg, t0;
        double dt0, r;
        int k0, k1;
        t += index;
        if (t < F107level)
            dt0 = 17.0 / 24.0;
        else
            dt0 = 20.0 / 24.0;
        t += dt0;
        if (tu < t) {
            k0 = index - 1;
            k1 = index;
            t0 = jdbeg;
            t0 += index - 1;
            if (t0 < F107level)
                dt0 = 17.0 / 24.0;
            else
                dt0 = 20.0 / 24.0;
            t0 += dt0;
        }
        else {
            k0 = index;
            k1 = index + 1;
            t0 = t;
            t = jdbeg;
            t += index + 1;
            if (t < F107level)
                dt0 = 17.0 / 24.0;
            else
                dt0 = 20.0 / 24.0;
            t += dt0;
        }
        r = (tu - t0).GetDouble() / (t - t0).GetDouble();
        RetVal.f135 = table[k0].f135 * (1.0 - r) + table[k1].f135 * r;
        RetVal.f90 = table[k0].f90 * (1.0 - r) + table[k1].f90 * r;
        RetVal.f81 = table[k0].f81 * (1.0 - r) + table[k1].f81 * r;
        RetVal.f135w = table[k0].f135w * (1.0 - r) + table[k1].f135w * r;
        RetVal.f81w = table[k0].f81w * (1.0 - r) + table[k1].f81w * r;
        RetVal.f01 = table[k0].f01 * (1.0 - r) + table[k1].f01 * r;
    }

    if (tu < Kfirst) {
        RetVal.ap = table[0].ap;
        RetVal.kp = table[0].kp;
    }
    else if (tu > Flast) {
        RetVal.ap = table[len - 1].ap;
        RetVal.kp = table[len - 1].kp;
    }
    else {
        TimeJD t = jdbeg, t0;
        double r;
        int k0, k1;
        t += index;
        t += 0.5;
        if (tu < t) {
            k0 = index - 1;
            k1 = index;
            t0 = t;
            t0 -= 1.0;
        }
        else {
            k0 = index;
            k1 = index + 1;
            t0 = t;
        }
        r = (tu - t0).GetDouble();
        RetVal.ap = table[index].ap;
        RetVal.kp = table[k0].kp * (1.0 - r) + table[k1].kp * r;
    }

    memcpy(RetVal.three_hours_kp, table[index].three_hours_kp, sizeof(RetVal.three_hours_kp));
    return &RetVal;
}

//-----------------------------------------------------------------------------
// Получение модифицированного трехчасового индекса
double TTableGeoSolarFlux::Get_Kp3hourMod(const TimeJD& t) const
{
    if (constUsing)
        return consts.three_hours_kp[0];
    TAssert((table != NULL) && (len > 0));

    TimeJD tu = t;

    if (tu < k3first)
        return table[0].three_hours_kp[0];
    if (tu > k3last)
        return table[len - 1].three_hours_kp[7];

    unsigned index;
    double daytime;
    GetIndex(tu, index, daytime);

    if (daytime >= 1.0 / 16. && daytime <= 1.0 - 1.0 / 16.) {
        unsigned kp = (int)((daytime - 1.0 / 16.0) * 8.0);
        double r = daytime * 8.0 - 0.5 - kp;
        return table[index].three_hours_kp[kp] * (1.0 - r) +
            table[index].three_hours_kp[kp + 1] * r;
    }

    if (daytime < 1.0 / 16.) {
        double r = daytime * 8.0 + 0.5;
        return table[index - 1].three_hours_kp[7] * (1.0 - r) +
            table[index].three_hours_kp[0] * r;
    }

    double r = daytime * 8.0 - 7.5;
    return table[index].three_hours_kp[7] * (1.0 - r) +
        table[index + 1].three_hours_kp[0] * r;
}

//-----------------------------------------------------------------------------
// Установка константных индексов
void TTableGeoSolarFlux::SetConstValue(double f135_, double f90_, double f81_,
                                       double f135w_, double f81w_, double f01_,
                                       double kp_, int ap_)
{
    consts.f135 = f135_;
    consts.f90 = f90_;
    consts.f81 = f81_;
    consts.f135w = f135w_;
    consts.f81w = f81w_;
    consts.f01 = f01_;
    consts.kp = kp_;
    consts.ap = ap_;
    for (int i = 0; i < 8; i++)
        consts.three_hours_kp[i] = kp_;
};
//-----------------------------------------------------------------------------
// Получение по дате индекса в таблице и времени суток (как double от 0 до 1)
void TTableGeoSolarFlux::GetIndex(const TimeJD& t, unsigned& index, double& daytime) const
{
    TimeJD tu = t;

    if (tu < jdbeg)
        index = 0;
    else {
        index = (long)((tu - jdbeg).GetDouble());
        if (index >= len)
            index = len - 1;
    }

    daytime = tu.GetFraction() - 0.5;
    if (daytime < 0.0)
        daytime += 1.0;
}

//-----------------------------------------------------------------------------
// Получение массива трехчасовых индексов
const double* TTableGeoSolarFlux::Get_Kp3hour(const TimeJD& tm) const
{
    if (constUsing)
        return consts.three_hours_kp;
    unsigned index;
    double daytime;
    GetIndex(tm, index, daytime);
    return table[index].three_hours_kp;
}

///////////////////////////////////////////////////////////////////////////////
