///////////////////////////////////////////////////////////////////////////////
// Библиотека алгоритмов небесной механики
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
// Таблица индексов солнечной активности и
// геомагнитной возмущенности
///////////////////////////////////////////////////////////////////////////////
#ifndef BNOTOOL_GEOSOLARFLUX_H
#define BNOTOOL_GEOSOLARFLUX_H
///////////////////////////////////////////////////////////////////////////////
#include "Defines.h"
#include "TM.h"
#include "IModel.h"
#include <string>
#include <vector>

///////////////////////////////////////////////////////////////////////////////
// Таблица индексов солнечной активности и геомагнитной возмущенности
///////////////////////////////////////////////////////////////////////////////
class TTableGeoSolarFlux : public ITableGeoSolarFlux
{
public:
    // Конструктор по умолчанию
    TTableGeoSolarFlux();
    // Деструктор
    virtual ~TTableGeoSolarFlux();

    // Получение индексов на указанную дату
    virtual const TGeoSolarFlux* Get_F107(const TimeJD& tm) const;
    // Получение трехчасовых индексов на указанную дату
    virtual const double* Get_Kp3hour(const TimeJD& tm) const;
    // Получение модифицированного трехчасового индекса
    virtual double Get_Kp3hourMod(const TimeJD& tm) const;

    // Инициализация из файла
    bool Init(const char* fname);
    // Сохранение в файле индексов для указанного диапазона дат
    void Save(const TimeJD& ts, const TimeJD& te, std::vector<std::string>& res);
    // Установка константных значений индексов
    void SetConstValue(double f135, double f90, double f81, double f135w, double f81w,
                       double f01, double kp, int ap);
    // Включение и выключение режима использования константных индексов
    void SetConstUsing(bool I)
    {
        constUsing = I;
    };

private:
    TimeJD jdbeg;
    TimeJD k3first, k3last;
    TimeJD Ffirst, Flast;
    TimeJD Kfirst, Klast;
    TimeJD F107level;
    TGeoSolarFlux* table;
    TGeoSolarFlux consts;
    unsigned long int len;
    bool constUsing;

    void GetIndex(const TimeJD& t, unsigned& index, double& daytime) const;
};

///////////////////////////////////////////////////////////////////////////////
#endif
