///////////////////////////////////////////////////////////////////////////////
// Библиотека алгоритмов небесной механики
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
// Определение класса Гринвичской замороженной системы координат
////////////////////////////////////////////////////////////////////////////////
#include "GreenwichFrame.h"
#include "FGreenwichFrame.h"
#include "METEFrame.h"
#include "NZEFrame.h"
#include "IAstroBase.h"
////////////////////////////////////////////////////////////////////////////////
// Получение объекта, который соответствует переходу от текущей системы отсчета 
// к системе отсчета J2000 на момент времени t0 (для 3-мерных векторов)
void GreenwichFrame::Get3DJ2000Converter(const TimeJD& t0, FrameConverter3D& result) const
{
    result.Init();

    GetAstroBase()->MtxJ20002GFCS(t0, result.Rotation);
    result.Rotation.Transp();
    return;
}

//------------------------------------------------------------------------------
// Получение объекта, который соответствует переходу от текущей системы отсчета 
// к системе отсчета J2000 на момент времени t0 (для 6-мерных векторов)
void GreenwichFrame::Get6DJ2000Converter(const TimeJD& t0, FrameConverter6D& result) const
{
    result.Init();
    Get3DJ2000Converter(t0, result.Converter3D);

    TransMtx wgf;
    GetAstroBase()->GCSVelocityMatrix(wgf);
    wgf.Transp();
    result.Converter3D.Rotation.mul(wgf, result.VRotation);
}

// Получение объекта, который соответствует переходу от текущей системы отсчета 
// к системе отсчета rf на момент времени t0 (для 3-мерных векторов)
//------------------------------------------------------------------------------
void GreenwichFrame::Get3DConverter(const IFrame& rf, const TimeJD& t0,
                                    FrameConverter3D& result) const
{
    if (rf.GetType() == csIdGFCS) {
        const FGreenwichFrame& gf = (const FGreenwichFrame &)rf;
        if (t0 == gf.GetFixTime()) {
            result.Init();
            return;
        }
    }

    if (rf.GetType() == csIdMETE) {
        const METEFrame& mf = (const METEFrame &)rf;
        if (t0 == mf.GetFixTime()) {
            result.Init();
            GetAstroBase()->MtxMETE2GCS(t0, result.Rotation);
            result.Rotation.Transp();
            return;
        }
    }

    if (rf.GetType() == csIdNZE) {
        const NZEFrame& lf = (const NZEFrame &)rf;
        lf.Get3DGreenwichConverter(result);
        result.Invert();
        return;
    }

    IFrame::Get3DConverter(rf, t0, result);
}

//------------------------------------------------------------------------------
// Получение объекта, который соответствует переходу от текущей системы отсчета 
// к системе отсчета rf на момент времени t0 (для 6-мерных векторов)
void GreenwichFrame::Get6DConverter(const IFrame& rf, const TimeJD& t0,
                                    FrameConverter6D& result) const
{
    if (rf.GetType() == csIdGFCS) {
        const FGreenwichFrame& gf = (const FGreenwichFrame &)rf;
        if (t0 == gf.GetFixTime()) {
            result.Init();
            GetAstroBase()->GCSVelocityMatrix(result.VRotation);
            result.VRotation.Transp();
            return;
        }
    }

    if (rf.GetType() == csIdMETE) {
        const METEFrame& mf = (const METEFrame &)rf;
        if (t0 == mf.GetFixTime()) {
            result.Init();
            Get3DConverter(rf, t0, result.Converter3D);
            TransMtx w;
            GetAstroBase()->GCSVelocityMatrix(w);
            w.Transp();
            result.Converter3D.Rotation.mul(w, result.VRotation);
            return;
        }
    }

    if (rf.GetType() == csIdNZE) {
        const NZEFrame& lf = (const NZEFrame &)rf;
        lf.Get6DGreenwichConverter(result);
        result.Invert();
        return;
    }

    IFrame::Get6DConverter(rf, t0, result);
}

////////////////////////////////////////////////////////////////////////////////
