///////////////////////////////////////////////////////////////////////////////
// Библиотека небесной механики
// (с) ЗАО "КИА Системы", 2005 г.  
///////////////////////////////////////////////////////////////////////////////
// Алгоритмы астробаллистики, не зависящие от внешних данных
///////////////////////////////////////////////////////////////////////////////
#include "IAstroBase.h"
#include "Defines.h"
///////////////////////////////////////////////////////////////////////////////
// Базовый класс астробаллистической базы
///////////////////////////////////////////////////////////////////////////////
// Вычисление матрицы поворота системы координат в плоскости 
// истинного экватора на угол dt
void IAstroBase::MtxRotation(double dt, TransMtx& mtx) const
{
    double c, s;

    c = cos(dt);
    s = sin(dt);
    mtx[0] = c;
    mtx[1] = s;
    mtx[2] = 0;
    mtx[3] = -s;
    mtx[4] = c;
    mtx[5] = 0;
    mtx[6] = 0;
    mtx[7] = 0;
    mtx[8] = 1;
}

//-----------------------------------------------------------------------------
// Вычисление матрицы скоростей при переходе от фиксированного Гринвича
// к вращающемуся
void IAstroBase::GCSVelocityMatrix(TransMtx& m) const
{
    double w = GetEarthRotation();

    m = 0;
    m[1] = w;
    m[3] = -w;
}

//-----------------------------------------------------------------------------
// Вычисление матрицы перехода от Гринвича к системе NEZ
void IAstroBase::MtxGCS2NZE(const Vect3& vect, TransMtx& mtx) const
{
    double c = 1.0 / (1.0 - GetEarthOblatness());
    c *= c;
    double d = sqrt(vect.x * vect.x + vect.y * vect.y + c * c * vect.z * vect.z);
    double f = sqrt(vect.x * vect.x + vect.y * vect.y);

    Vect3 ey(vect.x, vect.y, c * vect.z);
    ey *= 1.0 / d;

    Vect3 ez(-vect.y / f, vect.x / f, 0);
    Vect3 ex = ey;
    ex.MulVect(ez);

    mtx[0] = ex.x;
    mtx[1] = ex.y;
    mtx[2] = ex.z;
    mtx[3] = ey.x;
    mtx[4] = ey.y;
    mtx[5] = ey.z;
    mtx[6] = ez.x;
    mtx[7] = ez.y;
    mtx[8] = ez.z;
}

//-----------------------------------------------------------------------------
// Вычисление вектора сдвига при переходе от гелиоцентрической системы
// к геоцентрической
void IAstroBase::GetGelio2GeoShift(const Vect3& vEarthMoon, const Vect3& vMoon, Vect3& result) const
{
    result.x = -vEarthMoon.x + vMoon.x / RLT;
    result.y = -vEarthMoon.y + vMoon.y / RLT;
    result.z = -vEarthMoon.z + vMoon.z / RLT;
}

///////////////////////////////////////////////////////////////////////////////
