///////////////////////////////////////////////////////////////////////////////
// Библиотека алгоритмов небесной механики
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
// Реализация общих методов класса IFrame
////////////////////////////////////////////////////////////////////////////////
#include "IFrames.h"
////////////////////////////////////////////////////////////////////////////////
// Получение объекта, который соответствует переходу от текущей системы отсчета 
// к системе отсчета rf на момент времени t0 (для 3-мерных векторов)
void IFrame::Get3DConverter(const IFrame& rf, const TimeJD& t0,
                            FrameConverter3D& result) const
{
    if (IsEqual(rf)) {
        result.Init();
        return;
    }

    FrameConverter3D toj, j_res;

    Get3DJ2000Converter(t0, toj);
    rf.Get3DJ2000Converter(t0, j_res);
    j_res.Invert();

    toj.Multiply(j_res, result);
}

//------------------------------------------------------------------------------
// Получение объекта, который соответствует переходу от текущей системы отсчета 
// к системе отсчета rf на момент времени t0 (для 6-мерных векторов)
void IFrame::Get6DConverter(const IFrame& rf, const TimeJD& t0, FrameConverter6D& result) const
{
    if (IsEqual(rf)) {
        result.Init();
        return;
    }

    FrameConverter6D toj, j_res;

    Get6DJ2000Converter(t0, toj);
    rf.Get6DJ2000Converter(t0, j_res);
    j_res.Invert();

    toj.Multiply(j_res, result);
}

////////////////////////////////////////////////////////////////////////////////
