///////////////////////////////////////////////////////////////////////////////
// Библиотека алгоритмов небесной механики
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
#ifndef BNOTOOL_IKOPROP_H
#define BNOTOOL_IKOPROP_H
///////////////////////////////////////////////////////////////////////////////
#include "AccAdapter.h"
#include "Matrix.h"

///////////////////////////////////////////////////////////////////////////////
// Класс описывающий динамические свойства КО при движении
///////////////////////////////////////////////////////////////////////////////
class IKOProp
{
public:
    // Конструктор по умолчанию
    IKOProp()
    {
        MMode = NULL;
    };
    // Конструктор
    IKOProp(MotionMode& m)
    {
        MMode = &m;
    };
    // Деструктор
    virtual ~IKOProp()
    {
    };
    // Установка модели возмущений
    void Init(const MotionMode& m)
    {
        MMode = &m;
    };
    // Вычисление полного вектора ускорения в J2000
    virtual void GetAcc(const AccAdapter& p, Vect3& acc) const = 0;

protected:
    const MotionMode* MMode;
};

///////////////////////////////////////////////////////////////////////////////
// Класс описывающий динамические свойства КО при движении и получении 
// матрицы частных производных
///////////////////////////////////////////////////////////////////////////////
class IKOPropVar : public IKOProp
{
public:
    // Получение числа варьируемых параметров
    virtual int GetNVars() const = 0;
    // Вычисление добавок к правым частям уравнений в вариациях, зависящих
    // от свойств KO
    virtual void GetRPVars(const AccAdapter& p, TransMtx& dfdr, TransMtx& dfdv,
                           DMatrix& dfdc) const = 0;
};

///////////////////////////////////////////////////////////////////////////////
#endif
