///////////////////////////////////////////////////////////////////////////////
// Библиотека алгоритмов небесной механики
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
#include "IMeasurement.h"
///////////////////////////////////////////////////////////////////////////////
// Класс определяющий интерфейс измерения
// Реализованы наиболее общие методы для всех типов измерений
///////////////////////////////////////////////////////////////////////////////
// Конструктор
//-----------------------------------------------------------------------------
IMeasurement::IMeasurement() :
    Type(mtUnknown),
    MTime(),
    m_rearranged(true),
    m_droppable(true),
    counter(0)
{
}

//-----------------------------------------------------------------------------
// Метод возвращает в переменной p весовую матрицу измерения,
// которая является квадратной симметричной матрицей
//-----------------------------------------------------------------------------
void IMeasurement::GetWeightMatrix(DSymmMatrix& p) const
{
    p.Init(GetSize());
    unsigned i, j;

    DVector pm;
    GetMSEVector(pm);

    for (i = 0; i < GetSize(); i++)
        p(i, i) = 1.0 / (pm[i] * pm[i]);

    for (i = 0; i < GetSize() - 1; i++) {
        for (j = i + 1; j < GetSize(); j++)
            p(i, j) = 0.0;
    }
}

///////////////////////////////////////////////////////////////////////////////
// Класс, реализующий наиболее общие методы для всех типов измерений
///////////////////////////////////////////////////////////////////////////////
// Конструктор
//-----------------------------------------------------------------------------
AMeasurement::AMeasurement() : IMeasurement(),
                               // WrkFrame(0),
                               FrameConverter()
{
}

//-----------------------------------------------------------------------------
// Метод установки рабочей системы отсчета
//-----------------------------------------------------------------------------
AMeasurement::~AMeasurement()
{
    /*	if (WrkFrame != NULL){
            delete WrkFrame;
            WrkFrame = 0;
        }*/
}

//-----------------------------------------------------------------------------
// Метод установки рабочей системы отсчета
//-----------------------------------------------------------------------------
void AMeasurement::SetWrkFrame(const IFrame& fr) const
{
    //  TAssert(GetSrcFrame() != NULL);

    // if (WrkFrame != 0 && WrkFrame->IsEqual(fr))
    // return;

    //  if (WrkFrame != NULL)
    //  delete WrkFrame;
    ///////////////////////////////////////////////////////////////////////////////
    IFrame* WrkFrame = CreateIFrame(fr.GetParameters());
    WrkFrame->Get6DConverter(*GetSrcFrame(), GetTime(), FrameConverter);
}
