///////////////////////////////////////////////////////////////////////////////
// Библиотека алгоритмов небесной механики
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
#include "IODE.h"
#include "Defines.h"

#include <string.h>
///////////////////////////////////////////////////////////////////////////////
// Класс определяет интерфейс объекта, выполняющего решение системы дифф.
// уравнений
///////////////////////////////////////////////////////////////////////////////
// Конструктор
//-----------------------------------------------------------------------------
IODESolver::IODESolver()
{
    System = NULL;
    Y0 = NULL;
    Yk = NULL;
    IsInit = false;
}

//-----------------------------------------------------------------------------
// Деструктор
//-----------------------------------------------------------------------------
IODESolver::~IODESolver()
{
    System = NULL;
    if (Y0 != NULL) {
        delete[] Y0;
        delete[] Yk;
    }

    Y0 = NULL;
    Yk = NULL;
}

//-----------------------------------------------------------------------------
// Установка системы уравнений
//-----------------------------------------------------------------------------
void IODESolver::SetSystem(IODESystem* odesystem)
{
    TAssert(odesystem != NULL);

    if (Y0 != NULL) {
        delete[] Y0;
        delete[] Yk;
    }
    Y0 = new double[odesystem->GetDimension() + 1];
    Yk = new double[odesystem->GetDimension() + 1];

    System = odesystem;
    IsInit = false;
}

//-----------------------------------------------------------------------------
// Установка начальных условий
//-----------------------------------------------------------------------------
void IODESolver::SetInitial(double x, const double* y)
{
    TAssert(System != NULL);

    Y0[0] = x;
    memcpy(Y0 + 1, y, System->GetDimension() * sizeof(double));
    IsInit = true;
}

///////////////////////////////////////////////////////////////////////////////
