///////////////////////////////////////////////////////////////////////////////
// Библиотека алгоритмов небесной механики
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
// Интерфейсы для решения систем обыкновенных дифференциальных уравнений
///////////////////////////////////////////////////////////////////////////////
#ifndef BNOTOOL_IODE_H
#define BNOTOOL_IODE_H
///////////////////////////////////////////////////////////////////////////////
#include <stdlib.h>

///////////////////////////////////////////////////////////////////////////////
// Класс определяет интерфейс системы обыкновенных дифференциальных уравнений
///////////////////////////////////////////////////////////////////////////////
class IODESystem
{
public:
    // Получение размерности системы
    virtual ~IODESystem()
    {
    };
    // Получение размерности системы
    virtual int GetDimension() const = 0;
    // Вычисление правых частей
    // Метод возвращает false, если достигнута критичная точка,
    // после которой дальнейшее решение невозможно
    virtual bool GetRP(double t, const double* ph, double* f) = 0;
};

///////////////////////////////////////////////////////////////////////////////
// Класс определяет интерфейс объекта, выполняющего решение системы дифф. 
// уравнений
///////////////////////////////////////////////////////////////////////////////
class IODESolver
{
public:
    // Конструктор по умолчанию
    IODESolver();
    // Деструктор
    virtual ~IODESolver();

    // Установка системы уравнений
    virtual void SetSystem(IODESystem* odesystem);
    // Запрос системы уравнений
    virtual const IODESystem* GetSystem(IODESystem* odesystem)
    {
        return System;
    };
    // Установка начальных условий
    virtual void SetInitial(double x, const double* y);
    // Запуск процедуры решения до заданного значения аргумента
    virtual bool Run(double x) = 0;
    // Запрос полученного решения
    virtual const double* GetSolution() const
    {
        return Yk + 1;
    };
    // Запрос аргумента решения
    virtual double GetX() const
    {
        return Yk[0];
    };

protected:
    // Объект, определяющий вид системы уравнений
    IODESystem* System;
    // Начальные условия (аргумент + фаз. переменные)
    double* Y0;
    // Результат решения (аргумент + фаз. переменные)
    double* Yk;
    // Флаг инициализации начальных условий
    bool IsInit;
};

///////////////////////////////////////////////////////////////////////////////
#endif
