///////////////////////////////////////////////////////////////////////////////
// Библиотека алгоритмов небесной механики
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
// Общие интерфейсы прогноза
///////////////////////////////////////////////////////////////////////////////
#ifndef BNOTOOL_IPREDICT_H
#define BNOTOOL_IPREDICT_H
///////////////////////////////////////////////////////////////////////////////
#include "PhasePoint.h"
#include "Matrix.h"

///////////////////////////////////////////////////////////////////////////////
// Прогноз фазовых переменных (положение в протранстве и вектор скорости)
///////////////////////////////////////////////////////////////////////////////
class IPredict
{
public:
    // Собственно прогнозирование
    virtual ~IPredict()
    {
    };
    // Собственно прогнозирование
    virtual bool Run(const TimeJD& t) = 0;
    // Получение конечной точки
    virtual void GetDestPoint(PhasePoint6D& dest) = 0;

    virtual void GetDestPointEX(Vect6& dest)
    {
    };

    virtual void GetDestMatrix(DMatrix& m)
    {
    };

    virtual void ReRun()
    {
    };
};

///////////////////////////////////////////////////////////////////////////////
// Прогноз фазовых переменных и матрицы частных производных
// (положение в протранстве и вектор скорости)
///////////////////////////////////////////////////////////////////////////////
class IPredictVar : public IPredict
{
public:
    // Получение матрицы частных производных
    virtual void GetDestMatrix(DMatrix& m) = 0;
};

///////////////////////////////////////////////////////////////////////////////
#endif
