///////////////////////////////////////////////////////////////////////////////
// Библиотека программных инструментов
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
// Списки общепринятых идентификаторов комплекса - номера планет, атмосферных 
// и грав. моделей, и пр.
///////////////////////////////////////////////////////////////////////////////
#ifndef BNOTOOL_IDS_H
#define BNOTOOL_IDS_H

///////////////////////////////////////////////////////////////////////////////
// Номера планет (барицентров систем тел) Солнечной системы
///////////////////////////////////////////////////////////////////////////////
enum TPlanetId
{
    plIdMercury = 1,
    plIdVenus = 2,
    plIdEarth = 3,
    plIdEarthMoon = 399,
    plIdMoon = 301,
    plIdMars = 4,
    plIdJupiter = 5,
    plIdSaturn = 6,
    plIdUranus = 7,
    plIdNeptune = 8,
    plIdPluto = 9,
    plIdSun = 0,
    plIdSunBari = 99
};

///////////////////////////////////////////////////////////////////////////////
// Возможные типы моделей атмосферы
///////////////////////////////////////////////////////////////////////////////
enum TAtmosphereId
{
    atmIdNONE = 0,
    atmIdDMA90 = 1,
    atmIdDMA77 = 2,
    atmIdMSIS90 = 3,
    atmIdDMA2000 = 4,
    atmIdUSSA76 = 5,
    atmIdJ72 = 6,
    atmIdJR = 7,
    atmIdCIRA90 = 8,
    atmIdC81 = 9,
    atmIdDMA84 = 10,
    atmIdStatic = 11,
    atmIdDMA2004 = 12,
    atmIdDMA2004m = 13
};

///////////////////////////////////////////////////////////////////////////////
// Типы гравитационных моделей, известные базе данных
///////////////////////////////////////////////////////////////////////////////
enum TGravityModelId
{
    grIdPZ90 = 1,
    grIdEGM96 = 2,
    grIdJGM3 = 3,
    grIdIERS = 3,
    grIdJGM2 = 4,
    grIdJGM1 = 5,
    grIdGEMT3 = 6,
    grIdGEMT2 = 7,
    grIdGEMT1 = 8,
    grIdWGS84 = 9,
    grIdWGS72 = 10,
    grIdMu = 11,
    // модели для планет содержат только мю - гравитационную постоянную
    grIdPZ902s = 12,
    // Усеченная модель ПЗ-90-2 36х36
    grIdPZ902f = 13,
    grIdNONE = 0
};

///////////////////////////////////////////////////////////////////////////////
// Список возможных координатных систем
///////////////////////////////////////////////////////////////////////////////
enum TCoordSysId
{
    csIdJ2000 = 1,
    csIdB1950 = 2,
    csIdB1900 = 3,
    csIdMETE = 4,
    csIdGCS = 5,
    csIdGFCS = 6,
    csIdJ2000S = 7,
    csIdECI = 8,
    csIdNZE = 9,
    csIdRNB = 11,
    csIdLocalJ2000 = 12,
    csIdLocalRL = 13
};

///////////////////////////////////////////////////////////////////////////////
// Возможные признаки учета светового давления
///////////////////////////////////////////////////////////////////////////////
enum TTypeLight
{
    lightNONE,
    lightNoShadow,
    lightCylinderShadow
};

///////////////////////////////////////////////////////////////////////////////
// Возможные режимы печати в процедурах типа check...
///////////////////////////////////////////////////////////////////////////////
enum TCheckPrint
{
    chPrintNone,
    // не печатать ничего
    chPrintError,
    // печатать сообщения об ошибках
    chPrintFull // печатать общую информацию (что проверено, размер таблиц и т.п.)
};

///////////////////////////////////////////////////////////////////////////////
// Возможные значения признака способа расчета номера витка
///////////////////////////////////////////////////////////////////////////////
enum TMethodNRev
{
    methodNRevZ = 0,
    methodNRevG = 1
};

///////////////////////////////////////////////////////////////////////////////
// Типы обобщенных векторов состояния
///////////////////////////////////////////////////////////////////////////////
enum TUniVect
{
    uvIdSvd6 = 0,
    uvIdKepler = 1
};

///////////////////////////////////////////////////////////////////////////////
#endif
