///////////////////////////////////////////////////////////////////////////////
// Библиотека алгоритмов небесной механики
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
// Определение класса описания гелиоцентрической системы J2000
////////////////////////////////////////////////////////////////////////////////
#include "J2000SFrame.h"
#include "IAstroBase.h"
////////////////////////////////////////////////////////////////////////////////
// Получение объекта, который соответствует переходу от текущей системы отсчета 
// к системе отсчета J2000 на момент времени t0 (для 3-мерных векторов)
void J2000SFrame::Get3DJ2000Converter(const TimeJD& t0, FrameConverter3D& result) const
{
    IAstroBase* ab = GetAstroBase();
    Vect3 cEarthMoon, cMoon;

    ab->GetPosGelio(plIdEarthMoon, t0, &cEarthMoon);
    ab->GetPosGeo(plIdMoon, t0, &cMoon);

    result.Init();
    ab->GetGelio2GeoShift(cEarthMoon, cMoon, result.Shift);
}

//------------------------------------------------------------------------------
// Получение объекта, который соответствует переходу от текущей системы отсчета 
// к системе отсчета J2000 на момент времени t0 (для 6-мерных векторов)
void J2000SFrame::Get6DJ2000Converter(const TimeJD& t0, FrameConverter6D& result) const
{
    IAstroBase* ab = GetAstroBase();
    Vect3 cEarthMoon, cMoon;
    Vect3 vEarthMoon, vMoon;

    ab->GetPosGelio(plIdEarthMoon, t0, &cEarthMoon, &vEarthMoon);
    ab->GetPosGeo(plIdMoon, t0, &cMoon, &vMoon);

    result.Init();
    ab->GetGelio2GeoShift(cEarthMoon, cMoon, result.Converter3D.Shift);
    ab->GetGelio2GeoShift(vEarthMoon, vMoon, result.VShift);
}

////////////////////////////////////////////////////////////////////////////////
