///////////////////////////////////////////////////////////////////////////////
// Библиотека алгоритмов небесной механики
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
// Реализация модели КО с постоянными баллистическим коэффициентом и 
// коэффициентом светового давления
///////////////////////////////////////////////////////////////////////////////
#include "KOConst.h"
#include "Consts.h"
#include "IAstroBase.h"
///////////////////////////////////////////////////////////////////////////////
// Вычисление полного вектора ускорения в J2000
//-----------------------------------------------------------------------------
void KOConst::GetAcc(const AccAdapter& p, Vect3& acc) const
{
    TAssert(MMode != NULL);

    acc.x = acc.y = acc.z = 0;

    if (MMode->IsAtm) {
        Vect3 aacc;
        AtmAcc(p, aacc);
        acc += aacc;
    }

    if (MMode->LightMode != lightNONE) {
        Vect3 lacc;
        LightAcc(p, lacc);
        acc += lacc;
    }
}

//-----------------------------------------------------------------------------
// Вычисление атмосферного ускорения
//-----------------------------------------------------------------------------
void KOConst::AtmAcc(const AccAdapter& p, Vect3& acc) const
{
    double t1;
    t1 = -Kb * p.Density * p.VGr.Len() * 1.e6;

    acc.x = p.VGr.x * t1;
    acc.y = p.VGr.y * t1;
    acc.z = p.VGr.z * t1;

    // Ускорение получено в Гринвиче. Переводим его в J2000

    acc *= p.CGFCStoJ2000;
}

//-----------------------------------------------------------------------------
// Вычисление ускорения от светового давления
//-----------------------------------------------------------------------------
double KOConst::LightAcc(const AccAdapter& p, Vect3& acc) const
{
    acc.x = acc.y = acc.z = 0;

    if (MMode->LightMode == lightCylinderShadow) {
        double rs = p.SunJ2000 * p.RJ2000;
        if (rs < 0) {
            rs /= p.SunJ2000.Len();
            double rk = p.RJ2000.Len2();
            if (rs * rs > rk - REarthMid * REarthMid)
                return 0.0;
        }
    }

    Vect3 ps = p.RJ2000;
    ps -= p.SunJ2000;
    double len = ps.Len2();
    len *= sqrt(len);

    double t1 = Kappa * GetAstroBase()->GetPlanet(plIdSun)->GetGravityModel()->mu / len;

    acc.x = ps.x * t1;
    acc.y = ps.y * t1;
    acc.z = ps.z * t1;
    return 1.0;
}

///////////////////////////////////////////////////////////////////////////////
