///////////////////////////////////////////////////////////////////////////////
// Библиотека алгоритмов небесной механики
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
// Реализация модели КО для уравнений в вариациях с постоянными баллистическим 
// коэффициентом и коэффициентом светового давления
///////////////////////////////////////////////////////////////////////////////
#ifndef BNOTOOL_KOCONSTVAR_H
#define BNOTOOL_KOCONSTVAR_H
///////////////////////////////////////////////////////////////////////////////
#include "IKOProp.h"

///////////////////////////////////////////////////////////////////////////////
class KOConstVar : public IKOPropVar
{
public:
    // Конструктор по умолчанию
    KOConstVar();
    // Баллистический коэффициент
    double Kb;
    // Коэффициент светового давления
    double Kappa;
    // Флаг вычисления вариаций от балл. коэф-та
    bool IsKbVar;
    // Флаг вычисления вариаций от коэф-та светового давления
    bool IsKappaVar;

    // Вычисление полного вектора ускорения в J2000
    virtual void GetAcc(const AccAdapter& p, Vect3& acc) const;
    // Получение числа варьируемых параметров
    virtual int GetNVars() const;
    // Вычисление добавок к правым частям уравнений в вариациях, зависящих
    // от свойств KO
    virtual void GetRPVars(const AccAdapter& p, TransMtx& dfdr, TransMtx& dfdv,
                           DMatrix& dfdc) const;
    // Вычисление вариаций от атмосферы по координатам и скоростям
    void GetAtmVars(const AccAdapter& p, TransMtx& dfdr, TransMtx& dfdv) const;

protected:
    // Вычисление вариаций от светового давления по координатам и скоростям
    void GetSolarVars(const AccAdapter& p, TransMtx& dfdr) const;
    // Вычисление явных производных от баллистического коэффициента
    void GetKbVars(const AccAdapter& p, Vect3& var) const;
    // Вычисление явных производных от коэффициента св. давления
    void GetKappaVars(const AccAdapter& p, Vect3& var) const;
};

///////////////////////////////////////////////////////////////////////////////
#endif
