///////////////////////////////////////////////////////////////////////////////
// Библиотека алгоритмов небесной механики
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
// Класс орбитальных элементов Кеплера
///////////////////////////////////////////////////////////////////////////////
#ifndef BNOTOOL_KEPLER_H
#define BNOTOOL_KEPLER_H
///////////////////////////////////////////////////////////////////////////////
#include "ICoords6D.h"
#include "IModel.h"

///////////////////////////////////////////////////////////////////////////////
class Kepler : public ICoords6D
{
public:
    double A; // Большая полуось орбиты
    double Ex; // e * cos(omp)
    double Ey; // e * sin(omp)
    double CosI; // косинус наклонения
    double Omega; // Долгота восходящего узла
    double U; // Аргумент широты

    // Конструктор по умолчанию
    Kepler();

    TimeJD GetTimeFromAN(const TimeJD& base) const;

    // Установка гравитационной постоянной
    void SetParam(double mu)
    {
        Mu = mu;
    };
    // Получение гравитационной постоянной
    double GetMu() const
    {
        return Mu;
    };

    // Установка координат и скоростей по декартовым
    virtual void SetXYZ(const Vect6& src);
    // Получение декартовых координат и скоростей
    virtual bool GetXYZ(Vect6& dst) const;

    // Вычисляемые параметры
    // Период орбиты
    double GetPeriod() const;
    // Радиус перицентра
    double GetRP() const;
    // Радиус апоцентра
    double GetRA() const;
    // Высота перицентра над эллипсоидом
    double GetHP(const TGeometryModel* geom = NULL) const;
    // Высота апоцентра над эллипсоидом
    double GetHA(const TGeometryModel* geom = NULL) const;
    // Эксцентриситет орбиты
    double GetEcc() const;
    // Аргумент перицентра
    double GetArgP() const;
    // Наклонение относительно плоскости Лапласа
    double GetLaplaceInc() const;
    // Долгота восходящего узла в плоскости Лапласа
    double GetLaplaceLAN() const;
    // Время движения от восх. узла до текущей точки
    double GetTimeFromAN() const;
    // Истиная аномалия
    double GetTrueAnomaly() const;
    // чЩЮЙУМЕОЙЕ ЙОФЕТЧБМБ ЧТЕНЕОЙ НЕЦДХ РЕТЙГЕОФТПН Й ХЪМПН (бМЗПТЙФН EL23)
    double GetTimeP() const;
    double GetSPeriod(int NF) const;
protected:
    double Mu; // Гравитационная постоянная небесного тела 
};

///////////////////////////////////////////////////////////////////////////////
#endif
