///////////////////////////////////////////////////////////////////////////////
// Библиотека алгоритмов небесной механики
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
// Класс орбитальных элементов Лагранжа
///////////////////////////////////////////////////////////////////////////////
#ifndef BNOTOOL_LAGRANGE_H
#define BNOTOOL_LAGRANGE_H
///////////////////////////////////////////////////////////////////////////////
#include "ICoords6D.h"
#include "IModel.h"
#include "Matrix.h"

///////////////////////////////////////////////////////////////////////////////
class Lagrange : public ICoords6D
{
public:
    double Lambda;
    double L;
    double P;
    double Q;
    double H;
    double K;

    // Конструктор по умолчанию
    Lagrange();

    // Установка гравитационной постоянной
    void SetParam(double mu)
    {
        Mu = mu;
    };
    // Получение гравитационной постоянной
    double GetMu() const
    {
        return Mu;
    };

    // Установка координат и скоростей по декартовым
    virtual void SetXYZ(const Vect6& src);
    // Получение декартовых координат и скоростей
    virtual bool GetXYZ(Vect6& dst) const;

    // Вычисляемые параметры
    // Период орбиты
    double GetPeriod() const;
    // Радиус перицентра
    double GetRP() const;
    // Эксцентриситет орбиты
    double GetEcc() const;

    // Сохранение в файле
    void Save(FILE* f) const;
    // Загрузка из файла
    void Load(FILE* f);

protected:
    double Mu; // Гравитационная постоянная небесного тела 
    double Prec; // Точность при решении уравнения Кеплера
    int NMax; // Макс. число итераций при решении уравнения

    // Решение общего кеплеровского уравнения
    bool SolveKeplerEq(double* oke) const;
};

///////////////////////////////////////////////////////////////////////////////
// Вычисление матрицы производных от по прямоугольных координат
// по элементат Лагранжа
///////////////////////////////////////////////////////////////////////////////
bool DXYZDLagrange(const Lagrange& lagr, DSqrMatrix& d);
///////////////////////////////////////////////////////////////////////////////
#endif
