///////////////////////////////////////////////////////////////////////////////
// Библиотека алгоритмов небесной механики
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
// Определение класса описания локальной (топоцентрической) системы J2000
////////////////////////////////////////////////////////////////////////////////
#include "LocalJ2000Frame.h"
#include "FGreenwichFrame.h"
#include "GreenwichFrame.h"
#include "IAstroBase.h"
////////////////////////////////////////////////////////////////////////////////
// Установка параметров системы
//------------------------------------------------------------------------------
void LocalJ2000Frame::SetParameters(const IFrameParameters& p)
{
    TAssert(p.GetFrameType() == csIdLocalJ2000);
    const LocalJ2000FrameParameters& pm = (const LocalJ2000FrameParameters &)p;
    Pars = pm;
}

//------------------------------------------------------------------------------
// Получение объекта, который соответствует переходу от текущей системы отсчета 
// к системе отсчета J2000 на момент времени t0 (для 3-мерных векторов)
//------------------------------------------------------------------------------
void LocalJ2000Frame::Get3DJ2000Converter(const TimeJD& t0, FrameConverter3D& result) const
{
    TAssert(Pars.IsInit());

    result.Init();

    FGreenwichFrame fg;
    FrameConverter3D gj;

    fg.Init(t0);
    fg.Get3DJ2000Converter(t0, gj);

    Pars.GetPoint().GetXYZ(result.Shift);
    result.Shift.MulLeft(gj.Rotation);
}

//------------------------------------------------------------------------------
// Получение объекта, который соответствует переходу от текущей системы отсчета 
// к системе отсчета J2000 на момент времени t0 (для 6-мерных векторов)
//------------------------------------------------------------------------------
void LocalJ2000Frame::Get6DJ2000Converter(const TimeJD& t0, FrameConverter6D& result) const
{
    TAssert(Pars.IsInit());

    result.Init();

    FGreenwichFrame fg;
    FrameConverter3D gj;

    fg.Init(t0);
    fg.Get3DJ2000Converter(t0, gj);

    Vect3 rg;
    Pars.GetPoint().GetXYZ(rg);
    result.Converter3D.Shift = rg;
    result.Converter3D.Shift.MulLeft(gj.Rotation);

    TransMtx w;
    GetAstroBase()->GCSVelocityMatrix(w);
    w.Transp();

    result.VShift = rg;
    result.VShift.MulLeft(w);
    result.VShift.MulLeft(gj.Rotation);
}

////////////////////////////////////////////////////////////////////////////////
