///////////////////////////////////////////////////////////////////////////////
// Библиотека алгоритмов небесной механики
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
// Определение класса описания локальной (топоцентрической) системы J2000
////////////////////////////////////////////////////////////////////////////////
#ifndef BNOTOOL__LOCALJ2000FRAME_H
#define BNOTOOL__LOCALJ2000FRAME_H
////////////////////////////////////////////////////////////////////////////////
#include "Defines.h"
#include "IFrames.h"
#include "SFramePars.h"

////////////////////////////////////////////////////////////////////////////////
class LocalJ2000Frame : public IFrame
{
public:
    // Получение ссылки на параметры системы
    virtual const IFrameParameters& GetParameters() const
    {
        return Pars;
    };
    // Установка параметров системы
    virtual void SetParameters(const IFrameParameters& p);
    // Является ли система инерциальной ?
    virtual bool IsInertial() const
    {
        return false;
    };
    // Является ли система геоцентрической ?
    virtual bool IsGeocentric() const
    {
        return false;
    };

    // Получение объекта, который соответствует переходу от текущей системы отсчета 
    // к системе отсчета J2000 на момент времени t0 (для 3-мерных векторов)
    virtual void Get3DJ2000Converter(const TimeJD& t0, FrameConverter3D& result) const;
    // Получение объекта, который соответствует переходу от текущей системы отсчета 
    // к системе отсчета J2000 на момент времени t0 (для 6-мерных векторов)
    virtual void Get6DJ2000Converter(const TimeJD& t0, FrameConverter6D& result) const;

    // Инициалицазия системы с указанием координат ее начальной точки
    void Init(const GeoCoordsPrec& p)
    {
        Pars.Init(p);
    };

protected:
    // Параметры системы
    LocalJ2000FrameParameters Pars;
};

////////////////////////////////////////////////////////////////////////////////
#endif
