///////////////////////////////////////////////////////////////////////////////
// Библиотека алгоритмов небесной механики
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
// Определение класса описания геоцентрической системы METE
////////////////////////////////////////////////////////////////////////////////
#ifndef BNOTOOL__METEFRAME_H
#define BNOTOOL__METEFRAME_H
////////////////////////////////////////////////////////////////////////////////
#include "Defines.h"
#include "IFrames.h"
#include "SFramePars.h"

////////////////////////////////////////////////////////////////////////////////
class METEFrame : public IFrame
{
public:
    // Конструктор по умолчанию
    METEFrame()
    {
    };
    // Конструктор с указанием времени фиксации
    METEFrame(const TimeJD& t)
    {
        Init(t);
    };

    // Инициализация с указанием времени фиксации
    METEFrame& Init(const TimeJD& t)
    {
        Pars.TFrameParameters::Init(t);
        return *this;
    };

    // Получение ссылки на параметры системы
    virtual const IFrameParameters& GetParameters() const
    {
        return Pars;
    };
    // Установка параметров системы
    virtual void SetParameters(const IFrameParameters& p);
    // Является ли система инерциальной ?
    virtual bool IsInertial() const
    {
        return true;
    };
    // Является ли система геоцентрической ?
    virtual bool IsGeocentric() const
    {
        return true;
    };

    // Получение объекта, который соответствует переходу от текущей системы отсчета 
    // к системе отсчета J2000 на момент времени t0 (для 3-мерных векторов)
    virtual void Get3DJ2000Converter(const TimeJD& t0, FrameConverter3D& result) const;
    // Получение объекта, который соответствует переходу от текущей системы отсчета 
    // к системе отсчета J2000 на момент времени t0 (для 6-мерных векторов)
    virtual void Get6DJ2000Converter(const TimeJD& t0, FrameConverter6D& result) const;

    // Получение объекта, который соответствует переходу от текущей системы отсчета 
    // к системе отсчета rf на момент времени t0 (для 3-мерных векторов)
    virtual void Get3DConverter(const IFrame& rf, const TimeJD& t0,
                                FrameConverter3D& result) const;
    // Получение объекта, который соответствует переходу от текущей системы отсчета 
    // к системе отсчета rf на момент времени t0 (для 6-мерных векторов)
    virtual void Get6DConverter(const IFrame& rf, const TimeJD& t0,
                                FrameConverter6D& result) const;

    const TimeJD& GetFixTime() const
    {
        return Pars.GetTime();
    };

protected:
    // Параметры системы
    METEFrameParameters Pars;
};

////////////////////////////////////////////////////////////////////////////////
#endif
