///////////////////////////////////////////////////////////////////////////////
// Библиотека алгоритмов небесной механики
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
// Описание модели движения для численного прогноза
///////////////////////////////////////////////////////////////////////////////
#include "MMode.h"
///////////////////////////////////////////////////////////////////////////////
// Конструктор по умолчанию
//-----------------------------------------------------------------------------
MotionMode::MotionMode()
{
    Reset();
}

//-----------------------------------------------------------------------------
// Конструктор копирования
//-----------------------------------------------------------------------------
MotionMode& MotionMode::operator=(const MotionMode& src)
{
    IsAtm = src.IsAtm;
    LightMode = src.LightMode;
    HCrit = src.HCrit;
    GrN = src.GrN;
    GrM = src.GrM;
    PList.Clear();

    int i, n = src.GetNPlanet();
    for (i = 0; i < n; i++)
        AddPlanet(src.GetPlanet(i));

    return *this;
}

//-----------------------------------------------------------------------------
// Добавить в модель центр притяжения
//-----------------------------------------------------------------------------
void MotionMode::AddPlanet(TPlanetId p)
{
    int k = FindPlanet(p);
    if (k < 0) {
        TPlanetId* n = new TPlanetId;
        *n = p;
        PList.Add(n);
    }
}

//-----------------------------------------------------------------------------
// Удалить центр притяжения
//-----------------------------------------------------------------------------
void MotionMode::DeletePlanet(TPlanetId p)
{
    int k = FindPlanet(p);
    if (k >= 0)
        PList.Delete(k);
}

//-----------------------------------------------------------------------------
// Запрос установленной планеты по индексу
//-----------------------------------------------------------------------------
TPlanetId MotionMode::GetPlanet(int i) const
{
    TAssert(i >= 0 && i < PList.Len());
    return *(PList[i]);
}

//-----------------------------------------------------------------------------
// Сброс всех настроек в значения по умолчанию
//-----------------------------------------------------------------------------
void MotionMode::Reset()
{
    IsAtm = true;
    HCrit = 70e-3; // в тыс. километров
    LightMode = lightNONE;
    PList.Clear();
    AddPlanet(plIdEarth);
    GrN = GrM = 8;
}

//-----------------------------------------------------------------------------
// Поиск в списке планеты с указанным id
//-----------------------------------------------------------------------------
int MotionMode::FindPlanet(TPlanetId id) const
{
    int i, n = PList.Len();
    for (i = 0; i < n; i++) {
        if (*PList[i] == id)
            return i;
    }

    return -1;
}

//-----------------------------------------------------------------------------
// Проверка присутствия указанной планеты в модели
//-----------------------------------------------------------------------------
bool MotionMode::IsPlanetSet(TPlanetId p) const
{
    return FindPlanet(p) >= 0;
}

///////////////////////////////////////////////////////////////////////////////
