///////////////////////////////////////////////////////////////////////////////
// Библиотека алгоритмов небесной механики
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
// Описание модели движения для численного прогноза
///////////////////////////////////////////////////////////////////////////////
#ifndef BNOTOOL_MOTIONMODE_H
#define BNOTOOL_MOTIONMODE_H
///////////////////////////////////////////////////////////////////////////////
#include "Ids.h"
#include "SList.h"

///////////////////////////////////////////////////////////////////////////////
class MotionMode
{
public:
    // Конструктор по умолчанию
    MotionMode();
    // Деструктор
    virtual ~MotionMode()
    {
    };

    MotionMode& operator=(const MotionMode& src);
    // Публичные переменные
    // Режим учета солнечного давления
    TTypeLight LightMode;
    // Признак учета земной атмосферы
    bool IsAtm;
    // Критическая высота (высота до которой возможен расчет)
    double HCrit;
    // Число гармоник, которое учитывется в нецентральном поле Земли
    int GrN, GrM;

    // Методы установки тел притяжения
    // Добавить в модель центр притяжения
    void AddPlanet(TPlanetId p);
    // Удалить центр притяжения
    void DeletePlanet(TPlanetId p);
    // Запрос числа установленных планет
    int GetNPlanet() const
    {
        return PList.Len();
    };
    // Запрос установленной планеты по индексу
    TPlanetId GetPlanet(int i) const;
    // Проверка присутствия указанной планеты в модели
    bool IsPlanetSet(TPlanetId p) const;

    // Сброс всех настроек в значения по умолчанию
    void Reset();

private:
    // Список установленных центров притяжения
    TSListDef<TPlanetId, true> PList;
    // Поиск в списке планеты с указанным id
    int FindPlanet(TPlanetId id) const;
};

///////////////////////////////////////////////////////////////////////////////
#endif
