///////////////////////////////////////////////////////////////////////////////
// Библиотека алгоритмов небесной механики
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
// Классы, используемые для хранения информации об общенных измерениях
///////////////////////////////////////////////////////////////////////////////
#include "MeasData.h"
///////////////////////////////////////////////////////////////////////////////
// Класс, описывающий одиночное измерение
///////////////////////////////////////////////////////////////////////////////
// Конструктор по умолчанию
//-----------------------------------------------------------------------------
MeasData::MeasData()
{
    Type = mtUnknown;
    Size = 0;;
    LatArg = 0;
    Deleted = false;
}

//-----------------------------------------------------------------------------
// Оператор присваивания из абстрактного измерения
//-----------------------------------------------------------------------------
const MeasData& MeasData::operator=(IMeasurement& m)
{
    Type = m.GetType();
    Size = m.GetSize();
    Time = m.GetTime();

    Delta.Init(Size);
    Delta.Init(0.0);

    functionals.Init(Size);
    functionals.Init(0.0);

    m.GetRealVector(Meas);
    m.GetMSEVector(PMeas);

    Id = m.GetID();
    StationId = m.GetStationID();
    ObjectId = m.GetObjectID();
    NightId = m.GetNightID();
    PacketId = m.GetPacketID();

    m_meas = &m;

    lincrt.Init(Size);

    return *this;
}

///////////////////////////////////////////////////////////////////////////////
// Класс, хранящий данные об измерениях в форме связанного списка
///////////////////////////////////////////////////////////////////////////////
// Копирование списка
//-----------------------------------------------------------------------------
void MeasDataList::Copy(const MeasDataList& src)
{
    clear();

    MeasDataEntry *e, *n;

    for (e = src.first(); e != NULL; e = e->next()) {
        n = new MeasDataEntry;
        *n = *e;
        insert_last(n, 0);
    }
}

//-----------------------------------------------------------------------------
// Добавление измерения в список
//-----------------------------------------------------------------------------
MeasDataEntry* MeasDataList::Add(IMeasurement& m)
{
    MeasDataEntry* n = new MeasDataEntry;
    MeasDataEntry* e;

    *(MeasData *)n = m;

    for (e = first(); e != NULL; e = e->next()) {
        if (n->Time < e->Time) {
            insert_prev(e, n, 0);
            return n;
        }
    }

    insert_last(n, 0);

    return n;
}

//-----------------------------------------------------------------------------
// Поиск по идентификатору измерения
//-----------------------------------------------------------------------------
const MeasDataEntry* MeasDataList::FindByID(TMeasId id) const
{
    MeasDataEntry* e;

    for (e = first(); e != NULL; e = e->next()) {
        if (e->Id == id)
            return e;
    }

    return NULL;
}

///////////////////////////////////////////////////////////////////////////////
