///////////////////////////////////////////////////////////////////////////////
// Библиотека алгоритмов небесной механики
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
// Классы, используемые для хранения информации об общенных измерениях
///////////////////////////////////////////////////////////////////////////////
#ifndef BNOTOOL_MEASDATA_H
#define BNOTOOL_MEASDATA_H
///////////////////////////////////////////////////////////////////////////////
#include "IMeasurement.h"
#include "twll.h"

///////////////////////////////////////////////////////////////////////////////
// Класс, описывающий одиночное измерение
///////////////////////////////////////////////////////////////////////////////
class MeasData
{
public:
    // Конструктор по умолчанию
    MeasData();
    // Деструктор
    virtual ~MeasData()
    {
    };

    // Оператор присваивания из абстрактного измерения
    const MeasData& operator=(IMeasurement& m);

    IMeasurement* m_meas;
    // Тип измерения
    int Type;
    // Размерность измерения
    unsigned Size;
    // Время измерения
    TimeJD Time;
    // Измеренные значения
    DVector Meas;
    // MSE вектор
    DVector PMeas;
    // Невязки измерений
    DVector Delta;
    // Идентификатор измерения
    TMeasId Id;
    // Идентификатор станции
    TSiteId StationId;
    // Идентификатор ночи
    TNightId NightId;
    //идентификатор проводки
    TPacketId PacketId;
    // Идентификатор объекта
    TObjectId ObjectId;
    // Вклад в функционал всех компонент
    DVector functionals;
    //  double FuncAll;
    // Вклад в функционал только учтенных компонент
    //	double FuncActive;
    //	int activeCount;
    // Аргумент широты измерения
    double LatArg;
    bool Deleted;
    Vect6 objPos;
    DVector lincrt;
    //	DMatrix mm;
    //	DMatrix mx;
};

///////////////////////////////////////////////////////////////////////////////
// Вспомогательный класс для представления данных об измерении в качестве 
// элемента списка
///////////////////////////////////////////////////////////////////////////////
typedef Twll_Entry_Class<MeasData> MeasDataEntry;

///////////////////////////////////////////////////////////////////////////////
// Класс, хранящий данные об измерениях в форме связанного списка
///////////////////////////////////////////////////////////////////////////////
class MeasDataList : public Twll_List_Class<MeasDataEntry>
{
public:
    // Конструктор по умолчанию
    MeasDataList() : Twll_List_Class<MeasDataEntry>()
    {
    };
    // Конструктор корирования
    MeasDataList(const MeasDataList& src) : Twll_List_Class<MeasDataEntry>()
    {
        Copy(src);
    };

    // Добавление измерения в список
    MeasDataEntry* Add(IMeasurement& m);
    // Копирование списка
    void Copy(const MeasDataList& src);
    // Поиск по идентификатору измерения
    const MeasDataEntry* FindByID(TMeasId id) const;
};

///////////////////////////////////////////////////////////////////////////////
#endif
