///////////////////////////////////////////////////////////////////////////////
// Библиотека алгоритмов небесной механики
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
// Определение класса системы отсчета NEZ
////////////////////////////////////////////////////////////////////////////////
#ifndef BNOTOOL__NZEFRAME_H
#define BNOTOOL__NZEFRAME_H
////////////////////////////////////////////////////////////////////////////////
#include "Defines.h"
#include "IFrames.h"
#include "SFramePars.h"

////////////////////////////////////////////////////////////////////////////////
class NZEFrame : public IFrame
{
public:
    // Конструктор по умолчанию
    NZEFrame()
    {
    };
    // Конструктор с указанием точки начала системы в Гринвиче
    NZEFrame(const Vect3& p)
    {
        Init(p);
    };

    // Инициализация с указанием точки начала системы в Гринвиче
    NZEFrame& Init(const Vect3& p)
    {
        Pars.Init(p);
        return *this;
    };
    // Получение ссылки на параметры системы
    virtual const IFrameParameters& GetParameters() const
    {
        return Pars;
    };
    // Установка параметров системы
    virtual void SetParameters(const IFrameParameters& p);
    // Является ли система инерциальной ?
    virtual bool IsInertial() const
    {
        return false;
    };
    // Является ли система геоцентрической ?
    virtual bool IsGeocentric() const
    {
        return false;
    };

    // Получение объекта, который соответствует переходу от текущей системы отсчета 
    // к системе отсчета J2000 на момент времени t0 (для 3-мерных векторов)
    virtual void Get3DJ2000Converter(const TimeJD& t0, FrameConverter3D& result) const;
    // Получение объекта, который соответствует переходу от текущей системы отсчета 
    // к системе отсчета J2000 на момент времени t0 (для 6-мерных векторов)
    virtual void Get6DJ2000Converter(const TimeJD& t0, FrameConverter6D& result) const;
    // Получение объекта, который соответствует переходу от текущей системы отсчета 
    // к системе отсчета rf на момент времени t0 (для 3-мерных векторов)
    virtual void Get3DConverter(const IFrame& rf, const TimeJD& t0,
                                FrameConverter3D& result) const;
    // Получение объекта, который соответствует переходу от текущей системы отсчета 
    // к системе отсчета rf на момент времени t0 (для 6-мерных векторов)
    virtual void Get6DConverter(const IFrame& rf, const TimeJD& t0,
                                FrameConverter6D& result) const;
    // Получение объекта, который соответствует переходу от текущей системы отсчета 
    // к Гринвичу на момент времени t0 (для 3-мерных векторов)
    void Get3DGreenwichConverter(FrameConverter3D& result) const;
    // Получение объекта, который соответствует переходу от текущей системы отсчета 
    // к Гринвичу на момент времени t0 (для 6-мерных векторов)
    void Get6DGreenwichConverter(FrameConverter6D& result) const;

    const Vect3& GetPoint() const
    {
        return Pars.GetPoint();
    };

protected:
    // Параметры системы
    NZEFrameParameters Pars;
};

////////////////////////////////////////////////////////////////////////////////
#endif
