///////////////////////////////////////////////////////////////////////////////
//    
// ()  ...,  ""
///////////////////////////////////////////////////////////////////////////////
//       
///////////////////////////////////////////////////////////////////////////////
#ifndef BNOTOOL_ODCOREL_H
#define BNOTOOL_ODCOREL_H
///////////////////////////////////////////////////////////////////////////////
#include "ITraject.h"
#include "IMeasurementPacket.h"
#include "twll.h"
#include "Matrix.h"
#include "MeasData.h"
#include <stdio.h>
#include "AprioriConst.h"



class IODCorel
  {
  public:
	  void CalcCriteria(double& crt, double& sigma);

      //       
    typedef enum
      {
        //       
      UNDEFINED,
        //       ,   
      FIRST_MEAS,
        //       ,   
      LAST_MEAS,
        //          
      TRAJECT
      } EnEstimTimeType;

       //  ,    
    class FuncItem
      {
      public:
        double Value;
      };
    typedef Twll_Entry_Class<FuncItem> FuncEntry;
       //        
       //   
    class FunctHistoryType : public Twll_List_Class<FuncEntry>
      {
      public:
          //          
        void Add(double f);
          //     
        unsigned Size() const
          { return size();};
          //          index
        double Get(unsigned index) const;
          //    
        void Clear()
          { clear();};
      };

      //   
    IODCorel();
virtual ~IODCorel() {};
      //   . :
      // u0 -      
      // trj - c  ,     
      //                  
      // pack -    
      // etype -    
      //       ,     
      //   
    void Init(const DVector &u0, ITrajectVar &trj, IMeasurementPacket &pack,
          EnEstimTimeType etype = TRAJECT);
      //             
    void Clear();
      //             
      //   true,     
      //  false   
    bool DoOneStep();
      //   ,      ,
      //     DoOneStep() 
    void Accept();
      //         n.      
      //    DoOneStep()  Accept()   
      //  .
      //  ,    
    void Run(unsigned n, FILE *f = NULL, FILE *fm = NULL);
      //     
    void Stop();
      //     F
    double GetCurrentFunc() const ;
      //  true,      
    bool HasActive()
      { return HasActiveFlag;};
      //      
    DVector U0;
      //      
    DVector U0m;
      //     
    DVector Uc;
      //      ,    
    DVector Du;

      // ,           
    FunctHistoryType FunctHistory;
      //   
    double Criteria;
      //        
    double MatrCoeff;
      //          
    double DeltaCoeff;
      //   ,    
    const IFrame *BaseFrame;
      //     
    bool SolvFlag;

	double m_curSigma;
	double m_curCriteria;

      //   ,   
    MeasDataList MDataList;
      //     
    MeasDataList MInactiveDataList;
      //    
    bool FlagSaveInActive;
      //    
    const DSymmMatrix & GetBaseMatrix() const
      { return A;};
      //     
    const TimeJD & GetEstimTime() const
      { return EstimTime;};

      //    ( )
    void PrintVector(FILE *f = stdout);
      //   ( )
    void PrintFunc(FILE *f = stdout);
      //    ( )
    void PrintMeasSet(FILE *f = stdout);
      //   
    IAprioriConstList ACList;

  private:
      //   ,      
    TimeJD EstimTime;
      //   
    bool StopedFlag;
      //       
    bool HasActiveFlag;
      //      

  protected: 
	
      //      
   // DVector Up;
      //      
    double Fp;
    
      //      
    DVector Bp;
      //     
    double F2;
      //     
    DVector U2;
      //     
    DVector B2;
	   
    EnEstimTimeType EstimTimeMethod;
    //   ,   
public:
	int  matrixSize;
	DVector Up;
    IMeasurementPacket *MPacket;
protected:
      //   ,      
      //          
    ITraject *Trj;
      //    
    DSymmMatrix A;
      //      
    DVector B;
      //    
public:
    double F;
      //        
    void ClearWork();
      //         
    virtual bool AddMeasurement(IMeasurement &meas);
      //     
    bool ProcessPack();
      //     ,     
      //  
    bool ProcessToForward();
      //     ,     
      //  
    bool ProcessToBackward();
      //   
    void ProcessAprioriConsts();
      //     
   // double CalcCriteria();
      //   
    bool CalcIncrements();
      //  
    void PrintMeas(const MeasData &meas, FILE *f, bool active_flag);
    
    // solution of the linear system
    virtual bool LinSysSolution(const DVector &B, const CDSymmMatrix &A, DVector &X);
    //  ,    , 
    //	1 -   
    //	0 -   
    //	-1 -   
    int IsHyperbolic(DVector& U);

	double m_jacobiParam;
//	bool m_reset;
  };
///////////////////////////////////////////////////////////////////////////////
class ODCorel : public IODCorel
{
public:
  	virtual ~ODCorel() {};
      //   . :
      // u0 -      
      // trj - c  ,     
      //                  
      // pack -    
      // etype -    
      //       ,     
      //   
    void Init(const DVector &u0, ITrajectVar &trj, IMeasurementPacket &pack, TimeJD estimTime);
      //         
    virtual bool AddMeasurement(IMeasurement &meas);
      //   
    
	void SaveInActive(IMeasurement &meas, const TimeJD &tau, const DMatrix &d,
		const DVector &del, bool deleted);

	
};
#endif
