///////////////////////////////////////////////////////////////////////////////
// Библиотека алгоритмов небесной механики
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
// Реализация интегрирования системы обыкновенных уравнений
// по методике В.А.Степньянца
///////////////////////////////////////////////////////////////////////////////
#ifndef BNOTOOL_ODESTPNSOLVER_H
#define BNOTOOL_ODESTPNSOLVER_H
///////////////////////////////////////////////////////////////////////////////
#include "IODE.h"

///////////////////////////////////////////////////////////////////////////////
class ODEStpnSolver : public IODESolver
{
public:
    // Конструктор
    ODEStpnSolver();
    // Деструктор
    virtual ~ODEStpnSolver();

    // Установка системы уравнений
    virtual void SetSystem(IODESystem* odesystem);
    // Установка точности и шага
    void SetParams(const double* eps, double stepinit, double stepmin);
    // Установка начальных условий
    virtual void SetInitial(double x, const double* y);

    // Запуск процедуры решения до заданного значения аргумента
    virtual bool Run(double x);

    // Установка признака трассировки
    void SetTrace(bool f = true)
    {
        IsTrace = f;
    };

protected:
    // Допустимые ошибки при вычислении фазового вектора на одном 
    // шаге интегрирования
    double* Eps;
    // Шаг разгона
    double StepInit;
    // Минимально допустимый шаг
    double MinStep;
    // Флаг инициализации параметров
    bool IsParamsInit;
    // Рабочий буфер для хранения узлов
    double* WorkBuffer;
    // Флаг, указывающий, что разгон выполнен
    bool IsStarted;
    // Указатели на узловые точки (пр. части + аргумент)
    double* Nodes[10];
    // Признак наличия информации в кеше 4 доп. узлов
    bool HasCash;
    // Указатель на текущее состояние фазовых переменных
    double* Phase;
    // Текущий шаг
    double CurStep;
    // Признаки регулировки шага
    int SVar1, SVar2;
    // Указатели на временные вектора, используемые при вычислениях
    double *VTmp1, *VTmp2;
    // Указатель на вектор интерполяции
    double* IVect;
    // Признак трассировки
    bool IsTrace;
    // Погрешность на шаге
    double Delta;

    // Выполнение начального разгона
    bool Start(int dir);
    // Выполнение одного шага по методу Рунге-Кутта
    bool RKStep(const double* y0, double h, double* y, double* f);
    // Интерполяция конечной точки внутри вычисленных узлов
    void InterplateDestPoint(double x, int dir);
    // Выбор шага
    bool SetNewStep();
    // Сдвиг фазового вектора на шаг
    void ShiftPhase(int dir);
    // Выполнение очередного шага
    bool DoStep(int dir);
    // Изменение направления интегрирования
    bool ReverseNodes(int dir);

    // Отладка: вывод состояния в поток
    void Dump();
};

///////////////////////////////////////////////////////////////////////////////
#endif
