///////////////////////////////////////////////////////////////////////////////
// Библиотека алгоритмов небесной механики
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
#ifndef BNOTOOL_OBSERVEPOINT_H
#define BNOTOOL_OBSERVEPOINT_H
///////////////////////////////////////////////////////////////////////////////
#include "PhasePoint.h"
#include "GeoCoords.h"
#include "RAzGamCoords.h"
#include "AlphDelDCoords.h"
#include "AlphDelD6Coords.h"

///////////////////////////////////////////////////////////////////////////////
// Класс наблюдательного пункта
// Методы вычисляют параметры положения указанного объекта относительно
// пункта
///////////////////////////////////////////////////////////////////////////////
class ObservePoint
{
public:
    // Конструктор
    ObservePoint()
    {
        InitFlag = false;
    };

    // Инициализация с указанием координат в Гринвиче
    void Init(const Vect3& c)
    {
        PCoords = c;
        InitFlag = true;
    };
    // Инициализация с указанием географических координат
    void Init(const GeoCoordsPrec& g)
    {
        g.GetXYZ(PCoords);
        InitFlag = true;
    };

    // Вычисление дальности, азимута, угла места в системе NEZ
    void GetRAG(const PhasePoint3D& p, RAzGammaCoords& rag) const;
    void GetRAG(const PhasePoint3D& p, const FrameConverter6D& cnv, RAzGammaCoords& rag) const;
    void GetRAG(const PhasePoint6D& p, RAzGammaCoords& rag) const;
    void GetRAGt(const PhasePoint6D& p, const FrameConverter6D& cnv, RAzGamma6Coords& rag) const;
    void GetRAGt(const PhasePoint6D& p, RAzGamma6Coords& rag) const;
    void GetRAGtFast(const Vect6& p, const FrameConverter6D& cnv, RAzGamma6Coords& rag) const;

    void GetRAGtGCS(Vect6 gcsPos, RAzGamma6Coords& rag) const;
    // Вычисление дальности, alpha, delta в местной экв. системе (J2000) 
    void GetDAD(const PhasePoint3D& p, AlphDelDCoords& dad) const;
    void GetDAD(const PhasePoint6D& p, AlphDelDCoords& dad) const;
    // Вычисление дальности, alpha, delta и их производных 
    // в местной экв. системе (J2000) 
    void GetDADt(const Vect6& p, const FrameConverter6D& cnv, AlphDelD6Coords& dad) const;
    void GetDADt(const PhasePoint6D& p, AlphDelD6Coords& dad) const;
    void GetDADtPrtDrv(const PhasePoint6D& p, AlphDelD6Coords& dad, DMatrix& prtDrv) const;

    // Вычисление фазового угла на Солнце
    double PhaseSunAngle(const PhasePoint3D& p, int* sign) const;
    static double PhaseSunAngle(const Vect3& sunJ2000, const Vect3& siteJ2000, const Vect3& objectJ2000, int* sign);
    double GetGeoUserLon() const { return geo_userlon; }
    Vect3 GetCoords() const { return PCoords; }
protected:
    // Флаг инициализации
    bool InitFlag;
    // Координаты пункта в Гринвиче
    Vect3 PCoords;
    double geo_userlat;
    double geo_userlon;
    double geo_useralt;
};

///////////////////////////////////////////////////////////////////////////////
#endif
