///////////////////////////////////////////////////////////////////////////////
// Библиотека алгоритмов небесной механики
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
// Средства предварительного построения орбит по оптическим измерениям
// методом Лапласа
///////////////////////////////////////////////////////////////////////////////
#ifndef BNOTOOL_OPTORBIT_H
#define BNOTOOL_OPTORBIT_H

#include "SList.h"
#include "OpticMeasurement.h"

///////////////////////////////////////////////////////////////////////////////
// Вспомогательный класс, хранящий указатели на оптические измерения
///////////////////////////////////////////////////////////////////////////////
class OptList : public TSListDef<OpticMeasurement, false>
{
public:
};

///////////////////////////////////////////////////////////////////////////////
// Класс, выполняющий построение орбиты методом Лапласа
///////////////////////////////////////////////////////////////////////////////
class OptOrbit
{
public:
    // Конструктор по умолчанию
    OptOrbit();
    // Очистка класса
    void Clear();
    // Добавление измерения
    bool Add(const OpticMeasurement* m);
    // Построение орбиты
    bool Build(Vect6& res);
    TimeJD GetTime() const;

protected:
    // Список измерений
    OptList OList;
    // Построение полинома, аппроксиморуещего одну из компонент измерения
    static bool BuildPolynom(const OptList& meas, const TimeJD& t0, int k, int m, DVector& res);
    // Расчет единичного вектора и обратной матрицы Лапласа
    static bool CreateLMatrix(double a0, double a1, double a2, double d0, double d1, double d2, Vect3& e, Vect3& ed, DSqrMatrix& l1);
    // Вычисление расстояния от наблюдателя до КО и двух его производных
    // по времени
    static void DeltaKO(const DSqrMatrix& l1, double r, const Vect3& rip, const Vect3& wip, double earthMu, Vect3& ro);
    // Расчет невязки уравнения Лапласа
    static double DeltaL(double r, const DSqrMatrix& l1, const Vect3& e, const Vect3& rip, const Vect3& wip, double earthMu);
    static void e2a2d2(double a0, double a1, double d0, double d1, Vect3 e, double& a2, double& d2);
    static void GetMeasTimes(const OptList& meas, TimeJD& t0, double& tau);
    static int CalcPolyOrder(int vecsize, double interval);
    static void CalcDvsSec(double a0, double a1, double d0, double d1, Vect3 e, double r0, double mu, double& a2, double& d2);
    static bool CalcDvs(const OptList& meas, TimeJD t0, int m, double& a0, double& a1, double& a2, double& d0, double& d1, double& d2);
    static void CalcSiteCoords(const IFrame* srcfrm, TimeJD t0, Vect3 sitecoords, double earthRotationVel, Vect3& wip, Vect3& vip, Vect3& rip);
    static bool CalcVector(double a0, double a1, double a2, double d0, double d1, double d2, const Vect3& wip, const Vect3& vip, const Vect3& rip, double earthRadius, double earthMu, Vect6& res, Vect3& unitVector, double& r0);
};

///////////////////////////////////////////////////////////////////////////////
#endif
