///////////////////////////////////////////////////////////////////////////////
// Библиотека алгоритмов небесной механики
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
// Класс, моделирующий оптические измерения c учетом времени прохождения
// сигнала.
// Класс является абстрактным, поскольку в нем не реализован метод
// GetSrcFrame()
///////////////////////////////////////////////////////////////////////////////
#ifndef BNOTOOL_OPTICDELAYMESUREMENT_H
#define BNOTOOL_OPTICDELAYMESUREMENT_H
///////////////////////////////////////////////////////////////////////////////
#include "OpticMeasurement.h"

///////////////////////////////////////////////////////////////////////////////
class OpticDelayMeasurement : public OpticMeasurement {
public:
    // Конструктор по умолчанию
    OpticDelayMeasurement();
    // Установка точности опреления времени распространения сигнала
    void SetTimePrecision(double dt)
    {
        TimePrec = dt;
    };
    // Метод возвращает точность опреления времени распространения сигнала
    double GetTimePrecision() const
    {
        return TimePrec;
    };
    // Установка положения станции в Гринвиче
    virtual void SetStationCoords(const Vect3& pos)
    {
        RecPos = pos;
    };
    // Получение координат станции в Гринвиче
    virtual const Vect3& GetStationCoord() const
    {
        return RecPos;
    };

    bool PrepareDstFrame(const IFrame& fr);
    bool PrepareDstFrame_gr(const IFrame& fr);
protected:
    // Точность, с которой определяется время распространения сигнала
    double TimePrec;
    // Метод определяет время t, которое соответствует моменту излучения сигнала с объекта,
    // а также его положение на орбите в этот момент времени
    virtual bool DetectPosition(ITraject& trj, TimeJD& t, const IFrame* fr = NULL, Vect6* f = NULL) const;
    bool DetectPosition(ITraject& trj, TimeJD& t, Vect6* f = NULL) const;
    bool DetectPositionEx(ITraject& trj, TimeJD& t, Vect6& f) const;

    // Метод определяет положение приемника сигнала f в момент времени t
    // в системе отсчета, определяемой параметром fr
    virtual bool DetectRecieverPos(const TimeJD& t, const IFrame& fr, Vect3& f) const;
    // Положение станции наблюдения в Гринвиче
};

///////////////////////////////////////////////////////////////////////////////
#endif
