///////////////////////////////////////////////////////////////////////////////
// Библиотека алгоритмов небесной механики
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
// Классы фазовых точек
////////////////////////////////////////////////////////////////////////////////
#ifndef BNOTOOL_PHASEPOINT_H
#define BNOTOOL_PHASEPOINT_H
////////////////////////////////////////////////////////////////////////////////
#include "Defines.h"
#include "J2000Frame.h"

////////////////////////////////////////////////////////////////////////////////
// Фазовая точка
class IPhasePoint
{
public:
    // Конструктор
    IPhasePoint()
    {
        Frame = 0; //new J2000Frame;
    };

    IPhasePoint(int dummy) : Frame(0)
    {
    };
    // Конструктор копирования
    IPhasePoint(const IPhasePoint& src)
    {
        Frame = NULL;
        Init(src);
    };
    // Деструктор
    virtual ~IPhasePoint();

    // Время
    TimeJD T;

    // Инициализация
    void Init(const IPhasePoint& src);

    // Получение системы отсчета
    const IFrame* GetFrame() const
    {
        return Frame;
    };
    // Получение системы отсчета
    IFrame* GetFrame()
    {
        return Frame;
    };
    // Установка системы отсчета
    void SetFrame(const IFrame& f);

    IPhasePoint& operator=(const IPhasePoint& src)
    {
        Init(src);
        return *this;
    };

protected:
    IFrame* Frame;
};

////////////////////////////////////////////////////////////////////////////////
// Трехмерная фазовая точка
class PhasePoint3D : public IPhasePoint
{
public:
    // Конструктор
    PhasePoint3D() : IPhasePoint()
    {
    };
    // Конструктор копирования
    PhasePoint3D(const PhasePoint3D& src)
    {
        Frame = NULL;
        Init(src);
    };
    // Конструктор с параметрами
    PhasePoint3D(const IFrame& f, const Vect3& c, const TimeJD& t) : IPhasePoint(666)
    {
        Frame = NULL;
        Init(f, c, t);
    };

    // Инициализация
    void Init(const PhasePoint3D& src);
    // Инициализация
    void Init(const IFrame& f, const Vect3& c, const TimeJD& t);

    // Оператор присваивания
    PhasePoint3D& operator=(const PhasePoint3D& src)
    {
        Init(src);
        return *this;
    };

    // Декартовы координаты
    Vect3 Coords;

    // Конвертирование в указанную систему отсчета
    void Convert(const IFrame& f);
    // Присвоение точки с конвертированием в текущую систему отсчета
    void Assign(const PhasePoint3D& src);
};

////////////////////////////////////////////////////////////////////////////////
// Шестимерная фазовая точка
class PhasePoint6D : public IPhasePoint
{
public:
    // Конструктор

    PhasePoint6D(int dummy) : IPhasePoint(dummy)
    {
    };

    PhasePoint6D() : IPhasePoint()
    {
    };
    // Конструктор копирования
    PhasePoint6D(const PhasePoint6D& src)
    {
        Frame = NULL;
        Init(src);
    };
    // Конструктор с параметрами
    PhasePoint6D(const IFrame& f, const Vect6& cv, const TimeJD& t) : IPhasePoint(666)
    {
        Frame = NULL;
        Init(f, cv, t);
    };
    // Конструктор с параметрами
    PhasePoint6D(const IFrame& f, const Vect3& c, const Vect3& v, const TimeJD& t) : IPhasePoint(666)
    {
        Frame = NULL;
        Init(f, c, v, t);
    };

    // Инициализация
    void Init(const PhasePoint6D& src);
    // Инициализация
    void Init(const IFrame& f, const Vect6& cv, const TimeJD& t);
    // Инициализация
    void Init(const IFrame& f, const Vect3& c, const Vect3& v, const TimeJD& t);

    // Оператор присваивания
    PhasePoint6D& operator=(const PhasePoint6D& src)
    {
        Init(src);
        return *this;
    };

    // Декартовы координаты
    Vect6 CoordsVel;

    // Конвертирование в указанную систему отсчета
    void Convert(const IFrame& f);
    // Присвоение точки с конвертированием в текущую систему отсчета
    void Assign(const PhasePoint6D& src);
};

////////////////////////////////////////////////////////////////////////////////
#endif
