///////////////////////////////////////////////////////////////////////////////
// Библиотека алгоритмов небесной механики
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
// Классы, описывающие планеты и тела Солнечной системы
///////////////////////////////////////////////////////////////////////////////
#include "Planet.h"
#include "Consts.h"
///////////////////////////////////////////////////////////////////////////////
// Инициализация параметрами планеты
///////////////////////////////////////////////////////////////////////////////
void SPlanet::Init(const PlanetConsts& pc, const char* eph_name)
{
    Geometry.Re = pc.Re;
    Geometry.alpha = pc.Flat;
    omega = pc.MeanRot;
    RShad = pc.Re;
    GrModel.alpha = pc.Flat;
    GrModel.mu = pc.Mu;
    GrModel.Re = pc.Re;
    PName = pc.PlanetName;
    ID = pc.PlanetId;

    if (eph_name != NULL)
        Eph.Init(eph_name, pc.PlanetEphId);
}

///////////////////////////////////////////////////////////////////////////////
// Инициализация данных для Луны
///////////////////////////////////////////////////////////////////////////////
void MoonPlanet::Init(const char* eph_name)
{
    RShad = GrModel.Re = Geometry.Re = 1.7380;
    Geometry.alpha = GrModel.alpha = 0;
    omega = MoonMeanRot_Vallado;
    GrModel.mu = miuMoon;

    if (eph_name != NULL)
        Eph.Init(eph_name);
}

///////////////////////////////////////////////////////////////////////////////
// Инициализация данных для Солнца
///////////////////////////////////////////////////////////////////////////////
SunPlanet::SunPlanet()
{
    RShad = GrModel.Re = Geometry.Re = 696.0;
    Geometry.alpha = GrModel.alpha = 0;
    omega = 0;
    GrModel.mu = miuSun;
}

///////////////////////////////////////////////////////////////////////////////
