///////////////////////////////////////////////////////////////////////////////
// Библиотека алгоритмов небесной механики
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
// Класс координат дальность, азимут, угол места
///////////////////////////////////////////////////////////////////////////////
#include "RAzGamCoords.h"
#include "Consts.h"
///////////////////////////////////////////////////////////////////////////////
// Конструктор по умолчанию
//-----------------------------------------------------------------------------
RAzGammaCoords::RAzGammaCoords()
{
    Dist = Az = Gamma = 0.0;
}

//-----------------------------------------------------------------------------
// Установка координат по декартовым
//-----------------------------------------------------------------------------
void RAzGammaCoords::SetXYZ(const Vect3& src)
{
    Dist = src.Len();

    Az = atan2(src.z, src.x);
    Gamma = Dist != 0 ? asin(src.y / Dist) : 0;

    //  Az = atan2(src.y, src.x);
    //  Gamma = asin(src.z / Dist);
}

//-----------------------------------------------------------------------------
// Получение декартовых координат
//-----------------------------------------------------------------------------
void RAzGammaCoords::GetXYZ(Vect3& dst) const
{
    dst.y = Dist * sin(Gamma);
    dst.x = Dist * cos(Gamma) * cos(Az);
    dst.z = Dist * cos(Gamma) * sin(Az);
}

///////////////////////////////////////////////////////////////////////////////

RAzGamma6Coords::RAzGamma6Coords()
{
}

void RAzGamma6Coords::SetXYZ(const Vect6& src)
{
    Coords.SetXYZ(src);

    double x = src.x * 1000;
    double y = src.z * 1000;
    double z = src.y * 1000;
    double vx = src.vx;
    double vy = src.vz;
    double vz = src.vy;

    double x2_y2 = x * x + y * y;
    double xy = sqrt(x2_y2);
    double coord_norma_2 = x * x + y * y + z * z;
    double coord_norma_2_xy = coord_norma_2 * xy;
    double eldot = xy * vz / coord_norma_2 - (z * y / coord_norma_2_xy) * vy - (z * x / coord_norma_2_xy) * vx;
    double azdot = x * vy / x2_y2 - y * vx / x2_y2;
    double declination = Coords.Gamma;
    double dot_declination = eldot;
    double dot_ascension = azdot;
    double buf = dot_ascension * cos(declination);
    double angvel = sqrt(buf * buf + dot_declination * dot_declination);

    DElevation = eldot;
    DAzimuth = azdot;
    Velocity = angvel;
}

bool RAzGamma6Coords::GetXYZ(Vect6& dst) const
{
    return true;
}
