///////////////////////////////////////////////////////////////////////////////
// Библиотека алгоритмов небесной механики
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
// Определение класса системы отсчета NEZ
////////////////////////////////////////////////////////////////////////////////
#ifndef BNOTOOL__RNBFRAME_H
#define BNOTOOL__RNBFRAME_H
////////////////////////////////////////////////////////////////////////////////
#include "Defines.h"
#include "IFrames.h"
#include "SFramePars.h"

////////////////////////////////////////////////////////////////////////////////
class RNBFrame : public IFrame
{
public:
    // Конструктор по умолчанию
    RNBFrame()
    {
    };
    // Конструктор с указанием положения и скорости КА
    RNBFrame(const Vect3& p, const Vect3& v)
    {
        Init(p, v);
    };
    // Конструктор с указанием положения и скорости КА
    RNBFrame(const Vect6& rv)
    {
        Init(rv);
    };

    // Инициализация с указанием положения и скорости КА
    RNBFrame& Init(const Vect3& p, const Vect3& v)
    {
        Pars.Init(p, v);
        return *this;
    };
    // Инициализация с указанием положения и скорости КА
    RNBFrame& Init(const Vect6& rv)
    {
        Pars.Init(rv);
        return *this;
    };
    // Получение ссылки на параметры системы
    virtual const IFrameParameters& GetParameters() const
    {
        return Pars;
    };
    // Установка параметров системы
    virtual void SetParameters(const IFrameParameters& p);
    // Является ли система инерциальной ?
    virtual bool IsInertial() const
    {
        return false;
    };
    // Является ли система геоцентрической ?
    virtual bool IsGeocentric() const
    {
        return false;
    };

    // Получение объекта, который соответствует переходу от текущей системы отсчета 
    // к системе отсчета J2000 на момент времени t0 (для 3-мерных векторов)
    virtual void Get3DJ2000Converter(const TimeJD& t0, FrameConverter3D& result) const;
    // Получение объекта, который соответствует переходу от текущей системы отсчета 
    // к системе отсчета J2000 на момент времени t0 (для 6-мерных векторов)
    virtual void Get6DJ2000Converter(const TimeJD& t0, FrameConverter6D& result) const;

    const Vect3& GetPoint() const
    {
        return Pars.GetPoint();
    };

protected:
    // Параметры системы
    RNBFrameParameters Pars;
};

////////////////////////////////////////////////////////////////////////////////
#endif
