///////////////////////////////////////////////////////////////////////////////
// Библиотека алгоритмов небесной механики
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
// Представление продолжительности временных интервалов
///////////////////////////////////////////////////////////////////////////////
#ifndef BNOTOOL_TMCF_H
#define BNOTOOL_TMCF_H

#include <math.h>
///////////////////////////////////////////////////////////////////////////////
// Константы определяющие интервалы времени
///////////////////////////////////////////////////////////////////////////////
#define cfTmSec  1000
#define cfTmTSec 1000000
#define cfTmMin  60000
#define cfTmHour 3600000
#define cfTmDay  86400000

///////////////////////////////////////////////////////////////////////////////
// Предопределение классов временных
///////////////////////////////////////////////////////////////////////////////
/*
class TmMSec;
class TmSec;
class TmTSec;
class TmMin;
class TmHour;
class TmDay;*/
///////////////////////////////////////////////////////////////////////////////
// Шаблон для класса временных интервалов
///////////////////////////////////////////////////////////////////////////////
template <int CFM>
class TmClass
{
public:
    // Конструкторы
    explicit TmClass(double d1)
    {
        t = d1;
    };

    TmClass()
    {
        t = 0.0;
    };

    // Методы инициализации
    TmClass& Init(double d1)
    {
        t = d1;
        return *this;
    };

    TmClass& Init()
    {
        t = 0.0;
        return *this;
    };

    // Операторы приведения типа
    //    operator double() const
    //      { return (double) t; };
    double GetDouble() const
    {
        return t;
    };

    // Операторы присваивания
    TmClass& operator=(double ti)
    {
        t = ti;
        return *this;
    };

    // Унарные арифметические операторы
    TmClass& operator+=(TmClass const& ti)
    {
        t += ti.t;
        return *this;
    };

    TmClass& operator-=(TmClass const& ti)
    {
        t -= ti.t;
        return *this;
    };

    TmClass& operator*=(double mpu)
    {
        t *= mpu;
        return *this;
    };

    TmClass& operator/=(double div)
    {
        t /= div;
        return *this;
    };

    TmClass& operator-()
    {
        t = -t;
        return *this;
    };

    // Бинарные арифметические операторы
    friend TmClass operator+(TmClass const& ti1, TmClass const& ti2)
    {
        return TmClass(ti1.t - ti2.t);
    };

    friend TmClass operator-(TmClass const& ti1, TmClass const& ti2)
    {
        return TmClass(ti1.t - ti2.t);
    };

    friend TmClass operator*(TmClass const& ti1, double mpy1)
    {
        return TmClass(ti1.t * mpy1);
    };

    friend TmClass operator*(double mpy1, TmClass const& ti1)
    {
        return TmClass(ti1.t * mpy1);
    };

    friend TmClass operator/(TmClass const& ti1, double div)
    {
        return ti1.t / div;
    };

    // операторы сравнения
    friend bool operator>(TmClass const& ti1, TmClass const& ti2)
    {
        return ti1.t > ti2.t;
    };

    friend bool operator>(TmClass const& ti1, double ti2)
    {
        return ti1.t > ti2;
    };

    friend bool operator>(double ti1, TmClass const& ti2)
    {
        return ti1 > ti2.t;
    };

    friend bool operator>=(TmClass const& ti1, TmClass const& ti2)
    {
        return ti1.t >= ti2.t;
    };

    friend bool operator>=(TmClass const& ti1, double ti2)
    {
        return ti1.t >= ti2;
    };

    friend bool operator>=(double ti1, TmClass const& ti2)
    {
        return ti1 >= ti2.t;
    };

    friend bool operator<(TmClass const& ti1, TmClass const& ti2)
    {
        return ti1.t < ti2.t;
    };

    friend bool operator<(TmClass const& ti1, double ti2)
    {
        return ti1.t < ti2;
    };

    friend bool operator<(double ti1, TmClass const& ti2)
    {
        return ti1 < ti2.t;
    };

    friend bool operator<=(TmClass const& ti1, TmClass const& ti2)
    {
        return ti1.t <= ti2.t;
    };

    friend bool operator<=(TmClass const& ti1, double ti2)
    {
        return ti1.t <= ti2;
    };

    friend bool operator<=(double ti1, TmClass const& ti2)
    {
        return ti1 <= ti2.t;
    };

    // Операции приведения к другим единицам измерения
    TmClass<1> MSec() const
    {
        TmClass<1> buf(getVal());
        return buf;
    };

    TmClass<cfTmSec> Sec() const
    {
        TmClass<cfTmSec> buf(getVal() / cfTmSec);
        return buf;
    };

    TmClass<cfTmMin> Min() const
    {
        TmClass<cfTmMin> buf(getVal() / cfTmMin);
        return buf;
    };

    TmClass<cfTmHour> Hour() const
    {
        TmClass<cfTmHour> buf(getVal() / cfTmHour);
        return buf;
    };

    TmClass<cfTmTSec> TSec() const
    {
        TmClass<cfTmTSec> buf(getVal() / cfTmTSec);
        return buf;
    };

    TmClass<cfTmDay> Day() const
    {
        TmClass<cfTmDay> buf(getVal() / cfTmDay);
        return buf;
    }

protected:
    double t;

    double getVal() const
    {
        return t * CFM;
    }
};

///////////////////////////////////////////////////////////////////////////////
// Классы временных интервалов
///////////////////////////////////////////////////////////////////////////////
typedef TmClass<1> TmMSec;
typedef TmClass<cfTmSec> TmSec;
typedef TmClass<cfTmTSec> TmTSec;
typedef TmClass<cfTmMin> TmMin;
typedef TmClass<cfTmHour> TmHour;
typedef TmClass<cfTmDay> TmDay;
///////////////////////////////////////////////////////////////////////////////
#endif
