/***************************************************************************
 *   Copyright (C) 2008 by Golosova Natalia                                *
 *   nata@allerode                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef TRJNUM_H
#define TRJNUM_H
///////////////////////////////////////////////////////////////////////////////
#include "ITraject.h"
#include "DigPredict.h"

///////////////////////////////////////////////////////////////////////////////
class TrjNum : public ITraject
{
public:
    // Конструктор
    TrjNum();
    // Деструктор
    virtual ~TrjNum()
    {
    };
    // Метод устанавливает значение вектора уточняемых параметров
    virtual unsigned Size() const
    {
        return 6;
    };
    // Метод устанавливает значение вектора уточняемых параметров
    virtual bool Init(const DVector& u0);
    // Метод устанавливает время начальной точки траектории
    virtual void SetTime(const TimeJD& t)
    {
        InitTime = t;
    };
    // Метод возвращает время начальной точки траектории
    virtual const TimeJD& GetTime() const
    {
        return InitTime;
    };
    // Метод выполняет прогнозирование траектории на указанный момент времени t.
    // Параметр fr определяет систему отсчету, в которой должен быть возвращен
    // фазовый вектор f (т.е. координаты и скорости)
    virtual bool Predict(const TimeJD& t, const IFrame& fr, Vect6& f);
    virtual bool PredictMatrix(const TimeJD& t, const IFrame& fr, Vect6& f, DMatrix& m) { return 1; };
    // Метод выполняет вычисление вектора уточняемых параметров u для указанного
    // момента времени t.
    virtual bool Move(const TimeJD& t, DVector& u);
    // Получение ссылки на экземпляр прогноза (для внешних настроек)
    DigStPredict* GetPredictor()
    {
        return &Predictor;
    };
    // Установка значений коэффициентов и признаков их уточнения
    void SetKOProp(double kb, double kappa);

protected:
    // Класс численного прогноза 
    DigStPredict Predictor;
    // Время привязки начальных условий
    TimeJD InitTime;
    // Балл. коэффициент по умолчанию
    double Kb;
    // Коэффициент св. давления по умолчанию
    double Kappa;
};

///////////////////////////////////////////////////////////////////////////////
#endif
