///////////////////////////////////////////////////////////////////////////////
// Библиотека алгоритмов небесной механики
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
#include "msgp.h"
#include "msgpext/sgp4ext.h"
#include "METEFrame.h"
const char help = 'n';
#define pi 3.14159265358979323846
///////////////////////////////////////////////////////////////////////////////
// Прототипы вспомогательных функций (их реализации - файле msgpext/msputils.cpp)
///////////////////////////////////////////////////////////////////////////////
void MSGPInitl
(
    int, double, double, double, double&, int&,
    double&, double&, double&, double&, double&, double&, double&,
    double&, double&, double&, double&, double&, double&, double&
);
void MSGPDscom
(
    double, double, double, double, double, double, double,
    double&, double&, double&, double&, double&, double&, double&, double&,
    double&, double&, double&, double&, double&, double&, double&, double&,
    double&, double&, double&, double&, double&, double&, double&, double&,
    double&, double&, double&, double&, double&, double&, double&, double&,
    double&, double&, double&, double&, double&, double&, double&, double&,
    double&, double&, double&, double&, double&, double&, double&, double&,
    double&, double&, double&, double&, double&, double&, double&, double&,
    double&, double&, double&, double&, double&, double&, double&, double&,
    double&, double&, double&, double&, double&, double&, double&, double&,
    double&, double&, double&, double&, double&, double&, double&, double&,
    double&
);
void MSGPDpper
(
    double, double, double, double, double, double, double, double, double,
    double, double, double, double, double, double, double, double, double,
    double, double, double, double, double, double, double, double, double,
    double, double, double, double, double, int,
    double&, double&, double&, double&, double&
);
void MSGPDsinit
(
    double cosim, double emsq, double argpo, double s1, double s2, double s3,
    double s4, double s5, double sinim, double ss1, double ss2, double ss3,
    double ss4, double ss5, double sz1, double sz3, double sz11, double sz13,
    double sz21, double sz23, double sz31, double sz33, double t, double tc,
    double gsto, double mo, double mdot, double no, double omegao, double omegadot,
    double xpidot, double z1, double z3, double z11, double z13, double z21,
    double z23, double z31, double z33, double ecco, double eccsq,
    double& em, double& argpm, double& inclm, double& mm, double& nm, double& omegam,
    int& irez,
    double& atime, double& d2201, double& d2211, double& d3210, double& d3222,
    double& d4410, double& d4422, double& d5220, double& d5232, double& d5421,
    double& d5433, double& dedt, double& didt, double& dmdt, double& dndt,
    double& dnodt, double& domdt, double& del1, double& del2, double& del3,
    double& xfact, double& xlamo, double& xli, double& xni
);
void MSGPDspace
(
    int,
    double, double, double, double, double, double, double, double, double,
    double, double, double, double, double, double, double, double, double,
    double, double, double, double, double, double, double, double,
    double&, double&, double&, double&, double&, double&, double&, double&,
    double&, double&
);
///////////////////////////////////////////////////////////////////////////////
// Класс MSGP-прогноза
///////////////////////////////////////////////////////////////////////////////
// Инициализация прогноза
//-----------------------------------------------------------------------------
void MSGPPredict::Init(const TLESet& tle)
{
    TAssert(tle.IsInit());

    // Запоминаем переданные элементы внутри объекта

    TLE = tle;

    //  Локальные переменные

    double ao, ainv, einv, con42, cosio, sinio, cosio2, eccsq,
           omeosq, posq, rp, rteosq,
           cnodm, snodm, cosim, sinim, cosomm, sinomm, cc1sq,
           cc2, cc3, coef, coef1, cosio4, day, dndt,
           em, emsq, eeta, etasq, gam, argpm, omegam,
           inclm, mm, nm, perige, pinvsq, psisq, qzms24,
           rtemsq, s1, s2, s3, s4, s5, s6,
           s7, sfour, ss1, ss2, ss3, ss4, ss5,
           ss6, ss7, sz1, sz2, sz3, sz11, sz12,
           sz13, sz21, sz22, sz23, sz31, sz32, sz33,
           tc, temp, temp1, temp2, temp3, tsi, xpidot,
           xhdot1, z1, z2, z3, z11, z12, z13,
           z21, z22, z23, z31, z32, z33,
           qzms2t, ss, radiusearthkm, j2, j3oj2, j4, x2o3;
    double year, epoch;

    // Обнуление всех переменных в объекте Near

    Near.Isimp = 0;
    Near.Method = 0;
    Near.Aycof = 0.0;
    Near.Con41 = 0.0;
    Near.Cc1 = 0.0;
    Near.Cc4 = 0.0;
    Near.Cc5 = 0.0;
    Near.D2 = 0.0;
    Near.D3 = 0.0;
    Near.D4 = 0.0;
    Near.Delmo = 0.0;
    Near.Eta = 0.0;
    Near.Argpdot = 0.0;
    Near.Omgcof = 0.0;
    Near.Sinmao = 0.0;
    Near.T = 0.0;
    Near.T2cof = 0.0;
    Near.T3cof = 0.0;
    Near.T4cof = 0.0;
    Near.T5cof = 0.0;
    Near.X1mth2 = 0.0;
    Near.X7thm1 = 0.0;
    Near.Mdot = 0.0;
    Near.Omegadot = 0.0;
    Near.Xlcof = 0.0;
    Near.Xmcof = 0.0;
    Near.Omegacf = 0.0;

    // Обнуление всех переменных в объекте Near

    Deep.Irez = 0;
    Deep.D2201 = 0.0;
    Deep.D2211 = 0.0;
    Deep.D3210 = 0.0;
    Deep.D3222 = 0.0;
    Deep.D4410 = 0.0;
    Deep.D4422 = 0.0;
    Deep.D5220 = 0.0;
    Deep.D5232 = 0.0;
    Deep.D5421 = 0.0;
    Deep.D5433 = 0.0;
    Deep.Dedt = 0.0;
    Deep.Del1 = 0.0;
    Deep.Del2 = 0.0;
    Deep.Del3 = 0.0;
    Deep.Didt = 0.0;
    Deep.Dmdt = 0.0;
    Deep.Dnodt = 0.0;
    Deep.Domdt = 0.0;
    Deep.E3 = 0.0;
    Deep.Ee2 = 0.0;
    Deep.Peo = 0.0;
    Deep.Pgho = 0.0;
    Deep.Pho = 0.0;
    Deep.Pinco = 0.0;
    Deep.Plo = 0.0;
    Deep.Se2 = 0.0;
    Deep.Se3 = 0.0;
    Deep.Sgh2 = 0.0;
    Deep.Sgh3 = 0.0;
    Deep.Sgh4 = 0.0;
    Deep.Sh2 = 0.0;
    Deep.Sh3 = 0.0;
    Deep.Si2 = 0.0;
    Deep.Si3 = 0.0;
    Deep.Sl2 = 0.0;
    Deep.Sl3 = 0.0;
    Deep.Sl4 = 0.0;
    Deep.Gsto = 0.0;
    Deep.Xfact = 0.0;
    Deep.Xgh2 = 0.0;
    Deep.Xgh3 = 0.0;
    Deep.Xgh4 = 0.0;
    Deep.Xh2 = 0.0;
    Deep.Xh3 = 0.0;
    Deep.Xi2 = 0.0;
    Deep.Xi3 = 0.0;
    Deep.Xl2 = 0.0;
    Deep.Xl3 = 0.0;
    Deep.Xl4 = 0.0;
    Deep.Xlamo = 0.0;
    Deep.Zmol = 0.0;
    Deep.Zmos = 0.0;
    Deep.Atime = 0.0;
    Deep.Xli = 0.0;
    Deep.Xni = 0.0;

    radiusearthkm = 6378.135;
    ss = 78.0 / radiusearthkm + 1.0;
    qzms2t = pow(((120.0 - 78.0) / radiusearthkm), 4);
    j2 = 1.082616e-3;
    j3oj2 = -2.53881e-6 / j2;
    x2o3 = 2.0 / 3.0;
    j4 = -1.65597e-6;

    year = TLE.EpochYr;

    epoch = TLE.MjdSatEpoch - 33281.0;

    MSGPInitl
    (
        TLE.Satnum, TLE.Ecco, epoch, TLE.Inclo, TLE.No, Near.Method, ainv, ao,
        Near.Con41, con42, cosio, cosio2, einv, eccsq, omeosq,
        posq, rp, rteosq, sinio, Deep.Gsto
    );

    if ((omeosq >= 0.0) || (TLE.No >= 0.0)) {
        Near.Isimp = 0;
        if (rp < (220.0 / radiusearthkm + 1.0))
            Near.Isimp = 1;
        sfour = ss;
        qzms24 = qzms2t;
        perige = (rp - 1.0) * radiusearthkm;

        /* - for perigees below 156 km, s and qoms2t are altered - */
        if (perige < 156.0) {
            sfour = perige - 78.0;
            if (perige < 98.0)
                sfour = 20.0;
            qzms24 = pow(((120.0 - sfour) / radiusearthkm), 4.0);
            sfour = sfour / radiusearthkm + 1.0;
        }

        pinvsq = 1.0 / posq;

        tsi = 1.0 / (ao - sfour);
        Near.Eta = ao * TLE.Ecco * tsi;
        etasq = Near.Eta * Near.Eta;
        eeta = TLE.Ecco * Near.Eta;
        psisq = fabs(1.0 - etasq);
        coef = qzms24 * pow(tsi, 4.0);
        coef1 = coef / pow(psisq, 3.5);
        cc2 = coef1 * TLE.No * (ao * (1.0 + 1.5 * etasq + eeta *
                (4.0 + etasq)) + 0.375 * j2 * tsi / psisq * Near.Con41 *
            (8.0 + 3.0 * etasq * (8.0 + etasq)));
        Near.Cc1 = TLE.BStar * cc2;
        cc3 = 0.0;

        if (TLE.Ecco > 1.0e-4)
            cc3 = -2.0 * coef * tsi * j3oj2 * TLE.No * sinio / TLE.Ecco;
        Near.X1mth2 = 1.0 - cosio2;
        Near.Cc4 = 2.0 * TLE.No * coef1 * ao * omeosq *
        (Near.Eta * (2.0 + 0.5 * etasq) + TLE.Ecco *
            (0.5 + 2.0 * etasq) - j2 * tsi / (ao * psisq) *
            (-3.0 * Near.Con41 * (1.0 - 2.0 * eeta + etasq *
                    (1.5 - 0.5 * eeta)) + 0.75 * Near.X1mth2 *
                (2.0 * etasq - eeta * (1.0 + etasq)) * cos(2.0 * TLE.Argpo)));
        Near.Cc5 = 2.0 * coef1 * ao * omeosq * (1.0 + 2.75 *
            (etasq + eeta) + eeta * etasq);
        cosio4 = cosio2 * cosio2;
        temp1 = 1.5 * j2 * pinvsq * TLE.No;
        temp2 = 0.5 * temp1 * j2 * pinvsq;
        temp3 = -0.46875 * j4 * pinvsq * pinvsq * TLE.No;
        Near.Mdot = TLE.No + 0.5 * temp1 * rteosq * Near.Con41 + 0.0625 *
            temp2 * rteosq * (13.0 - 78.0 * cosio2 + 137.0 * cosio4);
        Near.Argpdot = -0.5 * temp1 * con42 + 0.0625 * temp2 *
            (7.0 - 114.0 * cosio2 + 395.0 * cosio4) +
            temp3 * (3.0 - 36.0 * cosio2 + 49.0 * cosio4);
        xhdot1 = -temp1 * cosio;
        Near.Omegadot = xhdot1 + (0.5 * temp2 * (4.0 - 19.0 * cosio2) +
            2.0 * temp3 * (3.0 - 7.0 * cosio2)) * cosio;
        xpidot = Near.Argpdot + Near.Omegadot;
        Near.Omgcof = TLE.BStar * cc3 * cos(TLE.Argpo);
        Near.Xmcof = 0.0;
        if (TLE.Ecco > 1.0e-4)
            Near.Xmcof = -x2o3 * coef * TLE.BStar / eeta;
        Near.Omegacf = 3.5 * omeosq * xhdot1 * Near.Cc1;
        Near.T2cof = 1.5 * Near.Cc1;
        Near.Xlcof = -0.25 * j3oj2 * sinio *
            (3.0 + 5.0 * cosio) / (1.0 + cosio);
        Near.Aycof = -0.5 * j3oj2 * sinio;
        Near.Delmo = pow((1.0 + Near.Eta * cos(TLE.Mo)), 3);
        Near.Sinmao = sin(TLE.Mo);
        Near.X7thm1 = 7.0 * cosio2 - 1.0;

        InitFlag = false;

        /* --------------- deep space initialization ------------- */

        if ((2 * pi / TLE.No) >= 225.0) {
            Near.Method = 2;
            Near.Isimp = 1;
            tc = 0.0;
            inclm = TLE.Inclo;

            MSGPDscom(
                epoch, TLE.Ecco, TLE.Argpo, tc, TLE.Inclo, TLE.Omegao, TLE.No,
                snodm, cnodm, sinim, cosim, sinomm, cosomm,
                day, Deep.E3, Deep.Ee2, em, emsq, gam,
                Deep.Peo, Deep.Pgho, Deep.Pho, Deep.Pinco,
                Deep.Plo, rtemsq, Deep.Se2, Deep.Se3,
                Deep.Sgh2, Deep.Sgh3, Deep.Sgh4,
                Deep.Sh2, Deep.Sh3, Deep.Si2, Deep.Si3,
                Deep.Sl2, Deep.Sl3, Deep.Sl4, s1, s2, s3, s4, s5,
                s6, s7, ss1, ss2, ss3, ss4, ss5, ss6, ss7, sz1, sz2, sz3,
                sz11, sz12, sz13, sz21, sz22, sz23, sz31, sz32, sz33,
                Deep.Xgh2, Deep.Xgh3, Deep.Xgh4, Deep.Xh2,
                Deep.Xh3, Deep.Xi2, Deep.Xi3, Deep.Xl2,
                Deep.Xl3, Deep.Xl4, nm, z1, z2, z3, z11,
                z12, z13, z21, z22, z23, z31, z32, z33,
                Deep.Zmol, Deep.Zmos);
            MSGPDpper(
                Deep.E3, Deep.Ee2, Deep.Peo, Deep.Pgho,
                Deep.Pho, Deep.Pinco, Deep.Plo, Deep.Se2,
                Deep.Se3, Deep.Sgh2, Deep.Sgh3, Deep.Sgh4,
                Deep.Sh2, Deep.Sh3, Deep.Si2, Deep.Si3,
                Deep.Sl2, Deep.Sl3, Deep.Sl4, Near.T,
                Deep.Xgh2, Deep.Xgh3, Deep.Xgh4, Deep.Xh2,
                Deep.Xh3, Deep.Xi2, Deep.Xi3, Deep.Xl2,
                Deep.Xl3, Deep.Xl4, Deep.Zmol, Deep.Zmos, 1,
                TLE.Ecco, TLE.Inclo, TLE.Omegao, TLE.Argpo, TLE.Mo);

            //                if (ecco < 0.0 ) or ( ecco > 1.0) then
            //                    writeln( 'error!!!! in sgp4init, ecco:=0 ' );

            argpm = 0.0;
            omegam = 0.0;
            mm = 0.0;

            MSGPDsinit(
                cosim, emsq, TLE.Argpo, s1, s2, s3, s4, s5, sinim, ss1, ss2, ss3, ss4, // 13
                ss5, sz1, sz3, sz11, sz13, sz21, sz23, sz31, sz33, Near.T, tc, // 11 24
                Deep.Gsto, TLE.Mo, Near.Mdot, TLE.No, TLE.Omegao, Near.Omegadot, //  6 30
                xpidot, z1, z3, z11, z13, z21, z23, z31, z33, //  9 39
                TLE.Ecco, eccsq, em, argpm, inclm, mm, nm, omegam, //  8 47
                Deep.Irez, Deep.Atime, //  2 49
                Deep.D2201, Deep.D2211, Deep.D3210, Deep.D3222, //  4 53
                Deep.D4410, Deep.D4422, Deep.D5220, Deep.D5232, //  4 57
                Deep.D5421, Deep.D5433, Deep.Dedt, Deep.Didt, //  4 61
                Deep.Dmdt, dndt, Deep.Dnodt, Deep.Domdt, //  4 65
                Deep.Del1, Deep.Del2, Deep.Del3, Deep.Xfact, //  4 69
                Deep.Xlamo, Deep.Xli, Deep.Xni); //  3 72
        }

        /* ----------- set variables if not deep space ----------- */
        if (Near.Isimp != 1) {
            cc1sq = Near.Cc1 * Near.Cc1;
            Near.D2 = 4.0 * ao * tsi * cc1sq;
            temp = Near.D2 * tsi * Near.Cc1 / 3.0;
            Near.D3 = (17.0 * ao + sfour) * temp;
            Near.D4 = 0.5 * temp * ao * tsi * (221.0 * ao + 31.0 * sfour) * Near.Cc1;
            Near.T3cof = Near.D2 + 2.0 * cc1sq;
            Near.T4cof = 0.25 * (3.0 * Near.D3 + Near.Cc1 * (12.0 * Near.D2 + 10.0 * cc1sq));
            Near.T5cof = 0.2 * (3.0 * Near.D4 +
                12.0 * Near.Cc1 * Near.D3 +
                6.0 * Near.D2 * Near.D2 +
                15.0 * cc1sq * (2.0 * Near.D2 + cc1sq));
        }
    }
    if (help == 'y')
        //    if (dbgfile != NULL)
    {
        printf("%7s\n",
               " ------------------after sgp4init :-------------");
        printf("    inputs  : \n");
        printf(
            "%7s%15ld%7s%15.0f%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f\n",
            "satn", TLE.Satnum, "yr", year, "bstar", TLE.BStar, "ecco", TLE.Ecco,
            "epoch", epoch, "argpo", TLE.Argpo);
        printf("%7s%15.9f%7s%15.9f\n", "inclo", TLE.Inclo, "mo", TLE.Mo);
        printf(" in and out variables \n");
        printf("%7s%15.9f\n", "no", TLE.No);
        printf("    outputs  :\n");
        printf("%7s%15d%7s%15d%7s%15d%7s%15.9f\n",
               "init", InitFlag, "isimp", Near.Isimp,
               "method", Near.Method, "aycof", Near.Aycof);
        printf("%7s%15.9f%7s%15.9f%7s%15.9f\n",
               "con41", Near.Con41, "cc1", Near.Cc1, "cc4", Near.Cc4);
        printf("%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f\n",
               "cc5", Near.Cc5, "d2", Near.D2, "d3", Near.D3,
               "d4", Near.D4, "delmo", Near.Delmo);
        printf("%7s%15.9f%7s%15.9f%7s%15.9f\n",
               "eta", Near.Eta, "argpdot", Near.Argpdot,
               "omgcof", Near.Omgcof);
        printf("%7s%15.9f%7s%15.9f%7s%15.9f\n",
               "sinmao", Near.Sinmao, "t2cof", Near.T2cof,
               "t3cof", Near.T3cof);
        printf(
            "%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f\n",
            "t4cof", Near.T4cof, "t5cof", Near.T5cof,
            "gsto", Deep.Gsto, "x1mth2", Near.X1mth2,
            "x7thm1", Near.X7thm1, "xlcof", Near.Xlcof);
        printf("%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f\n",
               "xmcof", Near.Xmcof, "mdot", Near.Mdot,
               "omegacf", Near.Omegacf, "omegadt", Near.Omegadot);
        printf("   in and outputs from deep space satellites :\n");
        printf("%7s%15.9f%7s%15.9f\n",
               "t", Near.T, "omegao", TLE.Omegao);
        printf("%7s%15d%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f\n",
               "irez", Deep.Irez, "atime", Deep.Atime,
               "d2201", Deep.D2201, "d2211", Deep.D2211,
               "d3210", Deep.D3210);
        printf(
            "%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f\n",
            "d3222", Deep.D3222, "d4410", Deep.D4410,
            "d4422", Deep.D4422, "d5220", Deep.D5220,
            "d5232", Deep.D5232, "d5421", Deep.D5421);
        printf("%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f\n",
               "d5433", Deep.D5433, "dedt", Deep.Dedt,
               "del1", Deep.Del1, "del2", Deep.Del2,
               "del3", Deep.Del3);
        printf("%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f\n",
               "didt", Deep.Didt, "dmdt", Deep.Dmdt,
               "dnodt", Deep.Dnodt, "domdt", Deep.Domdt,
               "e3", Deep.E3);
        printf(
            "%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f\n",
            "ee2", Deep.Ee2, "peo", Deep.Peo, "pgho", Deep.Pgho,
            "pho", Deep.Pho, "pinco", Deep.Pinco, "plo", Deep.Plo);
        printf(
            "%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f\n",
            "se2", Deep.Se2, "se3", Deep.Se3, "sgh2", Deep.Sgh2,
            "sgh3", Deep.Sgh3, "sgh4", Deep.Sgh4, "sh2", Deep.Sh2);
        printf(
            "%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f\n",
            "sh3", Deep.Sh3, "si2", Deep.Si2, "si3", Deep.Si3,
            "sl2", Deep.Sl2, "sl3", Deep.Sl3, "sl4", Deep.Sl4);
        printf("%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f\n",
               "xfact", Deep.Xfact, "xgh2", Deep.Xgh2, "xgh3", Deep.Xgh3,
               "xgh4", Deep.Xgh4, "xh2", Deep.Xh2);
        printf("%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f\n",
               "xh3", Deep.Xh3, "xi2", Deep.Xi2, "xi3", Deep.Xi3,
               "xl2", Deep.Xl2, "xl3", Deep.Xl3);
        printf(
            "%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f\n",
            "xl4", Deep.Xl4, "xli", Deep.Xli, "xlamo", Deep.Xlamo,
            "xni", Deep.Xni, "zmol", Deep.Zmol, "zmos", Deep.Zmos);
    }

    InitFlag = true;
}

//-----------------------------------------------------------------------------
// Собственно прогнозирование
//-----------------------------------------------------------------------------
bool MSGPPredict::Run(const TimeJD& t)
{
    double am, axnl, aynl, betal, cosim, cnod,
           cos2u, coseo1, cosi, cosip, cosisq, cossu, cosu,
           delm, delomg, em, emsq, ecose, el2, eo1,
           ep, esine, argpm, argpp, argpdf, pl, mrt,
           mvt, rdotl, rl, rvdot, rvdotl, sinim,
           sin2u, sineo1, sini, sinip, sinsu, sinu,
           snod, su, t2, t3, t4, tem5, temp,
           temp1, temp2, tempa, tempe, templ, u, ux,
           uy, uz, vx, vy, vz, inclm, mm,
           nm, omegam, xinc, xincp, xl, xlm, mp,
           xmdf, xmx, xmy, omegadf, xnode, omegap, tc, dndt,
           twopi, x2o3, j2, j3, xke, j3oj2;
    int ktr;

    TAssert(InitFlag);

    // Вычисляем длительность прогноза в минутах
    TimeTLE ttle(TLE.EpochYr, TLE.EpochDays);
    Near.T = (t - TimeJD(ttle)).GetDouble() * 1440.0;

    //double dt = floor(fabs(Near.T + 0.5));
    //Near.T = Near.T < 0 ? -dt : dt;
    /////////////////////////////////////////////////////

    /* -------------------- wgs-72 earth constants ----------------- */
    /* ------------------ set mathematical constants --------------- */
    twopi = 2.0 * pi;
    x2o3 = 2.0 / 3.0;
    xke = 7.43669161331734132e-2;
    j2 = 1.082616e-3;
    j3 = -2.53881e-6;
    j3oj2 = j3 / j2;

    /* --------------------- clear sgp4 error flag ----------------- */
    Error = 0;

    /* ------- update for secular gravity and atmospheric drag ----- */
    xmdf = TLE.Mo + Near.Mdot * Near.T;
    argpdf = TLE.Argpo + Near.Argpdot * Near.T;
    omegadf = TLE.Omegao + Near.Omegadot * Near.T;
    argpm = argpdf;
    mm = xmdf;
    t2 = Near.T * Near.T;
    omegam = omegadf + Near.Omegacf * t2;
    tempa = 1.0 - Near.Cc1 * Near.T;
    tempe = TLE.BStar * Near.Cc4 * Near.T;
    templ = Near.T2cof * t2;

    if (Near.Isimp != 1) {
        delomg = Near.Omgcof * Near.T;
        delm = Near.Xmcof *
            (pow((1.0 + Near.Eta * cos(xmdf)), 3) - Near.Delmo);
        temp = delomg + delm;
        mm = xmdf + temp;
        argpm = argpdf - temp;
        t3 = t2 * Near.T;
        t4 = t3 * Near.T;
        tempa = tempa - Near.D2 * t2 - Near.D3 * t3 - Near.D4 * t4;
        tempe = tempe + TLE.BStar * Near.Cc5 * (sin(mm) - Near.Sinmao);
        templ = templ + Near.T3cof * t3 + t4 * (Near.T4cof + Near.T * Near.T5cof);
    }

    nm = TLE.No;
    em = TLE.Ecco;
    inclm = TLE.Inclo;
    if (Near.Method == 2) {
        tc = Near.T;
        MSGPDspace
        (
            Deep.Irez,
            Deep.D2201, Deep.D2211, Deep.D3210,
            Deep.D3222, Deep.D4410, Deep.D4422,
            Deep.D5220, Deep.D5232, Deep.D5421,
            Deep.D5433, Deep.Dedt, Deep.Del1,
            Deep.Del2, Deep.Del3, Deep.Didt,
            Deep.Dmdt, Deep.Dnodt, Deep.Domdt,
            TLE.Argpo, Near.Argpdot, Near.T, tc,
            Deep.Gsto, Deep.Xfact, Deep.Xlamo,
            TLE.No, Deep.Atime,
            em, argpm, inclm, Deep.Xli, mm, Deep.Xni,
            omegam, dndt, nm);
        // add check here for incl too  - no.
    }

    if (nm <= 0.0) {
        Error = 2;
    }
    am = pow((xke / nm), x2o3) * tempa * tempa;
    nm = xke / pow(am, 1.5);
    em = em - tempe;

    if ((em >= 1.0) || (em < -0.001)) // || (am < 0.95))
        //psc correction  if ((em >= 1.0) || (em < -0.001))
    {
        Error = 1;
    }

    if (em < 0.0)
        em = 1.0e-6;
    mm = mm + TLE.No * templ;
    xlm = mm + argpm + omegam;
    emsq = em * em;
    temp = 1.0 - emsq;
    omegam = modfunc(omegam, twopi);
    argpm = modfunc(argpm, twopi);
    xlm = modfunc(xlm, twopi);
    mm = modfunc(xlm - argpm - omegam, twopi);

    /* ----------------- compute extra mean quantities ------------- */
    sinim = sin(inclm);
    cosim = cos(inclm);

    /* -------------------- add lunar-solar periodics -------------- */
    ep = em;
    xincp = inclm;
    argpp = argpm;
    omegap = omegam;
    mp = mm;
    sinip = sinim;
    cosip = cosim;

    if (Near.Method == 2) {
        MSGPDpper
        (
            Deep.E3, Deep.Ee2, Deep.Peo,
            Deep.Pgho, Deep.Pho, Deep.Pinco,
            Deep.Plo, Deep.Se2, Deep.Se3,
            Deep.Sgh2, Deep.Sgh3, Deep.Sgh4,
            Deep.Sh2, Deep.Sh3, Deep.Si2,
            Deep.Si3, Deep.Sl2, Deep.Sl3,
            Deep.Sl4, Near.T, Deep.Xgh2,
            Deep.Xgh3, Deep.Xgh4, Deep.Xh2,
            Deep.Xh3, Deep.Xi2, Deep.Xi3,
            Deep.Xl2, Deep.Xl3, Deep.Xl4,
            Deep.Zmol, Deep.Zmos,
            0, ep, xincp, omegap, argpp, mp
        );
        if (xincp < 0.0) {
            xincp = -xincp;
            omegap = omegap + pi;
            argpp = argpp - pi;
        }
        if ((ep < 0.0) || (ep > 1.0)) {
            Error = 3;
        }
    }
    /* -------------------- long period periodics ------------------ */
    if (Near.Method == 2) {
        sinip = sin(xincp);
        cosip = cos(xincp);
        Near.Aycof = -0.5 * j3oj2 * sinip;
        Near.Xlcof = -0.25 * j3oj2 * sinip * (3.0 + 5.0 * cosip) /
            (1.0 + cosip);
    }
    axnl = ep * cos(argpp);
    temp = 1.0 / (am * (1.0 - ep * ep));
    aynl = ep * sin(argpp) + temp * Near.Aycof;
    xl = mp + argpp + omegap + temp * Near.Xlcof * axnl;

    /* --------------------- solve kepler's equation --------------- */
    u = modfunc(xl - omegap, twopi);
    eo1 = u;
    tem5 = 9999.9;
    ktr = 1;
    while ((fabs(tem5) >= 1.0e-12) && (ktr <= 10)) {
        sineo1 = sin(eo1);
        coseo1 = cos(eo1);
        tem5 = 1.0 - coseo1 * axnl - sineo1 * aynl;
        tem5 = (u - aynl * coseo1 + axnl * sineo1 - eo1) / tem5;
        if (fabs(tem5) >= 0.95)
            tem5 = tem5 > 0.0 ? 0.95 : -0.95; //PSC - crude fix for convergance.
        eo1 = eo1 + tem5;
        ktr = ktr + 1;
    }

    /* ------------- short period preliminary quantities ----------- */
    ecose = axnl * coseo1 + aynl * sineo1;
    esine = axnl * sineo1 - aynl * coseo1;
    el2 = axnl * axnl + aynl * aynl;
    pl = am * (1.0 - el2);
    if (pl < 0.0) {
        Error = 4;
    }
    else {
        rl = am * (1.0 - ecose);
        rdotl = sqrt(am) * esine / rl;
        rvdotl = sqrt(pl) / rl;
        betal = sqrt(1.0 - el2);
        temp = esine / (1.0 + betal);
        sinu = am / rl * (sineo1 - aynl - axnl * temp);
        cosu = am / rl * (coseo1 - axnl + aynl * temp);
        su = atan2(sinu, cosu);
        sin2u = (cosu + cosu) * sinu;
        cos2u = 1.0 - 2.0 * sinu * sinu;
        temp = 1.0 / pl;
        temp1 = 0.5 * j2 * temp;
        temp2 = temp1 * temp;

        /* -------------- update for short period periodics ------------ */
        if (Near.Method == 2) {
            cosisq = cosip * cosip;
            Near.Con41 = 3.0 * cosisq - 1.0;
            Near.X1mth2 = 1.0 - cosisq;
            Near.X7thm1 = 7.0 * cosisq - 1.0;
        }
        mrt = rl * (1.0 - 1.5 * temp2 * betal * Near.Con41) +
            0.5 * temp1 * Near.X1mth2 * cos2u;
        su = su - 0.25 * temp2 * Near.X7thm1 * sin2u;
        xnode = omegap + 1.5 * temp2 * cosip * sin2u;
        xinc = xincp + 1.5 * temp2 * cosip * sinip * cos2u;
        mvt = rdotl - nm * temp1 * Near.X1mth2 * sin2u / xke;
        rvdot = rvdotl + nm * temp1 * (Near.X1mth2 * cos2u +
            1.5 * Near.Con41) / xke;

        /* --------------------- orientation vectors ------------------- */
        sinsu = sin(su);
        cossu = cos(su);
        snod = sin(xnode);
        cnod = cos(xnode);
        sini = sin(xinc);
        cosi = cos(xinc);
        xmx = -snod * cosi;
        xmy = cnod * cosi;
        ux = xmx * sinsu + cnod * cossu;
        uy = xmy * sinsu + snod * cossu;
        uz = sini * sinsu;
        vx = xmx * cossu - cnod * sinsu;
        vy = xmy * cossu - snod * sinsu;
        vz = sini * cossu;

        /* ------------------- position and velocity ------------------- */
        RV[0] = mrt * ux;
        RV[1] = mrt * uy;
        RV[2] = mrt * uz;
        RV[3] = mvt * ux + rvdot * vx;
        RV[4] = mvt * uy + rvdot * vy;
        RV[5] = mvt * uz + rvdot * vz;
        Tk = t;
    }

    if (help == 'y') {
        printf("%84s\n",
               "------------------after sgp4   :---------------");
        printf("    inputs : \n");
        printf("%7s%15d%7s%15d%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f\n",
               "isimp", Near.Isimp, "method", Near.Method,
               "aycof", Near.Aycof, "bstar", TLE.BStar,
               "con41", Near.Con41, "cc1", Near.Cc1);
        printf(
            "%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f\n",
            "cc4", Near.Cc4, "cc5", Near.Cc5,
            "d2", Near.D2, "d3", Near.D3,
            "d4", Near.D4, "delmo", Near.Delmo);
        printf(
            "%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f\n",
            "ecco", TLE.Ecco, "eta", Near.Eta,
            "argpo", TLE.Argpo, "argpdot", Near.Argpdot,
            "omgcof", Near.Omgcof, "sinmao", Near.Sinmao);
        printf(
            "%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f\n",
            "t", Near.T, "t2cof", Near.T2cof,
            "t3cof", Near.T3cof, "t4cof", Near.T4cof,
            "t5cof", Near.T5cof, "x1mth2", Near.X1mth2);
        printf(
            "%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f\n",
            "x7thm1", Near.X7thm1, "inclo", TLE.Inclo,
            "mo", TLE.Mo, "mdot", Near.Mdot,
            "xno", TLE.No, "omegao", TLE.Omegao);
        printf("%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f\n",
               "omegadt", Near.Omegadot, "xlcof", Near.Xlcof,
               "xmcof", Near.Xmcof,
               "omegacf", Near.Omegacf);
        printf("    outputs : \n");
        printf(
            "%7s%15d%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f\n",
            "error", Error, "x", RV[0], "y", RV[1], "z", RV[2],
            "xdot", RV[3], "ydot", RV[1], "zdot", RV[2]);
        printf("    extra inputs for ds : \n");
        printf("%7s%15d%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f\n",
               "irez", Deep.Irez, "d2201", Deep.D2201,
               "d2211", Deep.D2211, "d3210", Deep.D3210,
               "d3222", Deep.D3222);
        printf(
            "%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f\n",
            "d4410", Deep.D4410, "d4422", Deep.D4422,
            "d5220", Deep.D5220, "d5232", Deep.D5232,
            "d5421", Deep.D5421, "d5433", Deep.D5433);
        printf(
            "%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f\n",
            "dedt", Deep.Dedt, "del1", Deep.Del1,
            "del2", Deep.Del2, "del3", Deep.Del3,
            "didt", Deep.Didt, "dmdt", Deep.Dmdt);
        printf(
            "%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f\n",
            "dnodt", Deep.Dnodt, "domdt", Deep.Domdt,
            "e3", Deep.E3, "ee2", Deep.Ee2,
            "peo", Deep.Peo, "pgho", Deep.Pgho);
        printf(
            "%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f\n",
            "pho", Deep.Pho, "pinco", Deep.Pinco,
            "plo", Deep.Plo, "se2", Deep.Se2,
            "se3", Deep.Se3, "sgh2", Deep.Sgh2);
        printf(
            "%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f\n",
            "sgh3", Deep.Sgh3, "sgh4", Deep.Sgh4,
            "sh2", Deep.Sh2, "sh3", Deep.Sh3,
            "si2", Deep.Si2, "si3", Deep.Si3);
        printf(
            "%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f\n",
            "sl2", Deep.Sl2, "sl3", Deep.Sl3,
            "sl4", Deep.Sl4, "gsto", Deep.Gsto,
            "xfact", Deep.Xfact, "xgh2", Deep.Xgh2);
        printf(
            "%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f\n",
            "xgh3", Deep.Xgh3, "xgh4", Deep.Xgh4,
            "xh2", Deep.Xh2, "xh3", Deep.Xh3,
            "xi2", Deep.Xi2, "xi3", Deep.Xi3);
        printf(
            "%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f%7s%15.9f\n",
            "xl2", Deep.Xl2, "xl3", Deep.Xl3,
            "xl4", Deep.Xl4, "xlamo", Deep.Xlamo,
            "zmol", Deep.Zmol, "zmos", Deep.Zmos);
        printf("%7s%15.9f%7s%15.9f%7s%15.9f\n",
               "atime", Deep.Atime, "xli", Deep.Xli,
               "xni", Deep.Xni);
    }

    return Error == 0;
}

//-----------------------------------------------------------------------------
// Получение конечной точки
//-----------------------------------------------------------------------------
void MSGPPredict::GetDestPoint(PhasePoint6D& dest)
{
    // Результат прогнозирования хранится в RV в размерности SGP4

    dest.T = Tk;
    TimeTLE ttle(TLE.EpochYr, TLE.EpochDays);
    TimeJD t0(ttle);
    dest.SetFrame(METEFrame(t0));

    double radiusearthtkm = 6.378135; // !63 ! this does make a diff
    double vkmpersec = radiusearthtkm * 0.0743669161331734132 / 60.0e-3;

    dest.CoordsVel = RV;

    int i;
    for (i = 0; i < 3; i++) {
        dest.CoordsVel[i] *= radiusearthtkm;
        dest.CoordsVel[i + 3] *= vkmpersec;
    }
}

///////////////////////////////////////////////////////////////////////////////
