///////////////////////////////////////////////////////////////////////////////
// Библиотека алгоритмов небесной механики
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
// Реализация прогноза MSGP
///////////////////////////////////////////////////////////////////////////////
#ifndef BNOTOOL_MSGP_H
#define BNOTOOL_MSGP_H
///////////////////////////////////////////////////////////////////////////////
#include "IPredict.h"

///////////////////////////////////////////////////////////////////////////////
// Класс описывающий структуру TLE-элементов
///////////////////////////////////////////////////////////////////////////////
class TLESet
{
public:
    // Конструктор по умолчанию 
    TLESet()
    {
        SFlag = false;
    };

    // Заполнение элементов из двух текстовых строк
    // Никаких проверок на правильность формата строк не выполняется!!! 
    void SetFromStr(char longstr1[130], char longstr2[130]);
    // Заполнение вычислемых полей
    void Set();
    // Возврат флага инициализации вычисляемых параметров
    bool IsInit() const
    {
        return SFlag;
    };

    const TimeJD GetTime() const
    {
        return TimeJD(TimeTLE(EpochYr, EpochDays));
    };

    // Независимые параметры

    // Номер спутника
    long int Satnum;
    // Год эпохи
    int EpochYr;
    // День эпохи (в сутках)
    double EpochDays;
    // "N с точкой"
    double NDot;
    // "N с двумя точками"
    double NDDot;
    // "B со звездой"
    double BStar;
    // Номер элементов
    int ElementNum;
    // Наклонение
    double Inclo;
    // ДВУ
    double Omegao;
    // Эксцентриситет
    double Ecco;
    // Аргумент перицентра
    double Argpo;
    // Средняя аномалия
    double Mo;
    // Среднее движение
    double No;

    // Вычисляемые параметры

    // Большая полуось
    double A;
    // Высота апоцентра
    double Alta;
    // Высота перицентра
    double Altp;
    // Времена
    double EpTime;
    double MjdSatEpoch;

protected:
    // Флаг заполнения вычисляемых элементов
    bool SFlag;
};

///////////////////////////////////////////////////////////////////////////////
// Класс MSGP-прогноза
///////////////////////////////////////////////////////////////////////////////
class MSGPPredict : public IPredict
{
public:
    // Конструктор по умолчанию
    MSGPPredict() : IPredict()
    {
        InitFlag = false;
        Error = 0;
    };
    // Инициализация прогноза
    void Init(const TLESet& tle);

    // Вспомогательный класс для описания параметров в ближем космосе
    class NearEarthRecord
    {
    public:
        int Isimp, Method;
        double Aycof, Con41, Cc1, Cc4, Cc5, D2, D3, D4,
               Delmo, Eta, Argpdot, Omgcof, Sinmao, T, T2cof, T3cof,
               T4cof, T5cof, X1mth2, X7thm1, Mdot, Omegadot, Xlcof, Xmcof,
               Omegacf;
    };

    // Вспомогательный класс для описания параметров в дальнем космосе
    class DeepSpaceRecord
    {
    public:
        int Irez;
        double D2201, D2211, D3210, D3222, D4410, D4422, D5220, D5232, D5421,
               D5433, Dedt, Del1, Del2, Del3, Didt, Dmdt, Dnodt, Domdt,
               E3, Ee2, Peo, Pgho, Pho, Pinco, Plo, Se2, Se3,
               Sgh2, Sgh3, Sgh4, Sh2, Sh3, Si2, Si3, Sl2, Sl3,
               Sl4, Gsto, Xfact, Xgh2, Xgh3, Xgh4, Xh2, Xh3, Xi2,
               Xi3, Xl2, Xl3, Xl4, Xlamo, Zmol, Zmos, Atime, Xli,
               Xni;
    };

    // Собственно прогнозирование
    virtual bool Run(const TimeJD& t);
    // Получение конечной точки
    virtual void GetDestPoint(PhasePoint6D& dest);

    // Получение кода ошибки
    int GetError() const
    {
        return Error;
    };

protected:
    // Начальный набор TLE-элементов
    TLESet TLE;
    // Флаг инициализации
    bool InitFlag;
    // Объект для описания параметров в ближем космосе
    NearEarthRecord Near;
    // Объект для описания параметров в дальнем космосе
    DeepSpaceRecord Deep;
    // Код ошибки
    int Error;
    // Выходной вектор в METE
    Vect6 RV;
    // Время прогноза
    TimeJD Tk;
};

///////////////////////////////////////////////////////////////////////////////
#endif
