///////////////////////////////////////////////////////////////////////////////
// Библиотека программных инструментов
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
// Средства разбора командной строки
///////////////////////////////////////////////////////////////////////////////
#include "parsecmdline.h"
#include <string.h>
#include <stdio.h>
///////////////////////////////////////////////////////////////////////////////
// Статические переменные
///////////////////////////////////////////////////////////////////////////////
TArgsOptions noArgsOptions[] =
{
    {NULL, false, NULL}
};
int TArgs::UnResolved_Len = -1;
const char** TArgs::UnResolved = NULL;
TArgsOptions* TArgs::ArgsOptions = noArgsOptions;
FILE* TArgs::StreamOut = NULL;
///////////////////////////////////////////////////////////////////////////////
// Класс разбора параметров командной строки
///////////////////////////////////////////////////////////////////////////////
// Деструктор
//-----------------------------------------------------------------------------
TArgs::~TArgs()
{
    if (UnResolved_Len >= 0)
        delete[] UnResolved;
};
//-----------------------------------------------------------------------------
// Установка массива описания опций 
//-----------------------------------------------------------------------------
void TArgs::SetOptions(TArgsOptions opts[])
{
    ArgsOptions = opts;
};
//-----------------------------------------------------------------------------
// инициализация параметров из командной строки
//-----------------------------------------------------------------------------
bool TArgs::ParseCmdLine(int argc, const char* argv[], bool onlyOptions)
{
    bool wasError;
    int i, i1, iun;
    const char* s;
    const char* s1;

    // заказываем массив под неопознанные параметры
    if (UnResolved_Len <= argc) {
        if (UnResolved_Len >= 0)
            delete[] UnResolved;
        UnResolved_Len = argc + 1;
        UnResolved = new const char*[UnResolved_Len];
        UnResolved[0] = NULL;
    };

    wasError = false;
    for (i = 1, iun = 0; i < argc; i++) {
        s = argv[i];

        if (s[0] != '-') {
            // это не опция и не параметр
            UnResolved[iun++] = s;
            UnResolved[iun] = NULL;
            if (onlyOptions)
                wasError = true;
            continue;
        };

        // Проверка на выдачу Help'а
        if ((strncmp(s + 1, "?", 1) == 0) || (strncmp(s + 1, "-help", 5) == 0)) {
            Help(NULL);
            exit(0);
        };


        // Цикл поиска найденной опции в известных 
        for (i1 = 0; ; i1++) {
            s1 = ArgsOptions[i1].OptName;
            if (!s1)
                break;
            if (!*s1)
                continue;
            if (strncmp(s + 1, s1, strlen(s1)) == 0) {
                s = s + strlen(s1) + 1;
                break;
            };
        };

        // Встретили неизвестную опцию
        if (!s1) {
            if (onlyOptions) {
                UnResolved[iun++] = s;
                UnResolved[iun] = NULL;
                wasError = true;
            }
            continue;
        };

        // Опция требует указания параметра
        if (ArgsOptions[i1].NeedParam) {
            // Получение значения параметра
            s = s[0] == '\0' ? (i + 1 == argc ? NULL : argv[i + 1][0] == '-' ? NULL : argv[++i]) : s[0] == '=' ? s + 1 : s;
            // Не удалось получить параметр
            if (!s || (s[0] == '\0')) {
                fprintf(StreamOut ? StreamOut : stdout,
                        "Ошибка: после опции '%s' должен следовать параметр: %s. Опцию игнорируем",
                        s1, ArgsOptions[i1].Descr);
                wasError = true;
                continue;
            };
            // Установка параметра
            if (!SetParam(i1, s1, s))
                wasError = true;
        }
        else {
            if (!SetParam(i1, s1))
                wasError = true;
        };
    };

    if (onlyOptions)
        PrintUnresolved();
    if (StreamOut)
        fflush(StreamOut);
    else
        fflush(stdout);

    return !wasError;
};
//-----------------------------------------------------------------------------
// Показ подсказки. message - сообщение в начале подсказки.
//-----------------------------------------------------------------------------
void TArgs::Help(const char* message)
{
    if (message)
        fprintf(StreamOut ? StreamOut : stdout, "\n%s", message);
    fprintf(StreamOut ? StreamOut : stdout,
            "\n Запуск: <имя_программы> [<опция> [<параметр>]] ... | [<опция>] ... | [<параметр>] ...\n");
    for (int i = 0; ArgsOptions[i].OptName != NULL; i++) {
        if (ArgsOptions[i].OptName[0] != 0)
            fprintf(StreamOut ? StreamOut : stdout,
                    "\n -%s\t%s\t%s",
                    ArgsOptions[i].OptName, ArgsOptions[i].NeedParam ? "<параметр>" : "          ", ArgsOptions[i].Descr);
        else
            fprintf(StreamOut ? StreamOut : stdout,
                    "\n \t<параметр>\t%s", ArgsOptions[i].Descr);
    };
    fprintf(StreamOut ? StreamOut : stdout, "\n\n");
};
//-----------------------------------------------------------------------------
// Печать сообщения о нераспознанных параметрах
//-----------------------------------------------------------------------------
void TArgs::PrintUnresolved()
{
    for (int i1 = 0; UnResolved[i1] != NULL; i1++) {
        fprintf(StreamOut ? StreamOut : stdout,
                "\nНеизвестный параметр '%s'. Игнорируем\n", UnResolved[i1]);
    };
};
//-----------------------------------------------------------------------------
// разбор строки с параметрами доступа к БД формата "пользователь/пароль@схемаБД"
//-----------------------------------------------------------------------------
void ParseDBAddr(const char* dbline, TString& dbusername, TString& dbuserpass,
                 TString& dbaddr)
{
    dbusername = "";
    dbuserpass = "";
    dbaddr = "";
    char *s, *s1;

    s = strdup(dbline);
    s1 = strchr(s, '@');
    if (s1) {
        dbaddr = s1 + 1;
        s1[0] = 0;
    }

    s1 = strchr(s, '/');
    if (s1) {
        s1[0] = 0;
        dbuserpass = s1 + 1;
        dbusername = s;
    }
    else {
        dbusername = s;
    };

    free(s);
};
///////////////////////////////////////////////////////////////////////////////
