///////////////////////////////////////////////////////////////////////////////
// Библиотека программных инструментов
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
// Средства разбора командной строки
///////////////////////////////////////////////////////////////////////////////
#ifndef BNOTOOLS_PARSECMD_H
#define BNOTOOLS_PARSECMD_H
///////////////////////////////////////////////////////////////////////////////
#include "Defines.h"
#include "tstring.h"
#include <stdio.h>

///////////////////////////////////////////////////////////////////////////////
/// Описание одной опции командной строки
///////////////////////////////////////////////////////////////////////////////
class TArgsOptions
{
public:
    // имя опции
    const char* OptName;
    // если true - то после опции должен следовать параметр, 
    // отделенный пробелом или знаком =
    bool NeedParam;
    // строка - описание опции. Используется в сообщениях об ошибках и при 
    // показе подсказки в функции Help
    const char* Descr;
};

///////////////////////////////////////////////////////////////////////////////
// Класс разбора параметров командной строки
///////////////////////////////////////////////////////////////////////////////
class TArgs
{
public:
    // деструктор 
    virtual ~TArgs();
    // разбор параметров командной строки
    // argc - количество параметров
    // argv - указатель на массив параметров
    // onlyOptions - true, если в командной строке не должно быть ничего,
    //           кроме описанных опций и их параметров
    bool ParseCmdLine(int argc,
                      const char* argv[],
                      bool onlyOptions = false
    );
    // Показ подсказки. message - сообщение в начале подсказки.
    virtual void Help(const char* message);
    // процедура присваивания значений
    // paramNum - номер найденной опции в списке пользователя (нумерация от 0)
    // paramName - имя найденной опции
    // paramVal - параметр найденной опции (если указан needParam, иначе NULL)
    virtual bool SetParam(int paramNum,
                          const char* paramName,
                          const char* paramVal = NULL
    ) = 0;
    // установка массива описания опций 
    static void SetOptions(TArgsOptions opts[]);
    // печать сообщения о нераспознанных параметрах
    static void PrintUnresolved();
    // поток для вывода сообщений об ошибках или подсказки (по умолчанию - stdout)
    static FILE* StreamOut;
    // Число нераспознанных параметров 
    static int UnResolved_Len;
    // список указателей на нераспознанные параметры
    static const char** UnResolved;
private:
    static TArgsOptions* ArgsOptions;
};

///////////////////////////////////////////////////////////////////////////////
/// разбор строки с параметрами доступа к БД формата "пользователь/пароль@схемаБД"
///////////////////////////////////////////////////////////////////////////////
void ParseDBAddr(const char* dbline, ///< входная строка
                 TString& dbusername, ///< имя пользователя БД
                 TString& dbuserpass, ///< пароль пользователя БД
                 TString& dbaddr ///< адрес базы данных
);
///////////////////////////////////////////////////////////////////////////////
#endif
