///////////////////////////////////////////////////////////////////////////////
// Библиотека небесной механики
// (с) ЗАО "КИА Системы", 2005 г.  
///////////////////////////////////////////////////////////////////////////////
// Представление временных диапазонов с их границами
///////////////////////////////////////////////////////////////////////////////
#ifndef KIALIB_TIMERANGE_H
#define KIALIB_TIMERANGE_H
///////////////////////////////////////////////////////////////////////////////
#include "TM.h"
#include "Defines.h"
#include <math.h>
///////////////////////////////////////////////////////////////////////////////
// Константы, определяющие неопределенные границы
#define MAXJDDATE (TimeJD(9999999, 0))
#define MINJDDATE (TimeJD(0, 0))

///////////////////////////////////////////////////////////////////////////////
class TIME_RANGE
{
public:
    // Конструктор по умолчанию
    TIME_RANGE(void)
    {
        init();
    };
    // Конструктор с указанием границ диапазона
    TIME_RANGE(TimeJD const& beginning, TimeJD const& end)
    {
        init();
        Init(beginning, end);
    };

    // Инициализация с явным указанием границ
    void Init(TimeJD const& beginning, TimeJD const& in_end);
    // Запрос границ диапазона
    void GetRange(TimeJD& beginning, TimeJD& in_end) const
    {
        beginning = beg;
        in_end = end;
    };
    // Запрос левой границы
    TimeJD const& GetRangeBeg(void) const
    {
        return beg;
    };
    // Запрос правой границы
    TimeJD const& GetRangeEnd(void) const
    {
        return end;
    };

    // Установка границ диапазона
    void SetRange(TimeJD const& date_beg, TimeJD const& date_end)
    {
        beg = date_beg;
        end = date_end;
        init_flag = true;
    };
    // Установка левой границы диапазона
    void SetRangeBeg(TimeJD const& dat)
    {
        beg = dat;
        init_flag = true;
    }

    // Установка правой границы диапазона
    void SetRangeEnd(TimeJD const& dat)
    {
        end = dat;
        init_flag = true;
    };

    // Попадает ли указанная дата внутрь диапазона?
    bool IsTime(TimeJD const& date) const
    {
        return (less_or_equal(beg, date) && greater_or_equal(end, date));
    };

    // Был ли класс проинициализирован?
    bool IsClass(void) const
    {
        return init_flag;
    }

    bool operator!(void) const
    {
        return !IsClass();
    }

    // Является ли левая граница неопределенной?
    bool IsNullBeg() const
    {
        return beg == MINJDDATE;
    };
    // Является ли правая граница неопределенной?
    bool IsNullEnd() const
    {
        return end == MAXJDDATE;
    };

protected:
    bool init_flag;
    TimeJD beg;
    TimeJD end;

    void init(void);

private:
    bool less_or_equal(TimeJD const& __beg, TimeJD const& __date) const;
    bool greater_or_equal(TimeJD const& __beg, TimeJD const& __date) const;
};

///////////////////////////////////////////////////////////////////////////////
#endif
