///////////////////////////////////////////////////////////////////////////////
// Библиотека алгоритмов небесной механики
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
#include "msgp.h"
#include "msgpext/sgp4ext.h"
#include <string.h>
#include <stdlib.h>
#include <cmath>
///////////////////////////////////////////////////////////////////////////////
// Класс описывающий структуру TLE-элементов
///////////////////////////////////////////////////////////////////////////////
// Заполнение элементов из двух текстовых строк
// Никаких проверок на правильность формата строк не выполняется!!! 
//-----------------------------------------------------------------------------
void TLESet::SetFromStr(char longstr1[130], char longstr2[130])
{
    const double rad = 57.29577951308230;

    char str[20];
    int i, nexp, ibexp;

    const double xpdotp = 229.1831180523293;

    // Первая строка

    strncpy(str, &longstr1[2], 5);
    str[5] = '\0';
    Satnum = atol(str);

    strncpy(str, &longstr1[18], 2);
    str[2] = '\0';
    EpochYr = atoi(str);

    strncpy(str, &longstr1[20], 12);
    str[12] = '\0';
    EpochDays = atof(str);

    strncpy(str, &longstr1[33], 10);
    str[10] = '\0';
    NDot = atof(str);

    strncpy(str, &longstr1[44], 1);
    str[1] = '.';
    strncpy(&str[2], &longstr1[45], 5);
    str[7] = '\0';
    NDDot = atof(str);

    strncpy(str, &longstr1[50], 2);
    str[2] = '\0';
    nexp = atoi(str);

    strncpy(str, &longstr1[53], 1);
    str[1] = '.';
    strncpy(&str[2], &longstr1[54], 5);
    str[7] = '\0';
    BStar = atof(str);

    strncpy(str, &longstr1[59], 2);
    str[2] = '\0';
    ibexp = atol(str);

    strncpy(str, &longstr1[64], 4);
    str[4] = '\0';
    ElementNum = atoi(str);

    // Вторая строка

    strncpy(str, &longstr2[8], 8);
    str[8] = '\0';
    Inclo = atof(str);

    strncpy(str, &longstr2[17], 8);
    str[8] = '\0';
    Omegao = atof(str);

    str[0] = '.';
    strncpy(&str[1], &longstr2[26], 7);
    str[8] = '\0';

    // Заполнение лидирующих нулей

    for (i = 1; i < (int)strlen(str); i++) {
        if (str[i] == ' ')
            str[i] = '0';
    }
    Ecco = atof(str);

    strncpy(str, &longstr2[34], 8);
    str[8] = '\0';
    Argpo = atof(str);

    strncpy(str, &longstr2[43], 8);
    str[8] = '\0';
    Mo = atof(str);

    strncpy(str, &longstr2[52], 11);
    str[11] = '\0';
    No = atof(str);

    // Вычисление величин, относящихся к среднему движению

    NDDot = NDDot * pow(10.0, nexp);
    No = No / xpdotp; //* rad/min
    NDot = NDot / (xpdotp * 1440.0); // minperday
    NDDot = NDDot / (xpdotp * 1440.0 * 1440);

    // Вычисление стандартных элементов орбиты

    Inclo = Inclo / rad;
    Omegao = Omegao / rad;
    Argpo = Argpo / rad;
    Mo = Mo / rad;
    BStar = BStar * pow(10.0, ibexp);


    // Вычисление времен для набора элементов sgp4epoch
    // Имейте в виду, что sgp4 используюет в качестве единиц сутки, 
    // отсчитываемые от 0 jan 1950 (sgp4epoch) и минуты от эпохи (time)
    if (EpochYr < 50)
        EpochYr += 2000;
    else
        EpochYr += 1900;

    Set();
};
//-----------------------------------------------------------------------------
// Заполнение вычислемых полей
//-----------------------------------------------------------------------------
void TLESet::Set()
{
    const double tumin = 13.44654985511;
    double sec;
    int mon, day, hr, minute;

    A = pow(No * tumin, (-2.0 / 3.0));
    Alta = A * (1.0 + Ecco) - 1.0;
    Altp = A * (1.0 - Ecco) - 1.0;


    EpTime = (EpochYr - 1950) * 365 + (EpochYr - 1949) / 4 + EpochDays;

    days2mdhms(EpochYr, EpochDays, mon, day, hr, minute, sec);
    julianday(EpochYr, mon, day, hr, minute, sec, MjdSatEpoch);

    MjdSatEpoch = MjdSatEpoch - 2400000.5;
    SFlag = true;
}
