///////////////////////////////////////////////////////////////////////////////
// Библиотека программных инструментов
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
// Класс для работы со строкой произвольной длины
///////////////////////////////////////////////////////////////////////////////
#include "tstring.h"
#include <ctype.h>
#include <cstring>
///////////////////////////////////////////////////////////////////////////////
void TString::TrimLeftChar(char c)
{
    std::string::iterator p = begin();

    if (*p != c)
        return;

    while (*p == c && p != end())
        p++;

    if (p == end())
        *this = "";
    else {
        TString str = "";
        while (p != end()) {
            str += *p;
            p++;
        }
        *this = str;
    }
}

//---------------------------------------------------------------------------
void TString::TrimRightChar(char c)
{
    std::string::reverse_iterator p = rbegin();

    if (*p != c)
        return;

    int l = length();
    while (*p == c && p != rend()) {
        p++;
        l--;
    }

    if (*p == c)
        *this = "";
    else
        resize(l);
}

//---------------------------------------------------------------------------
static unsigned char table[] =
{
    254, /* Ч 192 */
    224, /* Ю 193 */
    225, /* А 194 */
    246, /* Ж 195 */
    228, /* Д 196 */
    229, /* e 197 */
    244, /* Т 198 */
    227, /* Ц 199 */
    245, /* x 200 */
    232, /* Х 201 */
    233, /* И 202 */
    234, /* Й 203 */
    235, /* К 204 */
    236, /* Л 205 */
    237, /* М 206 */
    238, /* Н 207 */
    239, /* О 208 */
    255, /* Ъ 209 */
    240, /* П 210 */
    241, /* Я 211 */
    242, /* Р 212 */
    243, /* С 213 */
    230, /* Ф 214 */
    226, /* Б 215 */
    252, /* Э 216 */
    251, /* Ш 217 */
    231, /* Г 218 */
    248, /* Ь 219 */
    253, /* Щ 220 */
    249, /* Ы 221 */
    247, /* В 222 */
    250, /* З 223 */
};
//---------------------------------------------------------------------------
static unsigned char tablek[] =
{
    193, /* Ю  224*/
    194, /* А  225*/
    215, /* Б  226*/
    199, /* Ц  227*/
    196, /* Д  228*/
    197, /* e  229*/
    214, /* Ф  230*/
    218, /* Г  231*/
    201, /* Х  232*/
    202, /* И  233*/
    203, /* Й  234*/
    204, /* К  235*/
    205, /* Л  236*/
    206, /* М  237*/
    207, /* Н  238*/
    208, /* О  239*/
    210, /* П  240*/
    211, /* Я  241*/
    212, /* Р  242*/
    213, /* С  243*/
    198, /* Т  244*/
    200, /* x  245*/
    195, /* Ж  246*/
    222, /* В  247*/
    219, /* Ь  248*/
    221, /* Ы  249*/
    223, /* З  250*/
    217, /* Ш  251*/
    216, /* Э  252*/
    220, /* Щ  253*/
    192, /* Ч  254*/
    209, /* Ъ  255*/
};
//---------------------------------------------------------------------------
static unsigned char FromDOSToKOI(unsigned char s)
{
    if (s < 128)
        return s;
    else if (s < 160)
        return tablek[s - 128] + 32;
    else if (s < 192)
        return tablek[s - 160];
    return s;
}

//---------------------------------------------------------------------------
unsigned char ToWin(unsigned char s)
{
    if (s < 192)
        return s;
    else if (s < 224)
        return table[s - 192];
    else return table[s - 224] - 32;
}

//---------------------------------------------------------------------------
unsigned char ToKOI(unsigned char s)
{
#if defined(_WIN32) || defined(_WINDOWS) || defined(_Windows)
    if (s < 192)
        return s;
    if (s < 224)
        return tablek[s - 192] + 32;
    return tablek[s - 224];
#else
    return s;
#endif
}

//---------------------------------------------------------------------------
int cmpstr(const char* str1, const char* str2)
{
#if defined(_WIN32) || defined(_WINDOWS) || defined(_Windows)
    return strcmp(str1, str2);
#else
    unsigned const char *s1, *s2;

    for (s1 = (unsigned const char *)str1, s2 = (unsigned const char *)str2;
        *s1 != '\0' && *s2 != '\0'; s1++, s2++)
    {
        if (ToWin(*s1) < ToWin(*s2))
            return -1;
        if (ToWin(*s1) > ToWin(*s2))
            return 1;
    }

    if (*s1 != '\0')
        return 1;

    if (*s2 != '\0')
        return -1;

    return 0;
#endif
}

//---------------------------------------------------------------------------
void TString::ToWin()
{
    if (empty())
        return;

    std::string::iterator p;

    for (p = begin(); p != end(); p++)
        *p = ::ToWin(*p);
}

//---------------------------------------------------------------------------
void TString::ToKOI()
{
    if (empty())
        return;

    std::string::iterator p;

    for (p = begin(); p != end(); p++)
        *p = ::ToKOI(*p);
}

//---------------------------------------------------------------------------
void TString::FromDOSToKOI()
{
    if (empty())
        return;

    std::string::iterator p;

    for (p = begin(); p != end(); p++)
        *p = ::FromDOSToKOI(*p);
}

//---------------------------------------------------------------------------
void TString::MakeLower()
{
    if (empty())
        return;

    std::string::iterator p;

    for (p = begin(); p != end(); p++)
        *p = tolower(*p);
}

///////////////////////////////////////////////////////////////////////////////
