///////////////////////////////////////////////////////////////////////////////
// Библиотека программных инструментов
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
// Класс для работы со строкой произвольной длины
///////////////////////////////////////////////////////////////////////////////
#ifndef BNOTOOL_STRING_H
#define BNOTOOL_STRING_H
///////////////////////////////////////////////////////////////////////////////
#include <string>
///////////////////////////////////////////////////////////////////////////////
typedef std::string bstring; // Для совместимости с MS VC++ 6.0
///////////////////////////////////////////////////////////////////////////////
//  Класс строки произвольной длины
class TString : public bstring
{
public:
    // Конструктор по умолчанию
    TString();
    // Конструктор
    TString(const char* str);
    // Конструктор копирования
    TString(const TString& str);
    // Конструктор
    TString(const char c);
    // Конструктор
    TString(const char* str, unsigned n) : bstring(str, n)
    {
    };
    // Оператор приведения к const char *
    operator const char *() const
    {
        return c_str();
    };
    // Установка буфера (введен для совместимости с пред. версией)
    void SetBuffer(unsigned n)
    {
    };

    unsigned GetLength() const
    {
        return (unsigned)length();
    };
    // Возврат длины строки
    unsigned Length() const
    {
        return (unsigned)length();
    };
    // Проверка на пусто
    bool IsEmpty() const
    {
        return empty();
    };
    // Возврат символа в указанной позиции
    char GetChar(int k) const
    {
        return at(k);
    };
    // Поиск символа
    int FindChar(char c) const
    {
        return (int)find(c);
    };
    // Выделение левой части строки
    TString Left(unsigned n) const
    {
        return TString(c_str(), n);
    };
    // Усечение повторяющегося лидирующего символа
    void TrimLeftChar(char c);
    // Усечение лидирующего пробела
    void TrimLeft()
    {
        TrimLeftChar(' ');
    };
    // Усечение хвостовых повторяющихся символов
    void TrimRightChar(char c);
    // Усечение хвостовых пробелов
    void TrimRight()
    {
        TrimRightChar(' ');
    };
    // Перекодировка в Windows-1251
    void ToWin();
    // Перекодировка в КОИ8
    void ToKOI();
    // Перекодировка из DOS в КОИ8
    void FromDOSToKOI();
    // Перевод всех символов в нижний регистр
    void MakeLower();
    // Операторы конкатенации
    friend TString operator+(const TString& string1,
                             const TString& string2);
    friend TString operator+(char c1, const TString& string2);
    friend TString operator+(const TString& string1, char c2);
    friend TString operator+(const TString& string1, const char* str2);
    friend TString operator+(const char* str1, const TString& string2);
};

///////////////////////////////////////////////////////////////////////////////
// Реализация inline методов класса TString
///////////////////////////////////////////////////////////////////////////////
inline TString::TString() : bstring("")
{
};
//---------------------------------------------------------------------------
inline TString::TString(const char* str) : bstring(str == NULL ? "" : str)
{
};
//---------------------------------------------------------------------------
inline TString::TString(const TString& str) : bstring(str)
{
};
//---------------------------------------------------------------------------
inline TString::TString(const char c) : bstring(1, c)
{
};
///////////////////////////////////////////////////////////////////////////////
// Реализация inline операторов класса TString
///////////////////////////////////////////////////////////////////////////////
inline TString operator+(const TString& string1, const TString& string2)
{
    TString res = string1;
    res += string2;
    return res;
}

//---------------------------------------------------------------------------
inline TString operator+(char c1, const TString& string2)
{
    TString res = c1;
    res += string2;
    return res;
}

//---------------------------------------------------------------------------
inline TString operator+(const TString& string1, char c2)
{
    TString res = string1;
    res += c2;
    return res;
}

//---------------------------------------------------------------------------
inline TString operator+(const char* str1, const TString& string2)
{
    TString res = str1;
    res += string2;
    return res;
}

//---------------------------------------------------------------------------
inline TString operator+(const TString& string1, const char* str2)
{
    TString res = string1;
    res += str2;
    return res;
}

///////////////////////////////////////////////////////////////////////////////
// Функция сравнения строк
///////////////////////////////////////////////////////////////////////////////
int cmpstr(const char* str1, const char* str2);
// Функция перекодировки в Windows-1251
unsigned char ToWin(unsigned char s);
// Функция перекодировки в КОИ8
unsigned char ToKOI(unsigned char s);
///////////////////////////////////////////////////////////////////////////////
#endif
