///////////////////////////////////////////////////////////////////////////////
// Библиотека программных инструментов
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
// Реализация класса двухсвязанными списка
///////////////////////////////////////////////////////////////////////////////
#include <stdlib.h>
///////////////////////////////////////////////////////////////////////////////
#include "twll.h"
///////////////////////////////////////////////////////////////////////////////
// Конструктор
CTwll_BList_Class::CTwll_BList_Class()
{
    plast = NULL;
    pfirst = NULL;
}

//-----------------------------------------------------------------------------
// Исключение элемента из списка
void CTwll_BList_Class::delete_node(TWLL_BENTRY_CLASS* e)
{
    TWLL_BENTRY_CLASS* p;

    if (e->next == NULL)
        plast = e->prev;
    else {
        p = e->next;
        p->prev = e->prev;
    }

    if (e->prev == NULL)
        pfirst = e->next;
    else {
        p = e->prev;
        p->next = e->next;
    }
}

//-----------------------------------------------------------------------------
// Вставка элемента перед указанным
void CTwll_BList_Class::insert_prev(TWLL_BENTRY_CLASS* c, TWLL_BENTRY_CLASS* n,
                                    unsigned id)
{
    TWLL_BENTRY_CLASS* p;

    if (!c->prev) {
        n->prev = NULL;
        pfirst = n;
    }
    else {
        n->prev = c->prev;
        p = c->prev;
        p->next = n;
    }

    n->next = c;
    c->prev = n;
    n->ident = id;
}

//-----------------------------------------------------------------------------
// Вставка элемента после указанного
void CTwll_BList_Class::insert_next(TWLL_BENTRY_CLASS* c, TWLL_BENTRY_CLASS* n,
                                    unsigned id)
{
    TWLL_BENTRY_CLASS* p;

    if (!c->next) {
        n->next = NULL;
        plast = n;
    }
    else {
        n->next = c->next;
        p = c->next;
        p->prev = n;
    }

    n->prev = c;
    c->next = n;
    n->ident = id;
}

//-----------------------------------------------------------------------------
// Вставка элемента в конец списка
void CTwll_BList_Class::insert_last(TWLL_BENTRY_CLASS* n, unsigned id)
{
    TWLL_BENTRY_CLASS* p;

    if (!plast) {
        pfirst = n;
        n->prev = NULL;
    }
    else {
        p = plast;
        p->next = n;
        n->prev = plast;
    }

    plast = n;
    n->next = NULL;
    n->ident = id;
}

//-----------------------------------------------------------------------------
// Перемещение элемента в конец списка
void CTwll_BList_Class::moveto_last(TWLL_BENTRY_CLASS* c)
{
    TWLL_BENTRY_CLASS* p;

    if (!c->next)
        return;

    p = c->next;
    p->prev = c->prev;

    p = c->prev;
    if (!p)
        pfirst = c->next;
    else
        p->next = c->next;

    p = plast;
    p->next = c;
    c->prev = plast;
    plast = c;
    c->next = NULL;
}

//-----------------------------------------------------------------------------
// Вставка элемента в начало списка
void CTwll_BList_Class::insert_first(TWLL_BENTRY_CLASS* n, unsigned id)
{
    TWLL_BENTRY_CLASS* p;

    if (!plast) {
        plast = n;
        n->next = NULL;
    }
    else {
        p = pfirst;
        p->prev = n;
        n->next = pfirst;
    }

    pfirst = n;
    n->prev = NULL;
    n->ident = id;
}

//-----------------------------------------------------------------------------
// Перемещение элемента в начало списка
void CTwll_BList_Class::moveto_first(TWLL_BENTRY_CLASS* c)
{
    TWLL_BENTRY_CLASS* p;

    if (!c->prev)
        return;

    if (!c->next)
        plast = c->prev;
    else {
        p = c->next;
        p->prev = c->prev;
    }

    p = c->prev;
    p->next = c->next;

    p = pfirst;
    p->prev = c;
    c->next = p;

    pfirst = c;
    c->prev = NULL;
}

//-----------------------------------------------------------------------------
// Поиск элемента элемента по его идентификатору
TWLL_BENTRY_CLASS* CTwll_BList_Class::search_id(unsigned id) const
{
    TWLL_BENTRY_CLASS* p;

    for (p = plast; p != NULL; p = p->prev)
        if (p->ident == id)
            break;

    return (p);
}

//-----------------------------------------------------------------------------
// Удаление элемента элемента по его идентификатору
void CTwll_BList_Class::delete_id(unsigned id)
{
    TWLL_BENTRY_CLASS* p;

    p = search_id(id);
    if (p != NULL)
        delete_node(p);

    return;
}

//-----------------------------------------------------------------------------
// Размер списка (число элементов в нем)
unsigned CTwll_BList_Class::size() const
{
    TWLL_BENTRY_CLASS* p;
    unsigned i;

    for (p = pfirst, i = 0; p != NULL; p = p->next, i++);
    return (i);
}

//-----------------------------------------------------------------------------
// Является ли список пустым
bool CTwll_BList_Class::isempty() const
{
    return (pfirst == NULL) && (plast == NULL);
}

//-----------------------------------------------------------------------------
// Принадлежит ли элемент списку
bool CTwll_BList_Class::isown(TWLL_BENTRY_CLASS* e)
{
    TWLL_BENTRY_CLASS* c = e;

    while (c->prev != NULL)
        c = c->prev;

    if (pfirst != c)
        return false;

    c = e;

    while (c->next != NULL)
        c = c->next;

    if (plast != c)
        return false;

    return true;
}

///////////////////////////////////////////////////////////////////////////////
