///////////////////////////////////////////////////////////////////////////////
// Библиотека программных инструментов
// (с) ИПМ им.М.В.Келдыша, ЗАО "Проекттехника"
///////////////////////////////////////////////////////////////////////////////
// Классы и шаблоны для работы со двухсвязанными списками
///////////////////////////////////////////////////////////////////////////////
#ifndef BNOTOOL_TWLL_H
#define BNOTOOL_TWLL_H
///////////////////////////////////////////////////////////////////////////////
#include <stdlib.h>

///////////////////////////////////////////////////////////////////////////////
// Базовый класс элемента двухсвязааного списка
class CTwll_BEntry_Class
{
public:
    // Конструктор
    CTwll_BEntry_Class()
    {
        next = prev = NULL;
        ident = 0;
    };
    // Деструктор
    virtual ~CTwll_BEntry_Class()
    {
    };
    // Указатель на следующий элемент
    class CTwll_BEntry_Class* next;
    // Указатель на предыдущий элемент
    class CTwll_BEntry_Class* prev;
    // Идентификатор элемента
    unsigned ident;
};

typedef class CTwll_BEntry_Class TWLL_BENTRY_CLASS;

///////////////////////////////////////////////////////////////////////////////
// Базовый класс двухсвязанного списка
///////////////////////////////////////////////////////////////////////////////
class CTwll_BList_Class
{
public:
    // Конструктор
    CTwll_BList_Class();
    // Деструктор
    virtual ~CTwll_BList_Class()
    {
    };
    // Исключение элемента из списка
    void delete_node(TWLL_BENTRY_CLASS* el);
    // Вставка элемента перед указанным
    void insert_prev(TWLL_BENTRY_CLASS* Cur, TWLL_BENTRY_CLASS* New, unsigned id = 0);
    // Вставка элемента после указанного
    void insert_next(TWLL_BENTRY_CLASS* Cur, TWLL_BENTRY_CLASS* New, unsigned id = 0);
    // Вставка элемента в конец списка
    void insert_last(TWLL_BENTRY_CLASS* New, unsigned id = 0);
    // Перемещение элемента в конец списка
    void moveto_last(TWLL_BENTRY_CLASS* Cur);
    // Вставка элемента в начало списка
    void insert_first(TWLL_BENTRY_CLASS* New, unsigned id = 0);
    // Перемещение элемента в начало списка
    void moveto_first(TWLL_BENTRY_CLASS* Cur);
    // Поиск элемента элемента по его идентификатору
    TWLL_BENTRY_CLASS* search_id(unsigned id) const;
    // Удаление элемента элемента по его идентификатору
    void delete_id(unsigned id);
    // Является ли список пустым
    bool isempty() const;
    // Принадлежит ли элемент списку
    bool isown(TWLL_BENTRY_CLASS* e);
    // Размер списка (число элементов в нем)
    unsigned size() const;
    // Сортировка списка сиспользованием указанной функции сравнения
    void sort(int compare(TWLL_BENTRY_CLASS*, TWLL_BENTRY_CLASS*));
    // Указатель на первый элемент
    TWLL_BENTRY_CLASS* pfirst;
    // Указатель на последний элемент
    TWLL_BENTRY_CLASS* plast;

private:
    // Смена местами двух элементов списка
    void swap(TWLL_BENTRY_CLASS*, TWLL_BENTRY_CLASS*);
    // Доступ к элементу по его порядковому номеру в списке
    TWLL_BENTRY_CLASS* set_number(int k);
};

typedef class Twll_BList_Class TWLL_BLIST_CLASS;

///////////////////////////////////////////////////////////////////////////////
// Шаблон динамического связанного списка
///////////////////////////////////////////////////////////////////////////////
template <class ENTRY>
class Twll_List_Class : public CTwll_BList_Class
{
public:
    // Деструктор
    virtual ~Twll_List_Class()
    {
        clear();
    }

    // Доступ к первому элементу списка
    ENTRY* first() const
    {
        return (ENTRY *)(CTwll_BList_Class::pfirst);
    }

    // Доступ к последнему элементу списка
    ENTRY* last() const
    {
        return (ENTRY *)(CTwll_BList_Class::plast);
    }

    // Поиск элемента элемента по его идентификатору
    ENTRY* search_id(unsigned id) const
    {
        return (ENTRY *)(CTwll_BList_Class::search_id(id));
    }

    // Исключение элемента из списка
    void delete_node(ENTRY* e)
    {
        CTwll_BList_Class::delete_node(e);
        delete e;
    };
    // Очистка списка
    void clear()
    {
        ENTRY *cur, *prev;

        cur = last();
        while (cur != NULL) {
            prev = cur->prev();
            delete cur;
            cur = prev;
        }

        pfirst = plast = NULL;
        return;
    };
};

///////////////////////////////////////////////////////////////////////////////
// Шаблон элементов динамического связанного списка
///////////////////////////////////////////////////////////////////////////////
template <class BASE>
class Twll_Entry_Class :
    public CTwll_BEntry_Class, public BASE
{
public:
    // Доступ к следующему элементу списка
    Twll_Entry_Class* next() const
    {
        return (Twll_Entry_Class *)
            (CTwll_BEntry_Class::next);
    }

    // Доступ к предыдущему элементу списка
    Twll_Entry_Class* prev() const
    {
        return (Twll_Entry_Class *)(CTwll_BEntry_Class::prev);
    }

    // Деструктор
    virtual ~Twll_Entry_Class()
    {
    };
};
#endif
///////////////////////////////////////////////////////////////////////////////
