#include "QualityOrbit.h"
#include "XMLFunctions.h"
#include "Propagator.h"

void QualityOrbit::AddValues(double dispR, double dispN, double dispB)
{
	m_r.AddValue(dispR);
	m_n.AddValue(dispN);
	m_b.AddValue(dispB);
}

void QualityOrbit::SetDate(double interval, TimeJD date)
{
	m_date = date;
	m_interval = interval;
}

void QualityOrbit::CalculateStatistics()
{
	m_r.CalculateStatistics();
	m_n.CalculateStatistics();
	m_b.CalculateStatistics();
}

void QualityOrbit::ToXml( std::ostream& outs ) const
{
	PutValue(outs,"R",m_r);
	PutValue(outs,"N",m_n);
	PutValue(outs,"B",m_b);
}

void QualityOrbit::Calculate(const SInitOrbit& orbit)
{
	if(!orbit.GetCovMtx().HasMtx() || orbit.GetMotionModel().GetType() != ST_PROP){
		return;
	}

	Propagator prop(orbit, true);

	double interval = 1*86400;
	TimeJD edate = ShiftDate(orbit.GetDate(),interval);

	TimeJD jd = orbit.GetDate();

	bool exit = false;
	while (!exit){
		if (jd >= edate){
			jd = edate;
			exit = true;
		}

		SInitOrbit norb;
		prop.Propagate(jd,norb);

		const CovMtx& mtx = norb.GetCovMtx();
		std::map<std::string, double> covrnb = mtx.GetCovRnbMap(norb.GetPhasePointJ2000().CoordsVel);

		AddValues(covrnb["RR"],covrnb["NN"],covrnb["BB"]);

		double step = 600;	
		jd = ShiftDate(jd,step);
	}

	CalculateStatistics();
	SetDate(interval, edate);
}
