#pragma once

#include "KepData.h"
#include "TM.h"
#include "StateVector.h"
#include "Propagator.h"
#include "XMLFunctions.h"

class AscNodeOrbit
{
public:
    AscNodeOrbit();
    explicit AscNodeOrbit(const PhasePoint6D& p);
    void calculate(Propagator& prop, const TimeJD& date, bool vzmode, bool gcs_node, bool calc_params = true);
    void calculate_desc(Propagator& prop, const TimeJD& date, bool vzmode, bool gcs_node, bool calc_params = true);
    void calculate_params(const PhasePoint6D& j2000AscNodePoint);
    double get_latitude() const { return m_latitude; }
    double get_longitude() const { return m_longitude; }
    double get_altitude() const { return m_altitude; }
    double get_dracp() const { return m_drperiod; }
    double get_vdracp() const { return m_drperiodDt; }
    KepData get_kd_j2000() const { return m_kdj2000; }
    KepData get_kd_eci() const { return m_kdeci; }
    TimeJD get_date() const { return m_date; }
    Vect6 get_j2000() const { return m_j2000; }
    Vect6 get_gcs() const { return m_gcs; }
    void set_rev(int rev);
    void set_age(double age_days);
    int get_rev() const { return m_rev; }
    double get_age() const { return m_age; }
    bool is_desc() const { return m_is_desc; }
    void ToXml(std::ostream& outs) const;
private:
    void calc_asc_node(Propagator& prop, const TimeJD& date, bool vzmode, bool gcs_node, PhasePoint6D& result) const;
    void calc_desc_node(Propagator& prop, const TimeJD& date, bool vzmode, bool gcs_node, PhasePoint6D& result) const;
    void calc_draconic_period(Propagator& prop, const TimeJD& asc_node_date, bool vzmode, bool gcs_node, double& dracp, double& vdracp) const;
    double get_kep_time(double TA, double e, double w) const;
    static KepData get_kep_data(const Vect6& pos);
    bool jump_arg_of_lat(const TimeJD& date, Propagator& prop, double dstU, bool gcs_node, SInitOrbit& result, int type = 0) const;
    double get_step(double period, double arg_of_per, double e, double u, double dst_u, bool positive) const;
    double get_step(const Kepler& kep, double dst_u, bool positive) const;

    double m_latitude;
    double m_longitude;
    double m_altitude;
    double m_drperiod;
    double m_drperiodDt;
    double m_age;
    int m_rev;
    KepData m_kdj2000;
    KepData m_kdeci;
    //KepData m_kdgcs;
    Vect6 m_j2000;
    Vect6 m_gcs;
    TimeJD m_date;
    bool m_is_desc;
};
