#ifndef ASTROBASEMODELS_H
#define ASTROBASEMODELS_H

class EnumToId
{
public:
    int id;
    const char* key;
};

static const int LightModelSize = 3;
static const int KpModelSize = 2;
static const int AtmModelSize = 14;
static const int GravModelSize = 14;

static const EnumToId LightModelIds[LightModelSize] = {
    {lightNONE, "LIGHT_NONE"},
    {lightNoShadow, "LIGHT_NOSHADOW"},
    {lightCylinderShadow, "LIGHT_CYLINDERSHADOW"},
};

static const EnumToId KpModelIds[KpModelSize] = {
    {0, "KP_AVERAGE"},
    {1, "KP_3H"},
};

static const EnumToId AtmModelIds[AtmModelSize] = {
    {atmIdNONE, "ATM_NONE"},
    {atmIdDMA90, "ATM_DMA90"},
    {atmIdDMA77, "ATM_DMA77"},
    {atmIdMSIS90, "ATM_MSIS90"},
    {atmIdDMA2000, "ATM_DMA2000"},
    {atmIdUSSA76, "ATM_USSA76"},
    {atmIdJ72, "ATM_J72"},
    {atmIdJR, "ATM_JR"},
    {atmIdCIRA90, "ATM_CIRA90"},
    {atmIdC81, "ATM_C81"},
    {atmIdDMA84, "ATM_DMA84"},
    {atmIdStatic, "ATM_STATIC"},
    {atmIdDMA2004, "ATM_DMA2004"},
    {atmIdDMA2004m, "ATM_DMA2004M"},
};

static const EnumToId GravModelIds[GravModelSize] = {
    {grIdPZ90, "GRAV_PZ90"},
    {grIdEGM96, "GRAV_EGM96"},
    {grIdJGM3, "GRAV_JGM3"},
    {grIdIERS, "GRAV_IERS"},
    {grIdJGM2, "GRAV_JGM2"},
    {grIdJGM1, "GRAV_JGM1"},
    {grIdGEMT3, "GRAV_GEMT3"},
    {grIdGEMT2, "GRAV_GEMT2"},
    {grIdGEMT1, "GRAV_GEMT1"},
    {grIdWGS84, "GRAV_WGS84"},
    {grIdWGS72, "GRAV_WGS72"},
    {grIdPZ902s, "GRAV_PZ9002"},
    {grIdMu, "GRAV_MU"},
    {grIdNONE, "GRAV_NONE"}
};

static int GetIdByKey(const EnumToId* ids, const char* key, int size)
{
    for (int i = 0; i < size; ++i) {
        if (!strcasecmp(ids[i].key, key)) {
            return ids[i].id;
        }
    }

    TAssert(0);

    return 0;
}

static const char* GetKeyById(const EnumToId* ids, int id, int size)
{
    for (int i = 0; i < size; ++i) {
        if (ids[i].id == id) {
            return ids[i].key;
        }
    }

    TAssert(0);

    return nullptr;
}

static const char* KpModelKeyById(int id)
{
    return GetKeyById(KpModelIds, id, KpModelSize);
}

static int KpModelIdByKey(const char* key)
{
    return GetIdByKey(KpModelIds, key, KpModelSize);
}

static const char* LightModelKeyById(int id)
{
    return GetKeyById(LightModelIds, id, LightModelSize);
}

static int LightModelIdByKey(const char* key)
{
    return GetIdByKey(LightModelIds, key, LightModelSize);
}

static const char* AtmModelKeyById(int id)
{
    return GetKeyById(AtmModelIds, id, AtmModelSize);
}

static int AtmModelIdByKey(const char* key)
{
    return GetIdByKey(AtmModelIds, key, AtmModelSize);
}

static const char* GravModelKeyById(int id)
{
    return GetKeyById(GravModelIds, id, GravModelSize);
}

static int GravModelIdByKey(const char* key)
{
    return GetIdByKey(GravModelIds, key, GravModelSize);
}

static bool IsLightModelUse(const char* key)
{
    return LightModelIdByKey(key) != lightNONE;
}

static bool IsAtmModelUse(const char* key)
{
    return AtmModelIdByKey(key) != atmIdNONE;
}

static bool IsGravModelUse(const char* key)
{
    return GravModelIdByKey(key) != grIdNONE;
}

#endif
