#ifndef __TIME_H_
#define __TIME_H_

#include <string>
#include <iomanip>
#include "globals.h"
#include <cmath>

class XTime
{
public:
    XTime()
    {
        Init();
    };

    int Round(double arg) const
    {
        double fFrac, fInt;
        fFrac = modf(arg, &fInt);
        if (fFrac >= 0.5) fInt++;
        return (int)fInt;
    }

    void Init()
    {
        Year = 0;
        Month = 0;
        Day = 0;
        Hour = 0;
        Min = 0;
        Sec = 0;
        Fraction = 0;
    }

    /*	XTime(int hour, int min, int sec, int msec)
        {
            m_year	= 0;
            m_month	= 0;
            m_day	= 0;
            m_hour	= hour;
            m_min	= min;
            m_sec	= sec;
            m_msec	= msec;
        };
    */
    XTime(int year, int month, int day, int hour, int min, int sec, double frac)
    {
        Year = year;
        Month = month;
        Day = day;
        Hour = hour;
        Min = min;
        Sec = sec;
        Fraction = frac;
    }

    int GetYear() const { return Year; };
    int GetMonth() const { return Month; };
    int GetDay() const { return Day; };
    int GetHour() const { return Hour; };
    int GetMin() const { return Min; };
    int GetSec() const { return Sec; };
    int GetMSec() const { return Round(Fraction * 1000); };

    std::string GetAsStr() const
    {
        std::ostringstream ss;

        ss
            << std::setfill('0')
            << std::setw(2) << GetDay() << "/"
            << std::setw(2) << GetMonth() << "/"
            << std::setw(4) << GetYear() << " "

            << std::setw(2) << GetHour() << ":"
            << std::setw(2) << GetMin() << ":"
            << std::setw(2) << GetSec() << "."
            << std::setw(3) << GetMSec();

        return ss.str();
    };

    void SetYear(int year) { Year = year; };
    void SetMonth(int month) { Month = month; };
    void SetDay(int day) { Day = day; };
    void SetHour(int hour) { Hour = hour; };
    void SetMin(int min) { Min = min; };
    void SetSec(int sec) { Sec = sec; };
    void SetMSec(int msec) { Fraction = msec * 1E-3; };

    int Year;
    unsigned short int Month;
    unsigned short int Day;
    unsigned short int Hour;
    unsigned short int Min;
    unsigned short int Sec;
    double Fraction;
};

#endif
