#include "CovMtx.h"
#include "XMLFunctions.h"
#include "StAlgorithms.h"
#include "BNOAstroBase.h"

CovMtx::CovMtx()
    : m_covdim(0)
{
}

CovMtx::CovMtx(const DSymmMatrix& mtx, bool kb, bool sp)
{
    SetCovRv(mtx, kb, sp);
}

CovMtx::CovMtx(const double* akx)
{
    SetCovRv(akx);
}

bool CovMtx::Load(myXML::Contain* parent, bool loadkb, bool loadsp)
{
    std::string conName = "CovRv";

    myXML::Contain* con;
    if (FindXMLContainer(parent, conName, con)) {

        std::string names[] = {"XX", "XY", "XZ", "XVX", "XVY", "XVZ", "YY", "YZ", "YVX", "YVY", "YVZ", "ZZ", "ZVX", "ZVY", "ZVZ", "VXVX", "VXVY", "VXVZ", "VYVY", "VYVZ", "VZVZ", "XS", "YS", "ZS", "VXS", "VYS", "VZS", "SK", "XK", "YK", "ZK", "VXK", "VYK", "VZK", "SS", "KK"};

        std::map<std::string, double> covmap;
        for (const auto& name : names) {
            double value;
            bool loaded;
            LoadXML(con, name, value, false, &loaded);
            if (loaded) {
                covmap[name] = value;
            }
        }

        if (!GetMapItem(covmap, std::string("XX"))) {
            m_covdim = 0;
            return true;
        }

        if (!SetCovRv(covmap, loadkb, loadsp)) {
            return false;
        }

    }
    else {
        m_covdim = 0;
        return true;
    }

    return true;
}


void CovMtx::load(const jsonval& parent)
{
    std::string names[] = {"XX", "XY", "XZ", "XVX", "XVY", "XVZ", "YY", "YZ", "YVX", "YVY", "YVZ", "ZZ", "ZVX", "ZVY", "ZVZ", "VXVX", "VXVY", "VXVZ", "VYVY", "VYVZ", "VZVZ", "XS", "YS", "ZS", "VXS", "VYS", "VZS", "SK", "XK", "YK", "ZK", "VXK", "VYK", "VZK", "SS", "KK"};

    std::map<std::string, double> covmap;
    for (const auto& name : names) {
        double value;
        if (loadjson(parent, name, value, false)) {
            covmap[name] = value;
        }
    }

    if (!SetCovRv(covmap, true, true)) {
        throw (Exp() << "bad cov");
    }
}

std::string CovMtx::CovIndexToKey(int i, int j, bool kb, bool sp)
{
    char tag[20];
    if (kb && sp) {
        static char const* const tag_XYZ[] = {"X", "Y", "Z", "VX", "VY", "VZ", "S", "K"};
        sprintf(tag, "%s%s", tag_XYZ[i], tag_XYZ[j]);
    }
    else if (kb && !sp) {
        static char const* const tag_XYZ[] = {"X", "Y", "Z", "VX", "VY", "VZ", "S"};
        sprintf(tag, "%s%s", tag_XYZ[i], tag_XYZ[j]);
    }
    else if (!kb && sp) {
        static char const* const tag_XYZ[] = {"X", "Y", "Z", "VX", "VY", "VZ", "K"};
        sprintf(tag, "%s%s", tag_XYZ[i], tag_XYZ[j]);
    }
    else {
        static char const* const tag_XYZ[] = {"X", "Y", "Z", "VX", "VY", "VZ"};
        sprintf(tag, "%s%s", tag_XYZ[i], tag_XYZ[j]);
    }

    return tag;
}

void CovMtx::GetCovRv(DSymmMatrix& mtx, bool& kb, bool& sp) const
{
    mtx = m_covmtx;
    kb = m_covHasKb;
    sp = m_covHasSp;
}

void CovMtx::SetCovRv(const DSymmMatrix& mtx, bool kb, bool sp)
{
    m_covHasKb = kb;
    m_covHasSp = sp;
    m_covmtx = mtx;

    if (m_covHasKb && m_covHasSp) {
        m_covdim = 8;
    }
    else if (!m_covHasKb && !m_covHasSp) {
        m_covdim = 6;
    }
    else {
        m_covdim = 7;
    }
}

bool CovMtx::SetCovRv(const std::map<std::string, double>& covmap, bool loadkb, bool loadsp)
{
    m_covHasKb = loadkb && GetMapItem(covmap, std::string("SS"));
    m_covHasSp = loadsp && GetMapItem(covmap, std::string("KK"));

    if (m_covHasKb && m_covHasSp) {
        m_covdim = 8;
    }
    else if (!m_covHasKb && !m_covHasSp) {
        m_covdim = 6;
    }
    else {
        m_covdim = 7;
    }

    m_covmtx.Init((unsigned)m_covdim);
    for (int i = 0; i < m_covdim; i++) {
        for (int j = i; j < m_covdim; j++) {
            std::string tag = CovIndexToKey(i, j, m_covHasKb, m_covHasSp);
            if (!GetMapItem(covmap, tag, m_covmtx(i, j))) {
                throw (Exp() << "Can't find cov item:" << tag);
            }
        }
    }

    return true;
}

void CovMtx::SetCovRv(const double* akx)
{
    std::map<std::string, double> covmap;
    FillCovMtx(akx, covmap);
    SetCovRv(covmap, false, false);
}

std::map<std::string, double> CovMtx::GetCovOrbMap(const Vect6& pos) const
{
    std::map<std::string, double> ret;
    std::map<std::string, double> covmap = GetCovRvMap();
    CreateCovOrb(pos, covmap, ret);

    return ret;
}

std::map<std::string, double> CovMtx::GetCovRnbMap(const Vect6& pos) const
{
    std::map<std::string, double> ret;
    std::map<std::string, double> covmap = GetCovRvMap();
    CreateCovRnb(pos, covmap, ret);

    return ret;
}

void CovMtx::FillVec(Vect6 pos, double* vec)
{
    for (int i = 0; i < 6; i++) { vec[i] = pos[i]; }
}

void CovMtx::FillCovMtx(std::map<std::string, double>& covmap, double* akx)
{
    akx[0] = covmap["XX"];
    akx[1] = covmap["XY"];
    akx[2] = covmap["XZ"];
    akx[3] = covmap["XVX"];
    akx[4] = covmap["XVY"];
    akx[5] = covmap["XVZ"];
    akx[6] = covmap["YY"];
    akx[7] = covmap["YZ"];
    akx[8] = covmap["YVX"];
    akx[9] = covmap["YVY"];
    akx[10] = covmap["YVZ"];
    akx[11] = covmap["ZZ"];
    akx[12] = covmap["ZVX"];
    akx[13] = covmap["ZVY"];
    akx[14] = covmap["ZVZ"];
    akx[15] = covmap["VXVX"];
    akx[16] = covmap["VXVY"];
    akx[17] = covmap["VXVZ"];
    akx[18] = covmap["VYVY"];
    akx[19] = covmap["VYVZ"];
    akx[20] = covmap["VZVZ"];
}

void CovMtx::FillCovMtx(const double* akx, std::map<std::string, double>& covmap)
{
    covmap["XX"] = akx[0];
    covmap["XY"] = akx[1];
    covmap["XZ"] = akx[2];
    covmap["XVX"] = akx[3];
    covmap["XVY"] = akx[4];
    covmap["XVZ"] = akx[5];
    covmap["YY"] = akx[6];
    covmap["YZ"] = akx[7];
    covmap["YVX"] = akx[8];
    covmap["YVY"] = akx[9];
    covmap["YVZ"] = akx[10];
    covmap["ZZ"] = akx[11];
    covmap["ZVX"] = akx[12];
    covmap["ZVY"] = akx[13];
    covmap["ZVZ"] = akx[14];
    covmap["VXVX"] = akx[15];
    covmap["VXVY"] = akx[16];
    covmap["VXVZ"] = akx[17];
    covmap["VYVY"] = akx[18];
    covmap["VYVZ"] = akx[19];
    covmap["VZVZ"] = akx[20];
}

int CovMtx::IsHyperbolic(const Vect6& U)
{
    double r = sqrt(U[0] * U[0] + U[1] * U[1] + U[2] * U[2]);
    double v = sqrt(U[3] * U[3] + U[4] * U[4] + U[5] * U[5]);
    double mu = GetAstroBase()->GetEarthMu();
    double b = 2 * mu / r;
    double v2 = v * v;
    if (v2 == b) return 0;
    if (v2 < b) return 1;
    else return -1;
}

bool CovMtx::TestOrbit(const Kepler& kepler)
{
    return !(
        kepler.A < 0 ||
        kepler.GetEcc() < 0 || kepler.GetEcc() > 1 ||
        kepler.CosI < -1 || kepler.CosI > 1 ||
        kepler.GetPeriod() < 0
    );
}

bool CovMtx::CreateCovOrb(Vect6 pos, std::map<std::string, double>& covmap, std::map<std::string, double>& covmtx) const
{
    double r423[22];
    double r700[41];
    double r701[10];
    double r707[36];
    double rab[100];
    double obr707[36];
    double akq[36];
    double akx[21];
    double x[6];

    FillVec(pos, x);
    FillCovMtx(covmap, akx);

    if (IsHyperbolic(pos) < 1) {
        return false;
    }
    else {
        Kepler kk;
        kk.SetXYZ(Vect6(x[0], x[1], x[2], x[3], x[4], x[5]));
        if (!TestOrbit(kk)) {
            return false;
        }
    }

    double t = 0;
    const double amu = GetAstroBase()->GetEarthMu();

    b423(amu, t, x, r423);
    b700(amu, &r423[1], r700);
    if (!b701(r700, t, r701)) { return false; }
    b707(r701, r700, r707);
    obgau(6, r707, rab, obr707);
    bpbt(obr707, akx, akq, 6, 6);

    const int nq = 36;
    double q2[nq];
    double akq2[nq];
    memset(q2, 0, sizeof(double) * nq);
    memset(akq2, 0, sizeof(double) * nq);

    double fi1 = r423[2];
    double fi2 = r423[3];

    q2[0] = 1;
    q2[21] = 1;
    q2[28] = 1;
    q2[35] = 1;
    double tv = fi1 * fi1 + fi2 * fi2;
    q2[7] = fi1 / sqrt(tv);
    q2[8] = fi2 / sqrt(tv);
    q2[13] = fi2 / (tv);
    q2[14] = -fi1 / (tv);

    bpbt(q2, akq, akq2, 6, 6);

    covmtx["AA"] = akq2[0];
    covmtx["AE"] = akq2[1];
    covmtx["AW"] = akq2[2];
    covmtx["AI"] = akq2[3];
    covmtx["AO"] = akq2[4];
    covmtx["AT"] = akq2[5];
    covmtx["EE"] = akq2[6];
    covmtx["EW"] = akq2[7];
    covmtx["EI"] = akq2[8];
    covmtx["EO"] = akq2[9];
    covmtx["ET"] = akq2[10];
    covmtx["WW"] = akq2[11];
    covmtx["WI"] = akq2[12];
    covmtx["WO"] = akq2[13];
    covmtx["WT"] = akq2[14];
    covmtx["II"] = akq2[15];
    covmtx["IO"] = akq2[16];
    covmtx["IT"] = akq2[17];
    covmtx["OO"] = akq2[18];
    covmtx["OT"] = akq2[19];
    covmtx["TT"] = akq2[20];
    covmtx["PP"] = sqrt(pow(akq2[0], 3) / amu);

    return true;
}

bool CovMtx::SetCovRvActive(Vect6 pos, double sigmaIncliantion)
{
    double r423[22];
    double r700[41];
    double r701[10];
    double r707[36];
    double rab[100];
    double obr707[36];
    double akq[36];
    double akx[21];
    double x[6];

    FillVec(pos, x);

    std::map<std::string, double> covmap = GetCovRvMap();
    FillCovMtx(covmap, akx);

    if (IsHyperbolic(pos) < 1) {
        return false;
    }
    else {
        Kepler kk;
        kk.SetXYZ(Vect6(x[0], x[1], x[2], x[3], x[4], x[5]));
        if (!TestOrbit(kk)) {
            return false;
        }
    }

    double t = 0;
    const double amu = GetAstroBase()->GetEarthMu();

    b423(amu, t, x, r423);
    b700(amu, &r423[1], r700);
    if (!b701(r700, t, r701)) { return false; }
    b707(r701, r700, r707);
    obgau(6, r707, rab, obr707);
    bpbt(obr707, akx, akq, 6, 6);

    const int nq = 36;
    double q2[nq];
    double q2obr[nq];
    double akq2[nq];
    double akq3[nq];
    double akx2[nq];
    memset(q2, 0, sizeof(double) * nq);
    memset(q2obr, 0, sizeof(double) * nq);
    memset(akq2, 0, sizeof(double) * nq);
    memset(akq3, 0, sizeof(double) * nq);
    memset(akx2, 0, sizeof(double) * nq);

    double fi1 = r423[2];
    double fi2 = r423[3];

    q2[0] = 1;
    q2[21] = 1;
    q2[28] = 1;
    q2[35] = 1;
    double tv = fi1 * fi1 + fi2 * fi2;
    q2[7] = fi1 / sqrt(tv);
    q2[8] = fi2 / sqrt(tv);
    q2[13] = fi2 / (tv);
    q2[14] = -fi1 / (tv);

    bpbt(q2, akq, akq2, 6, 6);

    /*covmtx["AA"]	= */ //akq2[0] = sqr(0.001);
    /*covmtx["AE"]	= */ //akq2[1] = 0;
    /*covmtx["AW"]	= */ //akq2[2] = 0;
    /*covmtx["AI"]	= */ //akq2[3] = 0;
    /*covmtx["AO"]	= */ //akq2[4] = 0;
    /*covmtx["AT"]	= */ //akq2[5] = 0;
    /*covmtx["EE"]	= */ //akq2[6] = sqr(0.0001);
    /*covmtx["EW"]	= */ //akq2[7] = 0;
    /*covmtx["EI"]	= */ //akq2[8] = 0;
    /*covmtx["EO"]	= */ //akq2[9] = 0;
    /*covmtx["ET"]	= */ //akq2[10]= 0;
    /*covmtx["WW"]	= */ //akq2[11]= sqr(0.1*deg2rad);
    /*covmtx["WI"]	= */ //akq2[12]= 0;
    /*covmtx["WO"]	= */ //akq2[13]= 0;
    /*covmtx["WT"]	= */ //akq2[14]= 0;
    /*covmtx["II"]	= */ //akq2[15]= sigmaIncliantion > 0 ? sqr(sigmaIncliantion) : akq2[15];
    /*covmtx["IO"]	= */ //akq2[16]= 0;
    /*covmtx["IT"]	= */ //akq2[17]= 0;
    /*covmtx["OO"]	= */ //akq2[18]= sqr(0.1*deg2rad);
    /*covmtx["OT"]	= */ //akq2[19]=0;
    /*covmtx["TT"]	= */ ///akq2[20]=1;
    /*covmtx["PP"]	= */ //sqrt(pow(akq2[0],3)/amu);

    obgau(6, q2, rab, q2obr);
    bpbt(q2obr, akq2, akq3, 6, 6);
    bpbt(r707, akq3, akx2, 6, 6);

    SetCovRv(akx2);

    return true;
}

bool CovMtx::CreateCovRnb(Vect6 pos, std::map<std::string, double>& covmap, std::map<std::string, double>& covmtx) const
{
    double akrnb[21];
    double akx[21];
    double x[6];

    FillVec(pos, x);
    FillCovMtx(covmap, akx);
    kx2krnb(x, akx, akrnb);

    covmtx["RR"] = akrnb[0];
    covmtx["RN"] = akrnb[1];
    covmtx["RB"] = akrnb[2];
    covmtx["RVR"] = akrnb[3];
    covmtx["RVN"] = akrnb[4];
    covmtx["RVB"] = akrnb[5];
    covmtx["NN"] = akrnb[6];
    covmtx["NB"] = akrnb[7];
    covmtx["NVR"] = akrnb[8];
    covmtx["NVN"] = akrnb[9];
    covmtx["NVB"] = akrnb[10];
    covmtx["BB"] = akrnb[11];
    covmtx["BVR"] = akrnb[12];
    covmtx["BVN"] = akrnb[13];
    covmtx["BVB"] = akrnb[14];
    covmtx["VRVR"] = akrnb[15];
    covmtx["VRVN"] = akrnb[16];
    covmtx["VRVB"] = akrnb[17];
    covmtx["VNVN"] = akrnb[18];
    covmtx["VNVB"] = akrnb[19];
    covmtx["VBVB"] = akrnb[20];

    return true;
}

std::map<std::string, double> CovMtx::GetCovRvMap() const
{
    std::map<std::string, double> ret;

    for (int i = 0; i < m_covdim; i++) {
        for (int j = i; j < m_covdim; j++) {
            ret[CovIndexToKey(i, j, m_covHasKb, m_covHasSp)] = m_covmtx(i, j);
        }
    }

    return ret;
}
