#ifndef COVMTX_H
#define COVMTX_H 

#include <myXML_Document.hpp>
#include "json_functions.h"
#include <map>
#include "Matrix.h"
#include "DVectors.h"
#include "Kepler.h"

class CovMtx
{
public:
    CovMtx();
    CovMtx(const DSymmMatrix& mtx, bool kb, bool sp);
    CovMtx(const double* akx);

    bool Load(myXML::Contain* parent, bool loadkb, bool loadsp);
    void load(const jsonval& parent);

    const DSymmMatrix& GetMtx() const { return m_covmtx; }
    void GetCovRv(DSymmMatrix& mtx, bool& kb, bool& sp) const;
    void SetCovRv(const DSymmMatrix& mtx, bool kb, bool sp);
    void SetCovRv(const double* akx);
    bool SetCovRv(const std::map<std::string, double>& covmap, bool loadkb, bool loadsp);
    bool SetCovRvActive(Vect6 pos, double sigmaIncliantion);

    std::map<std::string, double> GetCovRvMap() const;
    std::map<std::string, double> GetCovOrbMap(const Vect6& pos) const;
    std::map<std::string, double> GetCovRnbMap(const Vect6& pos) const;

    bool HasKb() const { return m_covHasKb; }
    bool HasSp() const { return m_covHasSp; }
    bool HasMtx() const { return m_covdim != 0; }
    int Dim() const { return m_covdim; }

protected:
    bool CreateCovOrb(Vect6 pos, std::map<std::string, double>& covmap, std::map<std::string, double>& covmtx) const;
    bool CreateCovRnb(Vect6 pos, std::map<std::string, double>& covmap, std::map<std::string, double>& covmtx) const;

    static std::string CovIndexToKey(int i, int j, bool kb, bool sp);
    static void FillVec(Vect6 pos, double* vec);
    static void FillCovMtx(std::map<std::string, double>& covmap, double* akx);
    static void FillCovMtx(const double* akx, std::map<std::string, double>& covmap);

    static bool TestOrbit(const Kepler& kepler);
    static int IsHyperbolic(const Vect6& U);

    DSymmMatrix m_covmtx;
    bool m_covHasKb;
    bool m_covHasSp;
    int m_covdim;
};

#endif
