#include "KepData.h"
#include "XMLFunctions.h"
#include "Consts.h"

void KepData::Init(const Kepler& kep)
{
    m_sma = kep.A;
    m_ecc = kep.GetEcc();
    m_incl = acos(kep.CosI);
    m_aop = kep.GetArgP();
    m_aol = kep.U;
    m_raan = kep.Omega;
    m_period = kep.GetPeriod();
    m_hApogee = kep.GetHA(nullptr);
    m_hPerigee = kep.GetHP(nullptr);
    m_rApogee = kep.GetRA();
    m_rPerigee = kep.GetRP();
}

bool KepData::Load(myXML::Contain* parent)
{
    LoadXML(parent, "SemimajorAxis", m_sma);
    LoadXML(parent, "Eccentricity", m_ecc);
    LoadXML(parent, "Inclination", m_incl);
    LoadXML(parent, "ArgOfPerigee", m_aop);
    LoadXML(parent, "ArgOfLatitude", m_aol);
    LoadXML(parent, "RAAN", m_raan);

    return true;
}

void KepData::load(const jsonval& parent)
{
    loadjson(parent, "a", m_sma);
    loadjson(parent, "e", m_ecc);
    loadjson(parent, "i", m_incl);
    loadjson(parent, "w", m_aop);
    loadjson(parent, "u", m_aol);
    loadjson(parent, "o", m_raan);

    m_sma /= 1000;
    m_incl *= deg2rad;
    m_aop *= deg2rad;
    m_aol *= deg2rad;
    m_raan *= deg2rad;
}

void KepData::ToXml(std::ostream& outs) const
{
    PutValue(outs, "Semiaxis", GetSemimajorAxis());
    PutValue(outs, "Inclination", GetInclination());
    PutValue(outs, "Eccentricity", GetEccentricity());
    PutValue(outs, "ArgOfPerigee", GetArgOfPerigee());
    PutValue(outs, "ArgOfLatitude", GetArgOfLatitude());
    PutValue(outs, "RAAN", GetRAAN());
    PutValue(outs, "Period", GetPeriod());
    PutValue(outs, "HApogee", GetHApogee());
    PutValue(outs, "HPerigee", GetHPerigee());
    PutValue(outs, "RApogee", GetRApogee());
    PutValue(outs, "RPerigee", GetRPerigee());
}
