#ifndef KEPLERTRJ_H
#define KEPLERTRJ_H

#include "ITraject.h"
#include "KeplPredict.h"

class KeplerTrj : public ITraject
{
public:
    // Конструктор
    KeplerTrj();
    // Деструктор
    virtual ~KeplerTrj()
    {
    };
    // Метод устанавливает значение вектора уточняемых параметров
    virtual bool Init(const DVector& u0);
    // Метод устанавливает время начальной точки траектории
    virtual void SetTime(const TimeJD& t)
    {
        InitTime = t;
    };
    // Метод возвращает время начальной точки траектории
    virtual const TimeJD& GetTime() const
    {
        return InitTime;
    }

    // Метод выполняет прогнозирование траектории на указанный момент времени t.
    // Параметр fr определяет систему отсчету, в которой должен быть возвращен
    // фазовый вектор f (т.е. координаты и скорости)
    virtual bool Predict(const TimeJD& t, const IFrame& fr, Vect6& f);
    //	bool PredictEx(const TimeJD &t, Vect6 &f);
    // Метод выполняет прогнозирование траектории на указанный момент времени t и
    // подсчет матрицы частных от фазового вектора по уточняемым параметрам.
    // Параметр fr определяет систему отсчету, в которой должен быть возвращен
    // фазовый вектор f (т.е. координаты и скорости) и матрица m.
    virtual bool PredictMatrix(const TimeJD& t, const IFrame& fr, Vect6& f, DMatrix& m);
    // Метод выполняет вычисление вектора уточняемых параметров u для указанного
    // момента времени t.
    virtual bool Move(const TimeJD& t, DVector& u);

    virtual IPredict* GetPredictorEx() { return &Predictor; }
    // Получение ссылки на экземпляр прогноза (для внешних настроек)
    KeplPredict* GetPredictor()
    {
        return &Predictor;
    };
    // Установка значений коэффициентов и признаков их уточнения

    virtual unsigned Size() const;
protected:
    // Класс численного прогноза уравнений в вариациях
    KeplPredict Predictor;
    // Время привязки начальных условий
    TimeJD InitTime;
    // Балл. коэффициент по умолчанию
    double Kb;
    // Коэффициент св. давления по умолчанию
    double Kappa;
    // Флаг уточнения Кb
    bool IsKbVar;
    // Флаг уточнения Кappa
    bool IsKappaVar;
};

///////////////////////////////////////////////////////////////////////////////
#endif
