#include "MagData.h"
#include "Consts.h"
#include "globals.h"

MagData::MagData() = default;

void MagData::override_stdmag(const double stdmag)
{
    m_stdmag = stdmag;
}

void MagData::SetOpticData(const double stdphase_deg, const double stdtrange_km, const double stdmag)
{
    m_hasData = true;
    m_stdphase_rad = stdphase_deg * deg2rad;
    m_stdrange_tkm = stdtrange_km * 1E-3;
    m_stdmag = stdmag;
}

void MagData::SetGeomData(const double rcs, const double albedo)
{
    m_hasGeomData = true;
    m_mag = 0;
    m_rcs = rcs;
    m_albedo = albedo;
}

void MagData::load(const jsonval& parent)
{
    loadjson(parent, "phase", m_stdphase_rad);
    loadjson(parent, "range", m_stdrange_tkm);
    loadjson(parent, "mag", m_stdmag);
    m_hasData = true;
}

double MagData::CalcA0g(const double stdmag, const double stdphase_rad, const double stdrange_tkm, const double sunMag)
{
    const double C = pow(10.0, ((sunMag - stdmag) / 2.5));
    const double F = 2 / 3.0 * sqr(Pi) * ((Pi - stdphase_rad) * cos(stdphase_rad) + sin(stdphase_rad)) * (1 - stdphase_rad / Pi);
    return C * sqr(stdrange_tkm * 1E6) / F;
}

double MagData::CalcRealMag(const double A0g, const double sunMag, const double phase_rad, const double range_tkm)
{
    const double F = 2 / 3.0 * sqr(Pi) * ((Pi - phase_rad) * cos(phase_rad) + sin(phase_rad)) * (1 - phase_rad / Pi);
    return sunMag - 2.5 * log10(F * A0g / sqr(range_tkm * 1E6));
}

void MagData::CalcMag(const double phase_rad, const double range_tkm)
{
    const double sunMag = -26.58;
    m_mag = CalcRealMag(CalcA0g(m_stdmag, m_stdphase_rad, m_stdrange_tkm, sunMag), sunMag, phase_rad, range_tkm);
    m_phase_rad = phase_rad;
    m_range_tkm = range_tkm;
}

void MagData::CalcGeomMag(const double phase_rad, const double range_tkm)
{
    const double sunMag = -26.58;
    const double F = 2 / 3.0 * sqr(Pi) * ((Pi - phase_rad) * cos(phase_rad) + sin(phase_rad)) * (1 - phase_rad / Pi);
    const double A0g = m_rcs * 0.15;
    m_mag = sunMag - 2.5 * log10(F * A0g / sqr(range_tkm * 1E6));
    m_phase_rad = phase_rad;
    m_range_tkm = range_tkm;
}
