#include "PrognozTrj.h"
#include "PhasePoint.h"
#include "METEFrame.h"
#include "ECIFrame.h"

PrognozTrj::PrognozTrj() : InitTime(0, 0.0)
{
    Kb = Kappa = 0.0;
    IsKbVar = IsKappaVar = false;
    m_periodChangePerRev = 0;
}

//-----------------------------------------------------------------------------
// Метод устанавливает значение вектора уточняемых параметров
//-----------------------------------------------------------------------------
bool PrognozTrj::Init(const DVector& u0)
{
    // Время должно быть установлено ранее с помощью SetTime() !!!!
    TAssert(InitTime != TimeJD(0, 0.0));

    // Проверка размерности переданного вектора
    unsigned n = 6;
    if (IsKbVar)
        n++;
    if (IsKappaVar)
        n++;
    TAssert(u0.Dim() == n);

    // Установка Kb и Карра в прогнозе
    if (IsKbVar)
        Kb = u0[6];

    if (IsKappaVar)
        Kappa = IsKbVar ? u0[7] : u0[6];

    PhasePoint6D ph;
    //	METEFrame j2000(InitTime);
    ECIFrame j2000(InitTime);
    //	J2000Frame j2000;

    Vect6 v;

    int i;
    for (i = 0; i < 6; i++)
        v[i] = u0[i];

    ph.Init(j2000, v, InitTime);

    Predictor.SetInitPoint(ph, m_periodChangePerRev);

    return true;
}

//-----------------------------------------------------------------------------
// Метод выполняет прогнозирование траектории на указанный момент времени t.
// Параметр fr определяет систему отсчету, в которой должен быть возвращен
// фазовый вектор f (т.е. координаты и скорости)
//-----------------------------------------------------------------------------
bool PrognozTrj::Predict(const TimeJD& t, const IFrame& fr, Vect6& f)
{
    if (!Predictor.Run(t))
        return false;

    PhasePoint6D pr(666);
    Predictor.GetDestPoint(pr);
    pr.Convert(fr);
    f = pr.CoordsVel;

    return true;
}

bool PrognozTrj::PredictA(const TimeJD& t, Vect6& f)
{
    if (!Predictor.Run(t))
        return false;

    Predictor.GetDestPointA(f);

    return true;
}

//-----------------------------------------------------------------------------
// Метод выполняет прогнозирование траектории на указанный момент времени t и
// подсчет матрицы частных от фазового вектора по уточняемым параметрам.
// Параметр fr определяет систему отсчету, в которой должен быть возвращен
// фазовый вектор f (т.е. координаты и скорости) и матрица m.
//-----------------------------------------------------------------------------
bool PrognozTrj::PredictMatrix(const TimeJD& t, const IFrame& fr, Vect6& f, DMatrix& m)
{
    if (!Predict(t, fr, f))
        return false;

    J2000Frame j2000;

    if (j2000.IsEqual(fr)) {
        //    Predictor.GetDestMatrix(m);
        //    m.Transpose();
        return true;
    }

    // Алгоритм с получением матрицы в произвольно СО пока не реализован
    TAssert(false);

    return false;
}

//-----------------------------------------------------------------------------
// Метод выполняет вычисление вектора уточняемых параметров u для указанного
// момента времени t.
//-----------------------------------------------------------------------------
bool PrognozTrj::Move(const TimeJD& t, DVector& u)
{
    if (!Predictor.Run(t))
        return false;

    unsigned n = 6;
    if (IsKbVar)
        n++;
    if (IsKappaVar)
        n++;

    u.Init(n);

    PhasePoint6D ph;

    Predictor.GetDestPoint(ph);
    int i;

    for (i = 0; i < 6; i++)
        u[i] = ph.CoordsVel[i];

    if (IsKbVar)
        u[6] = Kb;

    if (IsKappaVar) {
        if (IsKbVar)
            u[7] = Kappa;
        else
            u[6] = Kappa;
    }

    return true;
}

unsigned PrognozTrj::Size() const
{
    return 6;
}
