#ifndef PROPINFOBASE_H
#define PROPINFOBASE_H

#include <myXML_Document.hpp>
#include <map>
#include "AtmModel.h"
#include "EarthModel.h"
#include "json_functions.h"
#include "integratorcfg.h"

enum class proptypes
{
    ST_PROP = 0,
    KEPLER_PROP,
    PROGNOZ_PROP,
    SGP_PROP,
    UNKNOWN
};

class PropInfoBase
{
public:
    PropInfoBase()
    {
    };
    PropInfoBase(
        std::string lightModel,
        EarthModel earthModel,
        AtmModel atmModel,
        const bool sun,
        const bool moon,
        const IntegratorCfg& icfg,
        const proptypes type);

    bool Load(myXML::Contain* parent);
    void load(const jsonval& parent);
    bool IsSun() const { return m_sun; }
    bool IsMoon() const { return m_moon; }
    proptypes GetType() const { return m_type; }
    //bool is_prec_itrf() const { return m_prec_itrf; }
    const AtmModel& GetAtmModel() const { return m_atmModel; }
    const EarthModel& GetEarthModel() const { return m_earthModel; }
    std::string GetLightModel() const { return m_lightModel; }
    std::string GetAsXML() const;
    void ToXml(std::ostream& outs) const;
    const IntegratorCfg& get_integrator_cfg() const { return m_integrator_cfg; }
private:

    static std::string TypeConvert(proptypes value) ;
    static proptypes TypeConvert(const std::string& value);

    AtmModel m_atmModel;
    EarthModel m_earthModel;
    std::string m_lightModel;
    bool m_sun;
    bool m_moon;
    proptypes m_type;
    int m_id;
    IntegratorCfg m_integrator_cfg;
};

typedef std::map<int, PropInfoBase> PropInfoBases;
typedef PropInfoBase MotionModel;

#endif
