#ifndef PROPAGATOR_H
#define PROPAGATOR_H

#include "StateVector.h"
#include "TM.h"
#include "TrjNum.h"
#include "TrjNumVar.h"
#include "KeplerTrj.h"
#include "PrognozTrj.h"
#include "PropInfoBase.h"
#include "DigPredictVar.h"
#include "CovMtx.h"
#include "SGPTrj.h"

class Propagator
{
public:
    Propagator();
    Propagator(const SInitOrbit& initOrbit, bool variates);
    Propagator(ITraject* predictor);
    virtual bool Propagate(const TimeJD& jd, PhasePoint6D& ret);
    bool Propagate(const TimeJD& jd, SInitOrbit& res) const;
    bool Propagate(const TimeJD& jd, const IFrame& frame, PhasePoint6D& ret) const;
    bool Propagate(const TimeJD& jd, PhasePoint6D& ret, DSymmMatrix& covm) const;
    bool SetKeplerPropagation(TimeJD firstPointTime);
    bool Init(const PropInfoBase& propInfo, const SInitOrbit& initOrbit, bool variates = false);
    bool Init(const SInitOrbit& initOrbit, bool variates = false);
    static void InitStPredictor(TrjNum& trj, const SInitOrbit& initOrbit, const PropInfoBase& propInfo);
    static void InitStvPredictor(TrjNumVar& trj, const SInitOrbit& initOrbit, const PropInfoBase& propInfo);
    static void InitKepPredictor(KeplerTrj& trj, const SInitOrbit& initOrbit);
    static void InitPrognozPredictor(PrognozTrj& trj, const SInitOrbit& initOrbit);
    static void InitSGPPredictor(SGPTrj& trj, const SInitOrbit& initOrbit);
    static bool Propagate(ITraject& predictor, const TimeJD& jd, PhasePoint6D& ret);
    static bool Propagate(ITraject& predictor, const TimeJD& jd, const IFrame& frame, PhasePoint6D& ret);
    const SInitOrbit& GetInitOrbit() const { return m_initOrbit; }
    const ITraject* GetPredictor() const { return m_predictor; }
    ITraject* GetPredictor() { return m_predictor; }
    static void SetMotionModel(const PropInfoBase& PropInfoBase, DigPredict& predictor);
    static void SetMotionModel(const PropInfoBase& PropInfoBase, DigPredictVar& predictor);
    static void SetMotionModel(const PropInfoBase& PropInfoBase, IPredict& predictor, MotionMode& mode);
    static bool Propagate(ITraject& predictor, const TimeJD& jd, const DSymmMatrix& srccov, PhasePoint6D& ret, DSymmMatrix& dstcov);
    bool has_covmtx() const;
protected:
    TrjNumVar m_stvPredictor;
    TrjNum m_stPredictor;
    KeplerTrj m_kpPredictor;
    PrognozTrj m_prPredictor;
    ITraject* m_predictor;
    SInitOrbit m_initOrbit;
    SGPTrj m_sgpPredictor;
    bool m_variates;
    bool m_has_orbit;
};

#endif
