#include "Site.h"

#include "XMLFunctions.h"
#include "json_functions.h"
#include "GeoCoords.h"
#include "Consts.h"
#include "BNOAstroBase.h"

class SXYZCoordinates {
public:
    bool Load(myXML::Contain* parent)
    {
        LoadXML(parent, "x", m_x);
        LoadXML(parent, "y", m_y);
        LoadXML(parent, "z", m_z);
        return true;
    }

    double GetX() const { return m_x; }
    double GetY() const { return m_y; }
    double GetZ() const { return m_z; }
private:
    double m_x{0};
    double m_y{0};
    double m_z{0};
};

class SGeoCoordinates {
public:
    bool Load(myXML::Contain* parent)
    {
        LoadXML(parent, "Latitude", m_lat);
        LoadXML(parent, "Longitude", m_lon);
        LoadXML(parent, "Altitude", m_alt);
        return true;
    }

    void load(const jsonval& parent)
    {
        loadjson(parent, "lat", m_lat);
        loadjson(parent, "lon", m_lon);
        loadjson(parent, "alt", m_alt);

        m_lat *= deg2rad;
        m_lon *= deg2rad;
        m_alt /= 1E6;
    }

    double GetLat() const { return m_lat; }
    double GetLon() const { return m_lon; }
    double GetAlt() const { return m_alt; }
    std::string GetEllipsoid() const { return m_ellipsoid; }

private:
    double m_lat{0};
    double m_lon{0};
    double m_alt{0};
    std::string m_ellipsoid;
};

void Site::load(const jsonval& parent)
{
    loadjson(parent, "id", m_id, false);
    loadjson(parent, "abcor", m_needAberrationCorrection);
    loadjson(parent, "rarms", m_sigmaAlpha);
    loadjson(parent, "decrms", m_sigmaDelta);

    if (parent.FindMember("xyz") != parent.MemberEnd()) {

    }
    else {
        SGeoCoordinates sgc;
        loadjson(parent, "geo", sgc);

        double EarthRadius, EarthOblatness;
        ((BNOAstroBase*)GetAstroBase())->GetWGS84Geoid(EarthRadius, EarthOblatness);

        GeoCoordsPrec converter(EarthRadius, EarthOblatness);
        converter.Phi = sgc.GetLat();
        converter.Lam = sgc.GetLon();
        converter.H = sgc.GetAlt();
        converter.GetXYZ(m_coordinates);

        m_lat_deg = sgc.GetLat() * rad2deg;
        m_lon_deg = sgc.GetLon() * rad2deg;
        m_alt_m = sgc.GetAlt() * 1E6;
    }
}

bool Site::Load(myXML::Contain* parent)
{
    std::map<std::string, myXML::Contain*> items;
    GetChildren(parent, items);

    LoadXML(items, "Id", m_id, false);
    LoadXML(items, "NeedAberCor", m_needAberrationCorrection);
    LoadXML(items, "SigmaAlpha", m_sigmaAlpha);
    LoadXML(items, "SigmaDelta", m_sigmaDelta);

    if (GetMapItem(items, std::string("xyz"))) {

        SXYZCoordinates sgc;
        LoadXML(items, "xyz", sgc);
        m_coordinates = Vect3(sgc.GetX(), sgc.GetY(), sgc.GetZ());

    }
    else if (GetMapItem(items, std::string("GeoCoordinates"))) {

        SGeoCoordinates sgc;
        LoadXML(items, "GeoCoordinates", sgc);

        double EarthRadius, EarthOblatness;
        ((BNOAstroBase*)GetAstroBase())->GetWGS84Geoid(EarthRadius, EarthOblatness);

        GeoCoordsPrec converter(EarthRadius, EarthOblatness);
        converter.Phi = sgc.GetLat();
        converter.Lam = sgc.GetLon();
        converter.H = sgc.GetAlt();
        converter.GetXYZ(m_coordinates);
    }
    else {

    }

    return true;
}
