#ifndef STATEVECTOR_H
#define STATEVECTOR_H 

#include <myXML_Document.hpp>
#include "DVectors.h"
#include "TM.h"
#include "PhasePoint.h"
#include "KepData.h"
#include "TleData.h"
#include <map>
#include "PropInfoBase.h"
#include "CovMtx.h"
#include "uncert.h"
#include "OrbitId.h"
#include "integratorcfg.h"
#include "json_functions_bno.h"

enum OrbitTypes
{
    ORBSRCTYPE_TLE = 0,
    ORBSRCTYPE_OPT,
    ORBSRCTYPE_UNK,
};

class SInitOrbit
{
public:
    SInitOrbit(Vect6 pos, const TimeJD& date, double ballc, double solarpc, double periodChangePerRev, PropInfoBase propagator, CovMtx covmtx, int rev = 0, TOrbitId baseid = "");
    SInitOrbit();
    virtual ~SInitOrbit();

    bool Load(myXML::Contain* con);
    void load(const jsonval& parent);
    bool LoadNewStyle(myXML::Contain* con);
    Vect3 GetCoortinates() const { return m_coordinates; }
    Vect3 GetVelocities() const { return m_velocities; }
    Vect3 GetCoortinatesGSC() const;
    Vect3 GetVelocitiesGSC() const;
    Vect3 GetCoortinatesJ2000() const;
    Vect3 GetVelocitiesJ2000() const;
    Vect3 GetCoortinatesMETE() const;
    Vect3 GetVelocitiesMETE() const;
    Vect3 GetCoortinatesECI() const;
    Vect3 GetVelocitiesECI() const;

    const TimeJD& GetDate() const { return m_date; }

    double GetS() const { return m_ballisticCoefficient; }
    double GetBc() const { return m_ballisticCoefficient; }
    double GetSpc() const { return m_solarPresureCoefficient; }
    double GetKappa() const { return m_solarPresureCoefficient; }
    double GetK() const { return m_solarPresureCoefficient; }
    double GetBallisticCoefficient() const { return m_ballisticCoefficient; }
    double GetSolarPresureCoefficient() const { return m_solarPresureCoefficient; }
    double GetPeriodChangePerRev() const { return m_periodChangePerRev; }
    double GetInterval() const { return m_interval; }
    TOrbitId GetId() const { return m_id; }
    TOrbitId GetBaseId() const { return m_baseid; }
    PhasePoint6D GetPhasePointGSC() const;
    PhasePoint6D GetPhasePointJ2000() const;
    PhasePoint6D GetPhasePointMETE() const;
    PhasePoint6D GetPhasePointECI() const;
    int GetRev() const { return m_rev; }

    const KepData& GetKeplerData() const { return m_kepData; }
    const TleData& GetTleData() const { return m_tleData; }
    const CovMtx& GetCovMtx() const { return m_covmtx; }
    const Uncert& getUncert() const { return m_uncert; }
    std::map<std::string, double> GetCovRvMap() const;
    std::map<std::string, double> GetCovOrbMap() const;
    std::map<std::string, double> GetCovRnbMap() const;
    void SetCovMtx(const CovMtx& mtx) { m_covmtx = mtx; }
    void SetBaseOrbitId(TOrbitId baseid) { m_baseid = baseid; }
    const PropInfoBase& GetPropagatorData() const { return m_propInfo; }
    const PropInfoBase& GetModel() const { return m_propInfo; }
    const PropInfoBase& GetMotionModel() const { return m_propInfo; }
    double CalcLongitude() const;
    void SetMotionModel(const MotionModel& motionModel);
    void SetDate(const TimeJD& date);
    void SetId(TOrbitId id) { m_id = id; }
protected:
    static int ParseSrcType(const TOrbitId& id);
    void CreateFromKepler(const KepData& kd, const std::string& frame);
    void CreateFromPos(Vect6 dst, const std::string& frame);
    static bool CheckFrame(const std::string& frame);

    Vect3 m_coordinates;
    Vect3 m_velocities;
    TimeJD m_date;
    double m_ballisticCoefficient;
    double m_solarPresureCoefficient;
    double m_periodChangePerRev;
    double m_interval;
    KepData m_kepData;
    TleData m_tleData;
    CovMtx m_covmtx;
    Uncert m_uncert;
    int m_rev;
    TOrbitId m_id;
    TOrbitId m_baseid;
    PropInfoBase m_propInfo;
    int rev_;
};

typedef std::vector<SInitOrbit> SInitOrbits;

#endif
