#ifndef __TASKINIT_H_
#define __TASKINIT_H_

#include <string>
#include <sstream>
#include <fstream>
#include "BNOAstroBase.h"
#include "XMLFunctions.h"

static bool TestBreak(const std::string& tFile, bool lastCall = false)
{
    if (!tFile.empty()) {
        const std::ifstream ifile(tFile.c_str());
        if (ifile) {
            if (lastCall) {
                remove(tFile.c_str());
            }
            return true;
        }
        else {
            return false;
        }
    }
    else {
        return false;
    }
}

static void InitDataA(BNOAstroBase& ab, myXML::Document& doc, bool noAtm = false, bool loadPlanets = false)
{
    std::string configFile;
    LoadXML(doc.GetBody().GetRoot(), "Config", configFile);

    if (configFile == "%ASTROBASE%") {
        const char* env_astro = std::getenv("ASTROBASE");
        if (!env_astro) {
            throw (Exp() << "no env variable ASTROBASE found");
        }
        configFile = std::string(env_astro);
    }

    ab.SetCfgFile(configFile.c_str());

    if (ab.Init(TimeGD(1, 1, 1957, 0, 0, 0, 0), TimeGD(1, 1, 2500, 0, 0, 0, 0), noAtm, loadPlanets)) {
        SetAstroBase(ab);
    }
    else {
        throw (Exp() << "Failed to init astrobase:" << configFile);
    }
}

static void InitDataA(BNOAstroBase& ab, std::string configFile, bool noAtm = false, bool loadPlanets = false)
{
    if (configFile == "%ASTROBASE%") {
        const char* env_astro = std::getenv("ASTROBASE");
        if (!env_astro) {
            throw (Exp() << "no env variable ASTROBASE found");
        }
        configFile = std::string(env_astro);
    }

    ab.SetCfgFile(configFile.c_str());

    if (ab.Init(TimeGD(1, 1, 1957, 0, 0, 0, 0), TimeGD(1, 1, 2500, 0, 0, 0, 0), noAtm, loadPlanets)) {
        SetAstroBase(ab);
    }
    else {
        throw (Exp() << "Failed to init astrobase:" << configFile);
    }
}

static void InitData(const std::stringstream& data, BNOAstroBase& ab, myXML::Document& doc, bool noAtm = false, bool loadPlanets = false)
{
    LoadXmlString(data.str(), doc);
    InitDataA(ab, doc, noAtm, loadPlanets);
}

static void InitData(const std::string& file, BNOAstroBase& ab, myXML::Document& doc, bool noAtm = false, bool loadPlanets = false)
{
    LoadXmlFile(file, doc);
    InitDataA(ab, doc, noAtm, loadPlanets);
}

clock_t begin, end;

inline double diffclock(clock_t clock1, clock_t clock2)
{
    double diffticks = clock1 - clock2;
    double diffms = (diffticks) / CLOCKS_PER_SEC;
    return diffms;
}

inline void Tic(bool timing, const std::string& message)
{
    if (timing && !message.empty()) {
        end = clock();
        double dif = diffclock(end, begin);
        std::cerr << message << ": " << dif << std::endl;
        begin = clock();
    }
    else if (timing) {
        begin = clock();
    }
}

inline void Tic(bool timing)
{
    Tic(timing, "");
}


#endif
