#pragma once

#include <charconv>
#include <array>
#include <iostream>
#include <sstream>
#include <fstream>
#include <vector>
#include <string>
#include <map>
#include <list>
#include <stdio.h>
#include <iomanip>
#include "Converter.h"
#include "MException.h"
#include <algorithm>

#define SPACES " \t\r\n"

static std::string ToStr(const int arg, const int leading_zeros)
{
    std::ostringstream ss;
    if (leading_zeros) {
        ss << std::setw(leading_zeros) << std::setfill('0');
    }
    ss << arg;
    return ss.str();
}

static std::string ToStr(const size_t arg, const int leading_zeros)
{
    std::ostringstream ss;
    if (leading_zeros) {
        ss << std::setw(leading_zeros) << std::setfill('0');
    }
    ss << arg;
    return ss.str();
}

template <typename T>
void hangle_to_chars_error(const std::to_chars_result& res, const T& arg)
{
    if (res.ec == std::errc::invalid_argument) {
        throw Exp() << "failed to convert value [" << arg << "] to string, invalid argument";
    }

    if (res.ec == std::errc::result_out_of_range) {
        throw Exp() << "failed to convert value [" << arg << "] to string, out of range";
    }

    throw Exp() << "failed to convert value [" << arg << "] to string";
}

static std::string ToStr(const double value, const int precision = 0)
{
    std::ostringstream ss;

    if (precision) {
        ss << std::setprecision(precision);
    }
    else {
        ss << std::setprecision(std::numeric_limits<double>::digits10 + 2);
    }

    ss << value;
    return ss.str();
}

template <typename T>
static std::string ToStr(const T arg)
{
    std::array<char, 32> buf;

    const auto res = std::to_chars(buf.data(), buf.data() + buf.size(), arg);

    if (res.ec == std::errc()) {
        return std::string(buf.data(), res.ptr - buf.data());
    }

    hangle_to_chars_error(res, arg);

    return "";
}

static std::string ToStr(const bool arg)
{
    std::array<char, 32> buf;

    const auto res = std::to_chars(buf.data(), buf.data() + buf.size(), static_cast<int>(arg));

    if (res.ec == std::errc()) {
        return std::string(buf.data(), res.ptr - buf.data());
    }

    hangle_to_chars_error(res, arg);

    return "";
}

// static std::string ToStr(const size_t arg)
// {
//     std::array<char, 32> buf;
//
//     const auto res = std::to_chars(buf.data(), buf.data() + buf.size(), arg);
//
//     if (res.ec == std::errc()) {
//         return std::string(buf.data(), res.ptr - buf.data());
//     }
//
//     hangle_to_chars_error(res, arg);
//
//     return "";
// }
//
// static std::string ToStr(const double arg)
// {
//     std::array<char, 32> buf;
//
//     const auto res = std::to_chars(buf.data(), buf.data() + buf.size(), arg);
//
//     if (res.ec == std::errc()) {
//         return std::string(buf.data(), res.ptr - buf.data());
//     }
//
//     hangle_to_chars_error(res, arg);
//
//     return "";
// }

static void string_replace(std::string& str, const std::string& from, const std::string& to)
{
    if (from.empty())
        return;
    size_t start_pos = 0;
    while ((start_pos = str.find(from, start_pos)) != std::string::npos) {
        str.replace(start_pos, from.length(), to);
        start_pos += to.length(); // In case 'to' contains 'from', like replacing 'x' with 'yx'
    }
}

static std::string trim_right(const std::string& s, const std::string& t = SPACES)
{
    std::string d(s);
    const std::string::size_type i(d.find_last_not_of(t));
    if (i == std::string::npos) {
        return "";
    }
    else {
        return d.erase(d.find_last_not_of(t) + 1);
    }
}

static std::string trim_left(const std::string& s, const std::string& t = SPACES)
{
    std::string d(s);
    return d.erase(0, s.find_first_not_of(t));
}

template <typename T, typename T2>
bool GetMapItem(const std::map<T, T2>& map, const T& key, T2& ret)
{
    typename std::map<T, T2>::const_iterator it = map.find(key);
    if (it != map.end()) {
        ret = it->second;
        return true;
    }
    else {
        return false;
    }
}

template <typename T, typename T2>
bool GetMapItem(std::map<T, T2>& map, const T& key, T2*& ret)
{
    typename std::map<T, T2>::iterator it = map.find(key);
    if (it != map.end()) {
        ret = &it->second;
        return true;
    }
    else {
        ret = 0;
        return false;
    }
}

template <typename T, typename T2>
bool GetMapItem(const std::map<T, T2>& map, const T& key, const T2*& ret)
{
    typename std::map<T, T2>::const_iterator it = map.find(key);
    if (it != map.end()) {
        ret = &it->second;
        return true;
    }
    else {
        ret = 0;
        return false;
    }
}

template <typename T, typename T2>
bool GetMapItem(const std::map<T, T2>& map, const T& key)
{
    return map.find(key) != map.end();
}

static std::string trim(const std::string& s, const std::string& t = SPACES)
{
    return trim_left(trim_right(s, t), t);
}

template <typename ContainerT>
void splitstrf(const std::string& str, ContainerT& tokens, const std::string& delimiters = " ", const bool trimEmpty = false)
{
    std::string::size_type lastPos = 0;
    while (true) {
        std::string::size_type pos = str.find_first_of(delimiters, lastPos);
        if (pos == std::string::npos) {
            pos = str.length();

            if (pos != lastPos || !trimEmpty)
                tokens.push_back(typename ContainerT::value_type(str.data() + lastPos, (typename ContainerT::value_type::size_type)pos - lastPos));

            break;
        }
        else {
            if (pos != lastPos || !trimEmpty)
                tokens.push_back(typename ContainerT::value_type(str.data() + lastPos, (typename ContainerT::value_type::size_type)pos - lastPos));
        }

        lastPos = pos + 1;
    }
};

static std::vector<std::string> splitstr(const std::string& str_i, const char* const delims_i)
{
    std::vector<std::string> ret_vect;
    if (str_i.empty()) return ret_vect;

    splitstrf(str_i, ret_vect, std::string(delims_i));

    return ret_vect;
}

static bool ReadFile(const std::string& path, std::vector<std::string>& file)
{
    std::string line;
    file.clear();
    std::ifstream infile(path.c_str(), std::ios::in);
    if (infile.is_open()) {
        while (getline(infile, line, '\n')) {
            file.push_back(line);
        }
        infile.close();
        return true;
    }
    else {
        return false;
    }
}

static bool ReadFile(const std::string& path, std::string& str)
{
    FILE* fp = fopen(path.c_str(), "rb");
    if (!fp) {
        return false;
    }
    fseek(fp, 0, SEEK_END); //go to end
    const long len = ftell(fp); //get position at end (length)
    fseek(fp, 0, SEEK_SET); //go to beg.
    auto* buf = new char[len + 1]; //malloc buffer
    buf[len] = 0;
    size_t res = fread(buf, len, 1, fp); //read into buffer
    fclose(fp);
    str = std::string(buf);
    delete[] buf;
    return res;
}

static bool WriteFile(const std::string& path, const std::string& str)
{
    std::string line;
    std::ofstream file(path.c_str(), std::ios::out);

    if (file.is_open()) {
        file << str.c_str();
        file.close();
        return true;
    }
    else {
        std::cerr << "Error while writing to file [" + path + "]";
        return false;
    }
}

template <typename T, typename Tid>
void GetAsMap(const std::vector<T>& vec, std::map<Tid, T>& res)
{
    for (typename std::vector<T>::const_iterator it = vec.begin(); it != vec.end(); ++it) {
        res[(*it).GetId()] = *it;
    }
}

static void checkBounds(int value, const int* min, const int* max, bool strict = false)
{
    if (!strict && min && value < *min || strict && min && value <= *min) {
        throw Exp() << "value [" + ToStr(value) + "] less than " + ToStr(*min);
    }

    if (!strict && max && value > *max || strict && max && value >= *max) {
        throw Exp() << "value [" + ToStr(value) + "] greater than " + ToStr(*max);
    }
}

static void checkBounds(double value, const double* min, const double* max, bool strict = false)
{
    if (!strict && min && value < *min || strict && min && value <= *min) {
        throw Exp() << "value [" + ToStr(value) + "] less than " + ToStr(*min);
    }

    if (!strict && max && value > *max || strict && max && value >= *max) {
        throw Exp() << "value [" + ToStr(value) + "] greater than " + ToStr(*max);
    }
}

static void parseStrA(const std::string& data, int pos1, int pos2, int& ret, int* min, int* max, bool strict)
{
    std::string part;
    if (pos1 == -1 && pos2 == -1) {
        part = data;
    }
    else {
        part = pos2 >= 0 ? data.substr(pos1, pos2) : data.substr(pos1);
    }

    if (!Converter::ToNumber(part, ret)) {
        const std::string errstr = pos2 ? (ToStr(pos1) + ":" + ToStr(pos2)) : (ToStr(pos1) + ":-");
        throw Exp() << "bad substr format in line [" + data + "] at position" << errstr << " " << part;
    }

    if (min || max) {
        try {
            checkBounds(ret, min, max, strict);
        }
        catch (Exp& e) {
            const std::string errstr = pos2 ? (ToStr(pos1) + ":" + ToStr(pos2)) : (ToStr(pos1) + ":-");
            throw Exp() << "bad substr format in line [" + data + "] at position" << errstr << " " << part << ". " << e.what();
        }
    }
}

static void parseStrA(const std::string& data, int pos1, int pos2, double& ret, double* min, double* max, bool strict)
{
    std::string part;
    if (pos1 == -1 && pos2 == -1) {
        part = data;
    }
    else {
        part = pos2 >= 0 ? data.substr(pos1, pos2) : data.substr(pos1);
    }

    if (!Converter::ToNumber(part, ret)) {
        const std::string errstr = pos2 ? (ToStr(pos1) + ":" + ToStr(pos2)) : (ToStr(pos1) + ":-");
        throw (Exp() << "bad substr format in line [" + data + "] at position" << errstr << " " << part);
    }

    if (min || max) {
        try {
            checkBounds(ret, (double*)min, (double*)max, strict);
        }
        catch (Exp& e) {
            const std::string errstr = pos2 ? (ToStr(pos1) + ":" + ToStr(pos2)) : (ToStr(pos1) + ":-");
            throw Exp() << "bad substr format in line [" + data + "] at position" << errstr << " " << part << ". " << e.what();
        }
    }
}

static void parseStr(const std::string& data, int pos1, int pos2, int& ret, int min, int max, bool strict = false)
{
    parseStrA(data, pos1, pos2, ret, &min, &max, strict);
}

static void parseStr(const std::string& data, int pos1, int pos2, double& ret, double min, double max, bool strict = false)
{
    parseStrA(data, pos1, pos2, ret, &min, &max, strict);
}

static void parseStr(const std::string& data, int pos1, int pos2, int& ret)
{
    parseStrA(data, pos1, pos2, ret, nullptr, nullptr, false);
}

static void parseStr(const std::string& data, int pos1, int pos2, double& ret)
{
    parseStrA(data, pos1, pos2, ret, nullptr, nullptr, false);
}

static void parseStr(const std::string& data, int& ret, int min, int max, bool strict = false)
{
    parseStrA(data, -1, -1, ret, &min, &max, false);
}

static void parseStr(const std::string& data, double& ret, double min, double max, bool strict = false)
{
    parseStrA(data, -1, -1, ret, &min, &max, false);
}

static void parseStr(const std::string& data, int& ret)
{
    parseStrA(data, -1, -1, ret, nullptr, nullptr, false);
}

static void parseStr(const std::string& data, double& ret)
{
    parseStrA(data, -1, -1, ret, nullptr, nullptr, false);
}

template <typename T, typename T2>
std::vector<T> map_keys(const std::map<T, T2>& map)
{
    std::vector<T> result;
    result.reserve(map.size());

    for (const auto& it : map) {
        result.push_back(it.first);
    }

    return result;
}

template <typename T>
std::string implode(std::string delim, const std::vector<T>& vec)
{
    std::ostringstream ss;

    for (typename std::vector<T>::const_iterator it = vec.begin(); it != vec.end(); ++it) {
        ss << *it;
        if (it != vec.end() - 1) {
            ss << delim;
        }
    }

    return ss.str();
}


template <typename T>
bool in_array(const std::vector<T>& src, T subject)
{
    return std::find(std::begin(src), std::end(src), subject) != std::end(src);
}

inline bool substring_between(
    const std::string& s,
    const std::string& start_delim,
    const std::string& stop_delim,
    std::string& result)
{
    const auto first_delim_pos = s.find(start_delim);

    if (first_delim_pos == std::string::npos) {
        return false;
    }

    const auto end_pos_of_first_delim = first_delim_pos + start_delim.length();

    if (end_pos_of_first_delim == std::string::npos) {
        return false;
    }

    unsigned last_delim_pos = s.find_first_of(stop_delim, end_pos_of_first_delim);

    if (last_delim_pos == std::string::npos) {
        return false;
    }

    result = s.substr(end_pos_of_first_delim, last_delim_pos - end_pos_of_first_delim);
    return true;
}
