#include "TleData.h"
#include "XMLFunctions.h"
#include "Consts.h"

void TleData::Init(const TleData& data)
{
    m_ndot = data.GetNDot();
    m_nddot = data.GetNDDot();
    m_bstar = data.GetBStar();
    m_inclination = data.GetInclination();
    m_raan = data.GetRAAN();
    m_eccentricity = data.GetEccentricity();
    m_argofperigee = data.GetArgOfPerigee();
    m_meananomaly = data.GetMeanAnomaly();
    m_meanmotion = data.GetMeanMotion();
    m_solution = data.GetSolution();
    m_rev = data.GetRev();
}

bool TleData::Load(myXML::Contain* parent)
{
    std::map<std::string, myXML::Contain*> items;
    GetChildren(parent, items);

    LoadXML(items, "NDOT", m_ndot);
    LoadXML(items, "NDDOT", m_nddot);
    LoadXML(items, "ECCENTRICITY", m_eccentricity);
    LoadXML(items, "INCLINATION", m_inclination);
    LoadXML(items, "ARGOFPERIGEE", m_argofperigee);
    LoadXML(items, "RAAN", m_raan);
    LoadXML(items, "MEANANOMALY", m_meananomaly);
    LoadXML(items, "MEANMOTION", m_meanmotion);
    LoadXML(items, "BSTAR", m_bstar);
    LoadXML(items, "SOLUTION", m_solution);

    return true;
}

TLESet TleData::CreateTleSet(const TimeJD& date) const
{
    TLESet tle;
    TimeTLE tt = date;

    tle.EpochYr = tt.Year;
    tle.EpochDays = tt.Day;

    tle.No = m_meanmotion * 2.0 * Pi / 1440.0;
    tle.Ecco = m_eccentricity;
    tle.Omegao = m_raan * deg2rad;
    tle.Inclo = m_inclination * deg2rad;
    tle.Argpo = m_argofperigee * deg2rad;
    tle.Mo = m_meananomaly * deg2rad;

    tle.BStar = m_bstar;
    tle.NDot = m_ndot;
    tle.NDDot = m_nddot;
    tle.Satnum = 0;
    tle.ElementNum = m_solution;
    tle.Set();

    return tle;
}
