#pragma once

#include <myXML_Document.hpp>
#include "MException.h"
#include <vector>
#include <string>
#include "TM.h"
#include "TextFunctions.h"
#include "TimeFunctions.h"
#include <limits>

void GetChildren(myXML::Contain* parent, std::map<std::string, myXML::Contain*>& items);
void GetChildren(const std::map<std::string, myXML::Contain*>& parent, std::map<std::string, myXML::Contain*>& items);
void GenerateEmergeXML(const std::string& msg, const std::string& outputfile);
bool FindFirstContainer(myXML::Contain* parent, const std::string& name, myXML::Contain*& ret);
bool FindXMLContainer(const std::map<std::string, myXML::Contain*>& items, const std::string& name, myXML::Contain*& result);
bool FindXMLContainer(myXML::Contain* parent, const std::string& name, myXML::Contain*& result);

template <typename T>
void ReadXML(T storage, TimeGD& result);
template <typename T>
void ReadXML(T storage, double& result);
template <typename T>
void ReadXML(T storage, int& result);
template <typename T>
void ReadXML(T storage, bool& result);
template <typename T>
void ReadXML(T storage, std::string& result);
template <typename T, typename T2>
void ReadXML(T storage, T2& result);
template <typename T, typename T2>
void ReadXML(T storage, std::string subname, std::vector<T2>& result);
template <typename T, typename T2, typename T3>
void ReadXML(T storage, std::string subname, std::map<T2, T3>& result);

template <typename T>
void ReadXMLVector(myXML::Contain* con, std::string subname, std::vector<T>& result)
{
    myXML::ChildIter it = con->BeginChildIter();
    while (it != con->EndChildIter()) {
        if ((*it)->GetType() == myXML::Document::Container && (*it)->Get() == subname) {
            T item;
            item.Load((myXML::Contain*)(*it));
            result.push_back(item);
        }
        ++it;
    }
}

template <typename T>
void ReadXMLVectorSingle(myXML::Contain* con, std::string subname, std::vector<T>& result)
{
    myXML::ChildIter it = con->BeginChildIter();
    while (it != con->EndChildIter()) {
        if ((*it)->GetType() == myXML::Document::Container && (*it)->Get() == subname) {
            T item;
            ReadXML((myXML::Contain*)(*it), item);
            result.push_back(item);
        }
        ++it;
    }
}

template <typename T, typename T1>
void ReadXMLMap(myXML::Contain* con, std::string subname, std::map<T1, T>& result)
{
    myXML::ChildIter it = con->BeginChildIter();
    while (it != con->EndChildIter()) {
        if ((*it)->GetType() == myXML::Document::Container && (*it)->Get() == subname) {
            T item;
            item.Load((myXML::Contain*)(*it));
            result.insert(std::pair<T1, T>(item.GetId(), item));
        }
        ++it;
    }
}

template <typename T>
void ReadXMLClass(myXML::Contain* con, T& result)
{
    result.Load(con);
}

static void ReadXMLString(myXML::Contain* con, std::string& result)
{
    myXML::ChildIter begin = con->BeginChildIter();
    myXML::ChildIter end = con->EndChildIter();

    if (begin != end && (*begin)->GetType() == myXML::Document::Text) {
        result = (*begin)->Get();
    }
    else {
        throw (Exp() << ("Value '" + con->Get() + "' not found"));
    }
}

static void ReadXMLDate(myXML::Contain* con, TimeGD& result)
{
    myXML::ChildIter begin = con->BeginChildIter();
    myXML::ChildIter end = con->EndChildIter();

    if (begin != end && (*begin)->GetType() == myXML::Document::Text) {
        std::string str = (*begin)->Get();
        if (!StrToDate(str, result)) {
            throw (Exp() << "Bad value format '" << str << "'" << " in container '" << con->Get() << "'");
        }
    }
    else {
        throw (Exp() << "Value not found in container '" << con->Get() << "'");
    }
}

static void ReadXMLDate(myXML::Contain* con, TimeJD& result)
{
    TimeGD gd;
    ReadXMLDate(con, gd);
    result = gd;
}

template <typename T>
void ReadXMLNumeric(myXML::Contain* con, T& result)
{
    myXML::ChildIter begin = con->BeginChildIter();
    myXML::ChildIter end = con->EndChildIter();

    if (begin != end && (*begin)->GetType() == myXML::Document::Text) {
        std::string str = (*begin)->Get();
        if (!Converter::ToNumber(str, result)) {
            throw (Exp() << "Bad value format '" << str << "'" << " in container '" << con->Get() << "'");
        }
    }
    else {
        throw (Exp() << "Value not found in container '" << con->Get() << "'");
    }
}

template <typename T1>
std::string CreateExceptionCNF(T1 storage, std::string path)
{
    std::map<std::string, myXML::Contain*> children;
    GetChildren(storage, children);

    std::stringstream exp;
    exp << "Container '" << path << "' not found in list:\n";
    for (std::map<std::string, myXML::Contain*>::const_iterator it = children.begin(); it != children.end(); ++it) {
        exp << "'" << it->first << "'\n";
    }
    return exp.str();
}

template <typename T1, typename T2>
void LoadXMLUnit(T1 storage, std::string path, T2& result, bool exist, bool* loaded)
{
    if (loaded) {
        *loaded = false;
    }

    myXML::Contain* con;
    if (FindXMLContainer(storage, path, con)) {
        ReadXML(con, result);
        if (loaded) {
            *loaded = true;
        }
    }
    else if (exist) {
        throw (Exp() << CreateExceptionCNF(storage, path));
    }
}

template <typename T1, typename T2>
void LoadXMLUnitVector(T1 storage, std::string name, std::string subname, T2& result, bool exist, bool* loaded)
{
    if (loaded) {
        *loaded = false;
    }

    myXML::Contain* con;
    if (FindXMLContainer(storage, name, con)) {
        ReadXML(con, subname, result);
        if (loaded) {
            *loaded = true;
        }
    }
    else if (exist) {
        throw (Exp() << CreateExceptionCNF(storage, name));
    }
}

template <typename T1, typename T2>
void LoadXMLUnitVector(T1 storage, std::string subname, T2& result, bool exist, bool* loaded = nullptr)
{
    if (loaded) {
        *loaded = false;
    }

    ReadXML(storage, subname, result);
    if (loaded) {
        *loaded = true;
    }
}

template <typename T1, typename T2>
void LoadXMLUnitVectorSingle(T1 storage, std::string name, std::string subname, T2& result, bool exist, bool* loaded = nullptr)
{
    if (loaded) {
        *loaded = false;
    }

    myXML::Contain* con;
    if (FindXMLContainer(storage, name, con)) {
        ReadXMLVectorSingle(con, subname, result);
        if (loaded) {
            *loaded = true;
        }
    }
    else if (exist) {
        throw (Exp() << CreateExceptionCNF(storage, name));
    }
}

template <typename T1, typename T2>
void LoadXMLVector(T1 storage, std::string name, std::string subname, std::vector<T2>& result, bool exist = true)
{
    LoadXMLUnitVector(storage, name, subname, result, exist, 0);
}

template <typename T1, typename T2>
void LoadXML(T1 storage, std::string name, std::string subname, std::vector<T2>& result, bool exist = true)
{
    LoadXMLUnitVector(storage, name, subname, result, exist, 0);
}

template <typename T1, typename T2>
void LoadXMLVector(T1 storage, std::string subname, std::vector<T2>& result, bool exist = true)
{
    LoadXMLUnitVector(storage, subname, result, exist, 0);
}

template <typename T1, typename T2>
void LoadXMLVectorSingle(T1 storage, std::string name, std::string subname, std::vector<T2>& result, bool exist = true)
{
    LoadXMLUnitVectorSingle(storage, name, subname, result, exist, 0);
}

template <typename T1, typename T2, typename T3>
void LoadXMLMap(T1 storage, std::string name, std::string subname, std::map<T2, T3>& result, bool exist = true)
{
    LoadXMLUnitVector(storage, name, subname, result, exist, 0);
}

template <typename T1, typename T2, typename T3>
void LoadXMLMap(T1 storage, std::string subname, std::map<T2, T3>& result, bool exist = true)
{
    LoadXMLUnitVector(storage, subname, result, exist, 0);
}

template <typename T>
void LoadXML(T storage, std::string name, double& result, bool exist = true, bool* loaded = nullptr)
{
    LoadXMLUnit(storage, name, result, exist, loaded);
}

template <typename T>
void LoadXML(T storage, std::string name, int& result, bool exist = true, bool* loaded = nullptr)
{
    LoadXMLUnit(storage, name, result, exist, loaded);
}

template <typename T>
void LoadXML(T storage, std::string name, bool& result, bool exist = true, bool* loaded = nullptr)
{
    LoadXMLUnit(storage, name, result, exist, loaded);
}

template <typename T>
void LoadXML(T storage, std::string name, std::string& result, bool exist = true, bool* loaded = nullptr)
{
    LoadXMLUnit(storage, name, result, exist, loaded);
}

template <typename T>
void LoadXML(T storage, std::string name, TimeGD& result, bool exist = true, bool* loaded = nullptr)
{
    LoadXMLUnit(storage, name, result, exist, loaded);
}

template <typename T>
void LoadXML(T storage, std::string name, TimeJD& result, bool exist = true, bool* loaded = nullptr)
{
    TimeGD date;
    LoadXMLUnit(storage, name, date, exist, loaded);
    result = date;
}

template <typename T1, typename T2>
void LoadXML(T1 storage, std::string name, T2& result, bool exist = true, bool* loaded = nullptr)
{
    LoadXMLUnit(storage, name, result, exist, loaded);
}

template <typename T>
void ReadXML(T storage, TimeJD& result)
{
    ReadXMLDate(storage, result);
}

template <typename T>
void ReadXML(T storage, TimeGD& result)
{
    ReadXMLDate(storage, result);
}

template <typename T>
void ReadXML(T storage, double& result)
{
    ReadXMLNumeric(storage, result);
}

template <typename T>
void ReadXML(T storage, int& result)
{
    ReadXMLNumeric(storage, result);
}

template <typename T>
void ReadXML(T storage, bool& result)
{
    ReadXMLNumeric(storage, result);
}

template <typename T>
void ReadXML(T storage, std::string& result)
{
    ReadXMLString(storage, result);
}

template <typename T, typename T2>
void ReadXML(T storage, T2& result)
{
    ReadXMLClass(storage, result);
}

template <typename T, typename T2>
void ReadXML(T storage, std::string subname, std::vector<T2>& result)
{
    ReadXMLVector(storage, subname, result);
}

template <typename T, typename T2, typename T3>
void ReadXML(T storage, std::string subname, std::map<T2, T3>& result)
{
    ReadXMLMap(storage, subname, result);
}

template <typename T>
void PutValue(std::ostream& ss, std::string conName, const T& object)
{
    ss << "<" << conName << ">";
    object.ToXml(ss);
    ss << "</" << conName << ">";
}

template <typename T>
void PutValue(std::ostream& ss, std::string conName, const T* object)
{
    ss << "<" << conName << ">";
    object->ToXml(ss);
    ss << "</" << conName << ">";
}

template <typename T>
void PutValueAttr(std::ostream& ss, std::string conName, const T& object)
{
    if (object.HasXmlAttributes()) {
        ss << "<" << conName << " ";
        object.SetXmlAttributes(ss);
        ss << ">";
    }
    else {
        ss << "<" << conName << ">";
    }

    object.ToXml(ss);
    ss << "</" << conName << ">";
}

static void PutValue(std::ostream& ss, const std::string& conName, size_t value)
{
    ss << "<" << conName << ">" << value << "</" << conName << ">";
}

static void PutValue(std::ostream& ss, const std::string& conName, bool value)
{
    ss << "<" << conName << ">" << (value ? 1 : 0) << "</" << conName << ">";
}

static void PutValue(std::ostream& ss, const std::string& conName, int value)
{
    ss << "<" << conName << ">" << value << "</" << conName << ">";
}

static void PutValue(std::ostream& ss, const std::string& conName, const std::string& value)
{
    ss << "<" << conName << ">" << value << "</" << conName << ">";
}

static void PutValue(std::ostream& ss, const std::string& conName, const TimeGD& value)
{
    ss << "<" << conName << ">" << DateToStr(value) << "</" << conName << ">";
}

static void PutValue(std::ostream& ss, const std::string& conName, const TimeJD& value)
{
    ss << "<" << conName << ">" << DateToStr(value) << "</" << conName << ">";
}

template <typename T>
static void PutValue(std::ostream& ss, std::string conName, std::string subName, const std::vector<T>& vec)
{
    ss << "<" << conName << ">";
    for (typename std::vector<T>::const_iterator rec = vec.begin(); rec != vec.end(); ++rec) {
        PutValue(ss, subName, *rec);
    }
    ss << "</" << conName << ">";
}

template <typename T>
static void PutValue(std::ostream& ss, std::string conName, std::string subName, const std::vector<T*>& vec)
{
    ss << "<" << conName << ">";
    for (typename std::vector<T*>::const_iterator rec = vec.begin(); rec != vec.end(); ++rec) {
        PutValue(ss, subName, *(*rec));
    }
    ss << "</" << conName << ">";
}

static void PutValue(std::ostream& ss, const std::string& conName, const double value, const std::string& attributes = "")
{
    ss << std::setprecision(std::numeric_limits<double>::digits10 + 2);
    if (attributes.empty()) {
        ss << "<" << conName << ">" << value << "</" << conName << ">";
    }
    else {
        ss << "<" << conName << " " << attributes << ">" << value << "</" << conName << ">";
    }
}

/*static void PutValue(std::ostream& ss, std::string conName, double value, int precision, std::string attributes = "")
{
    ss << std::setprecision(precision);
    if (!attributes.size()){
        ss << "<" << conName << ">" << value << "</" << conName << ">";
    }else{
        ss << "<" << conName << " " << attributes << ">" << value << "</" << conName << ">";
    }
}*/

static void LoadXmlFile(const std::string& filename, myXML::Document& doc)
{
    if (!doc.LoadFile(filename)) {
        throw Exp() << "Bad XML format in file:" << filename;
    }
}

static void LoadXmlString(const std::string& str, myXML::Document& doc)
{
    if (!doc.LoadStringX(str.c_str())) {
        throw Exp() << "Bad XML string:\n" << str;
    }
}

