#ifndef XMLVECTOR_H
#define XMLVECTOR_H

#include <myXML_Document.hpp>
#include <map>
#include "XMLFunctions.h"


template <class T, typename TId>
class XMLVector
{
public:

    XMLVector(std::string label)
    {
        m_label = label;
        m_hasId = false;
    }

    XMLVector() { m_hasId = false; }
    XMLVector(T id) { SetId(id); }

    bool LoadRecords(myXML::Contain* parent)
    {
        return LoadRecords(parent, GetLabel());
    }

    bool LoadRecords(myXML::Contain* parent, std::string containerName)
    {
        myXML::Contain* con;

        if (GetContainer(parent, containerName, con)) {
            std::vector<myXML::Contain*> list = GetChildren(con, T::GetRecordLabel());
            for (int i = 0; i < (int)list.size(); ++i) {
                T rec;
                if (rec.Load(list[i])) {
                    m_records.push_back(rec);
                }
                else {
                    return false;
                }
            }
            return CheckValues();
        }
        else {
            errors->push_back(ErrorMsg(EM_CON_NOT_FOUND, containerName));
            return false;
        }
    }

    virtual bool CheckValues()
    {
        //	if (m_records.size() < 1){
        //		if(errors)errors->push_back(ErrorMsg(EM_RECORDS_NOT_FOUND,T::GetRecordLabel()));
        //		return false;
        //	}

        bool allHasId = true;
        bool oneHasId = false;
        for (int i = 0; i < (int)m_records.size(); ++i) {
            allHasId = allHasId && m_records[i].HasId();
            oneHasId = oneHasId || m_records[i].HasId();
        }
        if (oneHasId && !allHasId) {
            if (errors)errors->push_back(ErrorMsg(EM_RECORDS_NO_ID, T::GetRecordLabel()));
            return false;
        }

        /*	for (int i = 0; i < (int)m_records.size(); ++i){
                T &ag1 = m_records[i];
                if (!ag1.HasInternalId()){
                    ag1.SetInternalId(i);
                    for (int j = 0; j < (int)m_records.size() ; ++j){
                        T &ag2 = m_records[j];
                        if (!ag2.HasInternalId() && ag1.IsEqual(ag2)){
                            ag2.SetInternalId(ag1.GetInternalId());
                        }else if (i != j && ag1.GetInternalId() != ag2.GetInternalId() && ag1.HasId() && ag2.HasId() && ag1.GetId() == ag2.GetId()){
                            std::ostringstream ss;
                            ss << T::GetRecordLabel() << ":" << i << "," << j;
                        //	if(errors)errors->push_back(ErrorMsg(EM_RECORDS_SAME_ID,ss.str()));
                        //	return false;
                        }
                    }
                }
            }*/

        /*	for (int i = 0; i < (int)m_records.size() ; ++i){
                T ag = m_records[i];
                std::map<int,int>::const_iterator it = m_uniques.find(ag.GetInternalId());
                m_uniques[ag.GetInternalId()] = it != m_uniques.end() ? m_uniques[ag.GetInternalId()]+1 : 1;
            }*/

        return true;
    }

    void GetAsMap(std::map<TId, T>& res) const
    {
        for (size_t i = 0; i < m_records.size(); ++i) {
            res[m_records[i].GetId()] = m_records[i];
        }
    }

    bool HasId() const { return m_hasId; }
    TId GetId() const { return m_id; }

    void SetId(TId id)
    {
        m_id = id;
        m_hasId = true;
    }

    const T& GetActualRecord() const { return m_records[m_actualRecordIndex]; }
    std::vector<T>& GetVector() { return m_records; }
    const std::vector<T>& GetVector_const() const { return m_records; }
    std::string GetLabel() const { return m_label; }
protected:

    std::string m_label;
    std::vector<T> m_records;
    std::map<int, int> m_uniques;
    TId m_id;
    bool m_hasId;
    int m_actualRecordIndex;
};

template <typename T>
class XMLVectorItem
{
public:
    XMLVectorItem()
    {
        m_hasId = false;
        m_hasInternalId = false;
    }

    XMLVectorItem(T id)
    {
        m_id = id;
        m_hasId = true;
    }

    bool HasId() const { return m_hasId; }
    T GetId() const { return m_id; }
    int GetInternalId() const { return m_internalId; }
    bool HasInternalId() const { return m_hasInternalId; }

    void SetInternalId(int id)
    {
        m_internalId = id;
        m_hasInternalId = true;
    }

private:
    int m_internalId;
    bool m_hasInternalId;
    T m_id;
    bool m_hasId;

protected:
    void LoadRecords(myXML::Contain* parent)
    {
        m_hasId = XMLReadParam(parent, Param("Id", m_id), 0);
    }
};

#endif
