#include "XYZ.h"
#include "cmath"

XYZ::XYZ(double px, double py, double pz)
{
    x = px;
    y = py;
    z = pz;
}

XYZ::XYZ()
{
    x = 0;
    y = 0;
    z = 0;
}

XYZ::XYZ(const XYZ& right)
{
    x = right.x;
    y = right.y;
    z = right.z;
}

XYZ::~XYZ()
{
}

void XYZ::Normalize()
{
    double mod = sqrt((x * x) + (y * y) + (z * z));
    if (mod == 0) {
        x = 0;
        y = 0;
        z = 0;
        return;
    }
    double cd = 1 / mod;
    x = x * cd;
    y = y * cd;
    z = z * cd;
}

double XYZ::MagnitudeSquared() const
{
    return (x * x + y * y + z * z);
}

double XYZ::Magnitude() const
{
    return sqrt(x * x + y * y + z * z);
}

XYZ& XYZ::operator =(const XYZ& right)
{
    x = right.x;
    y = right.y;
    z = right.z;

    return *this;
}

bool XYZ::operator ==(const XYZ& right) const
{
    return x == right.x && y == right.y && z == right.z;
}

bool XYZ::operator !=(const XYZ& right) const
{
    return x != right.x || y != right.y || z != right.z;
}

XYZ XYZ::operator +(const XYZ& right) const
{
    return XYZ(x + right.x, y + right.y, z + right.z);
}

XYZ XYZ::operator -(const XYZ& right) const
{
    return XYZ(x - right.x, y - right.y, z - right.z);
}

XYZ XYZ::operator *(double c) const
{
    return XYZ(x * c, y * c, z * c);
}

XYZ XYZ::operator /(double c) const
{
    return XYZ(x / c, y / c, z / c);
}

XYZ XYZ::cross(const XYZ& right) const
{
    XYZ ret;

    ret.x = y * right.z - z * right.y;
    ret.y = z * right.x - x * right.z;
    ret.z = x * right.y - y * right.x;

    return ret;
}
