#include "globals.h"
#include <cmath>

void Topo(double el, double az, double ksi, double teta, double dzita, double& rel, double& raz)
{
    double cosf = cos(el);
    double sinf = sin(el);
    double cosl = cos(az);
    double sinl = sin(az);

    double Xn = cosf * cosl;
    double Yn = cosf * sinl;
    double Zn = sinf;

    double a1 = cosf * cosl * ksi - sinl * teta - sinf * cosl * dzita;
    double a2 = cosf * sinl * ksi + cosl * teta - sinf * sinl * dzita;
    double a3 = sinf * ksi + cosf * dzita;

    double x1 = a1 + Xn;
    double y1 = a2 + Yn;
    double z1 = a3 + Zn;

    double rr = sqrt(x1 * x1 + y1 * y1 + z1 * z1);
    rel = asin(z1 / rr);
    raz = atan2(y1, x1);
}

void Topo2(double lat, double lon, double R, double el, double az, double& teta, double& dzita)
{
    double la, fi, x1, y1, z1, Xn, Yn, Zn, b, l;

    la = lon;
    fi = lat;

    double cosfi = cos(fi);
    double sinfi = sin(fi);
    double cosla = cos(la);
    double sinla = sin(la);

    Xn = R * cosfi * cosla;
    Yn = R * cosfi * sinla;
    Zn = R * sinfi;

    b = el;
    l = az;

    double cosb = cos(b);
    x1 = R * cosb * cos(l);
    y1 = R * cosb * sin(l);
    z1 = R * sin(b);

    double a21, a22, a23, a31, a32, a33;

    //	a11 = cosfi*cosla;
    a21 = -sinla;
    a31 = -sinfi * cosla;

    //	a12 = cosfi*sinla;
    a22 = cosla;
    a32 = -sinfi * sinla;

    //	a13 = sinfi;
    a23 = 0;
    a33 = cosfi;

    teta = a21 * (x1 - Xn) + a22 * (y1 - Yn) + a23 * (z1 - Zn);
    dzita = a31 * (x1 - Xn) + a32 * (y1 - Yn) + a33 * (z1 - Zn);
}

double Min(double arg1, double arg2)
{
    if (arg1 >= arg2) {
        return arg2;
    }
    else {
        return arg1;
    }
}

double Max(double arg1, double arg2)
{
    if (arg1 <= arg2) {
        return arg2;
    }
    else {
        return arg1;
    }
}

double Modulus(double arg1, double arg2)
{
    double modu;
    modu = arg1 - (long)(arg1 / arg2) * arg2;
    if (modu >= 0.0) return modu;
    else return modu + arg2;
}

double Distance(double el1, double az1, double el2, double az2)
{
    double acs = cos(el1) * cos(el2) * cos(az2 - az1) + sin(el1) * sin(el2);

    if (acs > 1) {
        acs = 1;
    }
    else if (acs < -1) {
        acs = -1;
    }

    return acos(acs);
}

double DistanceCos(double el1, double az1, double el2, double az2)
{
    double acs = cos(el1) * cos(el2) * cos(az2 - az1) + sin(el1) * sin(el2);

    if (acs > 1) {
        acs = 1;
    }
    else if (acs < -1) {
        acs = -1;
    }

    return acs;
}

double sqr(double arg)
{
    return arg * arg;
}

double sqr3(double arg)
{
    return arg * arg * arg;
}

double sqr4(double arg)
{
    return arg * arg * arg * arg;
}

int Round(double arg)
{
    double fFrac, fInt;
    fFrac = modf(arg, &fInt);
    if (fFrac >= 0.5) fInt++;
    return (int)fInt;
}

void AlongAcross(Vect6 satPosJ2000, Vect3 sitePosJ2000, double ra, double de, double& along, double& across, double& alongra, double& alongde, double& acrossra, double& acrossde)
{
    double x = satPosJ2000.x;
    double y = satPosJ2000.y;
    double z = satPosJ2000.z;
    double vx = satPosJ2000.vx;
    double vy = satPosJ2000.vy;
    double vz = satPosJ2000.vz;
    double x0 = sitePosJ2000.x;
    double y0 = sitePosJ2000.y;
    double z0 = sitePosJ2000.z;

    double x1, y1, z1, x2, y2, z2, x3, y3, z3, xi, yi, zi, r1, r2, s;

    double cosde = cos(de);
    double sinde = sin(de);
    double cosra = cos(ra);
    double sinra = sin(ra);

    double cosdecosra = cosde * cosra;
    double cosdesinra = cosde * sinra;
    double sindecosra = sinde * cosra;
    double sindesinra = sinde * sinra;

    xi = cosdecosra;
    yi = cosdesinra;
    zi = sinde;

    x1 = x - x0;
    y1 = y - y0;
    z1 = z - z0;

    r1 = sqrt(x1 * x1 + y1 * y1 + z1 * z1);
    x1 = x1 / r1;
    y1 = y1 / r1;
    z1 = z1 / r1;

    s = vx * x1 + vy * y1 + vz * z1;
    x2 = vx - s * x1;
    y2 = vy - s * y1;
    z2 = vz - s * z1;

    r2 = sqrt(x2 * x2 + y2 * y2 + z2 * z2);
    x2 = x2 / r2;
    y2 = y2 / r2;
    z2 = z2 / r2;

    x3 = y1 * z2 - y2 * z1;
    y3 = z1 * x2 - z2 * x1;
    z3 = x1 * y2 - x2 * y1;

    along = xi * x2 + yi * y2 + zi * z2;
    across = xi * x3 + yi * y3 + zi * z3;

    if (x1 * xi + y1 * yi + z1 * zi < 0) {
        along = 1.0;
        across = 1.0;
    }

    alongra = y2 * cosdecosra - x2 * cosdesinra;
    acrossra = y3 * cosdecosra - x3 * cosdesinra;
    alongde = z2 * cosde - x2 * sindecosra - y2 * sindesinra;
    acrossde = z3 * cosde - x3 * sindecosra - y3 * sindesinra;
}
