#include "ires.h"
#include <utility>
#include "json_functions_bno.h"

IRes::IRes() = default;

IRes::IRes(const Vect6& vec, const TimeJD& date, const double age, std::string orbitid, const bool addon)
    :
    m_orbitid(std::move(orbitid)),
    m_vec(vec),
    m_date(date),
    m_age(age),
    m_addon(addon)
{
}

IRes::IRes(const Vect6& vec, const Vect3& rnb, const TimeJD& date, double age, std::string orbitid, bool addon)
    :
    m_orbitid(std::move(orbitid)),
    m_vec(vec),
    m_rnb(rnb),
    m_date(date),
    m_age(age),
    m_addon(addon),
    m_has_rnb(true)
{
}

void IRes::load(const jsonval& parent)
{
    double x, y, z;
    loadjson(parent, "x_km", x);
    loadjson(parent, "y_km", y);
    loadjson(parent, "z_km", z);

    double vx = 0, vy = 0, vz = 0;
    loadjson(parent, "vx_km/s", vx, false);
    loadjson(parent, "vy_km/s", vy, false);
    loadjson(parent, "vz_km/s", vz, false);

    loadjson(parent, "date", m_date);
    loadjson(parent, "orbitid", m_orbitid, false);
    loadjson(parent, "age_days", m_age, false);

    m_vec = Vect6(x / 1E3, y / 1E3, z / 1E3, vx, vy, vz);
}
