#include "orbBlock.h"
#include "TextFunctions.h"

OrbBlock::OrbBlock()
    :
    m_ip(nullptr)
{
}

OrbBlock::OrbBlock(
    const SInitOrbit& orbit,
    const TimeJD& beg,
    const TimeJD& end
)
    :
    m_orbit(orbit),
    m_ip(nullptr)
{
    m_data = orb_block_data(orbit.GetId(), orbit.GetDate(), beg, end);

#ifdef _DEBUG
    m_beg_s = DateToStr(beg);
    m_end_s = DateToStr(end);
#endif
}

OrbBlock::OrbBlock(
    const SInitOrbit& orbit,
    const TimeJD& beg,
    const TimeJD& end,
    const double mind,
    const std::string& next_orbitid
)
    :
    m_orbit(orbit),
    m_ip(nullptr)
{
    m_data = orb_block_data(orbit.GetId(), orbit.GetDate(), beg, end, mind, next_orbitid);

#ifdef _DEBUG
    m_beg_s = DateToStr(beg);
    m_end_s = DateToStr(end);
#endif
}

OrbBlock::OrbBlock(const orb_block_data& data)
    :m_data(data)
{
}

OrbBlock::~OrbBlock()
{
    delete m_ip;
}

void OrbBlock::init_by_orbit(const bool calc_rnb)
{
    m_ip = new XInterpolator(m_orbit, get_beg(), get_end(), calc_rnb);
}

void OrbBlock::init_by_data()
{
    m_ip = new XInterpolator(m_data.get_vecs());
}

void OrbBlock::load(const jsonval& parent)
{
    m_data.load(parent); 
}

