#pragma once

#include "TM.h"
#include "StateVector.h"
#include "interpolate.h"
#include "json_functions.h"
#include "orb_block_data.h"

class OrbBlock
{
public:
    OrbBlock();
    OrbBlock(const SInitOrbit& orbit, const TimeJD& beg, const TimeJD& end);
    OrbBlock(const SInitOrbit& orbit, const TimeJD& beg, const TimeJD& end, double mind, const std::string& next_orbitid);
    OrbBlock(const orb_block_data& data);
    ~OrbBlock();

    TimeJD get_beg() const { return get_data().get_beg(); }
    TimeJD get_end() const { return get_data().get_end(); }
    const SInitOrbit& get_orbit() const { return m_orbit; }
    double get_min_distance() const { return get_data().get_min_distance(); }
    std::string get_next_orbitid() const { return get_data().get_next_orbitid(); }
    void init_by_orbit(bool calc_rnb);
    void init_by_data();
    const XInterpolator* get_interpolator() const { return m_ip; }
    void load(const jsonval& parent);
    const orb_block_data& get_data() const { return m_data; }
private:
    SInitOrbit m_orbit;
    orb_block_data m_data;
    XInterpolator* m_ip{nullptr};

#ifdef _DEBUG
    std::string m_beg_s;
    std::string m_end_s;
#endif
};
