#pragma once

#include "TM.h"
#include <vector>
#include "orbBlock.h"
#include "StateVector.h"
#include "json_functions.h"

typedef std::vector<OrbBlock> OrbBlocks;

class OrbBlocksStore {
public:
    OrbBlocksStore();
    OrbBlocksStore(const OrbBlocks& blocks);
    void init(const SInitOrbits& orbits, const TimeJD* beg, const TimeJD* end, bool filter_orbits, bool min_dist_forced, bool interpolate, bool calc_rnb);
    void init(const SInitOrbits& orbits, const TimeJD& beg, const TimeJD& end, bool filter_orbits, bool min_dist_forced, bool interpolate, bool calc_rnb);
    void init();
    SInitOrbits calc_orbits(std::vector<TimeJD> dates, bool variates) const;
    Vect6 get_pos(const TimeJD& date, bool tks = false) const;
    std::pair<Vect6, Vect3> get_pos_rnb(const TimeJD& date, bool tks = false) const;
    const SInitOrbit& get_orbit(const TimeJD& date) const;
    const OrbBlocks& get_blocks() const { return m_blocks; }
    std::vector<orb_block_data> get_orb_data() const;
    OrbBlocks& get_blocks() { return m_blocks; }
    const TimeJD& get_beg_before_filter() const { return m_beg_before_filter; }
    const TimeJD& get_end_before_filter() const { return m_end_before_filter; }
    void load(const jsonval& parent);
    bool is_initiated() const { return m_initiated; }
    bool has_rnb() const { return m_has_rnb; }
    void set_rnb_flag(const bool flag) { m_has_rnb = flag; }
private:
    void init_min_dist(const SInitOrbits& orbits, const TimeJD& beg, const TimeJD& end, bool forced);
    void calc_min_dist(const SInitOrbit& orb1, const SInitOrbit& orb2, const TimeJD& beg, const TimeJD& end, TimeJD& result_jd, double& result_dist) const;
    SInitOrbits filter(const SInitOrbits& orbits) const;
    OrbBlocks m_blocks;
    TimeJD m_beg_before_filter;
    TimeJD m_end_before_filter;
    bool m_initiated{false};
    bool m_has_rnb{false};
};
