#include "orb_block_data.h"
#include "json_functions_bno.h"

orb_block_data::orb_block_data(
    std::string orbitid,
    const TimeJD& date,
    const TimeJD& beg,
    const TimeJD& end
)
    :
    m_orbitid(std::move(orbitid)),
    m_date(date),
    m_beg(beg),
    m_end(end),
    m_mind(-1)
{
}

orb_block_data::orb_block_data(
    std::string orbitid,
    const TimeJD& date,
    const TimeJD& beg,
    const TimeJD& end,
    const double mind,
    std::string next_orbitid
)
    :
    m_orbitid(std::move(orbitid)),
    m_next_orbitid(std::move(next_orbitid)),
    m_date(date),
    m_beg(beg),
    m_end(end),
    m_mind(mind)
{
}

orb_block_data::orb_block_data(
    std::string orbitid, 
    const TimeJD& date, 
    const TimeJD& beg, 
    const TimeJD& end,
    const double mind, 
    std::string next_orbitid, 
    const std::vector<IRes>& vecs)
    :
    m_orbitid(std::move(orbitid)),
    m_next_orbitid(std::move(next_orbitid)),
    m_date(date),
    m_beg(beg),
    m_end(end),
    m_mind(mind),
    m_vecs(vecs)
{
}

void orb_block_data::load(const jsonval& parent)
{
    loadjson(parent, "id", m_orbitid);
    loadjson(parent, "date", m_date);
    loadjson(parent, "beg", m_beg);
    loadjson(parent, "end", m_end);
    loadjson(parent, "mind_km", m_mind, false);

    std::string vecs;
    loadjson(parent, "vecs", vecs, false);

    if(vecs.empty()) {
        return;
    }

    std::vector<std::string> parts;
    splitstrf(vecs, parts, ",");

    for (size_t i = 0; i < parts.size(); i += 8) {
        int days;
        Converter::ToNumber(parts[i], days);

        double fraction;
        Converter::ToNumber(parts[i + 1], fraction);

        double x;
        Converter::ToNumber(parts[i + 2], x);

        double y;
        Converter::ToNumber(parts[i + 3], y);

        double z;
        Converter::ToNumber(parts[i + 4], z);

        double vx;
        Converter::ToNumber(parts[i + 5], vx);

        double vy;
        Converter::ToNumber(parts[i + 6], vy);

        double vz;
        Converter::ToNumber(parts[i + 7], vz);

        m_vecs.emplace_back(Vect6(x, y, z, vx, vy, vz), TimeJD(days, fraction), 0, "");
    }
}
