#pragma once

template <typename T> class svalue
{
public:
    svalue()
    {
    }

    svalue(const T& v)
        :
        m_has_data(true),
        m_value(v)
    {
    }

    friend bool operator<=(double l, const svalue& r) = delete;
    friend bool operator>=(double l, const svalue& r) = delete;
    friend bool operator==(double l, const svalue& r) = delete;
    friend bool operator!=(double l, const svalue& r) = delete;

    friend bool operator<=(float l, const svalue& r) = delete;
    friend bool operator>=(float l, const svalue& r) = delete;
    friend bool operator==(float l, const svalue& r) = delete;
    friend bool operator!=(float l, const svalue& r) = delete;

    friend bool operator<=(const svalue& l, double r) = delete;
    friend bool operator>=(const svalue& l, double r) = delete;
    friend bool operator==(const svalue& l, double r) = delete;
    friend bool operator!=(const svalue& l, double r) = delete;

    friend bool operator<=(const svalue& l, float r) = delete;
    friend bool operator>=(const svalue& l, float r) = delete;
    friend bool operator==(const svalue& l, float r) = delete;
    friend bool operator!=(const svalue& l, float r) = delete;

    friend bool operator!=(const svalue& l, const svalue& r) { return l.get() != r.get(); }
    friend bool operator==(const svalue& l, const svalue& r) { return l.get() == r.get(); }
    friend bool operator<(const svalue& l, const svalue& r) { return l.get() < r.get(); }
    friend bool operator<=(const svalue& l, const svalue& r) { return l.get() <= r.get(); }
    friend bool operator>(const svalue& l, const svalue& r) { return l.get() > r.get(); }
    friend bool operator>=(const svalue& l, const svalue& r) { return l.get() >= r.get(); }
    friend bool operator*(const svalue& l, const svalue& r) { return l.get() * r.get(); }
    friend bool operator/(const svalue& l, const svalue& r) { return l.get() / r.get(); }
    friend bool operator+(const svalue& l, const svalue& r) { return l.get() + r.get(); }
    friend bool operator-(const svalue& l, const svalue& r) { return l.get() - r.get(); }

    template <typename Tc> friend bool operator==(const Tc& l, const svalue& r) { return l == r.get(); }
    template <typename Tc> friend bool operator!=(const Tc& l, const svalue& r) { return l != r.get(); }
    template <typename Tc> friend bool operator<(const Tc& l, const svalue& r) { return l < r.get(); }
    template <typename Tc> friend bool operator>(const Tc& l, const svalue& r) { return l > r.get(); }
    template <typename Tc> friend bool operator<=(const Tc& l, const svalue& r) { return l <= r.get(); }
    template <typename Tc> friend bool operator>=(const Tc& l, const svalue& r) { return l >= r.get(); }
    template <typename Tc> friend bool operator*(const Tc& l, const svalue& r) { return l * r.get(); }
    template <typename Tc> friend bool operator/(const Tc& l, const svalue& r) { return l / r.get(); }
    template <typename Tc> friend bool operator+(const Tc& l, const svalue& r) { return l + r.get(); }
    template <typename Tc> friend bool operator-(const Tc& l, const svalue& r) { return l - r.get(); }

    template <typename Tc> friend bool operator==(const svalue& l, const Tc& r) { return l.get() == r; }
    //template <typename Tc> friend bool operator!=(const svalue& l, const Tc& r) { return l.get() != r; }
    template <typename Tc> friend bool operator<(const svalue& l, const Tc& r) { return l.get() < r; }
    template <typename Tc> friend bool operator>(const svalue& l, const Tc& r) { return l.get() > r; }
    template <typename Tc> friend bool operator<=(const svalue& l, const Tc& r) { return l.get() <= r; }
    template <typename Tc> friend bool operator>=(const svalue& l, const Tc& r) { return l.get() >= r; }
    template <typename Tc> friend bool operator*(const svalue& l, const Tc& r) { return l.get() * r; }
    template <typename Tc> friend bool operator/(const svalue& l, const Tc& r) { return l.get() / r; }
    template <typename Tc> friend bool operator+(const svalue& l, const Tc& r) { return l.get() + r; }
    template <typename Tc> friend bool operator-(const svalue& l, const Tc& r) { return l.get() - r; }

    bool has_data() const { return m_has_data; }
    const T& get() const { return m_value; }

    void set(const T& v)
    {
        m_value = v;
        m_has_data = true;
    }

    void unset()
    {
        m_has_data = false;
    }

private:
    bool m_has_data{false};
    T m_value;
};
