/*
 * Copyright 2010 Boris Popov <popov.b@gmail.com>
 */

#include "builder.hpp"
#include <inttostring.hpp>

//-----------------------------------------------
dxml::Builder::Builder
(const TagNameRoot tnr):
m_TNR (tnr),
m_TNL (""),
m_CAN ("") {

	return;
}

dxml::Builder::Builder
(const TagNameRoot tnr,
const TagNameLine tnl):
m_TNR (tnr),
m_TNL (tnl),
m_CAN ("") {

	return;
}

dxml::Builder::Builder
(const TagNameRoot tnr,
const TagNameLine tnl, const CountAttName can):
m_TNR (tnr),
m_TNL (tnl),
m_CAN (can) {

	return;
}

dxml::Builder::~Builder () {

	return;
}

bool dxml::Builder::Build (CharReader& cr, Xml& xml) {

	bool Success = true;
	bool Fault = false;

	XmlSContainer xsc;
	m_SE.Extract (cr, xsc);

	if (! m_Prober.Probing (xsc) ) {

		return Fault;
	}
		
	m_Build (xsc, xml);
	return Success;
}
		
void dxml::Builder::m_Build
(XmlSContainer& xsc, Xml& xml) {

	xml = m_GetRootOpen ();

	XmlString* xs = xsc.Pop ();
	size_t n = 1;
	while (xs != NULL) {

		xml = xml + "\n" + m_DecorLine (*xs, n);
		xs = xsc.Pop ();
		n++;
	}

	xml = xml + "\n" + m_GetRootClose ();
	return;
}
		
dxml::String dxml::Builder::m_DecorLine
(const String& str, const size_t n) const {

	if (m_TNL == "") return str;

	String temp;

	if (m_CAN == "") {

		temp = m_GetLineOpen () +
			str + m_GetLineClose ();

	} else {
		
		temp = m_GetLineOpenWCount (n) +
			str + m_GetLineClose ();
	}
	return temp;
}

dxml::String dxml::Builder::m_GetRootOpen () const {

	if (m_TNR == "") return "";
	return ( m_GetOpen ((m_TNR)) );
}

dxml::String dxml::Builder::m_GetRootClose () const {
	
	if (m_TNR == "") return "";
	return ( m_GetClose ((m_TNR)) );
}

dxml::String dxml::Builder::m_GetLineOpen () const {

	if (m_TNL == "") return "";
	return ( m_GetOpen ((m_TNL)) );
}
	
dxml::String dxml::Builder::m_GetLineClose () const {

	if (m_TNL == "") return "";
	return ( m_GetClose ((m_TNL)) );
}

dxml::String dxml::Builder::m_GetClose
(const String str) const {
	
	return ( m_GetCloseFirst (str) + m_GetCloseSecond () );
}
		
dxml::String dxml::Builder::m_GetLineOpenWCount
(const size_t n) const {

	if (m_TNL == "") return "";
	if (m_CAN == "") return m_GetLineOpen ();

	return ( m_GetOpenFirst (m_TNL) +
			m_GetAttr (n) + m_GetOpenSecond () ) ;
}
		
dxml::String dxml::Builder::m_GetAttr
(const size_t n) const {
	
	String num = its::SizeT_to_String (n);
	return ( " " + m_CAN + "=\"" + num + "\"" );
}

dxml::String dxml::Builder::m_GetOpen
(const String str) const {
	
	return ( m_GetOpenFirst (str) + m_GetOpenSecond () );
}
		
dxml::String dxml::Builder::m_GetOpenFirst
(const String str) const {
	
	return m_GetFirst ("<", str);
}
		
dxml::String dxml::Builder::m_GetCloseFirst
(const String str) const {

	return m_GetFirst ("</", str);
}
		
dxml::String dxml::Builder::m_GetOpenSecond () const {
	
	return ( ">");
}
		
dxml::String dxml::Builder::m_GetCloseSecond () const {

	return m_GetOpenSecond ();
}
		
dxml::String dxml::Builder::m_GetFirst
(const String str1, const String str2) const {

	return ( str1 + str2 );
}
//-----------------------------------------------
