/*
 * Copyright 2010 Boris Popov <popov.b@gmail.com>
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <mystring.hpp>
#include <charreader.hpp>
#include <subextractor.hpp>
#include <probing.hpp>

#ifndef __builder_hpp__
#define __builder_hpp__

namespace dxml {

//-----------------------------------------------
typedef common::String String;
typedef String Xml;
typedef String TagName;
typedef TagName TagNameRoot;
typedef TagName TagNameLine;
typedef String CountAttName;
//-----------------------------------------------

//-----------------------------------------------
class Builder {

	public:
		Builder (const TagNameRoot);

		Builder (const TagNameRoot,
				const TagNameLine);

		Builder (const TagNameRoot,
				const TagNameLine,
				const CountAttName);

		~Builder ();

		bool Build (CharReader&, Xml&);

	private:
		Builder (const Builder&);
		Builder& operator= (const Builder&);

		const TagNameRoot  m_TNR;
		const TagNameLine  m_TNL;
		const CountAttName m_CAN;
		
		SubstringExtractor m_SE;
		Prober m_Prober;

		void m_AddRootElement (XmlSContainer&);
		void m_AddLineElements (XmlSContainer&);

		String m_GetRootOpen () const;
		String m_GetRootClose () const;
		
		String m_GetLineOpen () const;
		String m_GetLineOpenWCount (const size_t) const;
		String m_GetLineClose () const;

		String m_GetOpen  (const String) const;
		String m_GetClose (const String) const;
		
		String m_GetOpenFirst  (const String) const;
		String m_GetOpenSecond () const;
		String m_GetCloseSecond () const;
		
		String m_GetCloseFirst  (const String) const;
		
		String m_GetFirst
			(const String, const String) const;

		String m_GetAttr (const size_t) const;

		void m_Build (XmlSContainer&, Xml&);

		String m_DecorLine
			(const String&, const size_t) const;
};
//-----------------------------------------------
}
#endif /* __builder_hpp__ */
