/*
 * Copyright 2010 Boris Popov <popov.b@gmail.com>
 */

#include "charreader.hpp"

//-----------------------------------------------
dxml::CharReader::CharReader (CBuffer buffer):
m_Buffer (buffer),
m_Pos (0) {

	return;
}

dxml::CharReader::~CharReader () {
		
	m_KillBuffer ();
	return;
}
		
bool dxml::CharReader::m_IsBufferLiving () const {

	if (m_Buffer == NULL) return false;
	return true;
}
		
void dxml::CharReader::m_KillBuffer () {

	if ( m_IsBufferLiving () ) {

		DestroyCBuffer (m_Buffer);
		m_Buffer = NULL;
	}
	return;
}
		
bool dxml::CharReader::Fetch
(char& ch, Position& pos) {

	bool Success = true;
	bool Fault = false;

	if (! m_IsBufferLiving () ) {

		return Fault;
	}

	Result result = GetChar (m_Buffer);
	if ( HasItEof (result) ) return Fault;

	ch = GetCharFromResult (result);
	pos = m_Pos;
	++m_Pos;
	
	return Success;
}
//-----------------------------------------------

//-----------------------------------------------
dxml::CharReaderFromString::CharReaderFromString
(const char* str):
CharReader ( InitCBufferFromString (str) ) {

	return;
}

dxml::CharReaderFromString::CharReaderFromString
(const String& str):
CharReader ( InitCBufferFromString (str.c_str ()) ) {

	return;
}
		
dxml::CharReaderFromString::~CharReaderFromString () {

	return;
}
//-----------------------------------------------

//-----------------------------------------------
dxml::CharReaderFromStdin::CharReaderFromStdin ():
CharReader ( InitCBufferFromSTDIN () ) {

	return;
}

dxml::CharReaderFromStdin::~CharReaderFromStdin () {

	return;
}
//-----------------------------------------------

//-----------------------------------------------
dxml::CharReaderFromFile::CharReaderFromFile
(FILE* fd):
CharReader ( InitCBufferFromFD (fd) ) {

	return;
}

dxml::CharReaderFromFile::~CharReaderFromFile () {

	return;
}
//-----------------------------------------------
