/*
 * Copyright 2010 Boris Popov <popov.b@gmail.com>
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <cstdlib>
#include <string>
#include <cbuffer.h>
#include <cstdio>

#ifndef __charreader__hpp__
#define __charreader__hpp__

namespace dxml {

//-----------------------------------------------
typedef size_t Position;
typedef std::string String;
//-----------------------------------------------

//-----------------------------------------------
class CharReader {

	public:
		CharReader (CBuffer);
		virtual ~CharReader () = 0;

		bool Fetch (char&, Position&);

	private:
		CBuffer m_Buffer;
		bool m_IsBufferLiving () const;
		void m_KillBuffer ();

		Position m_Pos;

		CharReader (const CharReader&);
		CharReader& operator= (const CharReader&);
};
//-----------------------------------------------

//-----------------------------------------------
class CharReaderFromString: public CharReader {

	public:
		CharReaderFromString (const char*);
		CharReaderFromString (const String&);

		virtual ~CharReaderFromString ();

	private:
		CharReaderFromString
			(const CharReaderFromString&);
			
		CharReaderFromString& operator=
			(const CharReaderFromString&);
};
//-----------------------------------------------

//-----------------------------------------------
class CharReaderFromStdin: public CharReader {

	public:
		CharReaderFromStdin ();

		virtual ~CharReaderFromStdin ();

	private:
		CharReaderFromStdin
			(const CharReaderFromStdin&);
			
		CharReaderFromStdin& operator=
			(const CharReaderFromStdin&);
};
//-----------------------------------------------

//-----------------------------------------------
class CharReaderFromFile: public CharReader {

	public:
		CharReaderFromFile (FILE*);

		virtual ~CharReaderFromFile ();

	private:
		CharReaderFromFile
			(const CharReaderFromFile&);
			
		CharReaderFromFile& operator=
			(const CharReaderFromFile&);
};
//-----------------------------------------------
}
#endif /* __charreader__hpp__ */
