/*
 * Copyright 2010 Boris Popov <popov.b@gmail.com>
 */

#include "dirtyxml.hpp"

//-----------------------------------------------
dxml::Dirty::Dirty ():
m_Builder ("DirtyRoot", "DirtyLine", "Id"),
m_CharReader (NULL) {

	return;
}

dxml::Dirty::Dirty (const DirtyRoot dr,
const DirtyLine dl, const Id id):
m_Builder (dr, dl, id),
m_CharReader (NULL) {

	return;
}

dxml::Dirty::~Dirty () {

	m_KillCharReader ();
	return;
}
		
void dxml::Dirty::m_KillCharReader () {

	if (m_CharReader == NULL) return;
	delete m_CharReader;
	m_CharReader = NULL;
	return;
}
		
void dxml::Dirty::m_Init () {

	m_KillCharReader ();
}

void dxml::Dirty::m_MakeCharReaderFromString
(const char* str) {

	m_CharReader = new CharReaderFromString (str);
	return;
}
	
void dxml::Dirty::m_MakeCharReaderFromStdin () {

	m_CharReader = new CharReaderFromStdin ();
	return;
}

void dxml::Dirty::m_MakeCharReaderFromFile
(FILE* file) {

	m_CharReader = new CharReaderFromFile (file);
	return;
}

bool dxml::Dirty::Parse (myXML::Document& xmldoc) {

	m_Init ();
	m_MakeCharReaderFromStdin ();
	return m_BuildAndLoad (xmldoc);
}

bool dxml::Dirty::Parse (XmlString& xmlstr) {
	
	m_Init ();
	m_MakeCharReaderFromStdin ();
	return m_BuildString (xmlstr);
}
		
bool dxml::Dirty::Parse (FILE* file,
myXML::Document& xmldoc) {
	
	m_Init ();
	m_MakeCharReaderFromFile (file);
	return m_BuildAndLoad (xmldoc);
}

bool dxml::Dirty::Parse (FILE* file,
XmlString& xmlstr) {
	
	m_Init ();
	m_MakeCharReaderFromFile (file);
	return m_BuildString (xmlstr);
}
		
bool dxml::Dirty::Parse (const char* cstr,
myXML::Document& xmldoc) {
	
	m_Init ();
	m_MakeCharReaderFromString (cstr);
	return m_BuildAndLoad (xmldoc);
}
		
bool dxml::Dirty::m_BuildAndLoad
(myXML::Document& xmldoc) {
	
	XmlString xml;
	if (! m_BuildString (xml) )  return false;
	return xmldoc.LoadStringX (xml.c_str ());
}
		
bool dxml::Dirty::Parse (const char* cstr,
XmlString& xmlstr) {
	
	m_Init ();
	m_MakeCharReaderFromString (cstr);
	return m_BuildString (xmlstr);
}
		
bool dxml::Dirty::m_BuildString (XmlString& xmlstr) {
	
	return m_Builder.Build (*m_CharReader, xmlstr);
}
//-----------------------------------------------
