/*
 * Copyright 2010 Boris Popov <popov.b@gmail.com>
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <builder.hpp>
#include <charreader.hpp>
#include <myXML_Document.hpp>

#ifndef __dirtyxml_hpp__
#define __dirtyxml_hpp__

///namespace of dirty xml
namespace dxml {

//-----------------------------------------------
///Имя корневого тега
typedef String DirtyRoot;

///Имя тега с идентифицированной строкой
typedef String DirtyLine;

///Имя атрибута с порядковым номером строки
typedef String Id;

///Строка с xml
typedef String XmlString;
//-----------------------------------------------

//-----------------------------------------------
/// Разборщик "грязного" xml
class Dirty {

	public:
		//@{
		/// Создание разборщика
		Dirty ();
		/// Создание разборщика
		/**
		  Здесь указываются необходимый корневой тег,
		  тег строки и имя атрибута счётчика.
		  */
		Dirty (const DirtyRoot, const DirtyLine,
				const Id);
		//@}

		~Dirty ();

		//@{
		/// Парсим из потока стандартного ввода
		/**

		  В случае успеха, результат в
		  объекте myXML::Document&.

		  */
		bool Parse (myXML::Document&);
		/// Парсим из потока стандартного ввода
		/**

		  В случае успеха, результат в строке.

		  */
		bool Parse (XmlString&);
		//@}
		
		//@{
		/// Парсим из файла
		/**

		  В случае успеха, результат в
		  объекте myXML::Document&.

		  */
		bool Parse (FILE*, myXML::Document&);
		/// Парсим из файла
		/**

		  В случае успеха, результат в строке.

		  */
		bool Parse (FILE*, XmlString&);
		//@}
	
		//@{
		///	Парсим из C-строки	
		/**

		  В случае успеха, результат в
		  объекте myXML::Document&.

		  */
		bool Parse (const char*, myXML::Document&);
		///	Парсим из C-строки	
		/**

		  В случае успеха, результат в строке.

		  */
		bool Parse (const char*, XmlString&);
		//@}

	private:
		Dirty (const Dirty&);
		Dirty& operator= (const Dirty&);

		Builder m_Builder;

		CharReader* m_CharReader;
		void m_KillCharReader ();
		void m_MakeCharReaderFromString (const char*);
		void m_MakeCharReaderFromStdin ();
		void m_MakeCharReaderFromFile (FILE*);

		void m_Init ();
		bool m_BuildAndLoad (myXML::Document&);
		bool m_BuildString (XmlString&);
};
//-----------------------------------------------
}
#endif /* __dirtyxml_hpp__ */


/**
  \mainpage DirtyXML
  
  \author Борис Попов

  \section about Краткое описание

  Библиотека предназначена для вытаскивания из текста,
  файла, потока ввода корректной xml структуры (при
  её наличии там).

  Пример:
  \code
  xwewl
  asd
  <root />
  asd
  sad
  <root>dsf</root>kjlfsd
  \endcode

  Из этого примера библиотека вытащит две
  корректные xml структуры:

  \code
  <root />
  \endcode
  \code
  <root>dsf</root>
  \endcode

  Вытащенные структуры могут быть представлены
  как текст, либо как внуртенняя древовидная
  структура.

  \warning
  В компаундном тексте очень желательно
  отсутствие угловых скобок.
  */
