/*
 * Copyright 2008, 2010  Boris Popov <popov.b@gmail.com>
 */

#include "myXML_BuildOfTree.hpp"

//-----------------------------------------------
myXML::BuildOfTree::BuildOfTree()
{
    zero_xmldeclar = true; //нет декларации
    root = NULL; //нет root
    last_container = root;
    message = "";
    last_in_declar = false; //последний - декларация
    xmldeclar = NULL;
    return;
}

//-----------------------------------------------

//-----------------------------------------------
void
myXML::BuildOfTree::make_open_teg(String t)
{
    if ((root != 0) && (container.empty())) {
        //корень закрылся
        message = message +
            "more than one root element: " +
            t + "\n";
        return;
    }
    last_in_declar = false;
    myXML::Contain* pC;
    pC = new myXML::Contain();
    (pC->Content) = t;
    if (root == 0) {
        root = pC;
    }
    last_container = pC;
    if (!container.empty()) {
        //тогда сцепить
        myXML::Contain* pParent;
        pParent = container.top();
        (*pParent) << (*pC);
    }
    container.push(pC);
    return;
}

//-----------------------------------------------

//-----------------------------------------------
void
myXML::BuildOfTree::make_close_teg(String t)
{
    if ((root == 0) ||
        (container.empty()) ||
        (t != ((container.top())->Content))) {
        message = message +
            "close teg in an unknown place: " +
            t + "\n";
        return;
    }
    container.pop();
    last_container = 0;
    return;
}

//-----------------------------------------------

//-----------------------------------------------
void
myXML::BuildOfTree::make_null_teg(String t)
{
    if ((root != 0) && (container.empty())) {
        //корень закрылся
        message = message +
            "more than one root element: " +
            t + "\n";
        return;
    }
    last_in_declar = false;
    myXML::Contain* pC;
    pC = new myXML::Contain();
    (pC->Content) = t;
    if (root == 0) {
        root = pC;
    }
    last_container = pC;
    if (!container.empty()) {
        //тогда сцепить
        myXML::Contain* pParent;
        pParent = container.top();
        (*pParent) << (*pC);
    }
    return;
}

//-----------------------------------------------

//-----------------------------------------------
void
myXML::BuildOfTree::make_text(String t)
{
    if ((root == 0) || (container.empty())) {
        message = message +
            "text in an unknown place: " +
            t + "\n";

    }
    myXML::Text* pT;
    pT = new myXML::Text();
    pT->Content = t;
    myXML::Contain* pC;
    pC = container.top();
    (*pC) << (*pT);
    last_container = 0;
    return;
}

//-----------------------------------------------

//-----------------------------------------------
void
myXML::BuildOfTree::put_att(String a,
                            String v)
{
    if ((last_container == 0) &&
        (root == 0) &&
        (container.empty())) {
        //в зоне пролога
        if (last_in_declar) {

            (xmldeclar->GetAttributes()).AddAtt(a, v);
        }

    }
    else {

        //в зоне боди
        (last_container->GetAttributes()).AddAtt(a, v);
    }
    return;
}

//-----------------------------------------------

#include <iostream>

void
myXML::BuildOfTree::make_decl()
{
    if (root != 0) {
        message = message +
            "declaration in an unknown place\n";
        std::cerr << "declaration in an unknown place\n";
        return;
    }
    if (zero_xmldeclar) {
        //создать
        xmldeclar = new myXML::Declar;
        zero_xmldeclar = false;
        last_in_declar = true;
    }
    else {
        //отказать и уведомит
        message = message +
            "more than one of declaration\n";
    }
    return;
}

//-----------------------------------------------

//-----------------------------------------------
myXML::BuildOfTreeResult
myXML::BuildOfTree::get()
{
    BuildOfTreeResult rez;
    rez.declar = xmldeclar;
    if (root == 0) {
        message = message +
            "root element is not found" +
            "\n";
    }
    rez.cont = root;
    rez.messg = &message;
    return rez;
}

//-----------------------------------------------

void myXML::BuildOfTree::Make_open_teg(const char* t)
{
    make_open_teg
        (clear_from_delim(String(t)));
    return;
}

void myXML::BuildOfTree::Make_close_teg(const char* t)
{
    make_close_teg
        (clear_from_delim(String(t)));
    return;
}

void myXML::BuildOfTree::Make_text(const char* t)
{
    String text1(t);
    text1 = clear_from_delim(String(text1));
    String text2 = m_fMirror.Fetch(text1);
    make_text(text2);
    return;
}

void myXML::BuildOfTree::Put_att(AttValue av)
{
    String at1(String(av.attribute));
    String va1(String(av.value));
    at1 = clear_from_delim(at1);
    va1 = clear_from_delim(va1);
    String va2 = m_fMirror.Fetch(va1);
    put_att(at1, va2);
    return;
}

void myXML::BuildOfTree::Make_decl()
{
    make_decl();
    return;
}

//-----------------------------------------------
bool
myXML::BuildOfTree::m_IsItDelimiter(const char ch)
{
    bool DELIMITER = true;
    bool NO_DELIMITER = false;
    switch (ch) {
    case ' ':
        return DELIMITER;
        break;
    case '\n':
        return DELIMITER;
        break;
    case '\t':
        return DELIMITER;
        break;
    case '\f':
        return DELIMITER;
        break;
    case '\r':
        return DELIMITER;
        break;
    default:
        return NO_DELIMITER;
    }
}

//-----------------------------------------------

//-----------------------------------------------
myXML::String myXML::BuildOfTree::clear_from_delim(String what)
{
    /* удалить разделители
     * в начале и конце what
     */

    String temp;
    size_t pos = 0;
    size_t start = 0;
    size_t end = what.size() - 1;
    const char* my_string = what.c_str();
    while (m_IsItDelimiter(my_string[pos])) {
        pos++;
    }
    start = pos; //начало без разделителей

    pos = end;
    while (m_IsItDelimiter(my_string[pos])) {
        pos--;
    }
    end = pos; //конец без разделителей
    temp.assign(what, start, end - start + 1);
    return temp;
}

//-----------------------------------------------

//-----------------------------------------------
void myXML::BuildOfTree::Put_message(const char* t)
{
    message = message + String(t);
    return;
}

//-----------------------------------------------
