/*
 * Copyright 2008, 2009, 2010 Boris Popov <popov.b@gmail.com>
 */

#include "myXML_Document.hpp"
#include "myXML_Mirroring.hpp"


//-----------------------------------------------
myXML::Contain::Contain() :
    myXML::Node(myXML::Document::Container)
{
    place = in;
    return;
}

//-----------------------------------------------

//-----------------------------------------------
myXML::Contain::Contain(String str) :
    myXML::Node(myXML::Document::Container)
{
    place = out;
    Set(str);
    return;
}

//-----------------------------------------------

//-----------------------------------------------
myXML::Contain::~Contain()
{
    if (!ListOfChildren.empty()) {
        std::list<myXML::Node*>::iterator p;
        p = ListOfChildren.begin();
        while (p != ListOfChildren.end()) {
            delete (*p);
            p++;
        }
        ListOfChildren.clear();
    }
    return;
}

//-----------------------------------------------

//-----------------------------------------------
myXML::BodyIter myXML::Contain::BeginBodyIter()
{
    myXML::Node* pN;
    if (ListOfChildren.empty()) {
        pN = 0;
    }
    else {
        pN = this;
    }
    myXML::BodyIter XBI(pN);
    if (!ListOfChildren.empty()) {
        ++XBI;
    }
    return XBI;
}

//-----------------------------------------------

//-----------------------------------------------
myXML::BodyIter myXML::Contain::EndBodyIter()
{
    myXML::Node* pN = 0;
    myXML::BodyIter XBI(pN);
    return XBI;
}

//-----------------------------------------------

//-----------------------------------------------
myXML::Contain& myXML::Contain::operator <<(Contain& e)
{
    //
    //возможно два варианта:
    //e внутри или снаружи.
    //
    if (e.place == in) {
        //сцепить
        ListOfChildren.push_back(&e);
        e.pParent = this;
    }
    else {
        //справа - контейнер снаружи
        //создать контейнер
        Contain* xmlspr = new Contain;
        xmlspr->Set(e.Get());
        (xmlspr->place) = in;
        //перенести атрибуты
        if (e.GetAttributes().HasItAttr()) {

            (xmlspr->GetAttributes()) = e.GetAttributes();
        }
        //перенос указателей///////////////////////////
        //
        //
        if (!e.ListOfChildren.empty()) {
            std::list<myXML::Node*>::iterator p;
            p = (e.ListOfChildren).begin();
            while (p != (e.ListOfChildren).end()) {
                (xmlspr->ListOfChildren).push_back(*p);
                (*p)->pParent = xmlspr;
                p++;
            }
            e.ListOfChildren.clear();
        }
        //
        ///////////////////////////////////////////////

        //сцепить
        ListOfChildren.push_back(xmlspr);
        (xmlspr->pParent) = this;
    }
    return *this;
}

//-----------------------------------------------

//-----------------------------------------------
myXML::Contain& myXML::Contain::operator <<(Text& t)
{
    if (t.place == in) {
        //сцепить
        ListOfChildren.push_back(&t);
        t.pParent = this;
    }
    else {
        Text* xmlspr = new Text;
        xmlspr->Set(t.Get());
        (xmlspr->place) = in;
        //сцепить
        ListOfChildren.push_back(xmlspr);
        (xmlspr->pParent) = this;
    }
    return *this;
}

//-----------------------------------------------

//-----------------------------------------------
myXML::String myXML::Contain::build()
{
    String shift;
    String akkum = "";
    size_t lev = GetLevel();
    for (size_t i = 0; i < lev; i++) {
        shift = shift + "    ";
    }
    // Для каждого ребёнка
    // вызываем этот метод
    if (!ListOfChildren.empty()) {
        std::list<myXML::Node*>::iterator p;
        p = ListOfChildren.begin();
        while (p != ListOfChildren.end()) {
            akkum = akkum + ((*p)->build());
            p++;
        }
    }
    String one_tag;
    String two_tag;
    if (ListOfChildren.empty()) {

        // нулевой тег
        if ((GetAttributes()).HasItAttr()) {

            // есть атрибуты
            String att;
            att = get_string_attr();
            one_tag = shift + "<" + Get() + att + " />\n";
        }
        else {
            one_tag = shift + "<" + Get() + " />\n";
        }
        return one_tag;
    }
    else {
        // полный тег
        two_tag = shift + "</" + Get() + ">\n";

        if ((GetAttributes()).HasItAttr()) {
            String att;
            att = get_string_attr();
            one_tag = shift + "<" + Get() + att + ">\n";
        }
        else {
            one_tag = shift + "<" + Get() + ">\n";
        }
        return one_tag + akkum + two_tag;
    }
}

//-----------------------------------------------

//-----------------------------------------------
myXML::String myXML::Contain::get_string_attr()
{
    mr::ToMirror tMirror;
    String out;

    AttrIter it = GetAttributes().Begin();
    AttrIter end = GetAttributes().End();

    while (it != end) {
        String attstring = (it.Get()).val;
        out = out + " " + (it.Get()).att + "=\"" +
            tMirror.Fetch(attstring)
            + "\"";
        ++it;
    }
    return out;
}

//-----------------------------------------------

//-----------------------------------------------
myXML::ChildIter myXML::Contain::BeginChildIter()
{
    ChildIter it;
    it.m_IOC = ListOfChildren.begin();
    return it;
}

//-----------------------------------------------

//-----------------------------------------------
myXML::ChildIter myXML::Contain::EndChildIter()
{
    ChildIter it;
    it.m_IOC = ListOfChildren.end();
    return it;
}

//-----------------------------------------------

myXML::Attributes& myXML::Contain::GetAttributes()
{
    return m_Attributes;
}
