/*
 * Copyright 2008, 2009, 2010  Boris Popov <popov.b@gmail.com>
 */

#include "myXML_Parser.hpp"

//myXML::BuildOfTree* XMLBOT;

//-----------------------------------------------
myXML::Parser::Parser(const char* ch) :

    m_InputString(ch)
{
    message = "";
    m_InitResult();
    return;
}

myXML::Parser::~Parser()
{
    return;
}

myXML::Parser::Parser() :

    m_InputString(NULL)
{
    message = "";
    m_InitResult();
    return;
}

//-----------------------------------------------

//-----------------------------------------------
void XML_PrintText(Text text, UserData* data)
{
    myXML::BuildOfTree* XMLBOT = (myXML::BuildOfTree*)data;
    XMLBOT->Make_text(text);
    return;
}

void XML_PrintAtt(AttValue av, UserData* data)
{
    myXML::BuildOfTree* XMLBOT = (myXML::BuildOfTree*)data;
    XMLBOT->Put_att(av);
    return;
}

void XML_PrintOpen(Text text, UserData* data)
{
    myXML::BuildOfTree* XMLBOT = (myXML::BuildOfTree*)data;
    XMLBOT->Make_open_teg(text);
    return;
}

void XML_PrintClose(Text text, UserData* data)
{
    myXML::BuildOfTree* XMLBOT = (myXML::BuildOfTree*)data;
    XMLBOT->Make_close_teg(text);
    return;
}

void XML_PrintD(UserData* data)
{
    myXML::BuildOfTree* XMLBOT = (myXML::BuildOfTree*)data;
    XMLBOT->Make_decl();
    return;
}

void XML_PrintE(Text text, UserData* data)
{
    myXML::BuildOfTree* XMLBOT = (myXML::BuildOfTree*)data;
    XMLBOT->Put_message(text);
    return;
}

//-----------------------------------------------

//-----------------------------------------------
myXML::ParserResult myXML::Parser::m_GetFromStdin()
{
    CBuffer buffer = InitCBufferFromSTDIN();
    if (buffer == NULL) {

        m_Result.messg = "Impossible to create a parsing buffer!\n";
        return m_Result;
    }

    EasyParser parser =
        InitEasyParser(buffer, &XML_PrintText, &XML_PrintOpen,
                       &XML_PrintClose, &XML_PrintAtt, &XML_PrintD, &XML_PrintE);

    if (parser == NULL) {

        m_Result.messg = "Impossible to create a sax parser!\n";
        DestroyCBuffer(buffer);
        return m_Result;
    }

    if (!Parse(parser, &xbot)) {

        XML_PrintE("Error of sax parsing!\n", &xbot);
    }

    DestroyEasyParser(parser);
    DestroyCBuffer(buffer);

    BuildOfTreeResult tr = xbot.get();
    m_Result.declar = tr.declar;
    m_Result.cont = tr.cont;
    m_Result.messg = message + (*(tr.messg));
    return m_Result;
}

myXML::ParserResult myXML::Parser::GetFromFile()
{
    if (m_InputString == NULL) {

        return m_GetFromStdin();
    }

    FILE* f = fopen(m_InputString, "rb");
    if (f == NULL) {

        m_Result.messg = "Impossible to open file!\n";
        return m_Result;
    }

    CBuffer buffer = InitCBufferFromFD(f);
    if (buffer == NULL) {

        fclose(f);
        m_Result.messg = "Impossible to create a parsing buffer!\n";
        return m_Result;
    }

    EasyParser parser =
        InitEasyParser(buffer, &XML_PrintText, &XML_PrintOpen,
                       &XML_PrintClose, &XML_PrintAtt, &XML_PrintD, &XML_PrintE);

    if (parser == NULL) {

        m_Result.messg = "Impossible to create a sax parser!\n";
        DestroyCBuffer(buffer);
        return m_Result;
    }

    if (!Parse(parser, &xbot)) {

        XML_PrintE("Error of sax parsing!\n", &xbot);
    }

    DestroyEasyParser(parser);
    DestroyCBuffer(buffer);
    fclose(f);

    BuildOfTreeResult tr = xbot.get();
    m_Result.declar = tr.declar;
    m_Result.cont = tr.cont;
    m_Result.messg = message + (*(tr.messg));
    return m_Result;
}

myXML::ParserResult myXML::Parser::Get()
{
    if (m_InputString == NULL) {

        return m_GetFromStdin();
    }

    CBuffer buffer = InitCBufferFromString(m_InputString);
    if (buffer == NULL) {

        m_Result.messg = "Impossible to create a parsing buffer!\n";
        return m_Result;
    }

    EasyParser parser =
        InitEasyParser(buffer, &XML_PrintText, &XML_PrintOpen,
                       &XML_PrintClose, &XML_PrintAtt, &XML_PrintD, &XML_PrintE);

    if (parser == NULL) {

        m_Result.messg = "Impossible to create a sax parser!\n";
        DestroyCBuffer(buffer);
        return m_Result;
    }

    if (!Parse(parser, &xbot)) {

        XML_PrintE("Error of sax parsing!\n", &xbot);
    }

    DestroyEasyParser(parser);
    DestroyCBuffer(buffer);

    BuildOfTreeResult tr = xbot.get();
    m_Result.declar = tr.declar;
    m_Result.cont = tr.cont;
    m_Result.messg = message + (*(tr.messg));
    return m_Result;
}

void myXML::Parser::m_InitResult()
{
    m_Result.declar = NULL;
    m_Result.cont = NULL;
    m_Result.messg = "";
    return;
}

//-----------------------------------------------
